@@ -40,7 +40,8 @@ const char* validNames[] =
 {
     "de",
     "den",
-    "van"
+    "van",
+    "von"
 };
 
 const char* invalidNames[] =
@@ -100,6 +101,7 @@ const char* invalidNames[] =
     "GraphicsContextImplementation",
     "GraphicsThread",
     "GraphicsWindowX11",
+    "GraphicsHandleX11",
     "Group",
     "Image",
     "ImageOptions",
@@ -168,6 +170,7 @@ const char* invalidNames[] =
     "Quake3",
     "Quicktime",
     "ReaderWriters",
+    "ReaderWriterTXP",
     "ReentrantMutex",
     "Referenced",
     "ReferenceFrame",
@@ -184,10 +187,12 @@ const char* invalidNames[] =
     "SimpleViewer",
     "SmokeTrailEffect",
     "Source",
+    "SoftImage",
     "SpotExponent",
     "State",
     "StateAttribute",
     "Stats",
+    "StateSet",
     "StatsVisitor",
     "Studio",
     "Support",
@@ -197,6 +202,7 @@ const char* invalidNames[] =
     "TessellationHints",
     "TestManipulator",
     "TestSupportCallback",
+    "Text",
     "TexEnv",
     "TexMat(Matrix",
     "Texture",
@@ -231,7 +237,17 @@ const char* invalidNames[] =
     "Windows",
     "X",
     "Xcode",
-    "Y"
+    "GraphicsWindowWin32",
+    "ImageLayer",
+    "Xml",
+    "Y",
+    "UpdateSkeleton",
+    "IncementalCompileOperation",
+    "Billboard",
+    "UpdateSkeleton",
+    "Vec3",
+    "Plugin",
+    "Get"
 };
 
 
@@ -243,6 +259,7 @@ struct TypoCorrection
 
 TypoCorrection typoCorrections[] =
 {
+    {"Alaxandre",      "Alexandre"},
     {"Aderian",        "Adrian"},
     {"Adndre",         "Andre"},
     {"Adrain",         "Adrian"},
@@ -291,15 +308,20 @@ TypoCorrection typoCorrections[] =
     {"Heirtlein",      "Hertlein"},
     {"Heirtlein",      "Hertlein"},
     {"Hertleinm",      "Hertlein"},
+    {"Happalahti",     "Haapalahti"},
     {"Hertlien",       "Hertlein"},
+    {"Hatwig",         "Hartwig"},
     {"Hi",             "He"},
     {"Hooper",         "Hopper"},
     {"Inverson",       "Iverson"},
     {"Iversion",       "Iverson"},
-    {"Jean-Sebastein", "Jean-Sebastien"},
-    {"Jean-Sebastian", "Jean-Sebastien"},
-    {"Jean-Sebastirn", "Jean-Sebastien"},
-    {"Jea-Sebastien",  "Jean-Sebastien"},
+    {"Jean-Sebastein", "Jean-Sbastien"},
+    {"Jean-Sebastian", "Jean-Sbastien"},
+    {"Jean-Sebastirn", "Jean-Sbastien"},
+    {"Jea-Sebastien",  "Jean-Sbastien"},
+    {"Jean-Sebasien",  "Jean-Sbastien"},
+    {"Jean-Sebastien", "Jean-Sbastien"},
+    {"Jean-Sebestien", "Jean-Sbastien"},
     {"Johhansen",      "Johansen"},
     {"Johnansen",      "Johansen"},
     {"Johnasen",       "Johansen"},
@@ -310,6 +332,7 @@ TypoCorrection typoCorrections[] =
     {"J",              "Jos"},
     {"Keuhne",         "Kuehne"},
     {"Kheune",         "Kuehne"},
+    {"Krulthof",       "Kruithof"},
     {"Lagrade",        "Lagarde"},
     {"Largade",        "Lagarde"},
     {"Larshkari",      "Lashkari"},
@@ -319,14 +342,19 @@ TypoCorrection typoCorrections[] =
     {"Laskari",        "Lashkari"},
     {"Leandowski",     "Lewandowski"},
     {"Lawandowski",    "Lewandowski"},
-    {"Lucas",          "Luaces"},
+    {"Luacas",         "Luaces"},
+    {"Luacas",         "Luaces"},
+    {"Liech",          "Leich"},
+    {"Lownman",        "Lowman"},
     {"Lugi",           "Luigi"},
     {"Lweandowski",    "Lewandowski"},
+    {"Lavingotte",     "Lavignotte"},
     {"Maceij",         "Maciej"},
     {"Machler",        "Mchler"},
     {"Macro",          "Marco"},
     {"Maechler",       "Mchler"},
     {"Mahai",          "Mihai"},
+    {"Magnes",          "Magnus"},
     {"Mammond",        "Marmond"},
     {"March",          "Marco"},
     {"Martsz",         "Martz"},
@@ -346,17 +374,23 @@ TypoCorrection typoCorrections[] =
     {"Narache",        "Marache"},
     {"Nicklov",        "Nikolov"},
     {"Nickolov",       "Nikolov"},
+    {"Nilson",         "Nilsson"},
     {"Olad",           "Olaf"},
     {"Olar",           "Olaf"},
     {"Oritz",          "Ortiz"},
     {"Osfeld",         "Osfield"},
     {"Osfied",         "Osfield"},
     {"Pail",           "Paul"},
+    {"Perciva",        "Peciva"},
+    {"Pecvia",         "Peciva"},
+    {"Priyadashi",     "Priyadarshi"},
     {"Rajce",          "Trajce"},
+    {"Raymon",         "Raymond"},
     {"Randal",         "Randall"},
     {"Robet",          "Robert"},
     {"Rodger",         "Roger"},
     {"Rolad",          "Roland"},
+    {"Riddel",         "Riddell"},
     {"Rucard",         "Richard"},
     {"Sekender",       "Sukender"},
     {"Sewel",          "Sewell"},
@@ -365,6 +399,7 @@ TypoCorrection typoCorrections[] =
     {"Sjolie",         "Sjlie"},
     {"Sjölie",        "Sjlie"},
     {"Skinnder",       "Skinner"},
+    {"Seberion",       "Seberino"},
     {"Sokolosky",      "Sokolowsky"},
     {"Sokolowky",      "Sokolowsky"},
     {"Sokolowki",      "Sokolowsky"},
@@ -389,10 +424,23 @@ TypoCorrection typoCorrections[] =
     {"Wieblen",        "Weiblen"},
     {"Woesnner",       "Woessner"},
     {"Wojchiech",      "Wojciech"},
+    {"Wojcoech",       "Wojciech"},
     {"Wojiech",        "Wojciech"},
     {"Xennon",         "Hanson"},
     {"Yefrei",         "Yefei"},
-    {"Yfei",           "Yefei"}
+    {"Yfei",           "Yefei"},
+    {"Gurhrie",        "Guthrie"},
+    {"Byran",          "Bryan"},
+    {"Fielder",        "Fiedler"},
+    {"Mathia",         "Mathias"},
+    {"Halgarth",       "Hogarth"},
+    {"Katherina",      "Katharina"},
+    {"Biyfarguine",    "Boufarguine"},
+    {"Dickenson",       "Dickinson"},
+    {"Jahannes","Johannes"},
+    {"Eskland","Ekstrand"},
+    {"Baeuerele","Baeuerle"},
+    {"Bauerle","Baeuerle"}
 };
 
 
@@ -406,10 +454,54 @@ struct NameCorrection
 
 NameCorrection nameCorrections[] =
 {
+    {"Philip","Lamp",
+     "Philip", "Lamb"},
+    {"Dimi","Christop",
+     "Dimi", "Christopoulos"},
+    {"Jorge","Ciges",
+     "Jorge", "Izquierdo"},
+    {"Rafa","Gata",
+     "Rafa", "Gaitan"},
+    {"Sukender","I",
+     "Sukender", ""},
+    {"Sukender","Here",
+     "Sukender", ""},
+    {"Ewe","Woessner",
+     "Uwe", "Woessner"},
+    {"Martin","von",
+     "Martin", "von Gargern"},
+    {"Alexandre","Irion",
+     "Alexander", "Irion"},
+    {"WojciechLewandowski","",
+     "Wojciech", "Lewandowski"},
+    {"Marin",          "Platings",
+     "Michael",        "Platings"},
+    {"Tomas",          "Holgarth",
+     "Thomas",         "Hogarth"},
+    {"Marin",          "Lavery",
+     "Martin",         "Lavery"},
+    {"Nguyen",         "Van",
+     "Nguyen",         "Van Truong"},
+    {"Thom",           "Carlo",
+     "Thom",           "DeCarlo"},
+    {"Stephan",        "Lamoliatte",
+     "Stephane",       "Lamoliatte"},
+    {"Ronald",         "van",
+     "Ronald",         "van Maarseveen"},
+    {"Lee",            "Bulter",
+     "Lee",            "Butler"},
+    {"Tery",           "Welsh",
+     "Terry",          "Welsh"},
+    {"Cesar",          "L",
+     "Csar",          "L. B. Silveira"},
     {"Marc",           "Sciabica",
      "Mark",           "Sciabica"},
+    {"Tom",            "Moore",
+     "Tim",            "Moore"},
+    {"Time",            "Moore",
+     "Tim",            "Moore"},
     {"Jean",           "Sebastien",
-     "Jean-Sebastien", "Guay"},
+     "Jean-Sbastien", "Guay"},
     {"Michale",        "Platings",
      "Michael",        "Platings"},
     {"Mick",           "",
@@ -492,6 +584,12 @@ NameCorrection nameCorrections[] =
      "Trajce",         "Nikolov"},
     {"Frauciel",       "Luc",
      "Luc",            "Frauciel"},
+    {"Alberto",        "Lucas",
+     "Alberto",        "Luaces"},
+    {"Tyge",           "",
+     "Tyge",           "Lvset"},
+    {"Ricard",         "Schmidt",
+     "Richard",        "Schmidt"}
 };
 
 
