@@ -1,7 +1,24 @@
 // Geo demo written by Geoff Michel, November 2002.
+/* OpenSceneGraph example, osggeodemo.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
 
 #include <stdio.h>
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osg/Node>
 #include <osg/Notify>
@@ -20,6 +37,8 @@
 
 #include <osgUtil/Optimizer>
 
+#include <iostream>
+
 // currently not a satisfactory solution, but this is early days for the
 // geo loader and having direct links with it. 
 #include "../../src/osgPlugins/geo/osgGeoAnimation.h"
@@ -41,11 +60,7 @@ public:
         return false;
         
     }
-    
-    virtual void accept(osgGA::GUIEventHandlerVisitor& v)
-    {
-        v.visit(*this);
-    }
+
     inline float getMouseX(void) {return mouse_x;}; 
     inline float getMouseY(void) {return mouse_y;};
     
@@ -88,52 +103,14 @@ int main( int argc, char **argv )
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
 
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example of how to control animation in Geo files.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-    arguments.getApplicationUsage()->addCommandLineOption("--fov <float>","Set the Field of View");
-   
-    float camera_fov=-1;
-    while (arguments.read("--fov",camera_fov)) {}
-
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-    
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
-
+    osgViewer::Viewer viewer;
 
     // load the nodes from the commandline arguments.
     osg::Node* rootnode = osgDB::readNodeFiles(arguments);
     if (!rootnode)
     {
+        osg::notify(osg::NOTICE)<<"Please specify a geo model filename on the command line."<<std::endl;
         return 1;
     }
     
@@ -149,7 +126,7 @@ int main( int argc, char **argv )
     { // it is a geo file, so set function to update its animation variables.
         ghand=new geodemoEventHandler();
         gh->setUserUpdate(dodynamics);
-        viewer.getEventHandlerList().push_front(ghand);
+        viewer.addEventHandler(ghand);
     }
     else
     { // maybe a group with geo models below.
@@ -165,37 +142,11 @@ int main( int argc, char **argv )
                 {
                     ghand=new geodemoEventHandler();
                     gh->setUserUpdate(dodynamics);
-                    viewer.getEventHandlerList().push_front(ghand);
+                    viewer.addEventHandler(ghand);
                 }
             }
         }
     }
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
