@@ -31,13 +31,16 @@
 #include <osg/PolygonOffset>
 #include <osg/MatrixTransform>
 #include <osg/Camera>
+#include <osg/RenderInfo>
+
+#include <osgDB/WriteFile>
 
 #include <osgText/Text>
 
 
 osg::Camera* createHUD()
 {
-    // create a camera to set up the projection and model view matrices, and the subgraph to drawn in the HUD
+    // create a camera to set up the projection and model view matrices, and the subgraph to draw in the HUD
     osg::Camera* camera = new osg::Camera;
 
     // set the projection matrix
@@ -65,7 +68,7 @@ osg::Camera* createHUD()
 
         std::string timesFont("fonts/arial.ttf");
 
-        // turn lighting off for the text and disable depth test to ensure its always ontop.
+        // turn lighting off for the text and disable depth test to ensure it's always ontop.
         osg::StateSet* stateset = geode->getOrCreateStateSet();
         stateset->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
 
@@ -138,7 +141,7 @@ osg::Camera* createHUD()
             text->setFont(timesFont);
             text->setPosition(position);
             text->setText("And finally set the Camera's RenderOrder to POST_RENDER\n"
-                          "to make sure its drawn last.");
+                          "to make sure it's drawn last.");
 
             position += delta;
         }    
@@ -187,6 +190,81 @@ osg::Camera* createHUD()
     return camera;
 }
 
+struct SnapImage : public osg::Camera::DrawCallback
+{
+    SnapImage(const std::string& filename):
+        _filename(filename),
+        _snapImage(false)
+    {
+        _image = new osg::Image;        
+    }
+
+    virtual void operator () (osg::RenderInfo& renderInfo) const
+    {
+
+        if (!_snapImage) return;
+    
+        osg::notify(osg::NOTICE)<<"Camera callback"<<std::endl;
+
+        osg::Camera* camera = renderInfo.getCurrentCamera();
+        osg::Viewport* viewport = camera ? camera->getViewport() : 0;
+
+        osg::notify(osg::NOTICE)<<"Camera callback "<<camera<<" "<<viewport<<std::endl;
+
+        if (viewport && _image.valid())
+        {
+            _image->readPixels(int(viewport->x()),int(viewport->y()),int(viewport->width()),int(viewport->height()),
+                               GL_RGBA,
+                               GL_UNSIGNED_BYTE);
+            osgDB::writeImageFile(*_image, _filename);
+            
+            osg::notify(osg::NOTICE)<<"Taken screenshot, and written to '"<<_filename<<"'"<<std::endl;             
+        }
+       
+        _snapImage = false;
+    }
+
+    std::string                         _filename;
+    mutable bool                        _snapImage;
+    mutable osg::ref_ptr<osg::Image>    _image;
+};
+
+struct SnapeImageHandler : public osgGA::GUIEventHandler
+{
+
+    SnapeImageHandler(int key,SnapImage* si):
+        _key(key),
+        _snapImage(si) {}
+
+    bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter&)
+    {
+        if (ea.getHandled()) return false;
+
+        switch(ea.getEventType())
+        {
+            case(osgGA::GUIEventAdapter::KEYUP):
+            {
+                if (ea.getKey() == _key)
+                {
+                    osg::notify(osg::NOTICE)<<"event handler"<<std::endl;
+                    _snapImage->_snapImage = true;
+                    return true;
+                }
+
+                break;
+            }
+        default:
+            break;
+        }
+
+        return false;
+    }
+    
+    int                     _key;
+    osg::ref_ptr<SnapImage> _snapImage;
+};
+
+
 int main( int argc, char **argv )
 {
     // use an ArgumentParser object to manage the program arguments.
@@ -196,8 +274,8 @@ int main( int argc, char **argv )
     // read the scene from the list of file specified commandline args.
     osg::ref_ptr<osg::Node> scene = osgDB::readNodeFiles(arguments);
     
-    // if not loaded assume no arguments passed in, try use default mode instead.
-    if (!scene) scene = osgDB::readNodeFile("dumptruck.osg");
+    // if not loaded assume no arguments passed in, try use default model instead.
+    if (!scene) scene = osgDB::readNodeFile("dumptruck.osgt");
     
     
     if (!scene)
@@ -223,7 +301,7 @@ int main( int argc, char **argv )
         
         osg::Camera* hudCamera = createHUD();
         
-        // set up cameras to rendering on the first window available.
+        // set up cameras to render on the first window available.
         hudCamera->setGraphicsContext(windows[0]);
         hudCamera->setViewport(0,0,windows[0]->getTraits()->width, windows[0]->getTraits()->height);
 
@@ -257,7 +335,7 @@ int main( int argc, char **argv )
         
         osg::Camera* hudCamera = createHUD();
         
-        // set up cameras to rendering on the first window available.
+        // set up cameras to render on the first window available.
         hudCamera->setGraphicsContext(windows[0]);
         hudCamera->setViewport(0,0,windows[0]->getTraits()->width, windows[0]->getTraits()->height);
         
@@ -273,6 +351,14 @@ int main( int argc, char **argv )
     {
         // construct the viewer.
         osgViewer::Viewer viewer;
+        
+        SnapImage* postDrawCallback = new SnapImage("PostDrawCallback.png");
+        viewer.getCamera()->setPostDrawCallback(postDrawCallback);        
+        viewer.addEventHandler(new SnapeImageHandler('p',postDrawCallback));
+        
+        SnapImage* finalDrawCallback = new SnapImage("FinalDrawCallback.png");
+        viewer.getCamera()->setFinalDrawCallback(finalDrawCallback);        
+        viewer.addEventHandler(new SnapeImageHandler('f',finalDrawCallback));
 
         osg::ref_ptr<osg::Group> group  = new osg::Group;
 
