@@ -22,8 +22,8 @@
 #include <osg/CoordinateSystemNode>
 #include <osgText/Text>
 
-#include <osgManipulator/CommandManager>
 #include <osgManipulator/TabBoxDragger>
+#include <osgManipulator/TabBoxTrackballDragger>
 #include <osgManipulator/TabPlaneDragger>
 #include <osgManipulator/TabPlaneTrackballDragger>
 #include <osgManipulator/TrackballDragger>
@@ -53,6 +53,12 @@ osgManipulator::Dragger* createDragger(const std::string& name)
         d->setupDefaultGeometry();
         dragger = d;
     }
+    else if ("TabBoxTrackballDragger" == name)
+    {
+        osgManipulator::TabBoxTrackballDragger* d = new osgManipulator::TabBoxTrackballDragger();
+        d->setupDefaultGeometry();
+        dragger = d;
+    }
     else if ("TrackballDragger" == name)
     {
         osgManipulator::TrackballDragger* d = new osgManipulator::TrackballDragger();
@@ -89,67 +95,104 @@ osgManipulator::Dragger* createDragger(const std::string& name)
     return dragger;
 }
 
-
-osg::Node* createHUD()
+// The DraggerContainer node is used to fix the dragger's size on the screen
+class DraggerContainer : public osg::Group
 {
-    osg::Geode* geode = new osg::Geode();
+public:
+    DraggerContainer() : _draggerSize(240.0f), _active(true) {}
     
-    std::string timesFont("fonts/arial.ttf");
-
-    osg::StateSet* stateset = geode->getOrCreateStateSet();
-    stateset->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
-
-    osgText::Text* text = new  osgText::Text;
-    geode->addDrawable( text );
-
-    osg::Vec3 position(50.0f,50.0f,0.0f);
-    text->setPosition(position);
-    text->setText("Use the Tab key to switch between the trackball and pick modes.");
-    text->setFont(timesFont);
-
-    osg::Camera* camera = new osg::Camera;
-
-    // set the projection matrix
-    camera->setProjectionMatrix(osg::Matrix::ortho2D(0,1280,0,1024));
-
-    // set the view matrix    
-    camera->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
-    camera->setViewMatrix(osg::Matrix::identity());
-
-    // only clear the depth buffer
-    camera->setClearMask(GL_DEPTH_BUFFER_BIT);
-
-    // draw subgraph after main camera view.
-    camera->setRenderOrder(osg::Camera::POST_RENDER);
-
-    camera->addChild(geode);
+    DraggerContainer( const DraggerContainer& copy, const osg::CopyOp& copyop=osg::CopyOp::SHALLOW_COPY )
+    :   osg::Group(copy, copyop),
+        _dragger(copy._dragger), _draggerSize(copy._draggerSize), _active(copy._active)
+    {}
     
-    return camera;
-}
+    META_Node( osgManipulator, DraggerContainer );
+    
+    void setDragger( osgManipulator::Dragger* dragger )
+    {
+        _dragger = dragger;
+        if ( !containsNode(dragger) ) addChild( dragger );
+    }
+    
+    osgManipulator::Dragger* getDragger() { return _dragger.get(); }
+    const osgManipulator::Dragger* getDragger() const { return _dragger.get(); }
+    
+    void setDraggerSize( float size ) { _draggerSize = size; }
+    float getDraggerSize() const { return _draggerSize; }
+    
+    void setActive( bool b ) { _active = b; }
+    bool getActive() const { return _active; }
+    
+    void traverse( osg::NodeVisitor& nv )
+    {
+        if ( _dragger.valid() )
+        {
+            if ( _active && nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR )
+            {
+                osgUtil::CullVisitor* cv = static_cast<osgUtil::CullVisitor*>(&nv);
+                
+                float pixelSize = cv->pixelSize(_dragger->getBound().center(), 0.48f);
+                if ( pixelSize!=_draggerSize )
+                {
+                    float pixelScale = pixelSize>0.0f ? _draggerSize/pixelSize : 1.0f;
+                    osg::Vec3d scaleFactor(pixelScale, pixelScale, pixelScale);
+                    
+                    osg::Vec3 trans = _dragger->getMatrix().getTrans();
+                    _dragger->setMatrix( osg::Matrix::scale(scaleFactor) * osg::Matrix::translate(trans) );
+                }
+            }
+        }
+        osg::Group::traverse(nv);
+    }
+    
+protected:
+    osg::ref_ptr<osgManipulator::Dragger> _dragger;
+    float _draggerSize;
+    bool _active;
+};
 
-osg::Node* addDraggerToScene(osg::Node* scene, osgManipulator::CommandManager* cmdMgr, const std::string& name)
+osg::Node* addDraggerToScene(osg::Node* scene, const std::string& name, bool fixedSizeInScreen)
 {
     scene->getOrCreateStateSet()->setMode(GL_NORMALIZE, osg::StateAttribute::ON);
 
-    osgManipulator::Selection* selection = new osgManipulator::Selection;
+    osg::MatrixTransform* selection = new osg::MatrixTransform;
     selection->addChild(scene);
 
     osgManipulator::Dragger* dragger = createDragger(name);
 
     osg::Group* root = new osg::Group;
-    root->addChild(dragger);
     root->addChild(selection);
-    root->addChild(createHUD());
+
+    if ( fixedSizeInScreen )
+    {
+        DraggerContainer* draggerContainer = new DraggerContainer;
+        draggerContainer->setDragger( dragger );
+        root->addChild(draggerContainer);
+    }
+    else
+        root->addChild(dragger);
 
     float scale = scene->getBound().radius() * 1.6;
     dragger->setMatrix(osg::Matrix::scale(scale, scale, scale) *
                        osg::Matrix::translate(scene->getBound().center()));
-    cmdMgr->connect(*dragger, *selection);
+
+    dragger->addTransformUpdating(selection);
+
+    // we want the dragger to handle it's own events automatically
+    dragger->setHandleEvents(true);
+
+    // if we don't set an activation key or mod mask then any mouse click on
+    // the dragger will activate it, however if do define either of ActivationModKeyMask or
+    // and ActivationKeyEvent then you'll have to press either than mod key or the specified key to
+    // be able to activate the dragger when you mouse click on it.  Please note the follow allows
+    // activation if either the ctrl key or the 'a' key is pressed and held down.
+    dragger->setActivationModKeyMask(osgGA::GUIEventAdapter::MODKEY_CTRL);
+    dragger->setActivationKeyEvent('a');
 
     return root;
 }
 
-osg::Node* createDemoScene(osgManipulator::CommandManager* cmdMgr) {
+osg::Node* createDemoScene(bool fixedSizeInScreen) {
  
     osg::Group* root = new osg::Group;
 
@@ -225,13 +268,13 @@ osg::Node* createDemoScene(osgManipulator::CommandManager* cmdMgr) {
     matirial->setShininess(osg::Material::FRONT_AND_BACK, 64.0f);
     root->getOrCreateStateSet()->setAttributeAndModes(matirial.get(), osg::StateAttribute::ON);
 
-      transform_1.get()->addChild(addDraggerToScene(geode_1.get(),cmdMgr,"TabBoxDragger"));
-    transform_2.get()->addChild(addDraggerToScene(geode_2.get(),cmdMgr,"TabPlaneDragger"));
-    transform_3.get()->addChild(addDraggerToScene(geode_3.get(),cmdMgr,"TabPlaneTrackballDragger"));
-    transform_4.get()->addChild(addDraggerToScene(geode_4.get(),cmdMgr,"TrackballDragger"));
-    transform_5.get()->addChild(addDraggerToScene(geode_5.get(),cmdMgr,"Translate1DDragger"));
-    transform_6.get()->addChild(addDraggerToScene(geode_6.get(),cmdMgr,"Translate2DDragger"));
-    transform_7.get()->addChild(addDraggerToScene(geode_7.get(),cmdMgr,"TranslateAxisDragger"));
+    transform_1.get()->addChild(addDraggerToScene(geode_1.get(),"TabBoxDragger",fixedSizeInScreen));
+    transform_2.get()->addChild(addDraggerToScene(geode_2.get(),"TabPlaneDragger",fixedSizeInScreen));
+    transform_3.get()->addChild(addDraggerToScene(geode_3.get(),"TabBoxTrackballDragger",fixedSizeInScreen));
+    transform_4.get()->addChild(addDraggerToScene(geode_4.get(),"TrackballDragger",fixedSizeInScreen));
+    transform_5.get()->addChild(addDraggerToScene(geode_5.get(),"Translate1DDragger",fixedSizeInScreen));
+    transform_6.get()->addChild(addDraggerToScene(geode_6.get(),"Translate2DDragger",fixedSizeInScreen));
+    transform_7.get()->addChild(addDraggerToScene(geode_7.get(),"TranslateAxisDragger",fixedSizeInScreen));
 
     root->addChild(transform_1.get());
     root->addChild(transform_2.get());
@@ -245,104 +288,7 @@ osg::Node* createDemoScene(osgManipulator::CommandManager* cmdMgr) {
  
     return root;
 }
-
-
-class PickModeHandler : public osgGA::GUIEventHandler
-{
-    public:
-        enum Modes
-        {
-            VIEW = 0,
-            PICK
-        };
-
-        PickModeHandler():
-            _mode(VIEW), 
-            _activeDragger(0)
-        {
-        }        
-        
-        bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa,
-                    osg::Object*, osg::NodeVisitor*)
-        {
-            osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
-            if (!view) return false;
-
-            if (ea.getKey() == osgGA::GUIEventAdapter::KEY_Tab &&
-                ea.getEventType() == osgGA::GUIEventAdapter::KEYDOWN &&
-                _activeDragger == 0)
-            {
-                _mode = ! _mode;
-            }
-            
-            if (VIEW == _mode) return false;
-
-            switch (ea.getEventType())
-            {
-                case osgGA::GUIEventAdapter::PUSH:
-                {
-                    osgUtil::LineSegmentIntersector::Intersections intersections;
-
-                    _pointer.reset();
-
-                    if (view->computeIntersections(ea.getX(),ea.getY(),intersections))
-                    {
-                        _pointer.setCamera(view->getCamera());
-                        _pointer.setMousePosition(ea.getX(), ea.getY());
-
-                        for(osgUtil::LineSegmentIntersector::Intersections::iterator hitr = intersections.begin();
-                            hitr != intersections.end();
-                            ++hitr)
-                        {
-                            _pointer.addIntersection(hitr->nodePath, hitr->getLocalIntersectPoint());
-                        }
-                        for (osg::NodePath::iterator itr = _pointer._hitList.front().first.begin();
-                             itr != _pointer._hitList.front().first.end();
-                             ++itr)
-                        {
-                            osgManipulator::Dragger* dragger = dynamic_cast<osgManipulator::Dragger*>(*itr);
-                            if (dragger)
-                            {
-
-                                dragger->handle(_pointer, ea, aa);
-                                _activeDragger = dragger;
-                                break;
-                            }                   
-                        }
-                    }
-                }
-                case osgGA::GUIEventAdapter::DRAG:
-                case osgGA::GUIEventAdapter::RELEASE:
-                {
-                    if (_activeDragger)
-                    {
-                        _pointer._hitIter = _pointer._hitList.begin();
-                        _pointer.setCamera(view->getCamera());
-                        _pointer.setMousePosition(ea.getX(), ea.getY());
-
-                        _activeDragger->handle(_pointer, ea, aa);
-                    }
-                    break;
-                }
-        default:
-            break;
-            }
-
-            if (ea.getEventType() == osgGA::GUIEventAdapter::RELEASE)
-            {
-                _activeDragger = 0;
-                _pointer.reset();
-            }
-
-            return true;
-        }
-        
-    private:
-        unsigned int _mode;
-        osgManipulator::Dragger* _activeDragger;
-        osgManipulator::PointerInfo _pointer;
-};
-
+// 
 int main( int argc, char **argv )
 {
 
@@ -360,7 +306,10 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("--help-all","Display all command line, env vars and keyboard & mouse bindings.");
 
     arguments.getApplicationUsage()->addCommandLineOption("--dragger <draggername>","Use the specified dragger for manipulation [TabPlaneDragger,TabPlaneTrackballDragger,TrackballDragger,Translate1DDragger,Translate2DDragger,TranslateAxisDragger,TabBoxDragger]");
+    arguments.getApplicationUsage()->addCommandLineOption("--fixedDraggerSize","Fix the size of the dragger geometry in the screen space");
     
+    bool fixedSizeInScreen = false;
+    while (arguments.read("--fixedDraggerSize")) { fixedSizeInScreen = true; }
 
     // construct the viewer.
     osgViewer::Viewer viewer;
@@ -394,16 +343,13 @@ int main( int argc, char **argv )
     // read the scene from the list of file specified command line args.
     osg::ref_ptr<osg::Node> loadedModel = osgDB::readNodeFiles(arguments);
 
-    // create a command manager
-    osg::ref_ptr<osgManipulator::CommandManager> cmdMgr = new osgManipulator::CommandManager;
-
     // if no model has been successfully loaded report failure.
     bool tragger2Scene(true);
     if (!loadedModel) 
     {
         //std::cout << arguments.getApplicationName() <<": No data loaded" << std::endl;
         //return 1;
-        loadedModel = createDemoScene(cmdMgr.get());
+        loadedModel = createDemoScene(fixedSizeInScreen);
         tragger2Scene=false;
     }
 
@@ -428,11 +374,11 @@ int main( int argc, char **argv )
     
     // pass the loaded scene graph to the viewer.
     if ( tragger2Scene ) {
-        viewer.setSceneData(addDraggerToScene(loadedModel.get(), cmdMgr.get(), dragger_name));
+        viewer.setSceneData(addDraggerToScene(loadedModel.get(), dragger_name, fixedSizeInScreen));
     } else { 
         viewer.setSceneData(loadedModel.get());
     }
-    viewer.addEventHandler(new PickModeHandler());
+
 
     return viewer.run();
 }
