@@ -28,7 +28,6 @@
 #include <osg/io_utils>
 
 #include <osgUtil/Optimizer>
-#include <osgUtil/IntersectVisitor>
 
 #include <osgDB/ReadFile>
 
@@ -80,7 +79,7 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius)
 
     osg::Group* model = new osg::Group;
 
-    osg::Node* glider = osgDB::readNodeFile("glider.osg");
+    osg::Node* glider = osgDB::readNodeFile("glider.osgt");
     if (glider)
     {
         const osg::BoundingSphere& bs = glider->getBound();
@@ -103,7 +102,7 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius)
         model->addChild(xform);
     }
  
-    osg::Node* cessna = osgDB::readNodeFile("cessna.osg");
+    osg::Node* cessna = osgDB::readNodeFile("cessna.osgt");
     if (cessna)
     {
         const osg::BoundingSphere& bs = cessna->getBound();
@@ -133,26 +132,20 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius)
 
 osg::Vec3 computeTerrainIntersection(osg::Node* subgraph,float x,float y)
 {
-    osgUtil::IntersectVisitor iv;
-    osg::ref_ptr<osg::LineSegment> segDown = new osg::LineSegment;
-
     const osg::BoundingSphere& bs = subgraph->getBound();
     float zMax = bs.center().z()+bs.radius();
     float zMin = bs.center().z()-bs.radius();
     
-    segDown->set(osg::Vec3(x,y,zMin),osg::Vec3(x,y,zMax));
-    iv.addLineSegment(segDown.get());
+    osg::ref_ptr<osgUtil::LineSegmentIntersector> intersector = 
+        new osgUtil::LineSegmentIntersector(osg::Vec3(x,y,zMin),osg::Vec3(x,y,zMax));
+
+    osgUtil::IntersectionVisitor iv(intersector.get());
 
     subgraph->accept(iv);
 
-    if (iv.hits())
+    if (intersector->containsIntersections())
     {
-        osgUtil::IntersectVisitor::HitList& hitList = iv.getHitList(segDown.get());
-        if (!hitList.empty())
-        {
-            osg::Vec3 ip = hitList.front().getWorldIntersectPoint();
-            return  ip;
-        }
+        return intersector->getFirstIntersection().getWorldIntersectPoint();
     }
 
     return osg::Vec3(x,y,0.0f);
@@ -323,7 +316,7 @@ public:
             {
                 // insert particle effects alongside the hit node, therefore able to track that nodes movement,
                 // however, this does require us to insert the ParticleSystem itself into the root of the scene graph
-                // seperately from the the main particle effects group which contains the emitters and programs.
+                // separately from the the main particle effects group which contains the emitters and programs.
                 // the follow code block implements this, note the path for handling particle effects which arn't attached to 
                 // moving models is easy - just a single line of code!
             
