@@ -30,46 +30,6 @@
 
 const std::string FILE_IDENTIFER("osgphotoalbum photo archive");
 
-class MyGraphicsContext {
-    public:
-        MyGraphicsContext()
-        {
-            osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
-            traits->x = 0;
-            traits->y = 0;
-            traits->width = 1;
-            traits->height = 1;
-            traits->windowDecoration = false;
-            traits->doubleBuffer = false;
-            traits->sharedContext = 0;
-            traits->pbuffer = true;
-
-            _gc = osg::GraphicsContext::createGraphicsContext(traits.get());
-
-            if (!_gc)
-            {
-                osg::notify(osg::NOTICE)<<"Failed to create pbuffer, failing back to normal graphics window."<<std::endl;
-                
-                traits->pbuffer = false;
-                _gc = osg::GraphicsContext::createGraphicsContext(traits.get());
-            }
-
-            if (_gc.valid()) 
-            
-            
-            {
-                _gc->realize();
-                _gc->makeCurrent();
-                std::cout<<"Realized window"<<std::endl;
-            }
-        }
-        
-        bool valid() const { return _gc.valid() && _gc->isRealized(); }
-        
-    private:
-        osg::ref_ptr<osg::GraphicsContext> _gc;
-};
-
 PhotoArchive::PhotoArchive(const std::string& filename)
 {
     readPhotoIndex(filename);
@@ -81,7 +41,12 @@ bool PhotoArchive::readPhotoIndex(const std::string& filename)
     
     char* fileIndentifier = new char [FILE_IDENTIFER.size()];
     in.read(fileIndentifier,FILE_IDENTIFER.size());
-    if (FILE_IDENTIFER!=fileIndentifier) return false;
+    if (FILE_IDENTIFER!=fileIndentifier)
+    {
+        delete [] fileIndentifier;
+        return false;
+    }
+    delete [] fileIndentifier;
     
     unsigned int numPhotos;
     in.read((char*)&numPhotos,sizeof(numPhotos));
@@ -206,10 +171,7 @@ void PhotoArchive::buildArchive(const std::string& filename, const FileNameList&
 
     std::cout<<"Building photo archive containing "<<photoIndex.size()<<" pictures"<<std::endl;
 
-    // create a graphics context so we can do data operations
-    MyGraphicsContext context;
-
-    // open up the archive for writing to    
+    // open up the archive for writing to
     osgDB::ofstream out(filename.c_str(), std::ios::out | std::ios::binary);
 
     // write out file indentifier.
@@ -258,10 +220,11 @@ void PhotoArchive::buildArchive(const std::string& filename, const FileNameList&
                 return;
             }
 
-            glPixelStorei(GL_PACK_ALIGNMENT,image->getPacking());
-            glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
+            osg::PixelStorageModes psm;
+            psm.pack_alignment = image->getPacking();
+            psm.unpack_alignment = image->getPacking();
 
-            GLint status = gluScaleImage(image->getPixelFormat(),
+            GLint status = osg::gluScaleImage(&psm, image->getPixelFormat(),
                 image->s(),
                 image->t(),
                 image->getDataType(),
@@ -273,9 +236,9 @@ void PhotoArchive::buildArchive(const std::string& filename, const FileNameList&
 
             if (status!=0)
             {
-               delete [] newData;
-
-                osg::notify(osg::WARN) << "Error scaleImage() did not succeed : errorString = "<<gluErrorString((GLenum)status)<<std::endl;
+                delete [] newData;
+                osg::notify(osg::WARN) << "Error scaleImage() did not succeed : errorString = "<<osg::gluErrorString((GLenum)status)<<std::endl;
+                return;
             }
     
             // now set up the photo header.
@@ -324,10 +287,11 @@ void PhotoArchive::buildArchive(const std::string& filename, const FileNameList&
                 return;
             }
 
-            glPixelStorei(GL_PACK_ALIGNMENT,image->getPacking());
-            glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
+            osg::PixelStorageModes psm;
+            psm.pack_alignment = image->getPacking();
+            psm.unpack_alignment = image->getPacking();
 
-            GLint status = gluScaleImage(image->getPixelFormat(),
+            GLint status = osg::gluScaleImage(&psm, image->getPixelFormat(),
                 image->s(),
                 image->t(),
                 image->getDataType(),
@@ -339,9 +303,9 @@ void PhotoArchive::buildArchive(const std::string& filename, const FileNameList&
 
             if (status!=0)
             {
-               delete [] newData;
-
-                osg::notify(osg::WARN) << "Error scaleImage() did not succeed : errorString = "<<gluErrorString((GLenum)status)<<std::endl;
+                delete [] newData;
+                osg::notify(osg::WARN) << "Error scaleImage() did not succeed : errorString = "<<osg::gluErrorString((GLenum)status)<<std::endl;
+                return;
             }
 
             ImageHeader imageHeader;
