@@ -467,7 +467,7 @@ int main( int argc, char **argv )
     osg::Node* loadedModel = osgDB::readNodeFiles(arguments);
     
     // if not loaded assume no arguments passed in, try use default mode instead.
-    if (!loadedModel) loadedModel = osgDB::readNodeFile("cessna.osg");
+    if (!loadedModel) loadedModel = osgDB::readNodeFile("cessna.osgt");
     
     if (!loadedModel)
     {
@@ -484,7 +484,7 @@ int main( int argc, char **argv )
     osg::Group* rootNode = new osg::Group();
     rootNode->addChild(createPreRenderSubGraph(loadedModelTransform,tex_width,tex_height, renderImplementation, useImage, useTextureRectangle, useHDR, samples, colorSamples));
 
-    osgDB::writeNodeFile(*rootNode, "test.ive");
+    //osgDB::writeNodeFile(*rootNode, "test.osgb");
 
     // add model to the viewer.
     viewer.setSceneData( rootNode );
