@@ -27,7 +27,6 @@
 #include <osg/Geometry>
 
 #include <osgUtil/SmoothingVisitor>
-#include <osgUtil/IntersectVisitor>
 
 #include <osgDB/ReadFile>
 
@@ -157,6 +156,24 @@ class IntersectionUpdateCallback : public osg::NodeCallback
     unsigned frameCount_;
 };
 
+class RotateUpdateCallback : public osg::NodeCallback
+{
+public:
+    RotateUpdateCallback() { i=0;}
+        virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
+        {
+            osgSim::SphereSegment * ss = dynamic_cast<osgSim::SphereSegment *>(node);
+            if (ss)
+            {
+                ss->setArea(osg::Vec3(cos(i/(2*osg::PI)),sin(i/(2*osg::PI)),0), osg::PI_2, osg::PI_2);
+                
+                i += 0.1f;
+            }
+
+        }
+protected:
+    float i;
+};
 
 osg::Node* createMovingModel(const osg::Vec3& center, float radius, osg::Geode * terrainGeode, osg::Group * root, bool createMovingRadar = false)
 {
@@ -166,7 +183,7 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius, osg::Geode *
 
     osg::Group* model = new osg::Group;
 
-    osg::Node* glider = osgDB::readNodeFile("glider.osg");
+    osg::Node* glider = osgDB::readNodeFile("glider.osgt");
     if (glider)
     {
         const osg::BoundingSphere& bs = glider->getBound();
@@ -219,7 +236,7 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius, osg::Geode *
         model->addChild(xform);
     }
  
-    osg::Node* cessna = osgDB::readNodeFile("cessna.osg");
+    osg::Node* cessna = osgDB::readNodeFile("cessna.osgt");
     if (cessna)
     {
         const osg::BoundingSphere& bs = cessna->getBound();
@@ -317,26 +334,20 @@ osg::Group* createOverlay(const osg::Vec3& center, float radius)
 
 osg::Vec3 computeTerrainIntersection(osg::Node* subgraph,float x,float y)
 {
-    osgUtil::IntersectVisitor iv;
-    osg::ref_ptr<osg::LineSegment> segDown = new osg::LineSegment;
-
     const osg::BoundingSphere& bs = subgraph->getBound();
     float zMax = bs.center().z()+bs.radius();
     float zMin = bs.center().z()-bs.radius();
     
-    segDown->set(osg::Vec3(x,y,zMin),osg::Vec3(x,y,zMax));
-    iv.addLineSegment(segDown.get());
+    osg::ref_ptr<osgUtil::LineSegmentIntersector> intersector = 
+        new osgUtil::LineSegmentIntersector(osg::Vec3(x,y,zMin),osg::Vec3(x,y,zMax));
+
+    osgUtil::IntersectionVisitor iv(intersector.get());
 
     subgraph->accept(iv);
 
-    if (iv.hits())
+    if (intersector->containsIntersections())
     {
-        osgUtil::IntersectVisitor::HitList& hitList = iv.getHitList(segDown.get());
-        if (!hitList.empty())
-        {
-            osg::Vec3 ip = hitList.front().getWorldIntersectPoint();
-            return  ip;
-        }
+        return intersector->getFirstIntersection().getWorldIntersectPoint();
     }
 
     return osg::Vec3(x,y,0.0f);
@@ -569,6 +580,20 @@ void build_world(osg::Group *root, unsigned int testCase, bool useOverlay, osgSi
                 root->addChild(mt.get());
                 break;
             }
+            case(7):        
+            {
+                ss = new osgSim::SphereSegment(
+                                computeTerrainIntersection(terrainGeode.get(),550.0f,780.0f), // center
+                                510.0f, // radius
+                                osg::DegreesToRadians(-240.0f),
+                                osg::DegreesToRadians(-135.0f),
+                                osg::DegreesToRadians(-10.0f),
+                                osg::DegreesToRadians(30.0f),
+                                60);
+                ss->setUpdateCallback(new RotateUpdateCallback());
+                root->addChild(ss.get());
+                break;
+            }
         };
         
         if (ss.valid())
