@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgtexture2D.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Node>
 #include <osg/Geometry>
 #include <osg/Notify>
@@ -12,7 +30,7 @@
 
 #include <osgText/Text>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //
@@ -63,7 +81,7 @@ public:
     {
         if (nv->getFrameStamp())
         {
-            double currTime = nv->getFrameStamp()->getReferenceTime();
+            double currTime = nv->getFrameStamp()->getSimulationTime();
             if (currTime-_prevTime>_delay) 
             {
                 // update filter modes and text.
@@ -222,7 +240,7 @@ public:
     {
         if (nv->getFrameStamp())
         {
-            double currTime = nv->getFrameStamp()->getReferenceTime();
+            double currTime = nv->getFrameStamp()->getSimulationTime();
             if (currTime-_prevTime>_delay) 
             {
                 // update filter modes and text.
@@ -379,7 +397,7 @@ public:
     {
         if (nv->getFrameStamp())
         {
-            double currTime = nv->getFrameStamp()->getReferenceTime();
+            double currTime = nv->getFrameStamp()->getSimulationTime();
             if (currTime-_prevTime>_delay) 
             {
                 // update filter modes and text.
@@ -532,7 +550,7 @@ public:
     {
         if (nv->getFrameStamp())
         {
-            double currTime = nv->getFrameStamp()->getReferenceTime();
+            double currTime = nv->getFrameStamp()->getSimulationTime();
             if (currTime-_prevTime>_delay) 
             {
                 // update filter modes and text.
@@ -639,6 +657,7 @@ osg::Node* createSubloadWall(osg::BoundingBox& bb)
     // create the text label.
     
     osgText::Text* text = new osgText::Text;
+    text->setDataVariance(osg::Object::DYNAMIC);
     text->setFont("fonts/arial.ttf");
     text->setPosition(center);
     text->setCharacterSize(height*0.03f);
@@ -680,76 +699,13 @@ osg::Node* createModel()
     return root;
 }
 
-int main( int argc, char **argv )
+int main(int , char **)
 {
-
-    // use an ArgumentParser object to manage the program arguments.
-    osg::ArgumentParser arguments(&argc,argv);
-
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates use of 2D textures.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-   
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-
-    // create a model from the images.
-    osg::Node* rootNode = createModel();
+    osgViewer::Viewer viewer;
 
     // add model to viewer.
-    viewer.setSceneData( rootNode );
+    viewer.setSceneData( createModel() );
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-
-    }
-
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-    
-    
-    
-    return 0;
+    return viewer.run();
 }
