@@ -29,6 +29,10 @@ AlphaFunc::~AlphaFunc()
 
 void AlphaFunc::apply(State&) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
     glAlphaFunc((GLenum)_comparisonFunc,_referenceValue);
+#else
+    OSG_NOTICE<<"Warning: AlphaFunc::apply(State&) - not supported."<<std::endl;
+#endif
 }
 
