@@ -18,15 +18,21 @@
 using namespace osg;
 
 AutoTransform::AutoTransform():
-    _autoUpdateEyeMovementTolerance(0.0f),
+    _autoUpdateEyeMovementTolerance(0.0),
     _autoRotateMode(NO_ROTATION),
     _autoScaleToScreen(false),
-    _scale(1.0f,1.0f,1.0f),
+    _scale(1.0,1.0,1.0),
     _firstTimeToInitEyePoint(true),
-    _minimumScale(0.0f),
-    _maximumScale(FLT_MAX),
-    _autoScaleTransitionWidthRatio(0.25f),
-    _matrixDirty(true)
+    _previousWidth(0),
+    _previousHeight(0),
+    _minimumScale(0.0),
+    _maximumScale(DBL_MAX),
+    _autoScaleTransitionWidthRatio(0.25),
+    _matrixDirty(true),
+    _axis(0.0f,0.0f,1.0f),
+    _normal(0.0f,-1.0f,0.0f),
+    _cachedMode(NO_ROTATION),
+    _side(1.0f,0.0,0.0f)
 {
 //    setNumChildrenRequiringUpdateTraversal(1);
 }
@@ -41,15 +47,58 @@ AutoTransform::AutoTransform(const AutoTransform& pat,const CopyOp& copyop):
     _rotation(pat._rotation),
     _scale(pat._scale),
     _firstTimeToInitEyePoint(true),
+    _previousWidth(0),
+    _previousHeight(0),
     _minimumScale(pat._minimumScale),
     _maximumScale(pat._maximumScale),
     _autoScaleTransitionWidthRatio(pat._autoScaleTransitionWidthRatio),
-    _matrixDirty(true)
+    _matrixDirty(true),
+    _axis(pat._axis),
+    _normal(pat._normal),
+    _cachedMode(pat._cachedMode),
+    _side(pat._side)
 {
 //    setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+1);            
 }
 
-void AutoTransform::setScale(const Vec3& scale)
+void AutoTransform::setAutoRotateMode(AutoRotateMode mode)
+{ 
+    _autoRotateMode = mode; 
+    _firstTimeToInitEyePoint = true; 
+    _cachedMode = CACHE_DIRTY;
+    updateCache();
+}
+
+void AutoTransform::setAxis(const Vec3& axis)
+{
+    _axis = axis;
+    _axis.normalize();
+    updateCache();
+}
+
+void AutoTransform::setNormal(const Vec3& normal)
+{
+    _normal = normal;
+    _normal.normalize();
+    updateCache();
+}
+
+void AutoTransform::updateCache()
+{
+    if (_autoRotateMode==ROTATE_TO_AXIS)
+    {
+        if      (_axis==Vec3(1.0f,0.0,0.0f) && _normal==Vec3(0.0f,-1.0,0.0f)) _cachedMode = AXIAL_ROT_X_AXIS;
+        else if (_axis==Vec3(0.0f,1.0,0.0f) && _normal==Vec3(1.0f, 0.0,0.0f)) _cachedMode = AXIAL_ROT_Y_AXIS;
+        else if (_axis==Vec3(0.0f,0.0,1.0f) && _normal==Vec3(0.0f,-1.0,0.0f)) _cachedMode = AXIAL_ROT_Z_AXIS;
+        else                                                                  _cachedMode = ROTATE_TO_AXIS;
+    }
+    else _cachedMode = _autoRotateMode;
+    
+    _side = _axis^_normal;
+    _side.normalize();   
+}
+
+void AutoTransform::setScale(const Vec3d& scale)
 {
     _scale = scale; 
     if (_scale.x()<_minimumScale) _scale.x() = _minimumScale;
@@ -83,19 +132,22 @@ bool AutoTransform::computeLocalToWorldMatrix(Matrix& matrix,NodeVisitor*) const
 
 bool AutoTransform::computeWorldToLocalMatrix(Matrix& matrix,NodeVisitor*) const
 {
+    if (_scale.x() == 0.0 || _scale.y() == 0.0 || _scale.z() == 0.0)
+        return false;
+
     if (_referenceFrame==RELATIVE_RF)
     {
-        matrix.postMult(osg::Matrix::translate(-_position)*
-                        osg::Matrix::rotate(_rotation.inverse())*
-                        osg::Matrix::scale(1.0f/_scale.x(),1.0f/_scale.y(),1.0f/_scale.z())*
-                        osg::Matrix::translate(_pivotPoint));
+        matrix.postMultTranslate(-_position);
+        matrix.postMultRotate(_rotation.inverse());
+        matrix.postMultScale(Vec3d(1.0/_scale.x(), 1.0/_scale.y(), 1.0/_scale.z()));
+        matrix.postMultTranslate(_pivotPoint);
     }
     else // absolute
     {
-        matrix = osg::Matrix::translate(-_position)*
-                 osg::Matrix::rotate(_rotation.inverse())*
-                 osg::Matrix::scale(1.0f/_scale.x(),1.0f/_scale.y(),1.0f/_scale.z())*
-                 osg::Matrix::translate(_pivotPoint);
+        matrix.makeRotate(_rotation.inverse());
+        matrix.preMultTranslate(-_position);
+        matrix.postMultScale(Vec3d(1.0/_scale.x(), 1.0/_scale.y(), 1.0/_scale.z()));
+        matrix.postMultTranslate(_pivotPoint);
     }
     return true;
 }
@@ -104,10 +156,10 @@ void AutoTransform::computeMatrix() const
 {
     if (!_matrixDirty) return;
     
-    _cachedMatrix.set(osg::Matrix::translate(-_pivotPoint)*
-                      osg::Matrix::scale(_scale)*
-                      osg::Matrix::rotate(_rotation)*
-                      osg::Matrix::translate(_position));
+    _cachedMatrix.makeRotate(_rotation);
+    _cachedMatrix.postMultTranslate(_position);
+    _cachedMatrix.preMultScale(_scale);
+    _cachedMatrix.preMultTranslate(-_pivotPoint);
     
     _matrixDirty = false;
 }
@@ -138,21 +190,21 @@ void AutoTransform::accept(NodeVisitor& nv)
                     height = viewport->height();
                 }
 
-                osg::Vec3 eyePoint = cs->getEyeLocal(); 
-                osg::Vec3 localUp = cs->getUpLocal(); 
-                osg::Vec3 position = getPosition();
+                osg::Vec3d eyePoint = cs->getEyeLocal(); 
+                osg::Vec3d localUp = cs->getUpLocal(); 
+                osg::Vec3d position = getPosition();
 
                 const osg::Matrix& projection = *(cs->getProjectionMatrix());
 
                 bool doUpdate = _firstTimeToInitEyePoint;
                 if (!_firstTimeToInitEyePoint)
                 {
-                    osg::Vec3 dv = _previousEyePoint-eyePoint;
+                    osg::Vec3d dv = _previousEyePoint-eyePoint;
                     if (dv.length2()>getAutoUpdateEyeMovementTolerance()*(eyePoint-getPosition()).length2())
                     {
                         doUpdate = true;
                     }
-                    osg::Vec3 dupv = _previousLocalUp-localUp;
+                    osg::Vec3d dupv = _previousLocalUp-localUp;
                     // rotating the camera only affects ROTATE_TO_*
                     if (_autoRotateMode &&
                         dupv.length2()>getAutoUpdateEyeMovementTolerance())
@@ -179,35 +231,35 @@ void AutoTransform::accept(NodeVisitor& nv)
 
                     if (getAutoScaleToScreen())
                     {
-                        float size = 1.0f/cs->pixelSize(getPosition(),0.48f);
+                        double size = 1.0/cs->pixelSize(getPosition(),0.48f);
 
-                        if (_autoScaleTransitionWidthRatio>0.0f)
+                        if (_autoScaleTransitionWidthRatio>0.0)
                         { 
-                            if (_minimumScale>0.0f)
+                            if (_minimumScale>0.0)
                             {
-                                float j = _minimumScale;
-                                float i = (_maximumScale<FLT_MAX) ? 
+                                double j = _minimumScale;
+                                double i = (_maximumScale<DBL_MAX) ? 
                                             _minimumScale+(_maximumScale-_minimumScale)*_autoScaleTransitionWidthRatio :
-                                            _minimumScale*(1.0f+_autoScaleTransitionWidthRatio);
-                                float c = 1.0f/(4.0f*(i-j));
-                                float b = 1.0f - 2.0f*c*i;
-                                float a = j + b*b / (4.0f*c);
-                                float k = -b / (2.0f*c);
+                                            _minimumScale*(1.0+_autoScaleTransitionWidthRatio);
+                                double c = 1.0/(4.0*(i-j));
+                                double b = 1.0 - 2.0*c*i;
+                                double a = j + b*b / (4.0*c);
+                                double k = -b / (2.0*c);
 
                                 if (size<k) size = _minimumScale;
                                 else if (size<i) size = a + b*size + c*(size*size);
                             }
 
-                            if (_maximumScale<FLT_MAX)
+                            if (_maximumScale<DBL_MAX)
                             {
-                                float n = _maximumScale;
-                                float m = (_minimumScale>0.0) ?
+                                double n = _maximumScale;
+                                double m = (_minimumScale>0.0) ?
                                             _maximumScale+(_minimumScale-_maximumScale)*_autoScaleTransitionWidthRatio :
-                                            _maximumScale*(1.0f-_autoScaleTransitionWidthRatio);
-                                float c = 1.0f / (4.0f*(m-n));
-                                float b = 1.0f - 2.0f*c*m;
-                                float a = n + b*b/(4.0f*c);
-                                float p = -b / (2.0f*c);
+                                            _maximumScale*(1.0-_autoScaleTransitionWidthRatio);
+                                double c = 1.0 / (4.0*(m-n));
+                                double b = 1.0 - 2.0*c*m;
+                                double a = n + b*b/(4.0*c);
+                                double p = -b / (2.0*c);
 
                                 if (size>p) size = _maximumScale;
                                 else if (size>m) size = a + b*size + c*(size*size);
@@ -230,13 +282,87 @@ void AutoTransform::accept(NodeVisitor& nv)
                     }
                     else if (_autoRotateMode==ROTATE_TO_CAMERA)
                     {
-                        osg::Vec3 PosToEye = _position - eyePoint;
+                        osg::Vec3d PosToEye = _position - eyePoint;
                         osg::Matrix lookto = osg::Matrix::lookAt(
-                            osg::Vec3(0,0,0), PosToEye, localUp);
+                            osg::Vec3d(0,0,0), PosToEye, localUp);
                         Quat q;
                         q.set(osg::Matrix::inverse(lookto));
                         setRotation(q);
                     }
+                    else if (_autoRotateMode==ROTATE_TO_AXIS)
+                    {
+                        Matrix matrix;
+                        Vec3 ev(eyePoint - _position);
+
+                        switch(_cachedMode)
+                        {
+                            case(AXIAL_ROT_Z_AXIS):
+                            {
+                                ev.z() = 0.0f;
+                                float ev_length = ev.length();
+                                if (ev_length>0.0f)
+                                {
+                                    //float rotation_zrotation_z = atan2f(ev.x(),ev.y());
+                                    //mat.makeRotate(inRadians(rotation_z),0.0f,0.0f,1.0f);
+                                    float inv = 1.0f/ev_length;
+                                    float s = ev.x()*inv;
+                                    float c = -ev.y()*inv;
+                                    matrix(0,0) = c;
+                                    matrix(1,0) = -s;
+                                    matrix(0,1) = s;
+                                    matrix(1,1) = c;
+                                }
+                                break;
+                            }
+                            case(AXIAL_ROT_Y_AXIS):
+                            {
+                                ev.y() = 0.0f;
+                                float ev_length = ev.length();
+                                if (ev_length>0.0f)
+                                {
+                                    //float rotation_zrotation_z = atan2f(ev.x(),ev.y());
+                                    //mat.makeRotate(inRadians(rotation_z),0.0f,0.0f,1.0f);
+                                    float inv = 1.0f/ev_length;
+                                    float s = -ev.z()*inv;
+                                    float c = ev.x()*inv;
+                                    matrix(0,0) = c;
+                                    matrix(2,0) = s;
+                                    matrix(0,2) = -s;
+                                    matrix(2,2) = c;
+                                }
+                                break;
+                            }
+                            case(AXIAL_ROT_X_AXIS):
+                            {
+                                ev.x() = 0.0f;
+                                float ev_length = ev.length();
+                                if (ev_length>0.0f)
+                                {
+                                    //float rotation_zrotation_z = atan2f(ev.x(),ev.y());
+                                    //mat.makeRotate(inRadians(rotation_z),0.0f,0.0f,1.0f);
+                                    float inv = 1.0f/ev_length;
+                                    float s = -ev.z()*inv;
+                                    float c = -ev.y()*inv;
+                                    matrix(1,1) = c;
+                                    matrix(2,1) = -s;
+                                    matrix(1,2) = s;
+                                    matrix(2,2) = c;
+                                }
+                                break;
+                            }
+                            case(ROTATE_TO_AXIS): // need to implement 
+                            {
+                                float ev_side = ev*_side;
+                                float ev_normal = ev*_normal;
+                                float rotation = atan2f(ev_side,ev_normal);
+                                matrix.makeRotate(rotation,_axis);
+                                break;
+                            }
+                        }
+                        Quat q;
+                        q.set(matrix);
+                        setRotation(q);
+                    }
 
                     _previousEyePoint = eyePoint;
                     _previousLocalUp = localUp;
