@@ -48,7 +48,7 @@ void ClampColor::apply(State& state) const
                                         
     if (!extensions->isClampColorSupported())
     {
-        notify(WARN)<<"Warning: ClampColor::apply(..) failed, ClampColor is not support by OpenGL driver."<<std::endl;
+        OSG_WARN<<"Warning: ClampColor::apply(..) failed, ClampColor is not support by OpenGL driver."<<std::endl;
         return;
     }
 
@@ -75,7 +75,7 @@ void ClampColor::setExtensions(unsigned int contextID,Extensions* extensions)
 
 ClampColor::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtenions(contextID);
+    setupGLExtensions(contextID);
 }
 
 ClampColor::Extensions::Extensions(const Extensions& rhs):
@@ -91,24 +91,24 @@ void ClampColor::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._glClampColor)          _glClampColor = 0;
 }
 
-void ClampColor::Extensions::setupGLExtenions(unsigned int contextID)
+void ClampColor::Extensions::setupGLExtensions(unsigned int contextID)
 {
-    _isClampColorSupported = isGLExtensionSupported(contextID,"GL_ARB_color_buffer_float") ||
+    _isClampColorSupported = OSG_GL3_FEATURES ||
+                             isGLExtensionSupported(contextID,"GL_ARB_color_buffer_float") ||
                              strncmp((const char*)glGetString(GL_VERSION),"2.0",3)>=0;
 
-    _glClampColor = getGLExtensionFuncPtr("glClampColor", "glClampColorARB");
+    setGLExtensionFuncPtr(_glClampColor, "glClampColor", "glClampColorARB");
 }
 
 void ClampColor::Extensions::glClampColor(GLenum target, GLenum mode) const
 {
     if (_glClampColor)
     {
-        typedef void (APIENTRY * GLClampColorProc) (GLenum target, GLenum mode);
-        ((GLClampColorProc)_glClampColor)(target,mode);
+        _glClampColor(target,mode);
     }
     else
     {
-        notify(WARN)<<"Error: glClampColor not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glClampColor not supported by OpenGL driver"<<std::endl;
     }
 }
 
