@@ -71,7 +71,7 @@ void DrawPixels::getSubImageDimensions(unsigned int& offsetX,unsigned int& offse
 
 BoundingBox DrawPixels::computeBound() const
 {
-    // really needs to be dependant of view poistion and projection... will implement simple version right now.
+    // really needs to be dependent of view position and projection... will implement simple version right now.
     BoundingBox bbox;
     float diagonal = 0.0f;
     if (_useSubImage)
@@ -90,6 +90,7 @@ BoundingBox DrawPixels::computeBound() const
 
 void DrawPixels::drawImplementation(RenderInfo&) const
 {
+#ifdef OSG_GL1_AVAILABLE
     glRasterPos3f(_position.x(),_position.y(),_position.z());
 
     if (_useSubImage)
@@ -109,5 +110,8 @@ void DrawPixels::drawImplementation(RenderInfo&) const
                      (GLenum)_image->getDataType(),
                      _image->data() );
     }
+#else
+    OSG_NOTICE<<"Warning: DrawPixels::drawImplementation(RenderInfo&) - not supported."<<std::endl;
+#endif
 }
 
