@@ -47,6 +47,7 @@ static DeletedDisplayListCache s_deletedDisplayListCache;
 
 GLuint Drawable::generateDisplayList(unsigned int contextID, unsigned int sizeHint)
 {
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedDisplayListCache);
 
     DisplayListMap& dll = s_deletedDisplayListCache[contextID];
@@ -60,7 +61,7 @@ GLuint Drawable::generateDisplayList(unsigned int contextID, unsigned int sizeHi
         DisplayListMap::iterator itr = dll.lower_bound(sizeHint);
         if (itr!=dll.end())
         {
-            // osg::notify(osg::NOTICE)<<"Reusing a display list of size = "<<itr->first<<" for requested size = "<<sizeHint<<std::endl;
+            // OSG_NOTICE<<"Reusing a display list of size = "<<itr->first<<" for requested size = "<<sizeHint<<std::endl;
 
             ++s_numberDrawablesReusedLastInLastFrame;
             
@@ -71,11 +72,15 @@ GLuint Drawable::generateDisplayList(unsigned int contextID, unsigned int sizeHi
         } 
         else
         {
-            // osg::notify(osg::NOTICE)<<"Creating a new display list of size = "<<sizeHint<<" although "<<dll.size()<<" are available"<<std::endl;
+            // OSG_NOTICE<<"Creating a new display list of size = "<<sizeHint<<" although "<<dll.size()<<" are available"<<std::endl;
             ++s_numberNewDrawablesInLastFrame;
             return  glGenLists( 1 );
         }
     }
+#else
+    OSG_NOTICE<<"Warning: Drawable::generateDisplayList(..) - not supported."<<std::endl;
+    return 0;
+#endif
 }
 
 unsigned int s_minimumNumberOfDisplayListsToRetainInCache = 0;
@@ -91,6 +96,7 @@ unsigned int Drawable::getMinimumNumberOfDisplayListsToRetainInCache()
 
 void Drawable::deleteDisplayList(unsigned int contextID,GLuint globj, unsigned int sizeHint)
 {
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     if (globj!=0)
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedDisplayListCache);
@@ -98,10 +104,14 @@ void Drawable::deleteDisplayList(unsigned int contextID,GLuint globj, unsigned i
         // insert the globj into the cache for the appropriate context.
         s_deletedDisplayListCache[contextID].insert(DisplayListMap::value_type(sizeHint,globj));
     }
+#else
+    OSG_NOTICE<<"Warning: Drawable::deleteDisplayList(..) - not supported."<<std::endl;
+#endif
 }
 
 void Drawable::flushAllDeletedDisplayLists(unsigned int contextID)
 {
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedDisplayListCache);
 
     DisplayListMap& dll = s_deletedDisplayListCache[contextID];
@@ -114,6 +124,9 @@ void Drawable::flushAllDeletedDisplayLists(unsigned int contextID)
     }
 
     dll.clear();         
+#else
+    OSG_NOTICE<<"Warning: Drawable::deleteDisplayList(..) - not supported."<<std::endl;
+#endif
 }
 
 void Drawable::discardAllDeletedDisplayLists(unsigned int contextID)
@@ -126,6 +139,7 @@ void Drawable::discardAllDeletedDisplayLists(unsigned int contextID)
 
 void Drawable::flushDeletedDisplayLists(unsigned int contextID, double& availableTime)
 {
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     // if no time available don't try to flush objects.
     if (availableTime<=0.0) return;
 
@@ -163,7 +177,7 @@ void Drawable::flushDeletedDisplayLists(unsigned int contextID, double& availabl
 
              if (noDeleted+dll.size() != prev_size)
              {
-                osg::notify(osg::WARN)<<"Error in delete"<<std::endl;
+                OSG_WARN<<"Error in delete"<<std::endl;
              }    
         }
         else
@@ -188,15 +202,18 @@ void Drawable::flushDeletedDisplayLists(unsigned int contextID, double& availabl
 
              if (noDeleted+dll.size() != prev_size)
              {
-                osg::notify(osg::WARN)<<"Error in delete"<<std::endl;
+                OSG_WARN<<"Error in delete"<<std::endl;
              }    
         }
     }
     elapsedTime = timer.delta_s(start_tick,timer.tick());
     
-    if (noDeleted!=0) notify(INFO)<<"Number display lists deleted = "<<noDeleted<<" elapsed time"<<elapsedTime<<std::endl;
+    if (noDeleted!=0) OSG_INFO<<"Number display lists deleted = "<<noDeleted<<" elapsed time"<<elapsedTime<<std::endl;
 
     availableTime -= elapsedTime;
+#else
+    OSG_NOTICE<<"Warning: Drawable::flushDeletedDisplayLists(..) - not supported."<<std::endl;
+#endif
 }
 
 Drawable::Drawable()
@@ -209,8 +226,13 @@ Drawable::Drawable()
     // to false in your constructor.  This will prevent any display
     // lists from being automatically created and safeguard the
     // dynamic updating of data.
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     _supportsDisplayList = true;
     _useDisplayList = true;
+#else
+    _supportsDisplayList = false;
+    _useDisplayList = false;
+#endif
 
     _supportsVertexBufferObjects = false;
     _useVertexBufferObjects = false;
@@ -443,19 +465,9 @@ void Drawable::dirtyBound()
 
 void Drawable::compileGLObjects(RenderInfo& renderInfo) const
 {
-    bool useVertexArrays = _supportsVertexBufferObjects && _useVertexBufferObjects && renderInfo.getState()->isVertexBufferObjectSupported();
-    if (useVertexArrays)
-    {
-        if (_drawCallback.valid())
-            _drawCallback->drawImplementation(renderInfo,this);
-        else
-            drawImplementation(renderInfo);
-
-        return;
-    }
-
     if (!_useDisplayList) return;
 
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     // get the contextID (user defined ID of 0 upwards) for the 
     // current OpenGL context.
     unsigned int contextID = renderInfo.getContextID();
@@ -478,7 +490,9 @@ void Drawable::compileGLObjects(RenderInfo& renderInfo) const
         drawImplementation(renderInfo);
 
     glEndList();
-
+#else
+    OSG_NOTICE<<"Warning: Drawable::compileGLObjects(RenderInfo&) - not supported."<<std::endl;
+#endif
 }
 
 void Drawable::setThreadSafeRefUnref(bool threadSafe)
@@ -486,13 +500,10 @@ void Drawable::setThreadSafeRefUnref(bool threadSafe)
     Object::setThreadSafeRefUnref(threadSafe);
 
     if (_stateset.valid()) _stateset->setThreadSafeRefUnref(threadSafe);
-
     if (_updateCallback.valid()) _updateCallback->setThreadSafeRefUnref(threadSafe);
     if (_eventCallback.valid()) _eventCallback->setThreadSafeRefUnref(threadSafe);
     if (_cullCallback.valid()) _cullCallback->setThreadSafeRefUnref(threadSafe);
     if (_drawCallback.valid()) _drawCallback->setThreadSafeRefUnref(threadSafe);
-    
-    if (_userData.valid()) _userData->setThreadSafeRefUnref(threadSafe);
 }
 
 void Drawable::resizeGLObjectBuffers(unsigned int maxSize)
@@ -537,7 +548,8 @@ void Drawable::setSupportsDisplayList(bool flag)
 {
     // if value unchanged simply return.
     if (_supportsDisplayList==flag) return;
-    
+
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     // if previously set to true then need to check about display lists.
     if (_supportsDisplayList)
     {
@@ -552,6 +564,9 @@ void Drawable::setSupportsDisplayList(bool flag)
     
     // set with new value.
     _supportsDisplayList=flag;
+#else
+    _supportsDisplayList=false;
+#endif
 }
 
 void Drawable::setUseDisplayList(bool flag)
@@ -559,6 +574,7 @@ void Drawable::setUseDisplayList(bool flag)
     // if value unchanged simply return.
     if (_useDisplayList==flag) return;
 
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     // if was previously set to true, remove display list.
     if (_useDisplayList)
     {
@@ -576,7 +592,7 @@ void Drawable::setUseDisplayList(bool flag)
     {
         if (flag)
         {
-            notify(WARN)<<"Warning: attempt to setUseDisplayList(true) on a drawable with does not support display lists."<<std::endl;
+            OSG_WARN<<"Warning: attempt to setUseDisplayList(true) on a drawable with does not support display lists."<<std::endl;
         }
         else 
         {
@@ -584,6 +600,9 @@ void Drawable::setUseDisplayList(bool flag)
             _useDisplayList = false;
         }
     }
+#else
+   _useDisplayList = false;
+#endif
 }
 
 
@@ -591,7 +610,7 @@ void Drawable::setUseVertexBufferObjects(bool flag)
 {
     // _useVertexBufferObjects = true;
 
-    // osg::notify(osg::NOTICE)<<"Drawable::setUseVertexBufferObjects("<<flag<<")"<<std::endl;
+    // OSG_NOTICE<<"Drawable::setUseVertexBufferObjects("<<flag<<")"<<std::endl;
 
     // if value unchanged simply return.
     if (_useVertexBufferObjects==flag) return;
@@ -601,13 +620,13 @@ void Drawable::setUseVertexBufferObjects(bool flag)
     {
         dirtyDisplayList();
     }
-    
+
     _useVertexBufferObjects = flag;
-    
 }
 
 void Drawable::dirtyDisplayList()
 {
+#ifdef OSG_GL_DISPLAYLISTS_AVAILABLE
     unsigned int i;
     for(i=0;i<_globjList.size();++i)
     {
@@ -617,6 +636,7 @@ void Drawable::dirtyDisplayList()
             _globjList[i] = 0;
         }
     }
+#endif
 }
 
 
@@ -773,9 +793,9 @@ BoundingBox Drawable::computeBound() const
     non_const_this->accept(cb);
 
 #if 0    
-    osg::notify(osg::NOTICE)<<"computeBound() "<<cb._bb.xMin()<<", "<<cb._bb.xMax()<<", "<<std::endl;
-    osg::notify(osg::NOTICE)<<"               "<<cb._bb.yMin()<<", "<<cb._bb.yMax()<<", "<<std::endl;
-    osg::notify(osg::NOTICE)<<"               "<<cb._bb.zMin()<<", "<<cb._bb.zMax()<<", "<<std::endl;
+    OSG_NOTICE<<"computeBound() "<<cb._bb.xMin()<<", "<<cb._bb.xMax()<<", "<<std::endl;
+    OSG_NOTICE<<"               "<<cb._bb.yMin()<<", "<<cb._bb.yMax()<<", "<<std::endl;
+    OSG_NOTICE<<"               "<<cb._bb.zMin()<<", "<<cb._bb.zMax()<<", "<<std::endl;
 #endif
     
     return cb._bb;
@@ -820,7 +840,9 @@ Drawable::Extensions::Extensions(const Extensions& rhs):
     _isFogCoordSupported = rhs._isFogCoordSupported;
     _isMultiTexSupported = rhs._isMultiTexSupported;
     _isOcclusionQuerySupported = rhs._isOcclusionQuerySupported;
+    _isARBOcclusionQuerySupported = rhs._isARBOcclusionQuerySupported;
     _isTimerQuerySupported = rhs._isTimerQuerySupported;
+    _isARBTimerQuerySupported = rhs._isARBTimerQuerySupported;
     
     _glFogCoordfv = rhs._glFogCoordfv;
     _glSecondaryColor3ubv = rhs._glSecondaryColor3ubv;
@@ -857,6 +879,7 @@ Drawable::Extensions::Extensions(const Extensions& rhs):
     _gl_get_query_objectiv_arb = rhs._gl_get_query_objectiv_arb;
     _gl_get_query_objectuiv_arb = rhs._gl_get_query_objectuiv_arb;
     _gl_get_query_objectui64v = rhs._gl_get_query_objectui64v;
+    _glGetInteger64v = rhs._glGetInteger64v;
 }
 
 
@@ -870,6 +893,7 @@ void Drawable::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._isARBOcclusionQuerySupported) _isARBOcclusionQuerySupported = false;
 
     if (!rhs._isTimerQuerySupported) _isTimerQuerySupported = false;
+    if (!rhs._isARBTimerQuerySupported) _isARBTimerQuerySupported = false;
 
     if (!rhs._glFogCoordfv) _glFogCoordfv = 0;
     if (!rhs._glSecondaryColor3ubv) _glSecondaryColor3ubv = 0;
@@ -916,6 +940,7 @@ void Drawable::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._gl_get_query_objectiv_arb) _gl_get_query_objectiv_arb = 0;
     if (!rhs._gl_get_query_objectuiv_arb) _gl_get_query_objectuiv_arb = 0;
     if (!rhs._gl_get_query_objectui64v) _gl_get_query_objectui64v = 0;
+    if (!rhs._glGetInteger64v) _glGetInteger64v = 0;
 }
 
 void Drawable::Extensions::setupGLExtensions(unsigned int contextID)
@@ -925,15 +950,17 @@ void Drawable::Extensions::setupGLExtensions(unsigned int contextID)
     _isFogCoordSupported = isGLExtensionSupported(contextID,"GL_EXT_fog_coord");
     _isMultiTexSupported = isGLExtensionSupported(contextID,"GL_ARB_multitexture");
     _isOcclusionQuerySupported = osg::isGLExtensionSupported(contextID, "GL_NV_occlusion_query" );
-    _isARBOcclusionQuerySupported = osg::isGLExtensionSupported(contextID, "GL_ARB_occlusion_query" );
+    _isARBOcclusionQuerySupported = OSG_GL3_FEATURES || osg::isGLExtensionSupported(contextID, "GL_ARB_occlusion_query" );
 
-    _isTimerQuerySupported = osg::isGLExtensionSupported(contextID, "GL_EXT_timer_query" );;
+    _isTimerQuerySupported = osg::isGLExtensionSupported(contextID, "GL_EXT_timer_query" );
+    _isARBTimerQuerySupported = osg::isGLExtensionSupported(contextID, "GL_ARB_timer_query");
 
 
     setGLExtensionFuncPtr(_glFogCoordfv, "glFogCoordfv","glFogCoordfvEXT");
     setGLExtensionFuncPtr(_glSecondaryColor3ubv, "glSecondaryColor3ubv","glSecondaryColor3ubvEXT");
     setGLExtensionFuncPtr(_glSecondaryColor3fv, "glSecondaryColor3fv","glSecondaryColor3fvEXT");
     setGLExtensionFuncPtr(_glMultiTexCoord1f, "glMultiTexCoord1f","glMultiTexCoord1fARB");
+    setGLExtensionFuncPtr(_glMultiTexCoord1fv, "glMultiTexCoord1fv","glMultiTexCoord1fvARB");
     setGLExtensionFuncPtr(_glMultiTexCoord2fv, "glMultiTexCoord2fv","glMultiTexCoord2fvARB");
     setGLExtensionFuncPtr(_glMultiTexCoord3fv, "glMultiTexCoord3fv","glMultiTexCoord3fvARB");
     setGLExtensionFuncPtr(_glMultiTexCoord4fv, "glMultiTexCoord4fv","glMultiTexCoord4fvARB");
@@ -945,6 +972,7 @@ void Drawable::Extensions::setupGLExtensions(unsigned int contextID)
     setGLExtensionFuncPtr(_glVertexAttrib1s, "glVertexAttrib1s","glVertexAttrib1sARB");
     setGLExtensionFuncPtr(_glVertexAttrib1f, "glVertexAttrib1f","glVertexAttrib1fARB");
     setGLExtensionFuncPtr(_glVertexAttrib1d, "glVertexAttrib1d","glVertexAttrib1dARB");
+    setGLExtensionFuncPtr(_glVertexAttrib1fv, "glVertexAttrib1fv","glVertexAttrib1fvARB");
     setGLExtensionFuncPtr(_glVertexAttrib2fv, "glVertexAttrib2fv","glVertexAttrib2fvARB");
     setGLExtensionFuncPtr(_glVertexAttrib3fv, "glVertexAttrib3fv","glVertexAttrib3fvARB");
     setGLExtensionFuncPtr(_glVertexAttrib4fv, "glVertexAttrib4fv","glVertexAttrib4fvARB");
@@ -983,6 +1011,8 @@ void Drawable::Extensions::setupGLExtensions(unsigned int contextID)
     setGLExtensionFuncPtr(_gl_get_query_objectiv_arb, "glGetQueryObjectiv","glGetQueryObjectivARB");
     setGLExtensionFuncPtr(_gl_get_query_objectuiv_arb, "glGetQueryObjectuiv","glGetQueryObjectuivARB");
     setGLExtensionFuncPtr(_gl_get_query_objectui64v, "glGetQueryObjectui64v","glGetQueryObjectui64vEXT");
+    setGLExtensionFuncPtr(_glQueryCounter, "glQueryCounter");
+    setGLExtensionFuncPtr(_glGetInteger64v, "glGetInteger64v");
 }
 
 void Drawable::Extensions::glFogCoordfv(const GLfloat* coord) const
@@ -993,7 +1023,7 @@ void Drawable::Extensions::glFogCoordfv(const GLfloat* coord) const
     }
     else
     {
-        notify(WARN)<<"Error: glFogCoordfv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glFogCoordfv not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1005,7 +1035,7 @@ void Drawable::Extensions::glSecondaryColor3ubv(const GLubyte* coord) const
     }
     else
     {
-        notify(WARN)<<"Error: glSecondaryColor3ubv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glSecondaryColor3ubv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1017,7 +1047,7 @@ void Drawable::Extensions::glSecondaryColor3fv(const GLfloat* coord) const
     }
     else
     {
-        notify(WARN)<<"Error: glSecondaryColor3fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glSecondaryColor3fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1029,7 +1059,7 @@ void Drawable::Extensions::glMultiTexCoord1f(GLenum target,GLfloat coord) const
     }
     else
     {
-        notify(WARN)<<"Error: glMultiTexCoord1f not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMultiTexCoord1f not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1041,7 +1071,7 @@ void Drawable::Extensions::glMultiTexCoord2fv(GLenum target,const GLfloat* coord
     }
     else
     {
-        notify(WARN)<<"Error: glMultiTexCoord2fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMultiTexCoord2fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1053,7 +1083,7 @@ void Drawable::Extensions::glMultiTexCoord3fv(GLenum target,const GLfloat* coord
     }
     else
     {
-        notify(WARN)<<"Error: _glMultiTexCoord3fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: _glMultiTexCoord3fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1065,7 +1095,7 @@ void Drawable::Extensions::glMultiTexCoord4fv(GLenum target,const GLfloat* coord
     }
     else
     {
-        notify(WARN)<<"Error: glMultiTexCoord4fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMultiTexCoord4fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1077,7 +1107,7 @@ void Drawable::Extensions::glMultiTexCoord1d(GLenum target,GLdouble coord) const
     }
     else
     {
-        notify(WARN)<<"Error: glMultiTexCoord1d not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMultiTexCoord1d not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1089,7 +1119,7 @@ void Drawable::Extensions::glMultiTexCoord2dv(GLenum target,const GLdouble* coor
     }
     else
     {
-        notify(WARN)<<"Error: glMultiTexCoord2dv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMultiTexCoord2dv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1101,7 +1131,7 @@ void Drawable::Extensions::glMultiTexCoord3dv(GLenum target,const GLdouble* coor
     }
     else
     {
-        notify(WARN)<<"Error: _glMultiTexCoord3dv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: _glMultiTexCoord3dv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1113,7 +1143,7 @@ void Drawable::Extensions::glMultiTexCoord4dv(GLenum target,const GLdouble* coor
     }
     else
     {
-        notify(WARN)<<"Error: glMultiTexCoord4dv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMultiTexCoord4dv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1125,7 +1155,7 @@ void Drawable::Extensions::glVertexAttrib1s(unsigned int index, GLshort s) const
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib1s not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib1s not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1137,7 +1167,7 @@ void Drawable::Extensions::glVertexAttrib1f(unsigned int index, GLfloat f) const
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib1f not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib1f not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1149,7 +1179,7 @@ void Drawable::Extensions::glVertexAttrib1d(unsigned int index, GLdouble f) cons
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib1d not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib1d not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1161,7 +1191,7 @@ void Drawable::Extensions::glVertexAttrib2fv(unsigned int index, const GLfloat *
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib2fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib2fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1173,7 +1203,7 @@ void Drawable::Extensions::glVertexAttrib3fv(unsigned int index, const GLfloat *
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib3fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib3fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1185,7 +1215,7 @@ void Drawable::Extensions::glVertexAttrib4fv(unsigned int index, const GLfloat *
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib4fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib4fv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1197,7 +1227,7 @@ void Drawable::Extensions::glVertexAttrib2dv(unsigned int index, const GLdouble
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib2dv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib2dv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1209,7 +1239,7 @@ void Drawable::Extensions::glVertexAttrib3dv(unsigned int index, const GLdouble
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib3dv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib3dv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1221,7 +1251,7 @@ void Drawable::Extensions::glVertexAttrib4dv(unsigned int index, const GLdouble
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib4dv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib4dv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1233,7 +1263,7 @@ void Drawable::Extensions::glVertexAttrib4ubv(unsigned int index, const GLubyte
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib4ubv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib4ubv not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -1245,38 +1275,38 @@ void Drawable::Extensions::glVertexAttrib4Nubv(unsigned int index, const GLubyte
     }
     else
     {
-        notify(WARN)<<"Error: glVertexAttrib4Nubv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glVertexAttrib4Nubv not supported by OpenGL driver"<<std::endl;
     }
 }
 
 void Drawable::Extensions::glGenBuffers(GLsizei n, GLuint *buffers) const
 {
     if (_glGenBuffers) _glGenBuffers(n, buffers); 
-    else notify(WARN)<<"Error: glGenBuffers not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glGenBuffers not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glBindBuffer(GLenum target, GLuint buffer) const
 {
     if (_glBindBuffer) _glBindBuffer(target, buffer); 
-    else notify(WARN)<<"Error: glBindBuffer not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glBindBuffer not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glBufferData(GLenum target, GLsizeiptrARB size, const GLvoid *data, GLenum usage) const
 {
     if (_glBufferData) _glBufferData(target, size, data, usage); 
-    else notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glBufferSubData(GLenum target, GLintptrARB offset, GLsizeiptrARB size, const GLvoid *data) const
 {
     if (_glBufferSubData) _glBufferSubData(target, offset, size, data); 
-    else notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glDeleteBuffers(GLsizei n, const GLuint *buffers) const
 {
     if (_glDeleteBuffers) _glDeleteBuffers(n, buffers); 
-    else notify(WARN)<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glBufferData not supported by OpenGL driver"<<std::endl;
 }
 
 GLboolean Drawable::Extensions::glIsBuffer (GLuint buffer) const
@@ -1284,7 +1314,7 @@ GLboolean Drawable::Extensions::glIsBuffer (GLuint buffer) const
     if (_glIsBuffer) return _glIsBuffer(buffer);
     else 
     {
-        notify(WARN)<<"Error: glIsBuffer not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glIsBuffer not supported by OpenGL driver"<<std::endl;
         return GL_FALSE;
     }
 }
@@ -1292,7 +1322,7 @@ GLboolean Drawable::Extensions::glIsBuffer (GLuint buffer) const
 void Drawable::Extensions::glGetBufferSubData (GLenum target, GLintptrARB offset, GLsizeiptrARB size, GLvoid *data) const
 {
     if (_glGetBufferSubData) _glGetBufferSubData(target,offset,size,data);
-    else notify(WARN)<<"Error: glGetBufferSubData not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glGetBufferSubData not supported by OpenGL driver"<<std::endl;
 }
 
 GLvoid* Drawable::Extensions::glMapBuffer (GLenum target, GLenum access) const
@@ -1300,7 +1330,7 @@ GLvoid* Drawable::Extensions::glMapBuffer (GLenum target, GLenum access) const
     if (_glMapBuffer) return _glMapBuffer(target,access);
     else 
     {
-        notify(WARN)<<"Error: glMapBuffer not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glMapBuffer not supported by OpenGL driver"<<std::endl;
         return 0;
     }
 }
@@ -1310,7 +1340,7 @@ GLboolean Drawable::Extensions::glUnmapBuffer (GLenum target) const
     if (_glUnmapBuffer) return _glUnmapBuffer(target);
     else 
     {
-        notify(WARN)<<"Error: glUnmapBuffer not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glUnmapBuffer not supported by OpenGL driver"<<std::endl;
         return GL_FALSE;
     }
 }
@@ -1318,13 +1348,13 @@ GLboolean Drawable::Extensions::glUnmapBuffer (GLenum target) const
 void Drawable::Extensions::glGetBufferParameteriv (GLenum target, GLenum pname, GLint *params) const
 {
     if (_glGetBufferParameteriv) _glGetBufferParameteriv(target,pname,params);
-    else notify(WARN)<<"Error: glGetBufferParameteriv not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glGetBufferParameteriv not supported by OpenGL driver"<<std::endl;
 }
 
 void Drawable::Extensions::glGetBufferPointerv (GLenum target, GLenum pname, GLvoid* *params) const
 {
     if (_glGetBufferPointerv) _glGetBufferPointerv(target,pname,params);
-    else notify(WARN)<<"Error: glGetBufferPointerv not supported by OpenGL driver"<<std::endl;
+    else OSG_WARN<<"Error: glGetBufferPointerv not supported by OpenGL driver"<<std::endl;
 }
 
 
@@ -1336,7 +1366,7 @@ void Drawable::Extensions::glGenOcclusionQueries( GLsizei n, GLuint *ids ) const
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glGenOcclusionQueries not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glGenOcclusionQueries not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1348,7 +1378,7 @@ void Drawable::Extensions::glDeleteOcclusionQueries( GLsizei n, const GLuint *id
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glDeleteOcclusionQueries not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glDeleteOcclusionQueries not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1360,7 +1390,7 @@ GLboolean Drawable::Extensions::glIsOcclusionQuery( GLuint id ) const
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glIsOcclusionQuery not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glIsOcclusionQuery not supported by OpenGL driver"<<std::endl;
     }    
 
     return GLboolean( 0 );
@@ -1374,7 +1404,7 @@ void Drawable::Extensions::glBeginOcclusionQuery( GLuint id ) const
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glBeginOcclusionQuery not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glBeginOcclusionQuery not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1386,7 +1416,7 @@ void Drawable::Extensions::glEndOcclusionQuery() const
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glEndOcclusionQuery not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glEndOcclusionQuery not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1398,7 +1428,7 @@ void Drawable::Extensions::glGetOcclusionQueryiv( GLuint id, GLenum pname, GLint
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glGetOcclusionQueryiv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glGetOcclusionQueryiv not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1410,7 +1440,7 @@ void Drawable::Extensions::glGetOcclusionQueryuiv( GLuint id, GLenum pname, GLui
     }
     else
     {
-        osg::notify(osg::WARN)<<"Error: glGetOcclusionQueryuiv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glGetOcclusionQueryuiv not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -1419,7 +1449,7 @@ void Drawable::Extensions::glGetQueryiv(GLenum target, GLenum pname, GLint *para
   if (_gl_get_queryiv_arb)
     _gl_get_queryiv_arb(target, pname, params);
   else
-    osg::notify(osg::WARN) << "Error: glGetQueryiv not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glGetQueryiv not supported by OpenGL driver" << std::endl;
 }
 
 void Drawable::Extensions::glGenQueries(GLsizei n, GLuint *ids) const
@@ -1427,7 +1457,7 @@ void Drawable::Extensions::glGenQueries(GLsizei n, GLuint *ids) const
   if (_gl_gen_queries_arb)
     _gl_gen_queries_arb(n, ids);
   else
-    osg::notify(osg::WARN) << "Error: glGenQueries not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glGenQueries not supported by OpenGL driver" << std::endl;
 }
 
 void Drawable::Extensions::glBeginQuery(GLenum target, GLuint id) const
@@ -1435,7 +1465,7 @@ void Drawable::Extensions::glBeginQuery(GLenum target, GLuint id) const
   if (_gl_begin_query_arb)
     _gl_begin_query_arb(target, id);
   else
-    osg::notify(osg::WARN) << "Error: glBeginQuery not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glBeginQuery not supported by OpenGL driver" << std::endl;
 }
 
 void Drawable::Extensions::glEndQuery(GLenum target) const
@@ -1443,14 +1473,22 @@ void Drawable::Extensions::glEndQuery(GLenum target) const
   if (_gl_end_query_arb)
     _gl_end_query_arb(target);
   else
-    osg::notify(osg::WARN) << "Error: glEndQuery not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glEndQuery not supported by OpenGL driver" << std::endl;
+}
+
+void Drawable::Extensions::glQueryCounter(GLuint id, GLenum target) const
+{
+    if (_glQueryCounter)
+        _glQueryCounter(id, target);
+    else
+        OSG_WARN << "Error: glQueryCounter not supported by OpenGL driver\n";
 }
 
 GLboolean Drawable::Extensions::glIsQuery(GLuint id) const
 {
   if (_gl_is_query_arb) return _gl_is_query_arb(id);
 
-  osg::notify(osg::WARN) << "Error: glIsQuery not supported by OpenGL driver" << std::endl;
+  OSG_WARN << "Error: glIsQuery not supported by OpenGL driver" << std::endl;
   return false;
 }
 
@@ -1459,7 +1497,7 @@ void Drawable::Extensions::glDeleteQueries(GLsizei n, const GLuint *ids) const
     if (_gl_delete_queries_arb) 
         _gl_delete_queries_arb(n, ids);
     else
-        osg::notify(osg::WARN) << "Error: glIsQuery not supported by OpenGL driver" << std::endl;
+        OSG_WARN << "Error: glIsQuery not supported by OpenGL driver" << std::endl;
 }
 
 void Drawable::Extensions::glGetQueryObjectiv(GLuint id, GLenum pname, GLint *params) const
@@ -1467,7 +1505,7 @@ void Drawable::Extensions::glGetQueryObjectiv(GLuint id, GLenum pname, GLint *pa
   if (_gl_get_query_objectiv_arb)
     _gl_get_query_objectiv_arb(id, pname, params);
   else
-    osg::notify(osg::WARN) << "Error: glGetQueryObjectiv not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glGetQueryObjectiv not supported by OpenGL driver" << std::endl;
 }
 
 void Drawable::Extensions::glGetQueryObjectuiv(GLuint id, GLenum pname, GLuint *params) const
@@ -1475,13 +1513,22 @@ void Drawable::Extensions::glGetQueryObjectuiv(GLuint id, GLenum pname, GLuint *
   if (_gl_get_query_objectuiv_arb)
     _gl_get_query_objectuiv_arb(id, pname, params);
   else
-    osg::notify(osg::WARN) << "Error: glGetQueryObjectuiv not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glGetQueryObjectuiv not supported by OpenGL driver" << std::endl;
 }
 
 void Drawable::Extensions::glGetQueryObjectui64v(GLuint id, GLenum pname, GLuint64EXT *params) const
 {
-  if (_gl_get_query_objectuiv_arb)
+  if (_gl_get_query_objectui64v)
     _gl_get_query_objectui64v(id, pname, params);
   else
-    osg::notify(osg::WARN) << "Error: glGetQueryObjectui64v not supported by OpenGL driver" << std::endl;
+    OSG_WARN << "Error: glGetQueryObjectui64v not supported by OpenGL driver" << std::endl;
+}
+
+void Drawable::Extensions::glGetInteger64v(GLenum pname, GLint64EXT *params)
+    const
+{
+    if (_glGetInteger64v)
+        _glGetInteger64v(pname, params);
+    else
+        OSG_WARN << "Error: glGetInteger64v not supported by OpenGL driver\n";
 }
