@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -13,6 +13,7 @@
 #include <osg/GLExtensions>
 #include <osg/State>
 #include <osg/Fog>
+#include <osg/Notify>
 
 using namespace osg;
 
@@ -28,6 +29,7 @@ Fog::Fog()
     _end     = 1.0f;
     _color.set( 0.0f, 0.0f, 0.0f, 0.0f);
     _fogCoordinateSource = FRAGMENT_DEPTH;
+    _useRadialFog = false;
 }
 
 
@@ -37,15 +39,35 @@ Fog::~Fog()
 
 void Fog::apply(State& state) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
+
+#ifdef OSG_GLES1_AVAILABLE
+    #define glFogi glFogx
+#endif
+
     glFogi( GL_FOG_MODE,     _mode );
+
     glFogf( GL_FOG_DENSITY,  _density );
     glFogf( GL_FOG_START,    _start );
     glFogf( GL_FOG_END,      _end );
     glFogfv( GL_FOG_COLOR,    (GLfloat*)_color.ptr() );
     
-    static bool fogCoordExtensionSuppoted = osg::isGLExtensionSupported(state.getContextID(),"GL_EXT_fog_coord");
-    if (fogCoordExtensionSuppoted)
+    static bool fogCoordExtensionSupported = osg::isGLExtensionSupported(state.getContextID(),"GL_EXT_fog_coord");
+    if (fogCoordExtensionSupported)
     {
         glFogi(GL_FOG_COORDINATE_SOURCE,_fogCoordinateSource);
     }
+
+    static bool fogDistanceExtensionSupported = osg::isGLExtensionSupported(state.getContextID(),"GL_NV_fog_distance");
+    if (fogDistanceExtensionSupported)
+    {
+        if(_useRadialFog)
+            glFogf(GL_FOG_DISTANCE_MODE_NV, GL_EYE_RADIAL_NV);
+        else
+            glFogf(GL_FOG_DISTANCE_MODE_NV, GL_EYE_PLANE_ABSOLUTE_NV);
+    }
+
+#else
+    OSG_NOTICE<<"Warning: Fog::apply(State&) - not supported."<<std::endl;
+#endif
 }
