@@ -31,7 +31,9 @@ ImageStream::ImageStream():
 
 ImageStream::ImageStream(const ImageStream& image,const CopyOp& copyop):
     Image(image,copyop),
-    _status(image._status)
+    _status(image._status),
+    _loopingMode(image._loopingMode),
+    _audioStreams(image._audioStreams)
 {
 }
 
