@@ -46,16 +46,16 @@ void Light::init( void )
     _linear_attenuation = 0.0f;
     _quadratic_attenuation = 0.0f;
 
-    //     notify(DEBUG) << "_ambient "<<_ambient<<std::endl;
-    //     notify(DEBUG) << "_diffuse "<<_diffuse<<std::endl;
-    //     notify(DEBUG) << "_specular "<<_specular<<std::endl;
-    //     notify(DEBUG) << "_position "<<_position<<std::endl;
-    //     notify(DEBUG) << "_direction "<<_direction<<std::endl;
-    //     notify(DEBUG) << "_spot_exponent "<<_spot_exponent<<std::endl;
-    //     notify(DEBUG) << "_spot_cutoff "<<_spot_cutoff<<std::endl;
-    //     notify(DEBUG) << "_constant_attenuation "<<_constant_attenuation<<std::endl;
-    //     notify(DEBUG) << "_linear_attenuation "<<_linear_attenuation<<std::endl;
-    //     notify(DEBUG) << "_quadratic_attenuation "<<_quadratic_attenuation<<std::endl;
+    //     OSG_DEBUG << "_ambient "<<_ambient<<std::endl;
+    //     OSG_DEBUG << "_diffuse "<<_diffuse<<std::endl;
+    //     OSG_DEBUG << "_specular "<<_specular<<std::endl;
+    //     OSG_DEBUG << "_position "<<_position<<std::endl;
+    //     OSG_DEBUG << "_direction "<<_direction<<std::endl;
+    //     OSG_DEBUG << "_spot_exponent "<<_spot_exponent<<std::endl;
+    //     OSG_DEBUG << "_spot_cutoff "<<_spot_cutoff<<std::endl;
+    //     OSG_DEBUG << "_constant_attenuation "<<_constant_attenuation<<std::endl;
+    //     OSG_DEBUG << "_linear_attenuation "<<_linear_attenuation<<std::endl;
+    //     OSG_DEBUG << "_quadratic_attenuation "<<_quadratic_attenuation<<std::endl;
 }
 
 void Light::setLightNum(int num) 
@@ -99,10 +99,9 @@ void Light::setLightNum(int num)
     }
 }
 
-
-
 void Light::captureLightState()
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
     glGetLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_AMBIENT, _ambient.ptr() );
     glGetLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_DIFFUSE, _diffuse.ptr() );
     glGetLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_SPECULAR, _specular.ptr() );
@@ -113,10 +112,14 @@ void Light::captureLightState()
     glGetLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_CONSTANT_ATTENUATION,   &_constant_attenuation );
     glGetLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_LINEAR_ATTENUATION,   &_linear_attenuation );
     glGetLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_QUADRATIC_ATTENUATION,   &_quadratic_attenuation );
+#else
+    OSG_NOTICE<<"Warning: Light::captureLightState() - not supported."<<std::endl;
+#endif
 }
 
 void Light::apply(State&) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
     glLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_AMBIENT,               _ambient.ptr() );
     glLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_DIFFUSE,               _diffuse.ptr() );
     glLightfv( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_SPECULAR,              _specular.ptr() );
@@ -127,4 +130,7 @@ void Light::apply(State&) const
     glLightf ( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_CONSTANT_ATTENUATION,  _constant_attenuation );
     glLightf ( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_LINEAR_ATTENUATION,    _linear_attenuation );
     glLightf ( (GLenum)((int)GL_LIGHT0 + _lightnum), GL_QUADRATIC_ATTENUATION, _quadratic_attenuation );
+#else
+    OSG_NOTICE<<"Warning: Light::apply(State&) - not supported."<<std::endl;
+#endif
 }
