@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -40,6 +40,10 @@ void LineStipple::setPattern(GLushort pattern)
 
 void LineStipple::apply(State&) const
 {
+#ifdef OSG_GL1_AVAILABLE
     glLineStipple(_factor, _pattern);
+#else
+    OSG_NOTICE<<"Warning: LineStipple::apply(State&) - not supported."<<std::endl;
+#endif
 }
 
