@@ -17,6 +17,7 @@
 #include <osg/Notify>
 #include <osg/OccluderNode>
 #include <osg/Transform>
+#include <osg/UserDataContainer>
 
 #include <algorithm>
 
@@ -28,7 +29,7 @@ namespace osg
     class CollectParentPaths : public NodeVisitor
     {
     public:
-        CollectParentPaths(osg::Node* haltTraversalAtNode=0) : 
+        CollectParentPaths(const osg::Node* haltTraversalAtNode=0) : 
             osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_PARENTS),
             _haltTraversalAtNode(haltTraversalAtNode)
         {
@@ -46,7 +47,7 @@ namespace osg
             }
        }
 
-        Node*           _haltTraversalAtNode;
+        const Node*     _haltTraversalAtNode;
         NodePath        _nodePath;
         NodePathList    _nodePaths;
     };
@@ -74,17 +75,16 @@ Node::Node(const Node& node,const CopyOp& copyop):
         _boundingSphere(node._boundingSphere),
         _boundingSphereComputed(node._boundingSphereComputed),
         _parents(), // leave empty as parentList is managed by Group.
-        _updateCallback(node._updateCallback),
+        _updateCallback(copyop(node._updateCallback.get())),
         _numChildrenRequiringUpdateTraversal(0), // assume no children yet.
         _numChildrenRequiringEventTraversal(0), // assume no children yet.
-        _cullCallback(node._cullCallback),
+        _cullCallback(copyop(node._cullCallback.get())),
         _cullingActive(node._cullingActive),
         _numChildrenWithCullingDisabled(0), // assume no children yet.
         _numChildrenWithOccluderNodes(0),
-        _nodeMask(node._nodeMask), 
-        _descriptions(node._descriptions),
-        _stateset(copyop(node._stateset.get()))
+        _nodeMask(node._nodeMask)
 {
+    setStateSet(copyop(node._stateset.get()));
 }
 
 Node::~Node()
@@ -95,32 +95,17 @@ Node::~Node()
 
 void Node::addParent(osg::Group* node)
 {
-    if (getRefMutex())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
 
-        _parents.push_back(node);
-    }
-    else
-    {
-        _parents.push_back(node);
-    }
+    _parents.push_back(node);
 }
 
 void Node::removeParent(osg::Group* node)
 {
-    if (getRefMutex())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
 
-        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
-        if (pitr!=_parents.end()) _parents.erase(pitr);
-    }
-    else
-    {
-        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
-        if (pitr!=_parents.end()) _parents.erase(pitr);
-    }
+    ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
+    if (pitr!=_parents.end()) _parents.erase(pitr);
 }
 
 void Node::accept(NodeVisitor& nv)
@@ -191,7 +176,7 @@ NodePathList Node::getParentalNodePaths(osg::Node* haltTraversalAtNode) const
     return cpp._nodePaths;
 }
 
-MatrixList Node::getWorldMatrices(osg::Node* haltTraversalAtNode) const
+MatrixList Node::getWorldMatrices(const osg::Node* haltTraversalAtNode) const
 {
     CollectParentPaths cpp(haltTraversalAtNode);
     const_cast<Node*>(this)->accept(cpp);
@@ -239,7 +224,7 @@ void Node::setUpdateCallback(NodeCallback* nc)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -275,7 +260,7 @@ void Node::setNumChildrenRequiringUpdateTraversal(unsigned int num)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -317,7 +302,7 @@ void Node::setEventCallback(NodeCallback* nc)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -353,7 +338,7 @@ void Node::setNumChildrenRequiringEventTraversal(unsigned int num)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -393,7 +378,7 @@ void Node::setCullingActive(bool active)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -428,7 +413,7 @@ void Node::setNumChildrenWithCullingDisabled(unsigned int num)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -465,7 +450,7 @@ void Node::setNumChildrenWithOccluderNodes(unsigned int num)
         {
             // the number of callbacks has changed, need to pass this
             // on to parents so they know whether app traversal is
-            // reqired on this subgraph.
+            // required on this subgraph.
             for(ParentList::iterator itr =_parents.begin();
                 itr != _parents.end();
                 ++itr)
@@ -488,6 +473,51 @@ bool Node::containsOccluderNodes() const
     return _numChildrenWithOccluderNodes>0 || dynamic_cast<const OccluderNode*>(this);
 }
 
+void Node::setDescriptions(const DescriptionList& descriptions)
+{
+    getOrCreateUserDataContainer()->setDescriptions(descriptions);
+}
+
+Node::DescriptionList& Node::getDescriptions()
+{
+    return getOrCreateUserDataContainer()->getDescriptions();
+}
+
+static OpenThreads::Mutex s_mutex_StaticDescriptionList;
+static const Node::DescriptionList& getStaticDescriptionList()
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_StaticDescriptionList);
+    static Node::DescriptionList s_descriptionList;
+    return s_descriptionList;
+}
+
+const Node::DescriptionList& Node::getDescriptions() const
+{
+    if (_userDataContainer) return _userDataContainer->getDescriptions();
+    else return getStaticDescriptionList();
+}
+
+std::string& Node::getDescription(unsigned int i)
+{
+    return getOrCreateUserDataContainer()->getDescriptions()[i];
+}
+
+const std::string& Node::getDescription(unsigned int i) const
+{
+    if (_userDataContainer) return _userDataContainer->getDescriptions()[i];
+    else return getStaticDescriptionList()[i];
+}
+
+unsigned int Node::getNumDescriptions() const
+{
+    return _userDataContainer ? _userDataContainer->getDescriptions().size() : 0;
+}
+
+void Node::addDescription(const std::string& desc)
+{
+    getOrCreateUserDataContainer()->getDescriptions().push_back(desc);
+}
+
 BoundingSphere Node::computeBound() const
 {
     return BoundingSphere();
@@ -516,12 +546,9 @@ void Node::setThreadSafeRefUnref(bool threadSafe)
     Object::setThreadSafeRefUnref(threadSafe);
     
     if (_stateset.valid()) _stateset->setThreadSafeRefUnref(threadSafe);
-
     if (_updateCallback.valid()) _updateCallback->setThreadSafeRefUnref(threadSafe);
     if (_eventCallback.valid()) _eventCallback->setThreadSafeRefUnref(threadSafe);
     if (_cullCallback.valid()) _cullCallback->setThreadSafeRefUnref(threadSafe);
-
-    if (_userData.valid()) _userData->setThreadSafeRefUnref(threadSafe);
 }
 
 void Node::resizeGLObjectBuffers(unsigned int maxSize)
