@@ -48,129 +48,60 @@ static OcclusionQueryBufferedExtensions s_OQ_bufferedExtensions;
 //   of these classes existed in their own separate header and
 //   source files.)
 
+namespace osg
+{
 
 // Create and return a StateSet appropriate for performing an occlusion
 //   query test (disable lighting, texture mapping, etc). Probably some
 //   room for improvement here. Could disable shaders, for example.
-osg::StateSet*
-initOQState()
+StateSet* initOQState()
 {
-    osg::StateSet* state = new osg::StateSet;
+    StateSet* state = new StateSet;
     // TBD Possible bug, need to allow user to set render bin number.
     state->setRenderBinDetails( 9, "RenderBin" );
 
-    state->setMode( GL_LIGHTING, osg::StateAttribute::OFF |
-        osg::StateAttribute::PROTECTED);
-    state->setTextureMode( 0, GL_TEXTURE_2D, osg::StateAttribute::OFF |
-        osg::StateAttribute::PROTECTED);
-    state->setMode( GL_CULL_FACE, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
+    state->setMode( GL_LIGHTING, StateAttribute::OFF | StateAttribute::PROTECTED);
+    state->setTextureMode( 0, GL_TEXTURE_2D, StateAttribute::OFF | StateAttribute::PROTECTED);
+    state->setMode( GL_CULL_FACE, StateAttribute::ON | StateAttribute::PROTECTED);
 
-    osg::ColorMask* cm = new osg::ColorMask( false, false, false, false );
-    state->setAttributeAndModes( cm, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
-    osg::Depth* d = new osg::Depth( osg::Depth::LEQUAL, 0.f, 1.f, false );
-    state->setAttributeAndModes( d, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
-    osg::PolygonMode* pm = new osg::PolygonMode(
-        osg::PolygonMode::FRONT_AND_BACK, osg::PolygonMode::FILL );
-    state->setAttributeAndModes( pm, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
+    ColorMask* cm = new ColorMask( false, false, false, false );
+    state->setAttributeAndModes( cm, StateAttribute::ON | StateAttribute::PROTECTED);
 
-    osg::PolygonOffset* po = new osg::PolygonOffset( -1., -1. );
-    state->setAttributeAndModes( po, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
+    Depth* d = new Depth( Depth::LEQUAL, 0.f, 1.f, false );
+    state->setAttributeAndModes( d, StateAttribute::ON | StateAttribute::PROTECTED);
+
+    PolygonMode* pm = new PolygonMode( PolygonMode::FRONT_AND_BACK, PolygonMode::FILL );
+    state->setAttributeAndModes( pm, StateAttribute::ON | StateAttribute::PROTECTED);
+
+    PolygonOffset* po = new PolygonOffset( -1., -1. );
+    state->setAttributeAndModes( po, StateAttribute::ON | StateAttribute::PROTECTED);
 
     return state;
 }
 
 // Create and return a StateSet for rendering a debug representation of query geometry.
-osg::StateSet*
-initOQDebugState()
+StateSet* initOQDebugState()
 {
     osg::StateSet* debugState = new osg::StateSet;
 
-    debugState->setMode( GL_LIGHTING, osg::StateAttribute::OFF |
-        osg::StateAttribute::PROTECTED);
-    debugState->setTextureMode( 0, GL_TEXTURE_2D, osg::StateAttribute::OFF |
-        osg::StateAttribute::PROTECTED);
-    debugState->setMode( GL_CULL_FACE, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
+    debugState->setMode( GL_LIGHTING, StateAttribute::OFF | StateAttribute::PROTECTED);
+    debugState->setTextureMode( 0, GL_TEXTURE_2D, StateAttribute::OFF | StateAttribute::PROTECTED);
+    debugState->setMode( GL_CULL_FACE, StateAttribute::ON | StateAttribute::PROTECTED);
 
-    osg::PolygonMode* pm = new osg::PolygonMode(
-        osg::PolygonMode::FRONT_AND_BACK, osg::PolygonMode::LINE );
-    debugState->setAttributeAndModes( pm, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
+    PolygonMode* pm = new PolygonMode( PolygonMode::FRONT_AND_BACK, PolygonMode::LINE );
+    debugState->setAttributeAndModes( pm, StateAttribute::ON | StateAttribute::PROTECTED);
 
-    osg::PolygonOffset* po = new osg::PolygonOffset( -1., -1. );
-    debugState->setAttributeAndModes( po, osg::StateAttribute::ON |
-        osg::StateAttribute::PROTECTED);
+    PolygonOffset* po = new PolygonOffset( -1., -1. );
+    debugState->setAttributeAndModes( po, StateAttribute::ON | StateAttribute::PROTECTED);
 
     return debugState;
 }
 
-
-// TestResult -- stores (per context) results of an occlusion query
-//   test performed by QueryGeometry. An OcclusionQueryNode has a
-//   Geode owning a single QueryGeometry that
-//   draws the occlusion query geometry. QueryGeometry keeps a
-//   TestResult per context to store the result/status of each query.
-// Accessed during the cull and draw traversals.
-class TestResult : public osg::Referenced
-{
-public:
-    TestResult() : _init( false ), _id( 0 ), _contextID( 0 ), _active( false ), _numPixels( 0 ) {}
-    ~TestResult() {}
-
-    bool _init;
-
-    // Query ID for this context.
-    GLuint _id;
-    // Context ID owning this query ID.
-    unsigned int _contextID;
-
-    // Set to true when a query gets issued and set to
-    //   false when the result is retrieved.
-    mutable bool _active;
-
-    // Result of last query.
-    GLint _numPixels;
-};
-
-// QueryGeometry -- A Drawable that performs an occlusion query,
-//   using its geometric data as the query geometry.
-class QueryGeometry : public osg::Geometry
-{
-public:
-    QueryGeometry( const std::string& oqnName=std::string("") );
-    ~QueryGeometry();
-
-    void reset();
-
-    // TBD implement copy constructor
-
-    virtual void drawImplementation( osg::RenderInfo& renderInfo ) const;
-
-    unsigned int getNumPixels( const osg::Camera* cam );
-
-
-    void releaseGLObjects( osg::State* state = 0 );
-    static void deleteQueryObject( unsigned int contextID, GLuint handle );
-    static void flushDeletedQueryObjects( unsigned int contextID, double currentTime, double& availableTime );
-    static void discardDeletedQueryObjects( unsigned int contextID );
-    
-protected:
-    typedef std::map< const osg::Camera*, TestResult > ResultMap;
-    mutable ResultMap _results;
-    mutable OpenThreads::Mutex _mapMutex;
-
-    // Needed for debug only
-    std::string _oqnName;
-};
+}
 
 struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
 {
-    typedef std::vector<TestResult*> ResultsVector;
+    typedef std::vector<osg::TestResult*> ResultsVector;
     ResultsVector _results;
 
     RetrieveQueriesCallback( osg::Drawable::Extensions* ext=NULL )
@@ -206,11 +137,11 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
             //   SceneView-based apps. Rely on the creating code to have passed
             //   in a valid Extensions pointer, and hope it's valid for any
             //   context that might be current.
-            osg::notify( osg::DEBUG_INFO ) << "osgOQ: RQCB: Using fallback path to obtain Extensions pointer." << std::endl;
+            OSG_DEBUG << "osgOQ: RQCB: Using fallback path to obtain Extensions pointer." << std::endl;
             ext = _extensionsFallback;
             if (!ext)
             {
-                osg::notify( osg::FATAL ) << "osgOQ: RQCB: Extensions pointer fallback is NULL." << std::endl;
+                OSG_FATAL << "osgOQ: RQCB: Extensions pointer fallback is NULL." << std::endl;
                 return;
             }
         }
@@ -218,7 +149,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
         ResultsVector::const_iterator it = _results.begin();
         while (it != _results.end())
         {
-            TestResult* tr = const_cast<TestResult*>( *it );
+            osg::TestResult* tr = const_cast<osg::TestResult*>( *it );
 
             if (!tr->_active || !tr->_init)
             {
@@ -231,7 +162,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
                 continue;
             }
 
-            osg::notify( osg::DEBUG_INFO ) <<
+            OSG_DEBUG <<
                 "osgOQ: RQCB: Retrieving..." << std::endl;
 
 #ifdef FORCE_QUERY_RESULT_AVAILABLE_BEFORE_RETRIEVAL
@@ -253,7 +184,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
 
             ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT, &(tr->_numPixels) );
             if (tr->_numPixels < 0)
-                osg::notify( osg::WARN ) << "osgOQ: RQCB: " <<
+                OSG_WARN << "osgOQ: RQCB: " <<
                 "glGetQueryObjectiv returned negative value (" << tr->_numPixels << ")." << std::endl;
 
             // Either retrieve last frame's results, or ignore it because the
@@ -265,7 +196,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
         }
 
         elapsedTime = timer.delta_s(start_tick,timer.tick());
-        osg::notify( osg::INFO ) << "osgOQ: RQCB: " << "Retrieved " << count <<
+        OSG_INFO << "osgOQ: RQCB: " << "Retrieved " << count <<
             " queries in " << elapsedTime << " seconds." << std::endl;
     }
 
@@ -274,7 +205,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
         _results.clear();
     }
 
-    void add( TestResult* tr )
+    void add( osg::TestResult* tr )
     {
         _results.push_back( tr );
     }
@@ -303,7 +234,7 @@ struct ClearQueriesCallback : public osg::Camera::DrawCallback
     {
         if (!_rqcb)
         {
-            osg::notify( osg::FATAL ) << "osgOQ: CQCB: Invalid RQCB." << std::endl;
+            OSG_FATAL << "osgOQ: CQCB: Invalid RQCB." << std::endl;
             return;
         }
         _rqcb->reset();
@@ -322,6 +253,10 @@ typedef osg::buffered_object< QueryObjectList > DeletedQueryObjectCache;
 static OpenThreads::Mutex s_mutex_deletedQueryObjectCache;
 static DeletedQueryObjectCache s_deletedQueryObjectCache;
 
+namespace osg
+{
+
+
 QueryGeometry::QueryGeometry( const std::string& oqnName )
   : _oqnName( oqnName )
 {
@@ -384,7 +319,7 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
         RetrieveQueriesCallback* >( cam->getPostDrawCallback() );
     if (!rqcb)
     {
-        osg::notify( osg::FATAL ) << "osgOQ: QG: Invalid RQCB." << std::endl;
+        OSG_FATAL << "osgOQ: QG: Invalid RQCB." << std::endl;
         return;
     }
     rqcb->add( tr );
@@ -398,7 +333,7 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
         tr->_init = true;
     }
 
-    osg::notify( osg::DEBUG_INFO ) <<
+    OSG_DEBUG <<
         "osgOQ: QG: Querying for: " << _oqnName << std::endl;
 
     ext->glBeginQuery( GL_SAMPLES_PASSED_ARB, tr->_id );
@@ -407,7 +342,7 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
     tr->_active = true;
 
 
-    osg::notify( osg::DEBUG_INFO ) <<
+    OSG_DEBUG <<
         "osgOQ: QG. OQNName: " << _oqnName <<
         ", Ctx: " << contextID <<
         ", ID: " << tr->_id << std::endl;
@@ -415,8 +350,9 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
     {
         GLenum err;
         if ((err = glGetError()) != GL_NO_ERROR)
-            osg::notify( osg::FATAL ) <<
-            "osgOQ: QG: OpenGL error: " << err << "." << std::endl;
+        {
+            OSG_FATAL << "osgOQ: QG: OpenGL error: " << err << "." << std::endl;
+        }
     }
 #endif
 
@@ -437,14 +373,17 @@ QueryGeometry::getNumPixels( const osg::Camera* cam )
 
 
 void
-QueryGeometry::releaseGLObjects( osg::State* state )
+QueryGeometry::releaseGLObjects( osg::State* state ) const
 {
     if (!state)
+    {
         // delete all query IDs for all contexts.
-        reset();
-
+        const_cast<QueryGeometry*>(this)->reset();
+    }
     else
     {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _mapMutex );
+
         // Delete all query IDs for the specified context.
         unsigned int contextID = state->getContextID();
         ResultMap::iterator it = _results.begin();
@@ -517,9 +456,6 @@ QueryGeometry::discardDeletedQueryObjects( unsigned int contextID )
 
 
 
-namespace osg
-{
-
 
 OcclusionQueryNode::OcclusionQueryNode()
   : _enabled( true ),
@@ -537,7 +473,7 @@ OcclusionQueryNode::~OcclusionQueryNode()
 {
 }
 
-OcclusionQueryNode::OcclusionQueryNode( const OcclusionQueryNode& oqn, const osg::CopyOp& copyop )
+OcclusionQueryNode::OcclusionQueryNode( const OcclusionQueryNode& oqn, const CopyOp& copyop )
   : Group( oqn, copyop ),
     _passed( false )
 {
@@ -551,8 +487,7 @@ OcclusionQueryNode::OcclusionQueryNode( const OcclusionQueryNode& oqn, const osg
 }
 
 
-bool
-OcclusionQueryNode::getPassed( const osg::Camera* camera, osg::NodeVisitor& nv )
+bool OcclusionQueryNode::getPassed( const Camera* camera, NodeVisitor& nv )
 {
     if ( !_enabled )
         // Queries are not enabled. The caller should be osgUtil::CullVisitor,
@@ -565,7 +500,7 @@ OcclusionQueryNode::getPassed( const osg::Camera* camera, osg::NodeVisitor& nv )
         //  2) we haven't rendered for an abnormally long time (probably because we're an out-of-range LOD child)
         // In these cases, assume we're visible to avoid blinking.
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _frameCountMutex );
-        const int& lastQueryFrame( _frameCountMap[ camera ] );
+        const unsigned int& lastQueryFrame( _frameCountMap[ camera ] );
         if( ( lastQueryFrame == 0 ) ||
             ( (nv.getTraversalNumber() - lastQueryFrame) >  (_queryFrameCount + 1) ) )
             return true;
@@ -573,8 +508,7 @@ OcclusionQueryNode::getPassed( const osg::Camera* camera, osg::NodeVisitor& nv )
 
     if (_queryGeode->getDrawable( 0 ) == NULL)
     {
-        osg::notify( osg::FATAL ) <<
-            "osgOQ: OcclusionQueryNode: No QueryGeometry." << std::endl;
+        OSG_FATAL << "osgOQ: OcclusionQueryNode: No QueryGeometry." << std::endl;
         // Something's broke. Return true so we at least render correctly.
         return true;
     }
@@ -605,15 +539,14 @@ OcclusionQueryNode::getPassed( const osg::Camera* camera, osg::NodeVisitor& nv )
     return _passed;
 }
 
-void
-OcclusionQueryNode::traverseQuery( const osg::Camera* camera, osg::NodeVisitor& nv )
+void OcclusionQueryNode::traverseQuery( const Camera* camera, NodeVisitor& nv )
 {
     bool issueQuery;
     {
         const int curFrame = nv.getTraversalNumber();
 
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _frameCountMutex );
-        int& lastQueryFrame = _frameCountMap[ camera ];
+        unsigned int& lastQueryFrame = _frameCountMap[ camera ];
         issueQuery = (curFrame - lastQueryFrame >= _queryFrameCount);
         if (issueQuery)
             lastQueryFrame = curFrame;
@@ -622,16 +555,14 @@ OcclusionQueryNode::traverseQuery( const osg::Camera* camera, osg::NodeVisitor&
         _queryGeode->accept( nv );
 }
 
-void
-OcclusionQueryNode::traverseDebug( osg::NodeVisitor& nv )
+void OcclusionQueryNode::traverseDebug( NodeVisitor& nv )
 {
     if (_debugBB)
         // If requested, display the debug geometry
         _debugGeode->accept( nv );
 }
 
-osg::BoundingSphere
-OcclusionQueryNode::computeBound() const
+BoundingSphere OcclusionQueryNode::computeBound() const
 {
     {
         // Need to make this routine thread-safe. Typically called by the update
@@ -644,22 +575,22 @@ OcclusionQueryNode::computeBound() const
         osg::OcclusionQueryNode* nonConstThis = const_cast<osg::OcclusionQueryNode*>( this );
 
 
-        osg::ComputeBoundsVisitor cbv;
+        ComputeBoundsVisitor cbv;
         nonConstThis->accept( cbv );
-        osg::BoundingBox bb = cbv.getBoundingBox();
+        BoundingBox bb = cbv.getBoundingBox();
 
-        osg::ref_ptr<osg::Vec3Array> v = new osg::Vec3Array;
+        osg::ref_ptr<Vec3Array> v = new Vec3Array;
         v->resize( 8 );
-        (*v)[0] = osg::Vec3( bb._min.x(), bb._min.y(), bb._min.z() );
-        (*v)[1] = osg::Vec3( bb._max.x(), bb._min.y(), bb._min.z() );
-        (*v)[2] = osg::Vec3( bb._max.x(), bb._min.y(), bb._max.z() );
-        (*v)[3] = osg::Vec3( bb._min.x(), bb._min.y(), bb._max.z() );
-        (*v)[4] = osg::Vec3( bb._max.x(), bb._max.y(), bb._min.z() );
-        (*v)[5] = osg::Vec3( bb._min.x(), bb._max.y(), bb._min.z() );
-        (*v)[6] = osg::Vec3( bb._min.x(), bb._max.y(), bb._max.z() );
-        (*v)[7] = osg::Vec3( bb._max.x(), bb._max.y(), bb._max.z() );
+        (*v)[0] = Vec3( bb._min.x(), bb._min.y(), bb._min.z() );
+        (*v)[1] = Vec3( bb._max.x(), bb._min.y(), bb._min.z() );
+        (*v)[2] = Vec3( bb._max.x(), bb._min.y(), bb._max.z() );
+        (*v)[3] = Vec3( bb._min.x(), bb._min.y(), bb._max.z() );
+        (*v)[4] = Vec3( bb._max.x(), bb._max.y(), bb._min.z() );
+        (*v)[5] = Vec3( bb._min.x(), bb._max.y(), bb._min.z() );
+        (*v)[6] = Vec3( bb._min.x(), bb._max.y(), bb._max.z() );
+        (*v)[7] = Vec3( bb._max.x(), bb._max.y(), bb._max.z() );
 
-        osg::Geometry* geom = static_cast< osg::Geometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
+        Geometry* geom = static_cast< Geometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
         geom->setVertexArray( v.get() );
 
         geom = static_cast< osg::Geometry* >( nonConstThis->_debugGeode->getDrawable( 0 ) );
@@ -671,100 +602,89 @@ OcclusionQueryNode::computeBound() const
 
 
 // Should only be called outside of cull/draw. No thread issues.
-void
-OcclusionQueryNode::setQueriesEnabled( bool enable )
+void OcclusionQueryNode::setQueriesEnabled( bool enable )
 {
     _enabled = enable;
 }
 
 // Should only be called outside of cull/draw. No thread issues.
-void
-OcclusionQueryNode::setDebugDisplay( bool debug )
+void OcclusionQueryNode::setDebugDisplay( bool debug )
 {
     _debugBB = debug;
 }
-bool
-OcclusionQueryNode::getDebugDisplay() const
+
+bool OcclusionQueryNode::getDebugDisplay() const
 {
     return _debugBB;
 }
 
-
-
-void
-OcclusionQueryNode::setQueryStateSet( osg::StateSet* ss )
+void OcclusionQueryNode::setQueryStateSet( StateSet* ss )
 {
     if (!_queryGeode)
     {
-        osg::notify( osg::WARN ) << "osgOQ: OcclusionQueryNode:: Invalid query support node." << std::endl;
+        OSG_WARN << "osgOQ: OcclusionQueryNode:: Invalid query support node." << std::endl;
         return;
     }
 
     _queryGeode->setStateSet( ss );
 }
-osg::StateSet*
-OcclusionQueryNode::getQueryStateSet()
+
+StateSet* OcclusionQueryNode::getQueryStateSet()
 {
     if (!_queryGeode)
     {
-        osg::notify( osg::WARN ) << "osgOQ: OcclusionQueryNode:: Invalid query support node." << std::endl;
+        OSG_WARN << "osgOQ: OcclusionQueryNode:: Invalid query support node." << std::endl;
         return NULL;
     }
     return _queryGeode->getStateSet();
 }
 
-const osg::StateSet*
-OcclusionQueryNode::getQueryStateSet() const
+const StateSet* OcclusionQueryNode::getQueryStateSet() const
 {
     if (!_queryGeode)
     {
-        osg::notify( osg::WARN ) << "osgOQ: OcclusionQueryNode:: Invalid query support node." << std::endl;
+        OSG_WARN << "osgOQ: OcclusionQueryNode:: Invalid query support node." << std::endl;
         return NULL;
     }
     return _queryGeode->getStateSet();
 }
 
-void
-OcclusionQueryNode::setDebugStateSet( osg::StateSet* ss )
+void OcclusionQueryNode::setDebugStateSet( StateSet* ss )
 {
     if (!_debugGeode)
     {
-        osg::notify( osg::WARN ) << "osgOQ: OcclusionQueryNode:: Invalid debug support node." << std::endl;
+        OSG_WARN << "osgOQ: OcclusionQueryNode:: Invalid debug support node." << std::endl;
         return;
     }
     _debugGeode->setStateSet( ss );
 }
 
-osg::StateSet*
-OcclusionQueryNode::getDebugStateSet()
+StateSet* OcclusionQueryNode::getDebugStateSet()
 {
     if (!_debugGeode.valid())
     {
-        osg::notify( osg::WARN ) << "osgOQ: OcclusionQueryNode:: Invalid debug support node." << std::endl;
+        OSG_WARN << "osgOQ: OcclusionQueryNode:: Invalid debug support node." << std::endl;
         return NULL;
     }
     return _debugGeode->getStateSet();
 }
-const osg::StateSet*
-OcclusionQueryNode::getDebugStateSet() const
+const StateSet* OcclusionQueryNode::getDebugStateSet() const
 {
     if (!_debugGeode.valid())
     {
-        osg::notify( osg::WARN ) << "osgOQ: OcclusionQueryNode:: Invalid debug support node." << std::endl;
+        OSG_WARN << "osgOQ: OcclusionQueryNode:: Invalid debug support node." << std::endl;
         return NULL;
     }
     return _debugGeode->getStateSet();
 }
 
-bool
-OcclusionQueryNode::getPassed() const
+bool OcclusionQueryNode::getPassed() const
 {
     return _passed;
 }
 
 
-void
-OcclusionQueryNode::createSupportNodes()
+void OcclusionQueryNode::createSupportNodes()
 {
     GLushort indices[] = { 0, 1, 2, 3,  4, 5, 6, 7,
         0, 3, 6, 5,  2, 1, 4, 7,
@@ -772,14 +692,13 @@ OcclusionQueryNode::createSupportNodes()
 
     {
         // Add the test geometry Geode
-        _queryGeode = new osg::Geode;
+        _queryGeode = new Geode;
         _queryGeode->setName( "OQTest" );
-        _queryGeode->setDataVariance( osg::Object::DYNAMIC );
+        _queryGeode->setDataVariance( Object::DYNAMIC );
 
-        osg::ref_ptr< QueryGeometry > geom = new QueryGeometry( getName() );
-        geom->setDataVariance( osg::Object::DYNAMIC );
-        geom->addPrimitiveSet( new osg::DrawElementsUShort(
-                    osg::PrimitiveSet::QUADS, 24, indices ) );
+        ref_ptr< QueryGeometry > geom = new QueryGeometry( getName() );
+        geom->setDataVariance( Object::DYNAMIC );
+        geom->addPrimitiveSet( new DrawElementsUShort( PrimitiveSet::QUADS, 24, indices ) );
 
         _queryGeode->addDrawable( geom.get() );
     }
@@ -787,20 +706,19 @@ OcclusionQueryNode::createSupportNodes()
     {
         // Add a Geode that is a visual representation of the
         //   test geometry for debugging purposes
-        _debugGeode = new osg::Geode;
+        _debugGeode = new Geode;
         _debugGeode->setName( "Debug" );
-        _debugGeode->setDataVariance( osg::Object::DYNAMIC );
+        _debugGeode->setDataVariance( Object::DYNAMIC );
 
-        osg::ref_ptr<osg::Geometry> geom = new osg::Geometry;
-        geom->setDataVariance( osg::Object::DYNAMIC );
+        ref_ptr<Geometry> geom = new Geometry;
+        geom->setDataVariance( Object::DYNAMIC );
 
-        osg::ref_ptr<osg::Vec4Array> ca = new osg::Vec4Array;
-        ca->push_back( osg::Vec4( 1.f, 1.f, 1.f, 1.f ) );
+        ref_ptr<Vec4Array> ca = new Vec4Array;
+        ca->push_back( Vec4( 1.f, 1.f, 1.f, 1.f ) );
         geom->setColorArray( ca.get() );
-        geom->setColorBinding( osg::Geometry::BIND_OVERALL );
+        geom->setColorBinding( Geometry::BIND_OVERALL );
 
-        geom->addPrimitiveSet( new osg::DrawElementsUShort(
-                    osg::PrimitiveSet::QUADS, 24, indices ) );
+        geom->addPrimitiveSet( new DrawElementsUShort( PrimitiveSet::QUADS, 24, indices ) );
 
         _debugGeode->addDrawable( geom.get() );
     }
@@ -813,8 +731,7 @@ OcclusionQueryNode::createSupportNodes()
 }
 
 
-void
-OcclusionQueryNode::releaseGLObjects( osg::State* state ) const
+void OcclusionQueryNode::releaseGLObjects( State* state ) const
 {
     if(_queryGeode->getDrawable( 0 ) != NULL)
     {
@@ -825,15 +742,13 @@ OcclusionQueryNode::releaseGLObjects( osg::State* state ) const
     }
 }
 
-void
-OcclusionQueryNode::flushDeletedQueryObjects( unsigned int contextID, double currentTime, double& availableTime )
+void OcclusionQueryNode::flushDeletedQueryObjects( unsigned int contextID, double currentTime, double& availableTime )
 {
     // Query object discard and deletion is handled by QueryGeometry support class.
     QueryGeometry::flushDeletedQueryObjects( contextID, currentTime, availableTime );
 }
 
-void
-OcclusionQueryNode::discardDeletedQueryObjects( unsigned int contextID )
+void OcclusionQueryNode::discardDeletedQueryObjects( unsigned int contextID )
 {
     // Query object discard and deletion is handled by QueryGeometry support class.
     QueryGeometry::discardDeletedQueryObjects( contextID );
