@@ -22,13 +22,18 @@ using namespace osg;
 PagedLOD::PerRangeData::PerRangeData():
     _priorityOffset(0.0f),
     _priorityScale(1.0f),
-    _timeStamp(0.0f) {}
+    _timeStamp(0.0f),
+    _frameNumber(0),
+    _frameNumberOfLastReleaseGLObjects(0) {}
 
 PagedLOD::PerRangeData::PerRangeData(const PerRangeData& prd):
     _filename(prd._filename),
     _priorityOffset(prd._priorityOffset),
     _priorityScale(prd._priorityScale),
-    _timeStamp(prd._timeStamp) {}
+    _timeStamp(prd._timeStamp),
+    _frameNumber(prd._frameNumber),
+    _frameNumberOfLastReleaseGLObjects(prd._frameNumberOfLastReleaseGLObjects),
+    _databaseRequest(prd._databaseRequest) {}
 
 PagedLOD::PerRangeData& PagedLOD::PerRangeData::operator = (const PerRangeData& prd)
 {
@@ -37,6 +42,9 @@ PagedLOD::PerRangeData& PagedLOD::PerRangeData::operator = (const PerRangeData&
     _priorityOffset = prd._priorityOffset;
     _priorityScale = prd._priorityScale;
     _timeStamp = prd._timeStamp;
+    _frameNumber = prd._frameNumber;
+    _frameNumberOfLastReleaseGLObjects = prd._frameNumberOfLastReleaseGLObjects;
+    _databaseRequest = prd._databaseRequest;
     return *this;
 }
 
@@ -46,17 +54,24 @@ PagedLOD::PagedLOD()
     _centerMode = USER_DEFINED_CENTER;
     _radius = -1;
     _numChildrenThatCannotBeExpired = 0;
+    _disableExternalChildrenPaging = false;
 }
 
 PagedLOD::PagedLOD(const PagedLOD& plod,const CopyOp& copyop):
     LOD(plod,copyop),
+    _databaseOptions(plod._databaseOptions),
     _databasePath(plod._databasePath),
     _frameNumberOfLastTraversal(plod._frameNumberOfLastTraversal),
     _numChildrenThatCannotBeExpired(plod._numChildrenThatCannotBeExpired),
+    _disableExternalChildrenPaging(plod._disableExternalChildrenPaging),
     _perRangeDataList(plod._perRangeDataList)
 {
 }
 
+PagedLOD::~PagedLOD()
+{
+}
+
 void PagedLOD::setDatabasePath(const std::string& path)
 {
     _databasePath = path;
@@ -105,9 +120,14 @@ void PagedLOD::traverse(NodeVisitor& nv)
 {
     // set the frame number of the traversal so that external nodes can find out how active this
     // node is.
-    if (nv.getFrameStamp()) setFrameNumberOfLastTraversal(nv.getFrameStamp()->getFrameNumber());
+    if (nv.getFrameStamp() && 
+        nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR) 
+    {
+        setFrameNumberOfLastTraversal(nv.getFrameStamp()->getFrameNumber());
+    }
 
     double timeStamp = nv.getFrameStamp()?nv.getFrameStamp()->getReferenceTime():0.0;
+    unsigned int frameNumber = nv.getFrameStamp()?nv.getFrameStamp()->getFrameNumber():0;
     bool updateTimeStamp = nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR;
 
     switch(nv.getTraversalMode())
@@ -148,7 +168,11 @@ void PagedLOD::traverse(NodeVisitor& nv)
                 {
                     if (i<_children.size())
                     {
-                        if (updateTimeStamp) _perRangeDataList[i]._timeStamp=timeStamp;
+                        if (updateTimeStamp)
+                        {
+                            _perRangeDataList[i]._timeStamp=timeStamp;
+                            _perRangeDataList[i]._frameNumber=frameNumber;
+                        }
 
                         _children[i]->accept(nv);
                         lastChildTraversed = (int)i;
@@ -167,12 +191,18 @@ void PagedLOD::traverse(NodeVisitor& nv)
                 // select the last valid child.
                 if (numChildren>0 && ((int)numChildren-1)!=lastChildTraversed)
                 {
-                    if (updateTimeStamp) _perRangeDataList[numChildren-1]._timeStamp=timeStamp;
+                    if (updateTimeStamp)
+                    {
+                        _perRangeDataList[numChildren-1]._timeStamp=timeStamp;
+                        _perRangeDataList[numChildren-1]._frameNumber=frameNumber;
+                    }
                     _children[numChildren-1]->accept(nv);
                 }
 
                 // now request the loading of the next unloaded child.
-                if (nv.getDatabaseRequestHandler() && numChildren<_perRangeDataList.size())
+                if (!_disableExternalChildrenPaging &&
+                    nv.getDatabaseRequestHandler() &&
+                    numChildren<_perRangeDataList.size())
                 {
                     // compute priority from where abouts in the required range the distance falls.
                     float priority = (_rangeList[numChildren].second-required_range)/(_rangeList[numChildren].second-_rangeList[numChildren].first);
@@ -188,16 +218,15 @@ void PagedLOD::traverse(NodeVisitor& nv)
 
                     if (_databasePath.empty())
                     {
-                        nv.getDatabaseRequestHandler()->requestNodeFile(_perRangeDataList[numChildren]._filename,this,priority,nv.getFrameStamp());
+                        nv.getDatabaseRequestHandler()->requestNodeFile(_perRangeDataList[numChildren]._filename,nv.getNodePath(),priority,nv.getFrameStamp(), _perRangeDataList[numChildren]._databaseRequest, _databaseOptions.get());
                     }
                     else
                     {
                         // prepend the databasePath to the child's filename.
-                        nv.getDatabaseRequestHandler()->requestNodeFile(_databasePath+_perRangeDataList[numChildren]._filename,this,priority,nv.getFrameStamp());
+                        nv.getDatabaseRequestHandler()->requestNodeFile(_databasePath+_perRangeDataList[numChildren]._filename,nv.getNodePath(),priority,nv.getFrameStamp(), _perRangeDataList[numChildren]._databaseRequest, _databaseOptions.get());
                     }
                 }
 
-
             }
 
 
@@ -255,15 +284,18 @@ bool PagedLOD::removeChildren( unsigned int pos,unsigned int numChildrenToRemove
     return Group::removeChildren(pos,numChildrenToRemove);
 }
 
-bool PagedLOD::removeExpiredChildren(double expiryTime,NodeList& removedChildren)
+bool PagedLOD::removeExpiredChildren(double expiryTime, unsigned int expiryFrame, NodeList& removedChildren)
 {
     if (_children.size()>_numChildrenThatCannotBeExpired)
     {
-        if (!_perRangeDataList[_children.size()-1]._filename.empty() && _perRangeDataList[_children.size()-1]._timeStamp<expiryTime)
-        {
-            osg::Node* nodeToRemove = _children[_children.size()-1].get();
+        unsigned cindex = _children.size() - 1;
+        if (!_perRangeDataList[cindex]._filename.empty() &&
+            _perRangeDataList[cindex]._timeStamp<expiryTime &&
+            _perRangeDataList[cindex]._frameNumber<expiryFrame)
+        {            
+            osg::Node* nodeToRemove = _children[cindex].get();
             removedChildren.push_back(nodeToRemove);
-            return Group::removeChildren(_children.size()-1,1);
+            return Group::removeChildren(cindex,1);
         }
     }
     return false;
