@@ -53,19 +53,22 @@ bool PolygonOffset::areFactorAndUnitsMultipliersSet()
 void PolygonOffset::setFactorAndUnitsMultipliersUsingBestGuessForDriver()
 {
     s_MultiplerSet = true;
-    // osg::notify(osg::NOTICE)<<"PolygonOffset::setFactorAndUnitMultipliersUsingBestGuessForDriver()"<<std::endl;
+    // OSG_NOTICE<<"PolygonOffset::setFactorAndUnitMultipliersUsingBestGuessForDriver()"<<std::endl;
 
+#if 0
     const GLubyte* renderer = glGetString(GL_RENDERER);
     if (renderer)
     {
         if ((strstr((const char*)renderer,"Radeon")!=0) ||
-            (strstr((const char*)renderer,"RADEON")!=0))
+            (strstr((const char*)renderer,"RADEON")!=0) ||
+            (strstr((const char*)renderer,"ALL-IN-WONDER")!=0))
         {
             setFactorMultiplier(1.0f);
             setUnitsMultiplier(128.0f);
-            osg::notify(osg::INFO)<<"PolygonOffset::setFactorAndUnitsMultipliersUsingBestGuessForDriver() apply ATI workaround."<<std::endl;
+            OSG_INFO<<"PolygonOffset::setFactorAndUnitsMultipliersUsingBestGuessForDriver() apply ATI workaround."<<std::endl;
         }
     }
+#endif
 }
 
 
