@@ -2,9 +2,10 @@
  * Copyright (C) 2003-2005 3Dlabs Inc. Ltd.
  * Copyright (C) 2004-2005 Nathan Cournia
  * Copyright (C) 2008 Zebra Imaging
+ * Copyright (C) 2010 VIRES Simulationstechnologie GmbH
  *
  * This application is open source and may be redistributed and/or modified   
- * freely and without restriction, both in commericial and non commericial
+ * freely and without restriction, both in commercial and non commercial
  * applications, as long as this copyright notice is maintained.
  * 
  * This application is distributed in the hope that it will be useful,
@@ -15,9 +16,11 @@
 
 /* file:        src/osg/Program.cpp
  * author:      Mike Weiblen 2008-01-19
+ *              Holger Helmich 2010-10-21
 */
 
 #include <list>
+#include <fstream>
 
 #include <osg/Notify>
 #include <osg/State>
@@ -91,13 +94,46 @@ void Program::discardDeletedGlPrograms(unsigned int contextID)
 }
 
 
+///////////////////////////////////////////////////////////////////////////
+// osg::Program::ProgramBinary
+///////////////////////////////////////////////////////////////////////////
+
+Program::ProgramBinary::ProgramBinary() : _format(0)
+{
+}
+
+Program::ProgramBinary::ProgramBinary(const ProgramBinary& rhs, const osg::CopyOp&) :
+    _data(rhs._data), _format(rhs._format)
+{
+}
+
+void Program::ProgramBinary::allocate(unsigned int size)
+{
+    _data.clear();
+    _data.resize(size);
+}
+
+void Program::ProgramBinary::assign(unsigned int size, const unsigned char* data)
+{
+    allocate(size);
+    if (data)
+    {
+        for(unsigned int i=0; i<size; ++i)
+        {
+            _data[i] = data[i];
+        }
+    }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////
 // osg::Program
 ///////////////////////////////////////////////////////////////////////////
 
 Program::Program() :
-    _geometryVerticesOut(0), _geometryInputType(GL_TRIANGLES),
-    _geometryOutputType(GL_TRIANGLE_STRIP)
+    _geometryVerticesOut(1), _geometryInputType(GL_TRIANGLES),
+    _geometryOutputType(GL_TRIANGLE_STRIP),
+    _patchVertices(3)
 {
 }
 
@@ -105,10 +141,28 @@ Program::Program() :
 Program::Program(const Program& rhs, const osg::CopyOp& copyop):
     osg::StateAttribute(rhs, copyop)
 {
-    osg::notify(osg::FATAL) << "how got here?" << std::endl;
+    for( unsigned int shaderIndex=0; shaderIndex < rhs.getNumShaders(); ++shaderIndex )
+    {
+        addShader( new osg::Shader( *rhs.getShader( shaderIndex ), copyop ) );
+    }
+
+    const osg::Program::AttribBindingList &abl = rhs.getAttribBindingList();
+    for( osg::Program::AttribBindingList::const_iterator attribute = abl.begin(); attribute != abl.end(); ++attribute )
+    {
+        addBindAttribLocation( attribute->first, attribute->second );
+    }
+
+    const osg::Program::FragDataBindingList &fdl = rhs.getFragDataBindingList();
+    for( osg::Program::FragDataBindingList::const_iterator fragdata = fdl.begin(); fragdata != fdl.end(); ++fragdata )
+    {
+        addBindFragDataLocation( fragdata->first, fragdata->second );
+    }
+
     _geometryVerticesOut = rhs._geometryVerticesOut;
     _geometryInputType = rhs._geometryInputType;
     _geometryOutputType = rhs._geometryOutputType;
+
+    _patchVertices = rhs._patchVertices;
 }
 
 
@@ -125,7 +179,7 @@ Program::~Program()
 int Program::compare(const osg::StateAttribute& sa) const
 {
     // check the types are equal and then create the rhs variable
-    // used by the COMPARE_StateAttribute_Parameter macro's below.
+    // used by the COMPARE_StateAttribute_Parameter macros below.
     COMPARE_StateAttribute_Types(Program,sa)
     
     if( _shaderList.size() < rhs._shaderList.size() ) return -1;
@@ -143,6 +197,9 @@ int Program::compare(const osg::StateAttribute& sa) const
     if( _geometryOutputType < rhs._geometryOutputType ) return -1;
     if( rhs._geometryOutputType < _geometryOutputType ) return 1;
 
+    if( _patchVertices < rhs._patchVertices ) return -1;
+    if( rhs._patchVertices < _patchVertices ) return 1;
+
     ShaderList::const_iterator litr=_shaderList.begin();
     ShaderList::const_iterator ritr=rhs._shaderList.begin();
     for(;
@@ -153,7 +210,7 @@ int Program::compare(const osg::StateAttribute& sa) const
         if (result!=0) return result;
     }
 
-    return 0; // passed all the above comparison macro's, must be equal.
+    return 0; // passed all the above comparison macros, must be equal.
 }
 
 
@@ -165,10 +222,10 @@ void Program::compileGLObjects( osg::State& state ) const
 
     for( unsigned int i=0; i < _shaderList.size(); ++i )
     {
-        _shaderList[i]->compileShader( contextID );
+        _shaderList[i]->compileShader( state );
     }
 
-    getPCP( contextID )->linkProgram();
+    getPCP( contextID )->linkProgram(state);
 }
 
 void Program::setThreadSafeRefUnref(bool threadSafe)
@@ -216,7 +273,6 @@ void Program::releaseGLObjects(osg::State* state) const
     }   
 }
 
-
 bool Program::addShader( Shader* shader )
 {
     if( !shader ) return false;
@@ -283,14 +339,47 @@ void Program::setParameter( GLenum pname, GLint value )
             break;
         case GL_GEOMETRY_OUTPUT_TYPE_EXT:
             _geometryOutputType = value;
-            dirtyProgram();    // needed?
+            //dirtyProgram();    // needed?
+            break;
+        case GL_PATCH_VERTICES:
+            _patchVertices = value;
+            dirtyProgram();
             break;
         default:
-            osg::notify(osg::WARN) << "setParameter invalid param " << pname << std::endl;
+            OSG_WARN << "setParameter invalid param " << pname << std::endl;
             break;
     }
 }
 
+void Program::setParameterfv( GLenum pname, const GLfloat* value )
+{
+    switch( pname )
+    {
+      // todo tessellation default level
+        case GL_PATCH_DEFAULT_INNER_LEVEL:
+            break;
+        case GL_PATCH_DEFAULT_OUTER_LEVEL:
+            break;
+        default:
+            OSG_WARN << "setParameter invalid param " << pname << std::endl;
+            break;
+    }
+}
+
+const GLfloat* Program::getParameterfv( GLenum pname ) const
+{
+    /*switch( pname )
+    {
+      ;
+      // todo tessellation default level
+      //        case GL_PATCH_DEFAULT_INNER_LEVEL: return _patchDefaultInnerLevel;
+      //        case GL_PATCH_DEFAULT_OUTER_LEVEL: return _patchDefaultOuterLevel;
+
+    }*/
+    OSG_WARN << "getParameter invalid param " << pname << std::endl;
+    return 0;
+}
+
 GLint Program::getParameter( GLenum pname ) const
 {
     switch( pname )
@@ -298,8 +387,9 @@ GLint Program::getParameter( GLenum pname ) const
         case GL_GEOMETRY_VERTICES_OUT_EXT: return _geometryVerticesOut;
         case GL_GEOMETRY_INPUT_TYPE_EXT:   return _geometryInputType;
         case GL_GEOMETRY_OUTPUT_TYPE_EXT:  return _geometryOutputType;
+        case GL_PATCH_VERTICES:            return _patchVertices; 
     }
-    osg::notify(osg::WARN) << "getParameter invalid param " << pname << std::endl;
+    OSG_WARN << "getParameter invalid param " << pname << std::endl;
     return 0;
 }
 
@@ -328,6 +418,21 @@ void Program::removeBindFragDataLocation( const std::string& name )
     dirtyProgram();
 }
 
+void Program::addBindUniformBlock(const std::string& name, GLuint index)
+{
+    _uniformBlockBindingList[name] = index;
+    dirtyProgram(); // XXX
+}
+
+void Program::removeBindUniformBlock(const std::string& name)
+{
+    _uniformBlockBindingList.erase(name);
+    dirtyProgram(); // XXX
+}
+
+
+
+
 void Program::apply( osg::State& state ) const
 {
     const unsigned int contextID = state.getContextID();
@@ -394,7 +499,7 @@ bool Program::getGlProgramInfoLog(unsigned int contextID, std::string& log) cons
     return getPCP( contextID )->getInfoLog( log );
 }
 
-const Program::ActiveVarInfoMap& Program::getActiveUniforms(unsigned int contextID) const
+const Program::ActiveUniformMap& Program::getActiveUniforms(unsigned int contextID) const
 {
     return getPCP( contextID )->getActiveUniforms();
 }
@@ -411,6 +516,7 @@ const Program::ActiveVarInfoMap& Program::getActiveAttribs(unsigned int contextI
 
 Program::PerContextProgram::PerContextProgram(const Program* program, unsigned int contextID ) :
         osg::Referenced(),
+        _loadedBinary(false),
         _contextID( contextID )
 {
     _program = program;
@@ -432,35 +538,58 @@ void Program::PerContextProgram::requestLink()
 }
 
 
-void Program::PerContextProgram::linkProgram()
+void Program::PerContextProgram::linkProgram(osg::State& state)
 {
     if( ! _needsLink ) return;
     _needsLink = false;
 
-    osg::notify(osg::INFO)
-        << "Linking osg::Program \"" << _program->getName() << "\""
-        << " id=" << _glProgramHandle
-        << " contextID=" << _contextID
-        <<  std::endl;
+    OSG_INFO << "Linking osg::Program \"" << _program->getName() << "\""
+             << " id=" << _glProgramHandle
+             << " contextID=" << _contextID
+             <<  std::endl;
 
-    if (_extensions->isGeometryShader4Supported())
+    const ProgramBinary* programBinary = _program->getProgramBinary();
+
+    _loadedBinary = false;
+    if (programBinary && programBinary->getSize())
     {
-        _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_VERTICES_OUT_EXT, _program->_geometryVerticesOut );
-        _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_INPUT_TYPE_EXT, _program->_geometryInputType );
-        _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_OUTPUT_TYPE_EXT, _program->_geometryOutputType );
+        GLint linked = GL_FALSE;
+        _extensions->glProgramBinary( _glProgramHandle, programBinary->getFormat(),
+            reinterpret_cast<const GLvoid*>(programBinary->getData()), programBinary->getSize() );
+        _extensions->glGetProgramiv( _glProgramHandle, GL_LINK_STATUS, &linked );
+        _loadedBinary = _isLinked = (linked == GL_TRUE);
     }
-    
-    // Detach removed shaders
-    for( unsigned int i=0; i < _shadersToDetach.size(); ++i )
+
+    if (!_loadedBinary)
     {
-        _shadersToDetach[i]->detachShader( _contextID, _glProgramHandle );
+        if (_extensions->isGeometryShader4Supported())
+        {
+            _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_VERTICES_OUT_EXT, _program->_geometryVerticesOut );
+            _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_INPUT_TYPE_EXT, _program->_geometryInputType );
+            _extensions->glProgramParameteri( _glProgramHandle, GL_GEOMETRY_OUTPUT_TYPE_EXT, _program->_geometryOutputType );
+        }
+
+        if (_extensions->areTessellationShadersSupported() )
+        {
+            _extensions->glPatchParameteri( GL_PATCH_VERTICES, _program->_patchVertices );
+            // todo: add default tessellation level
+        }
+
+        // Detach removed shaders
+        for( unsigned int i=0; i < _shadersToDetach.size(); ++i )
+        {
+            _shadersToDetach[i]->detachShader( _contextID, _glProgramHandle );
+        }
     }
     _shadersToDetach.clear();
-    
-    // Attach new shaders
-    for( unsigned int i=0; i < _shadersToAttach.size(); ++i )
+
+    if (!_loadedBinary)
     {
-        _shadersToAttach[i]->attachShader( _contextID, _glProgramHandle );
+        // Attach new shaders
+        for( unsigned int i=0; i < _shadersToAttach.size(); ++i )
+        {
+            _shadersToAttach[i]->attachShader( _contextID, _glProgramHandle );
+        }
     }
     _shadersToAttach.clear();
 
@@ -468,35 +597,59 @@ void Program::PerContextProgram::linkProgram()
     _attribInfoMap.clear();
     _lastAppliedUniformList.clear();
 
-    // set any explicit vertex attribute bindings
-    const AttribBindingList& bindlist = _program->getAttribBindingList();
-    for( AttribBindingList::const_iterator itr = bindlist.begin();
-        itr != bindlist.end(); ++itr )
+    if (!_loadedBinary)
     {
-        _extensions->glBindAttribLocation( _glProgramHandle, itr->second, itr->first.c_str() );
+        // set any explicit vertex attribute bindings
+        const AttribBindingList& programBindlist = _program->getAttribBindingList();
+        for( AttribBindingList::const_iterator itr = programBindlist.begin();
+            itr != programBindlist.end(); ++itr )
+        {
+            OSG_INFO<<"Program's vertex attrib binding "<<itr->second<<", "<<itr->first<<std::endl;
+            _extensions->glBindAttribLocation( _glProgramHandle, itr->second, reinterpret_cast<const GLchar*>(itr->first.c_str()) );
+        }
+
+        // set any explicit vertex attribute bindings that are set up via osg::State, such as the vertex arrays
+        //  that have been aliase to vertex attrib arrays
+        if (state.getUseVertexAttributeAliasing())
+        {
+            const AttribBindingList& stateBindlist = state.getAttributeBindingList();
+            for( AttribBindingList::const_iterator itr = stateBindlist.begin();
+                itr != stateBindlist.end(); ++itr )
+            {
+                OSG_INFO<<"State's vertex attrib binding "<<itr->second<<", "<<itr->first<<std::endl;
+                _extensions->glBindAttribLocation( _glProgramHandle, itr->second, reinterpret_cast<const GLchar*>(itr->first.c_str()) );
+            }
+        }
+
+        // set any explicit frag data bindings
+        const FragDataBindingList& fdbindlist = _program->getFragDataBindingList();
+        for( FragDataBindingList::const_iterator itr = fdbindlist.begin();
+            itr != fdbindlist.end(); ++itr )
+        {
+            _extensions->glBindFragDataLocation( _glProgramHandle, itr->second, reinterpret_cast<const GLchar*>(itr->first.c_str()) );
+        }
+
+        // if any program binary has been set then assume we want to retrieve a binary later.
+        if (programBinary)
+        {
+            _extensions->glProgramParameteri( _glProgramHandle, GL_PROGRAM_BINARY_RETRIEVABLE_HINT, GL_TRUE );
+        }
+
+        // link the glProgram
+        GLint linked = GL_FALSE;
+        _extensions->glLinkProgram( _glProgramHandle );
+        _extensions->glGetProgramiv( _glProgramHandle, GL_LINK_STATUS, &linked );
+        _isLinked = (linked == GL_TRUE);
     }
 
-    // set any explicit frag data bindings
-    const FragDataBindingList& fdbindlist = _program->getFragDataBindingList();
-    for( FragDataBindingList::const_iterator itr = fdbindlist.begin();
-        itr != fdbindlist.end(); ++itr )
-    {
-        _extensions->glBindFragDataLocation( _glProgramHandle, itr->second, itr->first.c_str() );
-    }
-
-    // link the glProgram
-    GLint linked = GL_FALSE;
-    _extensions->glLinkProgram( _glProgramHandle );
-    _extensions->glGetProgramiv( _glProgramHandle, GL_LINK_STATUS, &linked );
-    _isLinked = (linked == GL_TRUE);
     if( ! _isLinked )
     {
-        osg::notify(osg::WARN) << "glLinkProgram \""<< _program->getName() << "\" FAILED" << std::endl;
+        OSG_WARN << "glLinkProgram \""<< _program->getName() << "\" FAILED" << std::endl;
 
         std::string infoLog;
         if( getInfoLog(infoLog) )
         {
-            osg::notify(osg::WARN) << "Program \""<< _program->getName() << "\" " 
+            OSG_WARN << "Program \""<< _program->getName() << "\" " 
                                       "infolog:\n" << infoLog << std::endl;
         }
         
@@ -507,11 +660,63 @@ void Program::PerContextProgram::linkProgram()
         std::string infoLog;
         if( getInfoLog(infoLog) )
         {
-            osg::notify(osg::INFO) << "Program \""<< _program->getName() << "\" "<<
-                                      "link succeded, infolog:\n" << infoLog << std::endl;
+            OSG_INFO << "Program \""<< _program->getName() << "\" "<<
+                                      "link succeeded, infolog:\n" << infoLog << std::endl;
         }
     }
 
+    if (_extensions->isUniformBufferObjectSupported())
+    {
+        GLuint activeUniformBlocks = 0;
+        GLsizei maxBlockNameLen = 0;
+        _extensions->glGetProgramiv(_glProgramHandle, GL_ACTIVE_UNIFORM_BLOCKS,
+                                    reinterpret_cast<GLint*>(&activeUniformBlocks));
+        _extensions->glGetProgramiv(_glProgramHandle,
+                                    GL_ACTIVE_UNIFORM_MAX_LENGTH,
+                                    &maxBlockNameLen);
+        if (maxBlockNameLen > 0)
+        {
+            std::vector<GLchar> blockName(maxBlockNameLen);
+            for (GLuint i = 0; i < activeUniformBlocks; ++i)
+            {
+                GLsizei len = 0;
+                GLint blockSize = 0;
+                _extensions->glGetActiveUniformBlockName(_glProgramHandle, i,
+                                                         maxBlockNameLen, &len,
+                                                         &blockName[0]);
+                _extensions->glGetActiveUniformBlockiv(_glProgramHandle, i,
+                                                       GL_UNIFORM_BLOCK_DATA_SIZE,
+                                                       &blockSize);
+                _uniformBlockMap
+                    .insert(UniformBlockMap::value_type(&blockName[0],
+                                                        UniformBlockInfo(i, blockSize)));
+            }
+        }
+        // Bind any uniform blocks
+        const UniformBlockBindingList& bindingList = _program->getUniformBlockBindingList();
+        for (UniformBlockMap::iterator itr = _uniformBlockMap.begin(),
+                 end = _uniformBlockMap.end();
+             itr != end;
+            ++itr)
+        {
+            const std::string& blockName = itr->first;
+            UniformBlockBindingList::const_iterator bitr = bindingList.find(blockName);
+            if (bitr != bindingList.end())
+            {
+                _extensions->glUniformBlockBinding(_glProgramHandle, itr->second._index,
+                                                   bitr->second);
+                OSG_INFO << "uniform block " << blockName << ": " << itr->second._index
+                         << " binding: " << bitr->second << "\n";
+            }
+            else
+            {
+                OSG_WARN << "uniform block " << blockName << " has no binding.\n";
+            }
+
+        }
+
+    }
+
     // build _uniformInfoMap
     GLint numUniforms = 0;
     GLsizei maxLen = 0;
@@ -532,10 +737,9 @@ void Program::PerContextProgram::linkProgram()
             
             if( loc != -1 )
             {
-                _uniformInfoMap[name] = ActiveVarInfo(loc,type,size);
+                _uniformInfoMap[Uniform::getNameID(reinterpret_cast<const char*>(name))] = ActiveVarInfo(loc,type,size);
 
-                osg::notify(osg::INFO)
-                    << "\tUniform \"" << name << "\""
+                OSG_INFO << "\tUniform \"" << name << "\""
                     << " loc="<< loc
                     << " size="<< size
                     << " type=" << Uniform::getTypename((Uniform::Type)type)
@@ -564,18 +768,17 @@ void Program::PerContextProgram::linkProgram()
             
             if( loc != -1 )
             {
-                _attribInfoMap[name] = ActiveVarInfo(loc,type,size);
+                _attribInfoMap[reinterpret_cast<char*>(name)] = ActiveVarInfo(loc,type,size);
 
-                osg::notify(osg::INFO)
-                    << "\tAttrib \"" << name << "\""
-                    << " loc=" << loc
-                    << " size=" << size
-                    << std::endl;
+                OSG_INFO << "\tAttrib \"" << name << "\""
+                         << " loc=" << loc
+                         << " size=" << size
+                         << std::endl;
             }
         }
         delete [] name;
     }
-    osg::notify(osg::INFO) << std::endl;
+    OSG_INFO << std::endl;
 }
 
 bool Program::PerContextProgram::validateProgram()
@@ -586,17 +789,16 @@ bool Program::PerContextProgram::validateProgram()
     if( validated == GL_TRUE)
         return true;
 
-    osg::notify(osg::INFO)
-        << "glValidateProgram FAILED \"" << _program->getName() << "\""
-        << " id=" << _glProgramHandle
-        << " contextID=" << _contextID
-        <<  std::endl;
+    OSG_WARN << "glValidateProgram FAILED \"" << _program->getName() << "\""
+             << " id=" << _glProgramHandle
+             << " contextID=" << _contextID
+             <<  std::endl;
 
     std::string infoLog;
     if( getInfoLog(infoLog) )
-        osg::notify(osg::INFO) << "infolog:\n" << infoLog << std::endl;
+        OSG_WARN << "infolog:\n" << infoLog << std::endl;
 
-    osg::notify(osg::INFO) << std::endl;
+    OSG_WARN << std::endl;
     
     return false;
 }
@@ -606,6 +808,23 @@ bool Program::PerContextProgram::getInfoLog( std::string& infoLog ) const
     return _extensions->getProgramInfoLog( _glProgramHandle, infoLog );
 }
 
+Program::ProgramBinary* Program::PerContextProgram::compileProgramBinary(osg::State& state)
+{
+    linkProgram(state);
+    GLint binaryLength = 0;
+    _extensions->glGetProgramiv( _glProgramHandle, GL_PROGRAM_BINARY_LENGTH, &binaryLength );
+    if (binaryLength)
+    {
+        ProgramBinary* programBinary = new ProgramBinary;
+        programBinary->allocate(binaryLength);
+        GLenum binaryFormat = 0;
+        _extensions->glGetProgramBinary( _glProgramHandle, binaryLength, 0, &binaryFormat, reinterpret_cast<GLvoid*>(programBinary->getData()) );
+        programBinary->setFormat(binaryFormat);
+        return programBinary;
+    }
+    return 0;
+}
+
 void Program::PerContextProgram::useProgram() const
 {
     _extensions->glUseProgram( _glProgramHandle  );
