@@ -12,6 +12,7 @@
 */
 #include <osg/GL>
 #include <osg/ShadeModel>
+#include <osg/Notify>
 
 using namespace osg;
 
@@ -27,5 +28,9 @@ ShadeModel::~ShadeModel()
 
 void ShadeModel::apply(State&) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
     glShadeModel((GLenum)_mode);
+#else
+    OSG_NOTICE<<"Warning: ShadeModel::apply(State&) - not supported."<<std::endl;
+#endif
 }
