@@ -1,7 +1,7 @@
 /* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This application is open source and may be redistributed and/or modified   
- * freely and without restriction, both in commericial and non commericial
+ * freely and without restriction, both in commercial and non commercial
  * applications, as long as this copyright notice is maintained.
  * 
  * This application is distributed in the hope that it will be useful,
@@ -11,27 +11,39 @@
 
 #include <osg/StateAttribute>
 #include <osg/StateSet>
+#include <osg/State>
 #include <osg/Notify>
 
 #include <algorithm>
 
 using namespace osg;
 
+StateAttribute::StateAttribute()
+    :Object(true)
+{
+}
+
+
 void StateAttribute::addParent(osg::StateSet* object)
 {
+    OSG_DEBUG_FP<<"Adding parent"<<getRefMutex()<<std::endl;
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
+    
     _parents.push_back(object);
 }
 
 void StateAttribute::removeParent(osg::StateSet* object)
 {
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
+    
     ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
     if (pitr!=_parents.end()) _parents.erase(pitr);
 }
 
 
-void StateAttribute::setUpdateCallback(Callback* uc)
+void StateAttribute::setUpdateCallback(StateAttributeCallback* uc)
 {
-    osg::notify(osg::INFO)<<"StateAttribute::Setting Update callbacks"<<std::endl;
+    OSG_INFO<<"StateAttribute::Setting Update callbacks"<<std::endl;
 
     if (_updateCallback==uc) return;
     
@@ -43,22 +55,22 @@ void StateAttribute::setUpdateCallback(Callback* uc)
     
     if (delta!=0)
     {
-        osg::notify(osg::INFO)<<"Going to set StateAttribute parents"<<std::endl;
+        OSG_INFO<<"Going to set StateAttribute parents"<<std::endl;
 
         for(ParentList::iterator itr=_parents.begin();
             itr!=_parents.end();
             ++itr)
         {
-            osg::notify(osg::INFO)<<"   Setting StateAttribute parent"<<std::endl;
+            OSG_INFO<<"   Setting StateAttribute parent"<<std::endl;
 
             (*itr)->setNumChildrenRequiringUpdateTraversal((*itr)->getNumChildrenRequiringUpdateTraversal()+delta);
         }
     }
 }
 
-void StateAttribute::setEventCallback(Callback* ec)
+void StateAttribute::setEventCallback(StateAttributeCallback* ec)
 {
-    osg::notify(osg::INFO)<<"StateAttribute::Setting Event callbacks"<<std::endl;
+    OSG_INFO<<"StateAttribute::Setting Event callbacks"<<std::endl;
 
     if (_eventCallback==ec) return;
     
