@@ -45,6 +45,7 @@ TexEnvCombine::~TexEnvCombine()
 
 void TexEnvCombine::apply(State& state) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
     unsigned int contextID = state.getContextID();
 
     static bool s_isTexEnvCombineSupported =
@@ -103,6 +104,9 @@ void TexEnvCombine::apply(State& state) const
         // we will resort the settung the OpenGL default of GL_MODULATE.
         glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     }
+#else
+    OSG_NOTICE<<"Warning: TexEnvCombine::apply(State&) - not supported."<<std::endl;
+#endif
 }
 void TexEnvCombine::setCombine_RGB(GLint cm) { _combine_RGB = cm; }
 void TexEnvCombine::setCombine_Alpha(GLint cm) { _combine_Alpha = cm; }
@@ -123,7 +127,7 @@ static GLint Valid_Operand_Alpha(GLint op, const char* functionName)
 {
     if (op==TexEnvCombine::SRC_ALPHA || op==TexEnvCombine::ONE_MINUS_SRC_ALPHA) return op;
 
-    notify(WARN)<<"Warning:: TexEnvCombine::"<<functionName<<"("<<op<<") invalid parameter value,"<<std::endl<<
+    OSG_WARN<<"Warning:: TexEnvCombine::"<<functionName<<"("<<op<<") invalid parameter value,"<<std::endl<<
                   "          must be SRC_ALPHA or ONE_MINUS_SRC_ALPHA, resetting to SRC_ALPHA."<<std::endl;
     return TexEnvCombine::SRC_ALPHA;
 }
