@@ -21,18 +21,17 @@ TexGenNode::TexGenNode():
     setCullingActive(false);
     
     _textureUnit = 0;
-    _value = StateAttribute::ON;
-    _stateset = new StateSet;
+    setStateSet(new StateSet);
     _texgen = new TexGen;
 }
 
 TexGenNode::TexGenNode(const TexGenNode& cn, const CopyOp& copyop):
     Group(cn,copyop),
     _textureUnit(cn._textureUnit),
-    _value(cn._value),
     _texgen(static_cast<TexGen*>(copyop(cn._texgen.get()))),
     _referenceFrame(cn._referenceFrame)
 {
+    setStateSet(new StateSet);
 }
 
 TexGenNode::TexGenNode(TexGen *texgen):
@@ -42,8 +41,7 @@ TexGenNode::TexGenNode(TexGen *texgen):
     setCullingActive(false);
 
     _textureUnit = 0;
-    _value = StateAttribute::ON;
-    _stateset = new StateSet;
+    setStateSet(new StateSet);
     _texgen = texgen;
 }
 
