@@ -3,7 +3,7 @@
  * Copyright (C) 2008 Zebra Imaging
  *
  * This application is open source and may be redistributed and/or modified   
- * freely and without restriction, both in commericial and non commericial
+ * freely and without restriction, both in commercial and non commercial
  * applications, as long as this copyright notice is maintained.
  * 
  * This application is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 #include <osg/Program>
 #include <osg/StateSet>
 
+#include <limits.h>
 #include <algorithm>
 
 using namespace osg;
@@ -30,13 +31,13 @@ using namespace osg;
 ///////////////////////////////////////////////////////////////////////////
 
 Uniform::Uniform() :
-    _type(UNDEFINED), _numElements(0), _modifiedCount(0)
+    _type(UNDEFINED), _numElements(0), _nameID(UINT_MAX), _modifiedCount(0)
 {
 }
 
 
 Uniform::Uniform( Type type, const std::string& name, int numElements ) :
-    _type(type), _numElements(0), _modifiedCount(0)
+    _type(type), _numElements(0), _nameID(UINT_MAX), _modifiedCount(0)
 {
     setName(name);
     setNumElements(numElements);
@@ -55,7 +56,7 @@ Uniform::~Uniform()
 
 void Uniform::addParent(osg::StateSet* object)
 {
-    osg::notify(osg::INFO)<<"Uniform Adding parent"<<std::endl;
+    OSG_DEBUG_FP<<"Uniform Adding parent"<<std::endl;
 
     _parents.push_back(object);
 }
@@ -72,7 +73,7 @@ bool Uniform::setType( Type t )
 
     if( _type != UNDEFINED )
     {
-        osg::notify(osg::WARN) << "cannot change Uniform type" << std::endl;
+        OSG_WARN << "cannot change Uniform type" << std::endl;
         return false;
     }
     _type = t;
@@ -84,17 +85,18 @@ void Uniform::setName( const std::string& name )
 {
     if( _name != "" )
     {
-        osg::notify(osg::WARN) << "cannot change Uniform name" << std::endl;
+        OSG_WARN << "cannot change Uniform name" << std::endl;
         return;
     }
-    _name = name;
+    Object::setName(name);
+    _nameID = getNameID(_name);
 }
 
 void Uniform::setNumElements( unsigned int numElements )
 {
     if( numElements < 1 )
     {
-        osg::notify(osg::WARN) << "Uniform numElements < 1 is invalid" << std::endl;
+        OSG_WARN << "Uniform numElements < 1 is invalid" << std::endl;
         return;
     }
 
@@ -102,7 +104,7 @@ void Uniform::setNumElements( unsigned int numElements )
 
     if( _numElements>0 )
     {
-        osg::notify(osg::WARN) << "Warning: Uniform::setNumElements() cannot change Uniform numElements, size already fixed." << std::endl;
+        OSG_WARN << "Warning: Uniform::setNumElements() cannot change Uniform numElements, size already fixed." << std::endl;
         return;
     }
 
@@ -113,7 +115,7 @@ void Uniform::setNumElements( unsigned int numElements )
 void Uniform::allocateDataArray()
 {
     // if one array is already allocated, the job is done.
-    if( _floatArray.valid() != _intArray.valid() ) return;
+    if( _floatArray.valid() || _intArray.valid() || _uintArray.valid() ) return;
 
     // array cannot be created until _type and _numElements are specified
     int arrayNumElements = getInternalArrayNumElements();
@@ -124,11 +126,19 @@ void Uniform::allocateDataArray()
             case GL_FLOAT:
                 _floatArray = new FloatArray(arrayNumElements);
                 _intArray = 0;
+                _uintArray = 0;
                 return;
 
             case GL_INT:
                 _intArray = new IntArray(arrayNumElements);
                 _floatArray = 0;
+                _uintArray = 0;
+                return;
+
+            case GL_UNSIGNED_INT:
+                _uintArray = new UIntArray(arrayNumElements);
+                _floatArray = 0;
+                _intArray = 0;
                 return;
 
             default:
@@ -137,6 +147,7 @@ void Uniform::allocateDataArray()
     }
     _floatArray = 0;
     _intArray = 0;
+    _uintArray = 0;
 }
 
 bool Uniform::setArray( FloatArray* array )
@@ -146,12 +157,13 @@ bool Uniform::setArray( FloatArray* array )
     // incoming array must match configuration of the Uniform
     if( getInternalArrayType(getType())!=GL_FLOAT || getInternalArrayNumElements()!=array->getNumElements() )
     {
-        osg::notify(osg::WARN) << "Uniform::setArray : incompatible array" << std::endl;
+        OSG_WARN << "Uniform::setArray : incompatible array" << std::endl;
         return false;
     }
 
     _floatArray = array;
     _intArray = 0;
+    _uintArray = 0;
     dirty();
     return true;
 }
@@ -163,16 +175,34 @@ bool Uniform::setArray( IntArray* array )
     // incoming array must match configuration of the Uniform
     if( getInternalArrayType(getType())!=GL_INT || getInternalArrayNumElements()!=array->getNumElements() )
     {
-        osg::notify(osg::WARN) << "Uniform::setArray : incompatible array" << std::endl;
+        OSG_WARN << "Uniform::setArray : incompatible array" << std::endl;
         return false;
     }
 
     _intArray = array;
     _floatArray = 0;
+    _uintArray = 0;
     dirty();
     return true;
 }
 
+bool Uniform::setArray( UIntArray* array )
+{
+    if( !array ) return false;
+
+    // incoming array must match configuration of the Uniform
+    if( getInternalArrayType(getType())!=GL_UNSIGNED_INT || getInternalArrayNumElements()!=array->getNumElements() )
+    {
+        OSG_WARN << "Uniform::setArray : incompatible array" << std::endl;
+        return false;
+    }
+
+    _uintArray = array;
+    _floatArray = 0;
+    _intArray = 0;
+    dirty();
+    return true;
+}
 
 ///////////////////////////////////////////////////////////////////////////
 
@@ -211,6 +241,15 @@ int Uniform::compareData(const Uniform& rhs) const
         return memcmp( _intArray->getDataPointer(), rhs._intArray->getDataPointer(),
             _intArray->getTotalDataSize() );
     }
+
+    if( _uintArray.valid() )
+    {
+        if( ! rhs._uintArray ) return 1;
+        if( _uintArray == rhs._uintArray ) return 0;
+        return memcmp( _uintArray->getDataPointer(), rhs._uintArray->getDataPointer(),
+            _uintArray->getTotalDataSize() );
+    }
+    
     return -1;  // how got here?
 }
 
@@ -218,8 +257,11 @@ void Uniform::copyData(const Uniform& rhs)
 {
     // caller must ensure that _type==rhs._type
     _numElements = rhs._numElements;
+    _nameID = rhs._nameID;
+    if (rhs._floatArray.valid() || rhs._intArray.valid() || rhs._uintArray.valid()) allocateDataArray();
     if( _floatArray.valid() && rhs._floatArray.valid() ) *_floatArray = *rhs._floatArray;
     if( _intArray.valid() && rhs._intArray.valid() )     *_intArray = *rhs._intArray;
+    if( _uintArray.valid() && rhs._uintArray.valid() )     *_uintArray = *rhs._uintArray;
     dirty();
 }
 
@@ -229,9 +271,8 @@ bool Uniform::isCompatibleType( Type t ) const
     if( t == getType() ) return true;
     if( getGlApiType(t) == getGlApiType(getType()) ) return true;
 
-    osg::notify(osg::WARN)
-        << "Cannot assign between Uniform types " << getTypename(t)
-        << " and " << getTypename(getType()) << std::endl;
+    OSG_WARN << "Cannot assign between Uniform types " << getTypename(t)
+             << " and " << getTypename(getType()) << std::endl;
     return false;
 }
 
@@ -282,6 +323,7 @@ const char* Uniform::getTypename( Type t )
     case FLOAT_MAT4x3:  return "mat4x3";
     case SAMPLER_BUFFER:       return "samplerBuffer";
     case SAMPLER_CUBE_SHADOW:  return "samplerCubeShadow";
+    case UNSIGNED_INT:         return "unsigned int";
     case UNSIGNED_INT_VEC2:    return "uvec2";
     case UNSIGNED_INT_VEC3:    return "uvec3";
     case UNSIGNED_INT_VEC4:    return "uvec4";
@@ -311,6 +353,7 @@ int Uniform::getTypeNumComponents( Type t )
     {
     case FLOAT:
     case INT:
+    case UNSIGNED_INT:
     case BOOL:
     case SAMPLER_1D:
     case SAMPLER_2D:
@@ -419,6 +462,7 @@ Uniform::Type Uniform::getTypeId( const std::string& tname )
     if( tname == "mat4x3" )          return FLOAT_MAT4x3;
     if( tname == "samplerBuffer" )     return SAMPLER_BUFFER;
     if( tname == "samplerCubeShadow" ) return SAMPLER_CUBE_SHADOW;
+    if( tname == "unsigned int" )    return UNSIGNED_INT;
     if( tname == "uvec2" )           return UNSIGNED_INT_VEC2;
     if( tname == "uvec3" )           return UNSIGNED_INT_VEC3;
     if( tname == "uvec4" )           return UNSIGNED_INT_VEC4;
@@ -548,16 +592,36 @@ GLenum Uniform::getInternalArrayType( Type t )
     case UNSIGNED_INT_SAMPLER_BUFFER:
         return GL_INT;
 
-    // TODO integrate new types
+    case UNSIGNED_INT:
     case UNSIGNED_INT_VEC2:
     case UNSIGNED_INT_VEC3:
     case UNSIGNED_INT_VEC4:
+        return GL_UNSIGNED_INT;
+
     default:
         return 0;
     }
 }
 
 
+unsigned int Uniform::getNameID(const std::string& name)
+{
+    typedef std::map<std::string, unsigned int> UniformNameIDMap;
+    static OpenThreads::Mutex s_mutex_uniformNameIDMap;
+    static UniformNameIDMap s_uniformNameIDMap;
+
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_uniformNameIDMap);
+    UniformNameIDMap::iterator it = s_uniformNameIDMap.find(name);
+    if (it != s_uniformNameIDMap.end())
+    {
+        return it->second;
+    }
+    unsigned int id = s_uniformNameIDMap.size();
+    s_uniformNameIDMap.insert(UniformNameIDMap::value_type(name, id));
+    return id;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////
 // value constructors for single-element (ie: non-array) uniforms
 
@@ -657,6 +721,38 @@ Uniform::Uniform( const char* name, int i0, int i1, int i2, int i3 ) :
     set( i0, i1, i2, i3 );
 }
 
+Uniform::Uniform( const char* name, unsigned int i ) :
+    _type(UNSIGNED_INT), _numElements(1), _modifiedCount(0)
+{
+    setName(name);
+    allocateDataArray();
+    set( i );
+}
+
+Uniform::Uniform( const char* name, unsigned int i0, unsigned int i1 ) :
+    _type(UNSIGNED_INT_VEC2), _numElements(1), _modifiedCount(0)
+{
+    setName(name);
+    allocateDataArray();
+    set( i0, i1 );
+}
+
+Uniform::Uniform( const char* name, unsigned int i0, unsigned int i1, unsigned int i2 ) :
+    _type(UNSIGNED_INT_VEC3), _numElements(1), _modifiedCount(0)
+{
+    setName(name);
+    allocateDataArray();
+    set( i0, i1, i2 );
+}
+
+Uniform::Uniform( const char* name, unsigned int i0, unsigned int i1, unsigned int i2, unsigned int i3 ) :
+    _type(UNSIGNED_INT_VEC4), _numElements(1), _modifiedCount(0)
+{
+    setName(name);
+    allocateDataArray();
+    set( i0, i1, i2, i3 );
+}
+
 Uniform::Uniform( const char* name, bool b ) :
     _type(BOOL), _numElements(1), _modifiedCount(0)
 {
@@ -766,6 +862,30 @@ bool Uniform::set( int i0, int i1, int i2, int i3 )
     return isScalar() ? setElement(0,i0,i1,i2,i3) : false;
 }
 
+bool Uniform::set( unsigned int i )
+{
+    if( getNumElements() == 0 ) setNumElements(1);
+    return isScalar() ? setElement(0,i) : false;
+}
+
+bool Uniform::set( unsigned int i0, unsigned int i1 )
+{
+    if( getNumElements() == 0 ) setNumElements(1);
+    return isScalar() ? setElement(0,i0,i1) : false;
+}
+
+bool Uniform::set( unsigned int i0, unsigned int i1, unsigned int i2 )
+{
+    if( getNumElements() == 0 ) setNumElements(1);
+    return isScalar() ? setElement(0,i0,i1,i2) : false;
+}
+
+bool Uniform::set( unsigned int i0, unsigned int i1, unsigned int i2, unsigned int i3 )
+{
+    if( getNumElements() == 0 ) setNumElements(1);
+    return isScalar() ? setElement(0,i0,i1,i2,i3) : false;
+}
+
 bool Uniform::set( bool b )
 {
     if( getNumElements() == 0 ) setNumElements(1);
@@ -853,6 +973,26 @@ bool Uniform::get( int& i0, int& i1, int& i2, int& i3 ) const
     return isScalar() ? getElement(0,i0,i1,i2,i3) : false;
 }
 
+bool Uniform::get( unsigned int& i ) const
+{
+    return isScalar() ? getElement(0,i) : false;
+}
+
+bool Uniform::get( unsigned int& i0, unsigned int& i1 ) const
+{
+    return isScalar() ? getElement(0,i0,i1) : false;
+}
+
+bool Uniform::get( unsigned int& i0, unsigned int& i1, unsigned int& i2 ) const
+{
+    return isScalar() ? getElement(0,i0,i1,i2) : false;
+}
+
+bool Uniform::get( unsigned int& i0, unsigned int& i1, unsigned int& i2, unsigned int& i3 ) const
+{
+    return isScalar() ? getElement(0,i0,i1,i2,i3) : false;
+}
+
 bool Uniform::get( bool& b ) const
 {
     return isScalar() ? getElement(0,b) : false;
@@ -998,6 +1138,48 @@ bool Uniform::setElement( unsigned int index, int i0, int i1, int i2, int i3 )
     return true;
 }
 
+bool Uniform::setElement( unsigned int index, unsigned int i )
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    (*_uintArray)[j] = i;
+    dirty();
+    return true;
+}
+
+bool Uniform::setElement( unsigned int index, unsigned int i0, unsigned int i1 )
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT_VEC2) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    (*_uintArray)[j] = i0;
+    (*_uintArray)[j+1] = i1;
+    dirty();
+    return true;
+}
+
+bool Uniform::setElement( unsigned int index, unsigned int i0, unsigned int i1, unsigned int i2 )
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT_VEC3) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    (*_uintArray)[j] = i0;
+    (*_uintArray)[j+1] = i1;
+    (*_uintArray)[j+2] = i2;
+    dirty();
+    return true;
+}
+
+bool Uniform::setElement( unsigned int index, unsigned int i0, unsigned int i1, unsigned int i2, unsigned int i3 )
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT_VEC4) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    (*_uintArray)[j] = i0;
+    (*_uintArray)[j+1] = i1;
+    (*_uintArray)[j+2] = i2;
+    (*_uintArray)[j+3] = i3;
+    dirty();
+    return true;
+}
+
 bool Uniform::setElement( unsigned int index, bool b )
 {
     if( index>=getNumElements() || !isCompatibleType(BOOL) ) return false;
@@ -1151,6 +1333,44 @@ bool Uniform::getElement( unsigned int index, int& i0, int& i1, int& i2, int& i3
     return true;
 }
 
+bool Uniform::getElement( unsigned int index, unsigned int& i ) const
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    i = (*_uintArray)[j];
+    return true;
+}
+
+bool Uniform::getElement( unsigned int index, unsigned int& i0, unsigned int& i1 ) const
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT_VEC2) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    i0 = (*_uintArray)[j];
+    i1 = (*_uintArray)[j+1];
+    return true;
+}
+
+bool Uniform::getElement( unsigned int index, unsigned int& i0, unsigned int& i1, unsigned int& i2 ) const
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT_VEC3) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    i0 = (*_uintArray)[j];
+    i1 = (*_uintArray)[j+1];
+    i2 = (*_uintArray)[j+2];
+    return true;
+}
+
+bool Uniform::getElement( unsigned int index, unsigned int& i0, unsigned int& i1, unsigned int& i2, unsigned int& i3 ) const
+{
+    if( index>=getNumElements() || !isCompatibleType(UNSIGNED_INT_VEC4) ) return false;
+    unsigned int j = index * getTypeNumComponents(getType());
+    i0 = (*_uintArray)[j];
+    i1 = (*_uintArray)[j+1];
+    i2 = (*_uintArray)[j+2];
+    i3 = (*_uintArray)[j+3];
+    return true;
+}
+
 bool Uniform::getElement( unsigned int index, bool& b ) const
 {
     if( index>=getNumElements() || !isCompatibleType(BOOL) ) return false;
@@ -1189,11 +1409,16 @@ bool Uniform::getElement( unsigned int index, bool& b0, bool& b1, bool& b2, bool
     return true;
 }
 
+unsigned int Uniform::getNameID() const
+{
+    return _nameID;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 void Uniform::apply(const GL2Extensions* ext, GLint location) const
 {
-    // osg::notify(osg::NOTICE) << "uniform at "<<location<<" "<<_name<< std::endl;
+    // OSG_NOTICE << "uniform at "<<location<<" "<<_name<< std::endl;
 
     GLsizei num = getNumElements();
     if( num < 1 ) return;
@@ -1244,15 +1469,31 @@ void Uniform::apply(const GL2Extensions* ext, GLint location) const
         if( _intArray.valid() ) ext->glUniform4iv( location, num, &_intArray->front() );
         break;
 
+    case UNSIGNED_INT:
+        if( _uintArray.valid() ) ext->glUniform1uiv( location, num, &_uintArray->front() );
+        break;
+
+    case UNSIGNED_INT_VEC2:
+        if( _uintArray.valid() ) ext->glUniform2uiv( location, num, &_uintArray->front() );
+        break;
+
+    case UNSIGNED_INT_VEC3:
+        if( _uintArray.valid() ) ext->glUniform3uiv( location, num, &_uintArray->front() );
+        break;
+
+    case UNSIGNED_INT_VEC4:
+        if( _uintArray.valid() ) ext->glUniform4uiv( location, num, &_uintArray->front() );
+        break;
+
     default:
-        osg::notify(osg::FATAL) << "how got here? " __FILE__ ":" << __LINE__ << std::endl;
+        OSG_FATAL << "how got here? " __FILE__ ":" << __LINE__ << std::endl;
         break;
     }
 }
 
 void Uniform::setUpdateCallback(Callback* uc)
 {
-    osg::notify(osg::INFO)<<"Uniform::Setting Update callbacks"<<std::endl;
+    OSG_INFO<<"Uniform::Setting Update callbacks"<<std::endl;
 
     if (_updateCallback==uc) return;
     
@@ -1264,13 +1505,13 @@ void Uniform::setUpdateCallback(Callback* uc)
     
     if (delta!=0)
     {
-        osg::notify(osg::INFO)<<"Going to set Uniform parents"<<std::endl;
+        OSG_INFO<<"Going to set Uniform parents"<<std::endl;
 
         for(ParentList::iterator itr=_parents.begin();
             itr!=_parents.end();
             ++itr)
         {
-            osg::notify(osg::INFO)<<"   setting Uniform parent"<<std::endl;
+            OSG_INFO<<"   setting Uniform parent"<<std::endl;
             (*itr)->setNumChildrenRequiringUpdateTraversal((*itr)->getNumChildrenRequiringUpdateTraversal()+delta);
         }
     }
@@ -1278,7 +1519,7 @@ void Uniform::setUpdateCallback(Callback* uc)
 
 void Uniform::setEventCallback(Callback* ec)
 {
-    osg::notify(osg::INFO)<<"Uniform::Setting Event callbacks"<<std::endl;
+    OSG_INFO<<"Uniform::Setting Event callbacks"<<std::endl;
 
     if (_eventCallback==ec) return;
     
