@@ -125,6 +125,8 @@ void VertexProgram::dirtyVertexProgramObject()
 
 void VertexProgram::apply(State& state) const
 {
+#ifdef OSG_GL_FIXED_FUNCTION_AVAILABLE
+
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);
 
@@ -148,19 +150,19 @@ void VertexProgram::apply(State& state) const
                                      _vertexProgram.length(), _vertexProgram.c_str());
 
         // Check for errors
-        GLint errorposition;
+        GLint errorposition = 0;
         glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorposition);
         if (errorposition != -1)
         {
-            notify(osg::FATAL) << "VertexProgram: " << glGetString(GL_PROGRAM_ERROR_STRING_ARB) << std::endl;
+            OSG_FATAL << "VertexProgram: " << glGetString(GL_PROGRAM_ERROR_STRING_ARB) << std::endl;
 
             std::string::size_type start = _vertexProgram.rfind('\n', errorposition);
             std::string::size_type stop  = _vertexProgram.find('\n', errorposition);
             if (start!=std::string::npos && stop!=std::string::npos)
             {
-                notify(osg::FATAL) << "             : " << _vertexProgram.substr(start+1, stop-start-2) << std::endl;
+                OSG_FATAL << "             : " << _vertexProgram.substr(start+1, stop-start-2) << std::endl;
                 std::string pointAtproblem(errorposition-(start+1), ' ');
-                notify(osg::FATAL) << "             : " << pointAtproblem << '^' << std::endl;
+                OSG_FATAL << "             : " << pointAtproblem << '^' << std::endl;
             }
             return;
         }
@@ -188,6 +190,9 @@ void VertexProgram::apply(State& state) const
         }
         glMatrixMode(GL_MODELVIEW); // restore matrix mode
     }
+#else
+    OSG_NOTICE<<"Warning: VertexProgram::apply(State&) - not supported."<<std::endl;
+#endif
 }
 
 void VertexProgram::resizeGLObjectBuffers(unsigned int maxSize)
@@ -257,23 +262,22 @@ void VertexProgram::Extensions::setupGLExtensions(unsigned int contextID)
 {
     _isVertexProgramSupported = isGLExtensionSupported(contextID,"GL_ARB_vertex_program");
 
-    _glBindProgram = osg::getGLExtensionFuncPtr("glBindProgramARB");
-    _glGenPrograms = osg::getGLExtensionFuncPtr("glGenProgramsARB");
-    _glDeletePrograms = osg::getGLExtensionFuncPtr("glDeleteProgramsARB");
-    _glProgramString = osg::getGLExtensionFuncPtr("glProgramStringARB");
-    _glProgramLocalParameter4fv = osg::getGLExtensionFuncPtr("glProgramLocalParameter4fvARB");
+    setGLExtensionFuncPtr(_glBindProgram,"glBindProgramARB");
+    setGLExtensionFuncPtr(_glGenPrograms, "glGenProgramsARB");
+    setGLExtensionFuncPtr(_glDeletePrograms, "glDeleteProgramsARB");
+    setGLExtensionFuncPtr(_glProgramString, "glProgramStringARB");
+    setGLExtensionFuncPtr(_glProgramLocalParameter4fv, "glProgramLocalParameter4fvARB");
 }
 
 void VertexProgram::Extensions::glBindProgram(GLenum target, GLuint id) const
 {
     if (_glBindProgram)
     {
-        typedef void (APIENTRY * BindProgramProc) (GLenum target, GLuint id);
-        ((BindProgramProc)_glBindProgram)(target,id);
+        _glBindProgram(target,id);
     }
     else
     {
-        notify(WARN)<<"Error: glBindProgram not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glBindProgram not supported by OpenGL driver"<<std::endl;
     }    
 }
 
@@ -281,12 +285,11 @@ void VertexProgram::Extensions::glGenPrograms(GLsizei n, GLuint *programs) const
 {
     if (_glGenPrograms)
     {
-        typedef void (APIENTRY * GenProgramsProc) (GLsizei n, GLuint *programs);
-        ((GenProgramsProc)_glGenPrograms)(n,programs);
+        _glGenPrograms(n,programs);
     }
     else
     {
-        notify(WARN)<<"Error: glGenPrograms not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glGenPrograms not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -294,12 +297,11 @@ void VertexProgram::Extensions::glDeletePrograms(GLsizei n, GLuint *programs) co
 {
     if (_glDeletePrograms)
     {
-        typedef void (APIENTRY * DeleteProgramsProc) (GLsizei n, GLuint *programs);
-        ((DeleteProgramsProc)_glDeletePrograms)(n,programs);
+        _glDeletePrograms(n,programs);
     }
     else
     {
-        notify(WARN)<<"Error: glDeletePrograms not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glDeletePrograms not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -307,12 +309,11 @@ void VertexProgram::Extensions::glProgramString(GLenum target, GLenum format, GL
 {
     if (_glProgramString)
     {
-        typedef void (APIENTRY * ProgramStringProc) (GLenum target, GLenum format, GLsizei len, const void *string); 
-        ((ProgramStringProc)_glProgramString)(target,format, len, string); 
+        _glProgramString(target,format, len, string); 
     }
     else
     {
-        notify(WARN)<<"Error: glProgramString not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glProgramString not supported by OpenGL driver"<<std::endl;
     }
 }
 
@@ -320,11 +321,10 @@ void VertexProgram::Extensions::glProgramLocalParameter4fv(GLenum target, GLuint
 {
     if (_glProgramLocalParameter4fv)
     {
-        typedef void (APIENTRY * ProgramLocalParameter4fvProc) (GLenum target, GLuint index, const GLfloat *params);
-        ((ProgramLocalParameter4fvProc)_glProgramLocalParameter4fv)(target, index, params);
+        _glProgramLocalParameter4fv(target, index, params);
     }
     else
     {
-        notify(WARN)<<"Error: glProgramLocalParameter4fv not supported by OpenGL driver"<<std::endl;
+        OSG_WARN<<"Error: glProgramLocalParameter4fv not supported by OpenGL driver"<<std::endl;
     }
 }
