@@ -20,7 +20,7 @@ using namespace osg;
 View::View():
     Object(true)
 {
-    // osg::notify(osg::NOTICE)<<"Constructing osg::View"<<std::endl;
+    // OSG_NOTICE<<"Constructing osg::View"<<std::endl;
 
     setLightingMode(HEADLIGHT);
 
@@ -52,7 +52,7 @@ View::View(const osg::View& view, const osg::CopyOp& copyop):
 
 View::~View()
 {
-    osg::notify(osg::INFO)<<"Destructing osg::View"<<std::endl;
+    OSG_INFO<<"Destructing osg::View"<<std::endl;
 
     if (_camera.valid())
     {
@@ -82,7 +82,7 @@ View::~View()
     }
 #endif
     
-    osg::notify(osg::INFO)<<"Done destructing osg::View"<<std::endl;
+    OSG_INFO<<"Done destructing osg::View"<<std::endl;
 }
 
 void View::take(osg::View& rhs)
@@ -132,7 +132,6 @@ void View::setCamera(osg::Camera* camera)
     if (_camera.valid())
     {
         _camera->setView(this);
-        
         _camera->setRenderer(createRenderer(camera));
     }
 }
@@ -141,24 +140,22 @@ void View::updateSlaves()
 {
     for(unsigned int i=0; i<_slaves.size(); ++i)
     {
-        updateSlave(i);
+        Slave& slave = _slaves[i];
+        slave.updateSlave(*this);
     }
 }
 
-void View::updateSlave(unsigned int i)
+void View::Slave::updateSlaveImplementation(View& view)
 {
-    if (i >= _slaves.size() || !_camera) return;
+    if (!view.getCamera()) return;
 
-    Slave& slave = _slaves[i];
-
-    if (slave._camera->getReferenceFrame()==osg::Transform::RELATIVE_RF)
+    if (_camera->getReferenceFrame()==osg::Transform::RELATIVE_RF)
     {
-        slave._camera->setProjectionMatrix(_camera->getProjectionMatrix() * slave._projectionOffset);
-        slave._camera->setViewMatrix(_camera->getViewMatrix() * slave._viewOffset);
+        _camera->setProjectionMatrix(view.getCamera()->getProjectionMatrix() * _projectionOffset);
+        _camera->setViewMatrix(view.getCamera()->getViewMatrix() * _viewOffset);
     }
-    
-    slave._camera->inheritCullSettings(*_camera);
-    if (slave._camera->getInheritanceMask() & osg::CullSettings::CLEAR_COLOR) slave._camera->setClearColor(_camera->getClearColor());
+
+    _camera->inheritCullSettings(*(view.getCamera()), _camera->getInheritanceMask());
 }
 
 bool View::addSlave(osg::Camera* camera, const osg::Matrix& projectionOffset, const osg::Matrix& viewOffset, bool useMastersSceneData)
@@ -184,7 +181,7 @@ bool View::addSlave(osg::Camera* camera, const osg::Matrix& projectionOffset, co
 
     _slaves.push_back(Slave(camera, projectionOffset, viewOffset, useMastersSceneData));
 
-    updateSlave(i);
+    _slaves[i].updateSlave(*this);
     
     camera->setRenderer(createRenderer(camera));    
 
@@ -213,7 +210,7 @@ View::Slave * View::findSlaveForCamera(osg::Camera* camera)
     return &(_slaves[i]);
 }
 
-unsigned int View::findSlaveIndexForCamera(osg::Camera* camera)
+unsigned int View::findSlaveIndexForCamera(osg::Camera* camera) const
 {
     if (_camera == camera) return _slaves.size();
 
