@@ -34,7 +34,7 @@ Animation::Animation(const osgAnimation::Animation& anim, const osg::CopyOp& cop
 void Animation::addChannel(Channel* pChannel)
 {
     _channels.push_back(pChannel);
-    if (!_duration)
+    if (_duration == _originalDuration)
         computeDuration();
     else
         _originalDuration = computeDurationFromChannels();
@@ -80,7 +80,7 @@ void Animation::setDuration(double duration)
     _duration = duration;
 }
 
-float Animation::getDuration() const
+double Animation::getDuration() const
 {
     return _duration;
 }
@@ -95,40 +95,40 @@ void Animation::setWeight (float weight)
     _weight = weight;
 }
 
-bool Animation::update (float time, int priority)
+bool Animation::update (double time, int priority)
 {
     if (!_duration) // if not initialized then do it
         computeDuration();
 
     double ratio = _originalDuration / _duration;
 
-    float t = (time - _startTime) * ratio;
+    double t = (time - _startTime) * ratio;
     switch (_playmode) 
     {
     case ONCE:
-        if (t > _duration)
+        if (t > _originalDuration)
             return false;
         break;
     case STAY:
-        if (t > _duration)
-            t = _duration;
+        if (t > _originalDuration)
+            t = _originalDuration;
         break;
     case LOOP:
-        if (!_duration)
+        if (!_originalDuration)
             t = _startTime;
-        else if (t > _duration)
-            t = fmod(t, (float)_duration);
+        else if (t > _originalDuration)
+            t = fmod(t, _originalDuration);
         //      std::cout << "t " << t << " duration " << _duration << std::endl;
         break;
     case PPONG: 
-        if (!_duration)
+        if (!_originalDuration)
             t = _startTime;
         else 
         {
-            int tt = (int) (t / _duration);
-            t = fmod(t, (float)_duration);
+            int tt = (int) (t / _originalDuration);
+            t = fmod(t, _originalDuration);
             if (tt%2)
-                t = _duration - t;
+                t = _originalDuration - t;
         }
         break;
     }
