@@ -19,13 +19,18 @@
 using namespace osgAnimation;
 
 Skeleton::Skeleton() {}
+
 Skeleton::Skeleton(const Skeleton& b, const osg::CopyOp& copyop) : osg::MatrixTransform(b,copyop) {}
 
 Skeleton::UpdateSkeleton::UpdateSkeleton() : _needValidate(true) {}
-Skeleton::UpdateSkeleton::UpdateSkeleton(const UpdateSkeleton& us, const osg::CopyOp& copyop= osg::CopyOp::SHALLOW_COPY) : osg::Object(us, copyop), osg::NodeCallback(us, copyop)
+
+Skeleton::UpdateSkeleton::UpdateSkeleton(const UpdateSkeleton& us, const osg::CopyOp& copyop) :
+    osg::Object(us, copyop),
+    osg::NodeCallback(us, copyop)
 {
     _needValidate = true;
 }
+
 bool Skeleton::UpdateSkeleton::needToValidate() const
 {
     return _needValidate;
@@ -52,7 +57,7 @@ public:
             {
                 if (foundNonBone)
                 {
-                    osg::notify(osg::WARN) <<
+                    OSG_WARN <<
                         "Warning: a Bone was found after a non-Bone child "
                         "within a Skeleton. Children of a Bone must be ordered "
                         "with all child Bones first for correct update order." << std::endl;
