@@ -18,6 +18,7 @@
 
 #include <sstream>
 #include <stdio.h>
+#include <string.h>
 
 using namespace std;
 using namespace osgDB;
@@ -29,7 +30,7 @@ Output::Output()
     init();
 }
 
-Output::Output(const char* name) : ofstream(name)
+Output::Output(const char* name) : osgDB::ofstream(name)
 {
     init();
     _filename = name;
@@ -62,7 +63,7 @@ void Output::init()
     }
 }
 
-void Output::setOptions(const ReaderWriter::Options* options)
+void Output::setOptions(const Options* options)
 {
     _options = options;
 }
@@ -70,7 +71,7 @@ void Output::setOptions(const ReaderWriter::Options* options)
 void Output::open(const char *name)
 {
     init();
-    ofstream::open(name);
+    osgDB::ofstream::open(name);
     _filename = name;
 }
 
@@ -105,13 +106,24 @@ void Output::moveOut()
     if (_indent<0) _indent=0;
 }
 
+std::string Output::wrapString(const char* str)
+{
+    if (!str) return std::string("\"\"");
+    return wrapString(std::string(str));
+}
+
 std::string Output::wrapString(const std::string& str)
 {
     std::string newstring;
     newstring += '"';
     for(unsigned int i=0;i<str.size();++i)
     {
-        if (str[i]=='"')
+        if (str[i]=='\\')
+        {
+            newstring += '\\';
+            newstring += '\\';
+        }
+        else if (str[i]=='"')
         {
             newstring += '\\';
             newstring += '"';
@@ -124,7 +136,7 @@ std::string Output::wrapString(const std::string& str)
 
 bool Output::writeObject(const osg::Object& obj)
 {
-    return Registry::instance()->writeObject(obj,*this);
+    return Registry::instance()->getDeprecatedDotOsgObjectWrapperManager()->writeObject(obj,*this);
 }
 
 
@@ -182,13 +194,13 @@ std::string Output::getFileNameForOutput(const std::string& filename) const
     case(FULL_PATH):
         {
             // need to think about how best to implement this first...
-            osg::notify(osg::WARN)<<"Warning: Output::getFileNameForOutput() does not support FULL_PATH yet."<< std::endl;        
+            OSG_WARN<<"Warning: Output::getFileNameForOutput() does not support FULL_PATH yet."<< std::endl;        
             return filename;
         }
     case(RELATIVE_PATH):
         {
             // need to think about how best to implement this as well...
-            osg::notify(osg::WARN)<<"Warning: Output::getFileNameForOutput() does not support RELATIVE_PATH yet."<< std::endl;        
+            OSG_WARN<<"Warning: Output::getFileNameForOutput() does not support RELATIVE_PATH yet."<< std::endl;        
             return filename;
         }
     case(FILENAME_ONLY):
@@ -233,3 +245,14 @@ std::string Output::getShaderFileNameForOutput()
     return fileName;
 }
 
+void Output::setExternalFileWritten(const std::string& filename, bool hasBeenWritten)
+{
+    _externalFileWritten[filename] = hasBeenWritten;
+}
+
+bool Output::getExternalFileWritten(const std::string& filename) const
+{
+    ExternalFileWrittenMap::const_iterator itr = _externalFileWritten.find(filename);
+    if (itr != _externalFileWritten.end()) return itr->second;
+    return false;
+}
