@@ -100,7 +100,7 @@ class ReaderWriter3DC : public osgDB::ReaderWriter
             std::string fileName = osgDB::findDataFile( file, options );
             if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
 
-            osg::notify(osg::INFO) << "Reading file "<<fileName<<std::endl;
+            OSG_INFO << "Reading file "<<fileName<<std::endl;
 
             const int LINE_SIZE = 1024;
             char line[LINE_SIZE];
@@ -127,7 +127,7 @@ class ReaderWriter3DC : public osgDB::ReaderWriter
                 if (line[0]=='#')
                 {
                     // comment line
-                    osg::notify(osg::INFO) <<"Comment: "<<line<<std::endl;
+                    OSG_INFO <<"Comment: "<<line<<std::endl;
                 }
                 else if (strlen(line)>0)
                 {
@@ -142,8 +142,8 @@ class ReaderWriter3DC : public osgDB::ReaderWriter
                         if (vertices->size()>=targetNumVertices)
                         {
                             // finishing setting up the current geometry and add it to the geode.
-                            geometry->setUseDisplayList(true);    
-                            geometry->setUseVertexBufferObjects(true);    
+                            geometry->setUseDisplayList(true);
+                            geometry->setUseVertexBufferObjects(true);
                             geometry->setVertexArray(vertices);
                             geometry->setNormalArray(normals);
                             geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
@@ -175,7 +175,7 @@ class ReaderWriter3DC : public osgDB::ReaderWriter
 
 
             geometry->setUseDisplayList(true);
-            geometry->setUseVertexBufferObjects(true);    
+            geometry->setUseVertexBufferObjects(true);
             geometry->setVertexArray(vertices);
             geometry->setNormalArray(normals);
             geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
@@ -189,19 +189,19 @@ class ReaderWriter3DC : public osgDB::ReaderWriter
 
         }
 
-        virtual WriteResult writeNode(const osg::Node& node,const std::string& fileName,const Options* options =NULL) const 
-        { 
-            if (!acceptsExtension(osgDB::getFileExtension(fileName)))
-                return WriteResult(WriteResult::FILE_NOT_HANDLED);
+        virtual WriteResult writeNode(const osg::Node& node,const std::string& fileName,const Options* options =NULL) const
+        {
+            std::string ext = osgDB::getLowerCaseFileExtension(fileName);
+            if( !acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
             osgDB::ofstream f(fileName.c_str());
 
-            Writer3DCNodeVisitor nv(f); 
+            Writer3DCNodeVisitor nv(f);
 
             // we must cast away constness
             (const_cast<osg::Node*>(&node))->accept(nv);
 
-            return WriteResult(WriteResult::FILE_SAVED); 
+            return WriteResult::FILE_SAVED;
         }
 };
 
