@@ -1,7 +1,20 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 //
 // OpenFlight loader for OpenSceneGraph
 //
-//  Copyright (C) 2005-2006  Brede Johansen
+//  Copyright (C) 2005-2007  Brede Johansen
 //
 
 #include <assert.h>
@@ -25,7 +38,7 @@ class PushLevel : public Record
 
         META_Record(PushLevel)
 
-        virtual void read(RecordInputStream& /*in*/, Document& document)
+        virtual void readRecord(RecordInputStream& /*in*/, Document& document)
         {
             document.pushLevel();
         }
@@ -35,7 +48,8 @@ class PushLevel : public Record
         virtual ~PushLevel() {}
 };
 
-RegisterRecordProxy<PushLevel> g_PushLevel(PUSH_LEVEL_OP);
+REGISTER_FLTRECORD(PushLevel, PUSH_LEVEL_OP)
+
 
 
 /** PophLevel
@@ -50,6 +64,21 @@ class PopLevel : public Record
 
         virtual void read(RecordInputStream& /*in*/, Document& document)
         {
+            PrimaryRecord* parentPrimary = document.getTopOfLevelStack();
+            PrimaryRecord* currentPrimary = document.getCurrentPrimaryRecord();
+
+            // Call dispose() for primary without push, pop level pair. 
+            if (currentPrimary && currentPrimary!=parentPrimary)
+            {
+                currentPrimary->dispose(document);
+            }
+
+            // Call dispose() for primary with push, pop level pair. 
+            if (parentPrimary)
+            {
+                parentPrimary->dispose(document);
+            }
+
             document.popLevel();
         }
 
@@ -58,7 +87,8 @@ class PopLevel : public Record
         virtual ~PopLevel() {}
 };
 
-RegisterRecordProxy<PopLevel> g_PopLevel(POP_LEVEL_OP);
+REGISTER_FLTRECORD(PopLevel, POP_LEVEL_OP)
+
 
 
 /** PushSubface
@@ -81,7 +111,8 @@ class PushSubface : public Record
         virtual ~PushSubface() {}
 };
 
-RegisterRecordProxy<PushSubface> g_PushSubface(PUSH_SUBFACE_OP);
+REGISTER_FLTRECORD(PushSubface, PUSH_SUBFACE_OP)
+
 
 
 /** PopSubface
@@ -104,7 +135,8 @@ class PopSubface : public Record
         virtual ~PopSubface() {}
 };
 
-RegisterRecordProxy<PopSubface> g_PopSubface(POP_SUBFACE_OP);
+REGISTER_FLTRECORD(PopSubface, POP_SUBFACE_OP)
+
 
 
 /** PushExtension
@@ -128,7 +160,8 @@ class PushExtension : public Record
         virtual ~PushExtension() {}
 };
 
-RegisterRecordProxy<PushExtension> g_PushExtension(PUSH_EXTENSION_OP);
+REGISTER_FLTRECORD(PushExtension, PUSH_EXTENSION_OP)
+
 
 
 /** PopExtension
@@ -152,7 +185,8 @@ class PopExtension : public Record
         virtual ~PopExtension() {}
 };
 
-RegisterRecordProxy<PopExtension> g_PopExtension(POP_EXTENSION_OP);
+REGISTER_FLTRECORD(PopExtension, POP_EXTENSION_OP)
+
 
 
 /** PushAttribute - Reserved subtree
@@ -168,9 +202,6 @@ class PushAttribute : public Record
         virtual void read(RecordInputStream& in, Document& document)
         {
             readRecord(in,document);
-    //      in().seekg(in.getEndOfRecord(), std::ios_base::beg);
-            // loop until PopAttribute
-
         }
 
     protected:
@@ -178,7 +209,8 @@ class PushAttribute : public Record
         virtual ~PushAttribute() {}
 };
 
-RegisterRecordProxy<PushAttribute> g_PushAttribute(PUSH_ATTRIBUTE_OP);
+REGISTER_FLTRECORD(PushAttribute, PUSH_ATTRIBUTE_OP)
+
 
 
 /** PopAttribute
@@ -201,7 +233,8 @@ class PopAttribute : public Record
         virtual ~PopAttribute() {}
 };
 
-RegisterRecordProxy<PopAttribute> g_PopAttribute(POP_ATTRIBUTE_OP);
+REGISTER_FLTRECORD(PopAttribute, POP_ATTRIBUTE_OP)
+
 
 
 } // end namespace
