@@ -28,7 +28,6 @@
 
 #include <osgDB/FileUtils>
 #include <osgDB/WriteFile>
-#include <osg/Billboard>
 #include <osg/Geode>
 #include <osg/Geometry>
 #include <osg/LightSource>
@@ -51,7 +50,7 @@
 #include <osgSim/LightPointNode>
 #include <osgSim/ObjectRecordData>
 
-#ifdef _WIN32
+#ifdef _MSC_VER
 // Disable this warning. It's OK for us to use 'this' in initializer list,
 // as the texturePaletteManager merely stores a ref to it.
 #pragma warning( disable : 4355 )
@@ -65,8 +64,8 @@ FltExportVisitor::FltExportVisitor( DataOutputStream* dos,
                                 ExportOptions* fltOpt )
 
   : osg::NodeVisitor( osg::NodeVisitor::TRAVERSE_ALL_CHILDREN ),
-    _dos( *dos ),
     _fltOpt( fltOpt ),
+    _dos( *dos ),
     _materialPalette( new MaterialPaletteManager( *fltOpt ) ),
     _texturePalette( new TexturePaletteManager( *this, *fltOpt ) ),
     _lightSourcePalette( new LightSourcePaletteManager( *fltOpt ) ),
@@ -103,7 +102,7 @@ FltExportVisitor::FltExportVisitor( DataOutputStream* dos,
     _stateSetStack.push_back( ss );
 
 
-    // Temp file for storing records. Need a temp file because we candon't
+    // Temp file for storing records. Need a temp file because we don't
     // write header and palette until FltExportVisitor completes traversal.
     _recordsTempName = fltOpt->getTempDir() + "/ofw_temp_records";
     _recordsStr.open( _recordsTempName.c_str(), std::ios::out | std::ios::binary );
@@ -118,12 +117,12 @@ FltExportVisitor::~FltExportVisitor()
     // Delete our temp file.
     if (_recordsStr.is_open())
     {
-        osg::notify( osg::WARN ) << "fltexp: FltExportVisitor destructor has an open temp file." << std::endl;
+        OSG_WARN << "fltexp: FltExportVisitor destructor has an open temp file." << std::endl;
         // This should not happen. FltExportVisitor::complete should close
         // this file before we get to this destructor.
         return;
     }
-    osg::notify( osg::INFO ) << "fltexp: Deleting temp file " << _recordsTempName << std::endl;
+    OSG_INFO << "fltexp: Deleting temp file " << _recordsTempName << std::endl;
     FLTEXP_DELETEFILE( _recordsTempName.c_str() );
 }
 
@@ -148,7 +147,7 @@ FltExportVisitor::apply( osg::Group& node )
     // A Group node could indicate one of many possible records.
     //   Header record -- Don't need to support this here. We always output a header.
     //   Group record -- HIGH
-    //   Child of an LOD node -- HIGH Curently write out a Group record regardless.
+    //   Child of an LOD node -- HIGH Currently write out a Group record regardless.
     //   InstanceDefinition/InstanceReference -- MED --  multiparented Group is an instance
     //   Extension record -- MED
     //   Object record -- MED
@@ -213,7 +212,7 @@ FltExportVisitor::apply( osg::LOD& lodNode )
     _firstNode = false;
     ScopedStatePushPop guard( this, lodNode.getStateSet() );
 
-    // LOD center - same for all chilrden
+    // LOD center - same for all children
     osg::Vec3d center = lodNode.getCenter();
 
     // Iterate children of the LOD and write a separate LOD record for each,
@@ -231,7 +230,7 @@ FltExportVisitor::apply( osg::LOD& lodNode )
         writeComment( lodNode );
 
         // Traverse each child of the LOD
-        writePushTraverseWritePop( *lodChild );
+        writePushTraverseChildWritePop( *lodChild );
     }
 
 }
@@ -240,7 +239,7 @@ void
 FltExportVisitor::apply( osg::MatrixTransform& node )
 {
     // Importer reads a Matrix record and inserts a MatrixTransform above
-    //   the corrent node. We need to do the opposite: Write a Matrix record
+    //   the current node. We need to do the opposite: Write a Matrix record
     //   as an ancillary record for each child. We do that by storing the
     //   MatrixTransform in each child's UserData. Each child then checks
     //   UserData and writes a Matrix record if UserData is a MatrixTransform.
@@ -257,22 +256,24 @@ FltExportVisitor::apply( osg::MatrixTransform& node )
             (*m) *= *rm;
     }
 
-    std::vector< osg::Referenced* > saveUserDataList;
+    typedef std::vector< osg::ref_ptr< osg::Referenced > > UserDataList;
+    
+    UserDataList saveUserDataList( node.getNumChildren() );
+
     unsigned int idx;
-    for( idx=0; idx<node.getNumChildren(); idx++ )
+    for( idx=0; idx<node.getNumChildren(); ++idx )
     {
-        saveUserDataList.push_back( node.getChild( idx )->getUserData() );
+        saveUserDataList[ idx ] = node.getChild( idx )->getUserData();
         node.getChild( idx )->setUserData( m.get() );
     }
 
     traverse( (osg::Node&)node );
 
     // Restore saved UserData.
-    unsigned int nd = node.getNumChildren();
-    while (nd--)
-        node.getChild( nd )->setUserData(
-            saveUserDataList[ nd ] );
-
+    for( idx=0; idx< node.getNumChildren(); ++idx )
+    {
+      node.getChild( idx )->setUserData( saveUserDataList[ idx ].get() );
+    }
 }
 
 void
@@ -281,27 +282,29 @@ FltExportVisitor::apply( osg::PositionAttitudeTransform& node )
     _firstNode = false;
     ScopedStatePushPop guard( this, node.getStateSet() );
 
-    osg::ref_ptr<osg::RefMatrix> m = new osg::RefMatrix;
+    osg::ref_ptr<osg::RefMatrix> m = new osg::RefMatrix(
+        osg::Matrix::translate( -node.getPivotPoint() ) *
+        osg::Matrix::scale( node.getScale() ) *
+        osg::Matrix::rotate( node.getAttitude() ) *
+        osg::Matrix::translate( node.getPosition() ) );
 
-    const osg::Vec3d& trans = node.getPosition();
-    const osg::Quat& rot = node.getAttitude();
-    m->set(osg::Matrix::translate(trans) * osg::Matrix::rotate(rot) );
+    typedef std::vector< osg::ref_ptr< osg::Referenced > > UserDataList;
+    UserDataList saveUserDataList( node.getNumChildren() );
 
-    std::vector< osg::Referenced* > saveUserDataList;
     unsigned int idx;
-    for( idx=0; idx<node.getNumChildren(); idx++ )
+    for( idx=0; idx<node.getNumChildren(); ++idx )
     {
-        saveUserDataList.push_back( node.getChild( idx )->getUserData() );
+        saveUserDataList[ idx ] = node.getChild( idx )->getUserData();
         node.getChild( idx )->setUserData( m.get() );
     }
 
     traverse( (osg::Node&)node );
 
     // Restore saved UserData.
-    unsigned int nd = node.getNumChildren();
-    while (nd--)
-        node.getChild( nd )->setUserData(
-            saveUserDataList[ nd ] );
+    for( idx=0; idx<node.getNumChildren(); ++idx )
+    {
+        node.getChild( idx )->setUserData( saveUserDataList[ idx ].get() );
+    }
 
 }
 
@@ -336,6 +339,10 @@ FltExportVisitor::apply( osg::LightSource& node )
     writePushTraverseWritePop( node );
 }
 
+// Billboards also go through this code. The Geode is passed
+// to writeFace and writeMesh. If those methods successfully cast
+// the Geode to a Billboard, then they set the template mode
+// bit accordingly.
 void
 FltExportVisitor::apply( osg::Geode& node )
 {
@@ -349,60 +356,93 @@ FltExportVisitor::apply( osg::Geode& node )
         if (!geom)
         {
             std::string warning( "fltexp: Non-Geometry Drawable encountered. Ignoring." );
-            osg::notify( osg::WARN ) << warning << std::endl;
+            OSG_WARN << warning << std::endl;
             _fltOpt->getWriteResult().warn( warning );
             continue;
         }
 
         ScopedStatePushPop drawableGuard( this, geom->getStateSet() );
 
-        // Write vertex array data out to vertex palette manager
-        if (!isAllMesh( *geom ))
+        // Push and pop subfaces if polygon offset is on.
+        SubfaceHelper subface( *this, getCurrentStateSet() );
+
+        if (atLeastOneFace( *geom ))
+        {
             // If at least one record will be a Face record, then we
-            //   need to write to the vertex pool.
+            //   need to write to the vertex palette.
             _vertexPalette->add( *geom );
 
-        unsigned int jdx;
-        for (jdx=0; jdx < geom->getNumPrimitiveSets(); jdx++)
-        {
-            osg::PrimitiveSet* prim = geom->getPrimitiveSet( jdx );
-            if (prim->getType() == osg::PrimitiveSet::DrawArraysPrimitiveType)
-                handleDrawArrays( dynamic_cast<osg::DrawArrays*>( prim ), *geom, node );
-            else if (prim->getType() == osg::PrimitiveSet::DrawArrayLengthsPrimitiveType)
-                handleDrawArrayLengths( dynamic_cast<osg::DrawArrayLengths*>( prim ), *geom, node );
-            else if ( (prim->getType() == osg::PrimitiveSet::DrawElementsUBytePrimitiveType) ||
-                    (prim->getType() == osg::PrimitiveSet::DrawElementsUShortPrimitiveType) ||
-                    (prim->getType() == osg::PrimitiveSet::DrawElementsUIntPrimitiveType) )
-                handleDrawElements( dynamic_cast<osg::DrawElements*>( prim ), *geom, node );
-            else
+            // Iterate over all PrimitiveSets and output Face records.
+            unsigned int jdx;
+            for (jdx=0; jdx < geom->getNumPrimitiveSets(); jdx++)
             {
-                std::string warning( "fltexp: Unknown PrimitiveSet type." );
-                osg::notify( osg::WARN ) << warning << std::endl;
-                _fltOpt->getWriteResult().warn( warning );
-                return;
+                osg::PrimitiveSet* prim = geom->getPrimitiveSet( jdx );
+                if ( isMesh( prim->getMode() ) )
+                    continue;
+
+                if (prim->getType() == osg::PrimitiveSet::DrawArraysPrimitiveType)
+                    handleDrawArrays( dynamic_cast<osg::DrawArrays*>( prim ), *geom, node );
+                else if (prim->getType() == osg::PrimitiveSet::DrawArrayLengthsPrimitiveType)
+                    handleDrawArrayLengths( dynamic_cast<osg::DrawArrayLengths*>( prim ), *geom, node );
+                else if ( (prim->getType() == osg::PrimitiveSet::DrawElementsUBytePrimitiveType) ||
+                        (prim->getType() == osg::PrimitiveSet::DrawElementsUShortPrimitiveType) ||
+                        (prim->getType() == osg::PrimitiveSet::DrawElementsUIntPrimitiveType) )
+                    handleDrawElements( dynamic_cast<osg::DrawElements*>( prim ), *geom, node );
+                else
+                {
+                    std::string warning( "fltexp: Unknown PrimitiveSet type." );
+                    OSG_WARN << warning << std::endl;
+                    _fltOpt->getWriteResult().warn( warning );
+                    return;
+                }
             }
         }
+
+        if (atLeastOneMesh( *geom ))
+        {
+            // If at least one Mesh record, write out preamble mesh records
+            //   followed by a Mesh Primitive record per PrimitiveSet.
+            writeMesh( node, *geom );
+
+            writeMatrix( node.getUserData() );
+            writeComment( node );
+            writeMultitexture( *geom );
+            writeLocalVertexPool( *geom );
+
+            writePush();
+
+            unsigned int jdx;
+            for (jdx=0; jdx < geom->getNumPrimitiveSets(); jdx++)
+            {
+                osg::PrimitiveSet* prim = geom->getPrimitiveSet( jdx );
+                if ( !isMesh( prim->getMode() ) )
+                    continue;
+
+                if (prim->getType() == osg::PrimitiveSet::DrawArraysPrimitiveType)
+                    handleDrawArrays( dynamic_cast<osg::DrawArrays*>( prim ), *geom, node );
+                else if (prim->getType() == osg::PrimitiveSet::DrawArrayLengthsPrimitiveType)
+                    handleDrawArrayLengths( dynamic_cast<osg::DrawArrayLengths*>( prim ), *geom, node );
+                else if ( (prim->getType() == osg::PrimitiveSet::DrawElementsUBytePrimitiveType) ||
+                        (prim->getType() == osg::PrimitiveSet::DrawElementsUShortPrimitiveType) ||
+                        (prim->getType() == osg::PrimitiveSet::DrawElementsUIntPrimitiveType) )
+                    handleDrawElements( dynamic_cast<osg::DrawElements*>( prim ), *geom, node );
+                else
+                {
+                    std::string warning( "fltexp: Unknown PrimitiveSet type." );
+                    OSG_WARN << warning << std::endl;
+                    _fltOpt->getWriteResult().warn( warning );
+                    return;
+                }
+            }
+
+            writePop();
+        }
     }
 
     // Would traverse here if this node could have children.
     //   traverse( (osg::Node&)node );
 }
 
-void
-FltExportVisitor::apply( osg::Billboard& node )
-{
-    _firstNode = false;
-    ScopedStatePushPop guard( this, node.getStateSet() );
-
-    // TBD -- Not yet implemented, but HIGH priority.
-    //   Face record -- HIGH
-    //   Mesh record -- HIGH
-
-    writeMatrix( node.getUserData() );
-    writeComment( node );
-    writePushTraverseWritePop( node );
-}
-
 void
 FltExportVisitor::apply( osg::Node& node )
 {
@@ -417,11 +457,11 @@ FltExportVisitor::apply( osg::Node& node )
     else
     {
         // Unknown Node. Warn and return.
-        // (Note, if the base class of this Node was a Grouo, then apply(Group&)
+        // (Note, if the base class of this Node was a Group, then apply(Group&)
         //   would export a Group record then continue traversal. Because we are
         //   a Node, there's no way to continue traversal, so just return.)
         std::string warning( "fltexp: Unknown Node in OpenFlight export." );
-        osg::notify( osg::WARN ) << warning << std::endl;
+        OSG_WARN << warning << std::endl;
         _fltOpt->getWriteResult().warn( warning );
         return;
     }
@@ -450,7 +490,7 @@ FltExportVisitor::complete( const osg::Node& node )
     // Done writing records, close the record data temp file.
     _recordsStr.close();
 
-    // Write OpenFlight file front matter: heqader, vertex palette, etc.
+    // Write OpenFlight file front matter: header, vertex palette, etc.
     writeHeader( node.getName() );
 
     writeColorPalette();
@@ -465,7 +505,7 @@ FltExportVisitor::complete( const osg::Node& node )
     // Copy record data temp file into final OpenFlight file.
     // Yee-uck. TBD need better stream copy routine.
     char buf;
-    std::ifstream recIn;
+    osgDB::ifstream recIn;
     recIn.open( _recordsTempName.c_str(), std::ios::in | std::ios::binary );
     while (!recIn.eof() )
     {
@@ -607,24 +647,27 @@ FltExportVisitor::writeATTRFile( int unit, const osg::Texture2D* texture ) const
         const osg::StateSet* ss = getCurrentStateSet();
         const osg::TexEnv* texenv = dynamic_cast<const osg::TexEnv*>(
             ss->getTextureAttribute( unit, osg::StateAttribute::TEXENV ) );
-        switch( texenv->getMode())
+        if (texenv)
         {
-        case osg::TexEnv::DECAL:
-            ad.texEnvMode = AttrData::TEXENV_DECAL;
-            break;
-        case osg::TexEnv::MODULATE:
-        default:
-            ad.texEnvMode = AttrData::TEXENV_MODULATE;
-            break;
-        case osg::TexEnv::BLEND:
-            ad.texEnvMode = AttrData::TEXENV_BLEND;
-            break;
-        case osg::TexEnv::REPLACE:
-            ad.texEnvMode = AttrData::TEXENV_COLOR;
-            break;
-        case osg::TexEnv::ADD:
-            ad.texEnvMode = AttrData::TEXENV_ADD;
-            break;
+            switch( texenv->getMode())
+            {
+            case osg::TexEnv::DECAL:
+                ad.texEnvMode = AttrData::TEXENV_DECAL;
+                break;
+            case osg::TexEnv::MODULATE:
+            default:
+                ad.texEnvMode = AttrData::TEXENV_MODULATE;
+                break;
+            case osg::TexEnv::BLEND:
+                ad.texEnvMode = AttrData::TEXENV_BLEND;
+                break;
+            case osg::TexEnv::REPLACE:
+                ad.texEnvMode = AttrData::TEXENV_COLOR;
+                break;
+            case osg::TexEnv::ADD:
+                ad.texEnvMode = AttrData::TEXENV_ADD;
+                break;
+            }
         }
 
         osgDB::writeObjectFile( ad, name, _fltOpt.get() );
