@@ -24,6 +24,7 @@
 #include <osg/BlendFunc>
 #include <osg/Geometry>
 #include <osg/Geode>
+#include <osg/Billboard>
 #include <osg/io_utils>
 #include <osg/Material>
 #include <osg/Texture2D>
@@ -37,13 +38,15 @@ namespace flt
 
 // Bit flags for multitexturing
 static unsigned int LAYER_1( 0x80000000 >> 0 );
+#if 0
+// unused so if'deffing out
 static unsigned int LAYER_2( 0x80000000 >> 1 );
 static unsigned int LAYER_3( 0x80000000 >> 2 );
 static unsigned int LAYER_4( 0x80000000 >> 3 );
 static unsigned int LAYER_5( 0x80000000 >> 4 );
 static unsigned int LAYER_6( 0x80000000 >> 5 );
 static unsigned int LAYER_7( 0x80000000 >> 6 );
-
+#endif
 
 
 bool
@@ -67,19 +70,40 @@ FltExportVisitor::isTextured( int unit, const osg::Geometry& geom ) const
 }
 
 bool
-FltExportVisitor::isAllMesh( const osg::Geometry& geom ) const
+FltExportVisitor::isMesh( const GLenum mode ) const
 {
-    // Return true if all primitive types will use Mesh records for output.
+    return( (mode == GL_TRIANGLE_STRIP) ||
+        (mode == GL_TRIANGLE_FAN) ||
+        (mode == GL_QUAD_STRIP) );
+}
+
+bool
+FltExportVisitor::atLeastOneFace( const osg::Geometry& geom ) const
+{
+    // Return true if at least one PrimitiveSet mode will use a Face record.
     unsigned int jdx;
     for (jdx=0; jdx < geom.getNumPrimitiveSets(); jdx++)
     {
         const osg::PrimitiveSet* prim = geom.getPrimitiveSet( jdx );
-        if( (prim->getMode() != GL_TRIANGLE_STRIP) &&
-            (prim->getMode() != GL_TRIANGLE_FAN) &&
-            (prim->getMode() != GL_QUAD_STRIP) )
-            return false;
+        if( !isMesh( prim->getMode() ) )
+            return true;
     }
-    return true;
+    // All PrimitiveSet modes will use Mesh records.
+    return false;
+}
+bool
+FltExportVisitor::atLeastOneMesh( const osg::Geometry& geom ) const
+{
+    // Return true if at least one PrimitiveSet mode will use a Mesh record.
+    unsigned int jdx;
+    for (jdx=0; jdx < geom.getNumPrimitiveSets(); jdx++)
+    {
+        const osg::PrimitiveSet* prim = geom.getPrimitiveSet( jdx );
+        if( isMesh( prim->getMode() ) )
+            return true;
+    }
+    // All PrimitiveSet modes will use Face records.
+    return false;
 }
 
 void
@@ -104,13 +128,14 @@ FltExportVisitor::writeFace( const osg::Geode& geode, const osg::Geometry& geom,
         POINT_ROTATE_WITH_ALPHA_BLENDING = 4
     };
 
-    const unsigned int TERRAIN_BIT      = 0x80000000u >> 0;
-    const unsigned int NO_COLOR_BIT     = 0x80000000u >> 1;
-    const unsigned int NO_ALT_COLOR_BIT = 0x80000000u >> 2;
+    // const unsigned int TERRAIN_BIT      = 0x80000000u >> 0;
+    // const unsigned int NO_COLOR_BIT     = 0x80000000u >> 1;
+    // const unsigned int NO_ALT_COLOR_BIT = 0x80000000u >> 2;
     const unsigned int PACKED_COLOR_BIT = 0x80000000u >> 3;
-    const unsigned int FOOTPRINT_BIT    = 0x80000000u >> 4;    // Terrain culture cutout
+    // const unsigned int FOOTPRINT_BIT    = 0x80000000u >> 4;    // Terrain culture cutout
     const unsigned int HIDDEN_BIT       = 0x80000000u >> 5;
-    const unsigned int ROOFLINE_BIT     = 0x80000000u >> 6;
+    // const unsigned int ROOFLINE_BIT     = 0x80000000u >> 6;
+
     uint32 flags( PACKED_COLOR_BIT );
     if (geode.getNodeMask() == 0)
         flags |= HIDDEN_BIT;
@@ -139,7 +164,7 @@ FltExportVisitor::writeFace( const osg::Geode& geode, const osg::Geometry& geom,
         if (c && (c->size() > 0))
         {
             packedColorRaw = (*c)[0];
-            transparency = flt::uint16((1. - packedColorRaw[3]) * (double)0xffff);
+            transparency = uint16((1. - packedColorRaw[3]) * (double)0xffff);
         }
 
         if ( isLit( geom ) )
@@ -153,14 +178,14 @@ FltExportVisitor::writeFace( const osg::Geode& geode, const osg::Geometry& geom,
         (int)(packedColorRaw[0]*255);
 
 
-    int8 drawType;
+    int8 drawType = SOLID_NO_BACKFACE;
 
     switch( mode )
     {
     case GL_POINTS:
     {
         std::string warning( "fltexp: GL_POINTS not supported in FLT export." );
-        osg::notify( osg::WARN ) << warning << std::endl;
+        OSG_WARN << warning << std::endl;
         _fltOpt->getWriteResult().warn( warning );
         return;
         break;
@@ -170,7 +195,7 @@ FltExportVisitor::writeFace( const osg::Geode& geode, const osg::Geometry& geom,
     case GL_QUAD_STRIP:
     {
         std::string warning( "fltexp: Wrong mode in Face record." );
-        osg::notify( osg::WARN ) << warning << std::endl;
+        OSG_WARN << warning << std::endl;
         _fltOpt->getWriteResult().warn( warning );
         return;
         break;
@@ -223,15 +248,23 @@ FltExportVisitor::writeFace( const osg::Geode& geode, const osg::Geometry& geom,
         else
         {
             std::string warning( "fltexp: Face is textured, but Texture2D StateAttribute is NULL." );
-            osg::notify( osg::WARN ) << warning << std::endl;
+            OSG_WARN << warning << std::endl;
             _fltOpt->getWriteResult().warn( warning );
         }
     }
 
-    // Check for blending. We're not a Billboard (TBD?)
-    //   so use either FIXED_NO_ALPHA_BLENDING or FIXED_ALPHA_BLENDING.
+    // Set the appropriate template mode based
+    // on blending or Billboarding.
     TemplateMode templateMode( FIXED_NO_ALPHA_BLENDING );
-    if ( ss->getMode( GL_BLEND ) & osg::StateAttribute::ON )
+    const osg::Billboard* bb = dynamic_cast< const osg::Billboard* >( &geode );
+    if (bb != NULL)
+    {
+        if( bb->getMode() == osg::Billboard::AXIAL_ROT )
+            templateMode = AXIAL_ROTATE_WITH_ALPHA_BLENDING;
+        else
+            templateMode = POINT_ROTATE_WITH_ALPHA_BLENDING;
+    }
+    else if ( ss->getMode( GL_BLEND ) & osg::StateAttribute::ON )
     {
         const osg::BlendFunc* bf = static_cast<const osg::BlendFunc*>(
             ss->getAttribute(osg::StateAttribute::BLENDFUNC) );
@@ -275,14 +308,14 @@ FltExportVisitor::writeFace( const osg::Geode& geode, const osg::Geometry& geom,
     _records->writeInt32( -1 ); // Alternate color index
     // Next four bytes:
     //   15.8: two 2-byte "reserved" fields
-    //   15.9: one 4-byte "reserved" field
+    //   15.9: one 4-byte "reserved" field;
     _records->writeInt16( 0 ); // Reserved
     _records->writeInt16( -1 ); // Shader index
 }
 
 
 void
-FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom, GLenum mode )
+FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom )
 {
     enum DrawMode
     {
@@ -303,13 +336,13 @@ FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom,
         POINT_ROTATE_WITH_ALPHA_BLENDING = 4
     };
 
-    const unsigned int TERRAIN_BIT      = 0x80000000u >> 0;
-    const unsigned int NO_COLOR_BIT     = 0x80000000u >> 1;
-    const unsigned int NO_ALT_COLOR_BIT = 0x80000000u >> 2;
+    // const unsigned int TERRAIN_BIT      = 0x80000000u >> 0;
+    //const unsigned int NO_COLOR_BIT     = 0x80000000u >> 1;
+    //const unsigned int NO_ALT_COLOR_BIT = 0x80000000u >> 2;
     const unsigned int PACKED_COLOR_BIT = 0x80000000u >> 3;
-    const unsigned int FOOTPRINT_BIT    = 0x80000000u >> 4;    // Terrain culture cutout
+    //const unsigned int FOOTPRINT_BIT    = 0x80000000u >> 4;    // Terrain culture cutout
     const unsigned int HIDDEN_BIT       = 0x80000000u >> 5;
-    const unsigned int ROOFLINE_BIT     = 0x80000000u >> 6;
+    //const unsigned int ROOFLINE_BIT     = 0x80000000u >> 6;
     uint32 flags( PACKED_COLOR_BIT );
     if (geode.getNodeMask() == 0)
         flags |= HIDDEN_BIT;
@@ -337,7 +370,7 @@ FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom,
         if (c && (c->size() > 0))
         {
             packedColorRaw = (*c)[0];
-            transparency = flt::uint16((1. - packedColorRaw[3]) * (double)0xffff);
+            transparency = uint16((1. - packedColorRaw[3]) * (double)0xffff);
         }
 
         if (isLit( geom ))
@@ -354,32 +387,6 @@ FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom,
     int8 drawType;
     osg::StateSet const* ss = getCurrentStateSet();
 
-    switch( mode )
-    {
-    case GL_POINTS:
-    {
-        std::string warning( "fltexp: GL_POINTS not supported in FLT export." );
-        osg::notify( osg::WARN ) << warning << std::endl;
-        _fltOpt->getWriteResult().warn( warning );
-        return;
-        break;
-    }
-    case GL_LINES:
-    case GL_LINE_STRIP:
-    case GL_LINE_LOOP:
-    case GL_TRIANGLES:
-    case GL_QUADS:
-    {
-        std::string warning( "fltexp: Wrong mode in Mesh record." );
-        osg::notify( osg::WARN ) << warning << std::endl;
-        _fltOpt->getWriteResult().warn( warning );
-        return;
-        break;
-    }
-    case GL_TRIANGLE_STRIP:
-    case GL_TRIANGLE_FAN:
-    case GL_QUAD_STRIP:
-    case GL_POLYGON:
     {
         // Default to no facet culling
         drawType = SOLID_NO_BACKFACE;
@@ -394,8 +401,6 @@ FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom,
 
             // Note: OpenFlt can't handle FRONT or FRONT_AND_BACK settings, so ignore these(??)
         }
-        break;
-    }
     }
 
     // Determine the material properties for the face
@@ -418,15 +423,23 @@ FltExportVisitor::writeMesh( const osg::Geode& geode, const osg::Geometry& geom,
         else
         {
             std::string warning( "fltexp: Mesh is textured, but Texture2D StateAttribute is NULL." );
-            osg::notify( osg::WARN ) << warning << std::endl;
+            OSG_WARN << warning << std::endl;
             _fltOpt->getWriteResult().warn( warning );
         }
     }
 
-    // Check for blending. We're not a Billboard (TBD?)
-    //   so use either FIXED_NO_ALPHA_BLENDING or FIXED_ALPHA_BLENDING.
+    // Set the appropriate template mode based
+    // on blending or Billboarding.
     TemplateMode templateMode( FIXED_NO_ALPHA_BLENDING );
-    if ( ss->getMode( GL_BLEND ) & osg::StateAttribute::ON )
+    const osg::Billboard* bb = dynamic_cast< const osg::Billboard* >( &geode );
+    if (bb != NULL)
+    {
+        if( bb->getMode() == osg::Billboard::AXIAL_ROT )
+            templateMode = AXIAL_ROTATE_WITH_ALPHA_BLENDING;
+        else
+            templateMode = POINT_ROTATE_WITH_ALPHA_BLENDING;
+    }
+    else if ( ss->getMode( GL_BLEND ) & osg::StateAttribute::ON )
     {
         const osg::BlendFunc* bf = static_cast<const osg::BlendFunc*>(
             ss->getAttribute(osg::StateAttribute::BLENDFUNC) );
@@ -484,7 +497,7 @@ FltExportVisitor::writeVertexList( int first, unsigned int count )
 
     unsigned int idx;
     for( idx=0; idx<count; idx++)
-        // I'm imaginining that 'first' will be a 0-based index into the 
+        // I'm imagining that 'first' will be a 0-based index into the 
         //   'current' set of vertices held by the vertex palette manager.
         _records->writeInt32( _vertexPalette->byteOffset( first+idx ) );
 
@@ -499,7 +512,7 @@ FltExportVisitor::writeVertexList( const std::vector<unsigned int>& indices, uns
 
     unsigned int idx;
     for( idx=0; idx<count; idx++)
-        // I'm imaginining that 'first' will be a 0-based index into the 
+        // I'm imagining that 'first' will be a 0-based index into the 
         //   'current' set of vertices held by the vertex palette manager.
         _records->writeInt32( _vertexPalette->byteOffset( indices[ idx ] ) );
 
@@ -544,11 +557,11 @@ FltExportVisitor::writeMeshPrimitive( const std::vector<unsigned int>& indices,
 }
 
 void
-FltExportVisitor::writeLocalVertexPool( const osg::Geometry& geom, GLenum mode )
+FltExportVisitor::writeLocalVertexPool( const osg::Geometry& geom )
 {
     // Attribute Mask
     static const unsigned int HAS_POSITION      = 0x80000000u >> 0;
-    static const unsigned int HAS_COLOR_INDEX   = 0x80000000u >> 1;
+    // static const unsigned int HAS_COLOR_INDEX   = 0x80000000u >> 1;
     static const unsigned int HAS_RGBA_COLOR    = 0x80000000u >> 2;
     static const unsigned int HAS_NORMAL        = 0x80000000u >> 3;
     static const unsigned int HAS_BASE_UV       = 0x80000000u >> 4;
@@ -566,7 +579,7 @@ FltExportVisitor::writeLocalVertexPool( const osg::Geometry& geom, GLenum mode )
     if (!v3)
     {
         std::string warning( "fltexp: writeLocalVertexPool: VertexArray is not Vec3Array." );
-        osg::notify( osg::WARN ) << warning << std::endl;
+        OSG_WARN << warning << std::endl;
         _fltOpt->getWriteResult().warn( warning );
         return;
     }
@@ -742,7 +755,7 @@ FltExportVisitor::writeMultitexture( const osg::Geometry& geom )
             {
                 std::ostringstream warning;
                 warning << "fltexp: No Texture2D for unit " << idx;
-                osg::notify( osg::WARN ) << warning.str() << std::endl;
+                OSG_WARN << warning.str() << std::endl;
                 _fltOpt->getWriteResult().warn( warning.str() );
             }
 
@@ -757,7 +770,7 @@ FltExportVisitor::writeMultitexture( const osg::Geometry& geom )
 }
 
 void
-FltExportVisitor::writeUVList( int numVerts, const osg::Geometry& geom )
+FltExportVisitor::writeUVList( int numVerts, const osg::Geometry& geom, const std::vector<unsigned int>& indices )
 {
     unsigned int numLayers( 0 );
     uint32 flags( 0 );
@@ -780,33 +793,87 @@ FltExportVisitor::writeUVList( int numVerts, const osg::Geometry& geom )
     _records->writeInt32( flags );
 
     osg::Vec2 defaultCoord( 0., 0. );
-    const osg::StateSet* ss = getCurrentStateSet();
+    // const osg::StateSet* ss = getCurrentStateSet();
+    for( int vertexIdx=0; vertexIdx<numVerts; vertexIdx++)
+    {
+        for( idx=1; idx<8; idx++)
+        {
+            if( isTextured( idx, geom ) )
+            {
+                osg::Array* t = const_cast<osg::Array*>( geom.getTexCoordArray( idx ) );
+                osg::ref_ptr<osg::Vec2Array> t2 = dynamic_cast<osg::Vec2Array*>( t );
+                if (!t2.valid())
+                {
+                    std::ostringstream warning;
+                    warning << "fltexp: No Texture2D for unit " << idx;
+                    OSG_WARN << warning.str() << std::endl;
+                    _fltOpt->getWriteResult().warn( warning.str() );
+                    t2 = new osg::Vec2Array;
+                }
+
+                const int size = t2->getNumElements();
+                int vIdx = indices[ vertexIdx ];
+                osg::Vec2& tc( defaultCoord );
+                if (vIdx < size)
+                    tc = ( *t2 )[ vIdx ];
+                _records->writeFloat32( tc[ 0 ] );
+                _records->writeFloat32( tc[ 1 ] );
+            }
+        }
+    }
+}
+
+
+void
+FltExportVisitor::writeUVList( int numVerts, const osg::Geometry& geom, unsigned int first )
+{
+    unsigned int numLayers( 0 );
+    uint32 flags( 0 );
+    unsigned int idx;
     for( idx=1; idx<8; idx++)
     {
         if( isTextured( idx, geom ) )
         {
-            osg::Array* t = const_cast<osg::Array*>( geom.getTexCoordArray( idx ) );
-            osg::ref_ptr<osg::Vec2Array> t2 = dynamic_cast<osg::Vec2Array*>( t );
-            if (!t2.valid())
-            {
-                std::ostringstream warning;
-                warning << "fltexp: No Texture2D for unit " << idx;
-                osg::notify( osg::WARN ) << warning.str() << std::endl;
-                _fltOpt->getWriteResult().warn( warning.str() );
-                t2 = new osg::Vec2Array;
-            }
-            else if (t2->getNumElements() != numVerts)
-            {
-                std::ostringstream warning;
-                warning << "fltexp: Invalid number of texture coordinates for unit " << idx;
-                osg::notify( osg::WARN ) << warning.str() << std::endl;
-                _fltOpt->getWriteResult().warn( warning.str() );
-            }
+            flags |= LAYER_1 >> (idx-1);
+            numLayers++;
+        }
+    }
+    if( numLayers == 0 )
+        return;
 
-            const int size = t2->getNumElements();
-            int vIdx;
-            for( vIdx=0; vIdx<numVerts; vIdx++)
+    uint16 length( 8 + (8*numLayers*numVerts) );
+
+    _records->writeInt16( (int16) UV_LIST_OP );
+    _records->writeUInt16( length );
+    _records->writeInt32( flags );
+
+    osg::Vec2 defaultCoord( 0., 0. );
+    for( int vertexIdx=0; vertexIdx<numVerts; vertexIdx++)
+    {
+        for( idx=1; idx<8; idx++)
+        {
+            if( isTextured( idx, geom ) )
             {
+                osg::Array* t = const_cast<osg::Array*>( geom.getTexCoordArray( idx ) );
+                osg::ref_ptr<osg::Vec2Array> t2 = dynamic_cast<osg::Vec2Array*>( t );
+                if (!t2.valid())
+                {
+                    std::ostringstream warning;
+                    warning << "fltexp: No Texture2D for unit " << idx;
+                    osg::notify( osg::WARN ) << warning.str() << std::endl;
+                    _fltOpt->getWriteResult().warn( warning.str() );
+                    t2 = new osg::Vec2Array;
+                }
+                else if (t2->getNumElements() < first + numVerts)
+                {
+                    std::ostringstream warning;
+                    warning << "fltexp: Invalid number of texture coordinates for unit " << idx;
+                    OSG_WARN << warning.str() << std::endl;
+                    _fltOpt->getWriteResult().warn( warning.str() );
+                }
+
+                const int size = t2->getNumElements();
+                int vIdx = first + vertexIdx;
                 osg::Vec2& tc( defaultCoord );
                 if (vIdx < size)
                     tc = (*t2)[ vIdx ];
@@ -836,11 +903,7 @@ FltExportVisitor::handleDrawArrays( const osg::DrawArrays* da, const osg::Geomet
         useMesh = true;
         break;
     case GL_POINTS:
-    case GL_LINE_STRIP:
-    case GL_LINE_LOOP:
-    case GL_POLYGON:
-    default:
-        n = count;
+        n = 1;
         break;
     case GL_LINES:
         n = 2;
@@ -851,40 +914,27 @@ FltExportVisitor::handleDrawArrays( const osg::DrawArrays* da, const osg::Geomet
     case GL_QUADS:
         n = 4;
         break;
+    case GL_LINE_STRIP:
+    case GL_LINE_LOOP:
+    case GL_POLYGON:
+    default:
+        n = count;
+        break;
     }
 
-    // Push and pop subfaces if polygon offset is on.
-    SubfaceHelper subface( *this, getCurrentStateSet() );
-
     if (useMesh)
     {
-        writeMesh( geode, geom, mode );
-
-        writeMatrix( geode.getUserData() );
-        writeComment( geode );
-        writeMultitexture( geom );
-        writeLocalVertexPool( geom, mode );
-
-        writePush();
-
         std::vector< unsigned int > indices;
         int jdx;
         for (jdx=0; jdx<count; jdx++)
             indices.push_back( first+jdx );
         writeMeshPrimitive( indices, mode );
-
-        writePop();
     }
     else
     {
         const unsigned int max( first+count );
         while ((unsigned int)( first+n ) <= max)
         {
-            // Need:
-            // * Geode for record name (but also need to handle
-            //   multi Geometry objects and multi PrimitiveSet objects;
-            //   all Face records can't have the same name).
-            // * Mode
             writeFace( geode, geom, mode );
 
             writeMatrix( geode.getUserData() );
@@ -919,10 +969,7 @@ FltExportVisitor::handleDrawArrayLengths( const osg::DrawArrayLengths* dal, cons
         useMesh = true;
         break;
     case GL_POINTS:
-    case GL_LINE_STRIP:
-    case GL_LINE_LOOP:
-    case GL_POLYGON:
-    default:
+        n = 1;
         break;
     case GL_LINES:
         n = 2;
@@ -933,6 +980,11 @@ FltExportVisitor::handleDrawArrayLengths( const osg::DrawArrayLengths* dal, cons
     case GL_QUADS:
         n = 4;
         break;
+    case GL_LINE_STRIP:
+    case GL_LINE_LOOP:
+    case GL_POLYGON:
+    default:
+        break;
     }
 
     // Push and pop subfaces if polygon offset is on.
@@ -940,15 +992,6 @@ FltExportVisitor::handleDrawArrayLengths( const osg::DrawArrayLengths* dal, cons
 
     if (useMesh)
     {
-        writeMesh( geode, geom, mode );
-
-        writeMatrix( geode.getUserData() );
-        writeComment( geode );
-        writeMultitexture( geom );
-        writeLocalVertexPool( geom, mode );
-
-        writePush();
-
         int idx( 0 );
         for( osg::DrawArrayLengths::const_iterator itr=dal->begin();
              itr!=dal->end(); itr++ )
@@ -959,8 +1002,6 @@ FltExportVisitor::handleDrawArrayLengths( const osg::DrawArrayLengths* dal, cons
                 indices.push_back( idx );
             writeMeshPrimitive( indices, mode );
         }
-
-        writePop();
     }
     else
     {
@@ -1016,11 +1057,7 @@ FltExportVisitor::handleDrawElements( const osg::DrawElements* de, const osg::Ge
         useMesh = true;
         break;
     case GL_POINTS:
-    case GL_LINE_STRIP:
-    case GL_LINE_LOOP:
-    case GL_POLYGON:
-    default:
-        n = de->getNumIndices();
+        n = 1;
         break;
     case GL_LINES:
         n = 2;
@@ -1031,6 +1068,12 @@ FltExportVisitor::handleDrawElements( const osg::DrawElements* de, const osg::Ge
     case GL_QUADS:
         n = 4;
         break;
+    case GL_LINE_STRIP:
+    case GL_LINE_LOOP:
+    case GL_POLYGON:
+    default:
+        n = de->getNumIndices();
+        break;
     }
 
     // Push and pop subfaces if polygon offset is on.
@@ -1038,22 +1081,11 @@ FltExportVisitor::handleDrawElements( const osg::DrawElements* de, const osg::Ge
 
     if (useMesh)
     {
-        writeMesh( geode, geom, mode );
-
-        writeMatrix( geode.getUserData() );
-        writeComment( geode );
-        writeMultitexture( geom );
-        writeLocalVertexPool( geom, mode );
-
-        writePush();
-
         std::vector< unsigned int > indices;
         int idx;
         for (idx=0; idx<n; idx++)
             indices.push_back( de->index( idx ) );
         writeMeshPrimitive( indices, mode );
-
-        writePop();
     }
     else
     {
@@ -1080,7 +1112,7 @@ FltExportVisitor::handleDrawElements( const osg::DrawElements* de, const osg::Ge
             int numVerts = writeVertexList( indices, n );
             first += n;
 
-            writeUVList( numVerts, geom );
+            writeUVList( numVerts, geom, indices );
 
             writePop();
         }
