@@ -28,14 +28,14 @@
 #include <osg/Billboard>
 #include <osg/CameraView>
 
-using namespace osgdae;
+using namespace osgDAE;
 
 // Write non-standard node data as extra of type "Node" with "OpenSceneGraph" technique
 void daeWriter::writeNodeExtra(osg::Node &node)
 {
     unsigned int numDesc = node.getDescriptions().size();
     // Only create extra if descriptions are filled in
-    if (writeExtras && (numDesc > 0))
+    if (_pluginOptions.writeExtras && (numDesc > 0))
     {
         // Adds the following to a node
 
@@ -68,18 +68,12 @@ void daeWriter::writeNodeExtra(osg::Node &node)
 void daeWriter::apply( osg::Group &node )
 {
     debugPrint( node );
-
-    while ( lastDepth >= _nodePath.size() )
-    {
-        //We are not a child of previous node
-        currentNode = daeSafeCast< domNode >( currentNode->getParentElement() );
-        lastDepth--;
-    }
+    updateCurrentDaeNode();
     currentNode = daeSafeCast< domNode >(currentNode->add( COLLADA_ELEMENT_NODE ) );
     
     // If a multiswitch node, store it's data as extra "MultiSwitch" data in the "OpenSceneGraph" technique
     osgSim::MultiSwitch* multiswitch = dynamic_cast<osgSim::MultiSwitch*>(&node);
-    if (writeExtras && multiswitch)
+    if (_pluginOptions.writeExtras && multiswitch)
     {
         // Adds the following to a node
 
@@ -128,6 +122,8 @@ void daeWriter::apply( osg::Group &node )
     }
     else
     {
+        writeAnimations(node);
+
         currentNode->setId(getNodeName(node,"group").c_str());
     }
 
@@ -142,17 +138,11 @@ void daeWriter::apply( osg::Group &node )
 void daeWriter::apply( osg::Switch &node )
 {
     debugPrint( node );
-
-    while ( lastDepth >= _nodePath.size() )
-    {
-        //We are not a child of previous node
-        currentNode = daeSafeCast< domNode >( currentNode->getParentElement() );
-        lastDepth--;
-    }
+    updateCurrentDaeNode();
     currentNode = daeSafeCast< domNode >(currentNode->add( COLLADA_ELEMENT_NODE ) );
     currentNode->setId(getNodeName(node,"switch").c_str());
 
-    if (writeExtras)
+    if (_pluginOptions.writeExtras)
     {
         // Adds the following to a node
 
@@ -195,18 +185,12 @@ void daeWriter::apply( osg::Switch &node )
 void daeWriter::apply( osg::Sequence &node )
 {
     debugPrint( node );
-
-    while ( lastDepth >= _nodePath.size() )
-    {
-        //We are not a child of previous node
-        currentNode = daeSafeCast< domNode >( currentNode->getParentElement() );
-        lastDepth--;
-    }
+    updateCurrentDaeNode();
     currentNode = daeSafeCast< domNode >(currentNode->add( COLLADA_ELEMENT_NODE ) );
     currentNode->setId(getNodeName(node,"sequence").c_str());
 
     // If a sequence node, store it's data as extra "Sequence" data in the "OpenSceneGraph" technique
-    if (writeExtras)
+    if (_pluginOptions.writeExtras)
     {
         // Adds the following to a node
 
@@ -278,18 +262,12 @@ void daeWriter::apply( osg::Sequence &node )
 void daeWriter::apply( osg::LOD &node )
 {
     debugPrint( node );
-
-    while ( lastDepth >= _nodePath.size() )
-    {
-        //We are not a child of previous node
-        currentNode = daeSafeCast< domNode >( currentNode->getParentElement() );
-        lastDepth--;
-    }
-    currentNode = daeSafeCast< domNode >(currentNode->add( COLLADA_ELEMENT_NODE ) );
+    updateCurrentDaeNode();
     lastDepth = _nodePath.size();
+    currentNode = daeSafeCast< domNode >(currentNode->add( COLLADA_ELEMENT_NODE ) );
     currentNode->setId(getNodeName(node,"LOD").c_str());
 
-    if (writeExtras)
+    if (_pluginOptions.writeExtras)
     {
         // Store LOD data as extra "LOD" data in the "OpenSceneGraph" technique
         // Adds the following to a node
@@ -346,14 +324,15 @@ void daeWriter::apply( osg::LOD &node )
 
 void daeWriter::apply( osg::ProxyNode &node ) 
 {
-    osg::notify( osg::WARN ) << "ProxyNode. Missing " << node.getNumChildren() << " children" << std::endl;
+    OSG_WARN << "ProxyNode. Missing " << node.getNumChildren() << " children" << std::endl;
 }
 
 void daeWriter::apply( osg::LightSource &node )
 {
     debugPrint( node );
+    updateCurrentDaeNode();
 
-    domInstance_light *il = daeSafeCast< domInstance_light >( currentNode->add( "instance_light" ) );
+    domInstance_light *il = daeSafeCast< domInstance_light >( currentNode->add( COLLADA_ELEMENT_INSTANCE_LIGHT ) );
     std::string name = node.getName();
     if ( name.empty() )
     {
@@ -369,14 +348,124 @@ void daeWriter::apply( osg::LightSource &node )
     domLight *light = daeSafeCast< domLight >( lib_lights->add( COLLADA_ELEMENT_LIGHT ) );
     light->setId( name.c_str() );
     
+    osg::Light* pOsgLight = node.getLight();
+
+    domLight *pDomLight = daeSafeCast< domLight >( lib_lights->add( COLLADA_ELEMENT_LIGHT ) );
+    pDomLight->setId( name.c_str() );
+
+    domLight::domTechnique_common *domTechniqueCommon = daeSafeCast<domLight::domTechnique_common>(pDomLight->add(COLLADA_ELEMENT_TECHNIQUE_COMMON));
+
+    osg::Vec4 position = pOsgLight->getPosition();
+    osg::Vec3 direction = pOsgLight->getDirection();
+    osg::Vec4 ambientColor = pOsgLight->getAmbient();
+    osg::Vec4 diffuseColor = pOsgLight->getDiffuse();
+    osg::Vec4 specularColor = pOsgLight->getSpecular();
+
+    if (position.w() == 0)
+    {
+        // Directional light
+        domLight::domTechnique_common::domDirectional *domDirectional = daeSafeCast<domLight::domTechnique_common::domDirectional>(domTechniqueCommon->add(COLLADA_ELEMENT_DIRECTIONAL));
+
+        if ((position.x() != 0) || (position.y() != 0) || (position.z() != 0))
+        {
+            osg::Vec3 dir(-position.x(), -position.y(), -position.z());
+            // TODO wrap instance_light in a rotating node to translate default light [0,0,-1] into proper direction
+        }
+
+        domFloat3 color;
+        color.append3(diffuseColor.r(), diffuseColor.g(), diffuseColor.b());
+        domDirectional->add(COLLADA_ELEMENT_COLOR);
+        domDirectional->getColor()->setValue(color);
+    }
+    else if (direction.length() == 0)
+    {
+        // Omni/point light
+        domLight::domTechnique_common::domPoint *domPoint = daeSafeCast<domLight::domTechnique_common::domPoint>(domTechniqueCommon->add(COLLADA_ELEMENT_POINT));
+        domPoint->add(COLLADA_ELEMENT_CONSTANT_ATTENUATION);
+        domPoint->getConstant_attenuation()->setValue(pOsgLight->getConstantAttenuation());
+        domPoint->add(COLLADA_ELEMENT_LINEAR_ATTENUATION);
+        domPoint->getLinear_attenuation()->setValue(pOsgLight->getLinearAttenuation());
+        domPoint->add(COLLADA_ELEMENT_QUADRATIC_ATTENUATION);
+        domPoint->getQuadratic_attenuation()->setValue(pOsgLight->getQuadraticAttenuation());
+        
+        if ((position.x() != 0) || (position.y() != 0) || (position.z() != 0))
+        {
+            // TODO wrap instance_light in a transforming node to translate default light [0,0,0] into proper position
+        }
+
+        domFloat3 color;
+        color.append3(diffuseColor.r(), diffuseColor.g(), diffuseColor.b());
+        domPoint->add(COLLADA_ELEMENT_COLOR);
+        domPoint->getColor()->setValue(color);
+    }
+    else
+    {
+        // Spot light
+        domLight::domTechnique_common::domSpot *domSpot = daeSafeCast<domLight::domTechnique_common::domSpot>(domTechniqueCommon->add(COLLADA_ELEMENT_SPOT));
+        domSpot->add(COLLADA_ELEMENT_CONSTANT_ATTENUATION);
+        domSpot->getConstant_attenuation()->setValue(pOsgLight->getConstantAttenuation());
+        domSpot->add(COLLADA_ELEMENT_LINEAR_ATTENUATION);
+        domSpot->getLinear_attenuation()->setValue(pOsgLight->getLinearAttenuation());
+        domSpot->add(COLLADA_ELEMENT_QUADRATIC_ATTENUATION);
+        domSpot->getQuadratic_attenuation()->setValue(pOsgLight->getQuadraticAttenuation());
+
+        if ((position.x() != 0) || (position.y() != 0) || (position.z() != 0))
+        {
+            // TODO wrap instance_light in a transforming node to translate default light [0,0,0] into proper position
+            // and rotate default direction [0,0,-1] into proper dir
+        }
+
+        domFloat3 color;
+        color.append3(diffuseColor.r(), diffuseColor.g(), diffuseColor.b());
+        domSpot->add(COLLADA_ELEMENT_COLOR);
+        domSpot->getColor()->setValue(color);
+
+        domSpot->add(COLLADA_ELEMENT_FALLOFF_ANGLE);
+        domSpot->getFalloff_angle()->setValue(pOsgLight->getSpotCutoff());
+
+        domSpot->add(COLLADA_ELEMENT_FALLOFF_EXPONENT);
+        domSpot->getFalloff_exponent()->setValue(pOsgLight->getSpotExponent());
+    }
+
+    // Write ambient as a separate Collada <ambient> light
+    if ((ambientColor.r() != 0) || (ambientColor.g() != 0) || (ambientColor.b() != 0))
+    {
+        domInstance_light *ambientDomInstanceLight = daeSafeCast< domInstance_light >(currentNode->add( COLLADA_ELEMENT_INSTANCE_LIGHT ));
+        std::string name = node.getName();
+        if (name.empty())
+        {
+            name = uniquify( "light-ambient" );
+        }
+        else
+        {
+            name += "-ambient";
+        }
+        std::string url = "#" + name;
+        ambientDomInstanceLight->setUrl( url.c_str() );
+
+        domLight *ambientDomLight = daeSafeCast< domLight >( lib_lights->add( COLLADA_ELEMENT_LIGHT ) );
+        ambientDomLight->setId(name.c_str());
+        
+        domLight::domTechnique_common *ambientDomTechniqueCommon = daeSafeCast<domLight::domTechnique_common>(ambientDomLight->add(COLLADA_ELEMENT_TECHNIQUE_COMMON));
+
+        // Ambient light
+        domLight::domTechnique_common::domAmbient *domAmbient = daeSafeCast<domLight::domTechnique_common::domAmbient>(ambientDomTechniqueCommon->add(COLLADA_ELEMENT_AMBIENT));
+
+        domFloat3 color;
+        color.append3(ambientColor.r(), ambientColor.g(), ambientColor.b());
+        domAmbient->add(COLLADA_ELEMENT_COLOR);
+        domAmbient->getColor()->setValue(color);
+    }
+   
     traverse( node );
 }
 
 void daeWriter::apply( osg::Camera &node )
 {
     debugPrint( node );
+    updateCurrentDaeNode();
 
-    domInstance_camera *ic = daeSafeCast< domInstance_camera >( currentNode->add( "instance_camera" ) );
+    domInstance_camera *ic = daeSafeCast< domInstance_camera >( currentNode->add( COLLADA_ELEMENT_INSTANCE_CAMERA ) );
     std::string name = node.getName();
     if ( name.empty() )
     {
@@ -398,8 +487,9 @@ void daeWriter::apply( osg::Camera &node )
 void daeWriter::apply( osg::CameraView &node)
 {
     debugPrint( node );
+    updateCurrentDaeNode();
 
-    domInstance_camera *ic = daeSafeCast< domInstance_camera >( currentNode->add( "instance_camera" ) );
+    domInstance_camera *ic = daeSafeCast< domInstance_camera >( currentNode->add(COLLADA_ELEMENT_INSTANCE_CAMERA));
     std::string name = node.getName();
     if ( name.empty() )
     {
