@@ -24,7 +24,6 @@
 *          Wojtek Lewandowski 2009-05-22 
 *
 **********************************************************************/
-
 #include <osg/Texture>
 #include <osg/Notify>
 
@@ -32,11 +31,18 @@
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
 #include <osgDB/fstream>
-
 #include <iomanip>
 #include <stdio.h>
 #include <string.h>
 
+#if defined(OSG_GLES1_AVAILABLE) || defined(OSG_GLES2_AVAILABLE)
+    #define GL_RED                  0x1903
+    #define GL_LUMINANCE4_ALPHA4    0x8043
+#endif
+
+#if defined(OSG_GL3_AVAILABLE)
+    #define GL_LUMINANCE4_ALPHA4    0x8043
+#endif
 
 // NOTICE ON WIN32:
 // typedef DWORD unsigned long;
@@ -171,7 +177,8 @@ struct DDSURFACEDESC2
 
 //
 // Structure of a DXT-1 compressed texture block
-// see http://msdn.microsoft.com/library/default.asp?url=/library/en-us/directx9_c/Opaque_and_1_Bit_Alpha_Textures.asp
+// see page "Opaque and 1-Bit Alpha Textures (Direct3D 9)" on http://msdn.microsoft.com
+// url at time of writing http://msdn.microsoft.com/en-us/library/bb147243(v=VS.85).aspx
 //
 struct DXT1TexelsBlock
 {
@@ -228,10 +235,63 @@ struct DXT1TexelsBlock
 #define FOURCC_DXT4  (MAKEFOURCC('D','X','T','4'))
 #define FOURCC_DXT5  (MAKEFOURCC('D','X','T','5'))
 
+/*
+* FOURCC codes for 3dc compressed-texture pixel formats
+*/
+#define FOURCC_ATI1  (MAKEFOURCC('A','T','I','1'))
+#define FOURCC_ATI2  (MAKEFOURCC('A','T','I','2'))
+
+static unsigned int ComputeImageSizeInBytes
+    ( int width, int height, int depth,
+      unsigned int pixelFormat, unsigned int pixelType,
+      int packing = 1, int slice_packing = 1, int image_packing = 1 )
+{
+    if( width < 1 )  width = 1;
+    if( height < 1 ) height = 1;
+    if( depth < 1 )  depth = 1;
+
+    // Taking advantage of the fact that 
+    // DXT formats are defined as 4 successive numbers:
+    // GL_COMPRESSED_RGB_S3TC_DXT1_EXT         0x83F0
+    // GL_COMPRESSED_RGBA_S3TC_DXT1_EXT        0x83F1
+    // GL_COMPRESSED_RGBA_S3TC_DXT3_EXT        0x83F2
+    // GL_COMPRESSED_RGBA_S3TC_DXT5_EXT        0x83F3
+    if( pixelFormat >= GL_COMPRESSED_RGB_S3TC_DXT1_EXT &&
+        pixelFormat <= GL_COMPRESSED_RGBA_S3TC_DXT5_EXT )
+    {
+        width = (width + 3) & ~3;
+        height = (height + 3) & ~3;
+    }
+    // 3dc ATI formats
+    // GL_COMPRESSED_RED_RGTC1_EXT                     0x8DBB
+    // GL_COMPRESSED_SIGNED_RED_RGTC1_EXT              0x8DBC
+    // GL_COMPRESSED_RED_GREEN_RGTC2_EXT               0x8DBD
+    // GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT        0x8DBE
+    if( pixelFormat >= GL_COMPRESSED_RED_RGTC1_EXT &&
+        pixelFormat <= GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT )
+    {
+        width = (width + 3) & ~3;
+        height = (height + 3) & ~3;
+    }
+    // compute size of one row
+    unsigned int size = osg::Image::computeRowWidthInBytes
+                            ( width, pixelFormat, pixelType, packing );
+
+    // now compute size of slice
+    size *= height;
+    size += slice_packing - 1;
+    size -= size % slice_packing;
+
+    // compute size of whole image
+    size *= depth;
+    size += image_packing - 1;
+    size -= size % image_packing;
+
+    return size;
+}
 
 osg::Image* ReadDDSFile(std::istream& _istream)
 {
-
     DDSURFACEDESC2 ddsd;
 
     char filecode[4];
@@ -243,17 +303,6 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     // Get the surface desc.
     _istream.read((char*)(&ddsd), sizeof(ddsd));
 
-
-    // Size of 2d images - 3d images don't set dwLinearSize
-    //###[afarre_051904]
-    /*unsigned int size = ddsd.dwMipMapCount > 1 ? ddsd.dwLinearSize * (ddsd.ddpfPixelFormat.dwFourCC==FOURCC_DXT1 ? 2: 4) : ddsd.dwLinearSize;
-
-    if(size <= 0)
-    {
-        osg::notify(osg::WARN)<<"Warning:: dwLinearSize is not defined in dds file, image not loaded."<<std::endl;
-        return NULL;
-    }*/
-
     osg::ref_ptr<osg::Image> osgImage = new osg::Image();    
     
     //Check valid structure sizes
@@ -262,13 +311,11 @@ osg::Image* ReadDDSFile(std::istream& _istream)
         return NULL;
     }
 
-    bool is3dImage = false;
     int depth = 1;
 
     // Check for volume image
     if( ddsd.dwDepth > 0 && (ddsd.dwFlags & DDSD_DEPTH))
     {
-        is3dImage = true;
         depth = ddsd.dwDepth;
     }
 
@@ -283,7 +330,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     // Handle some esoteric formats
     if(ddsd.ddpfPixelFormat.dwFlags & DDPF_BUMPDUDV) 
     {
-        osg::notify(osg::WARN) << "ReadDDSFile warning: DDPF_BUMPDUDV format is not supported" << std::endl;
+        OSG_WARN << "ReadDDSFile warning: DDPF_BUMPDUDV format is not supported" << std::endl;
         return NULL;
 //         ddsd.ddpfPixelFormat.dwFlags =
 //             DDPF_LUMINANCE + DDPF_ALPHAPIXELS;
@@ -294,7 +341,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     }
     if(ddsd.ddpfPixelFormat.dwFlags & DDPF_BUMPLUMINANCE) 
     {
-        osg::notify(osg::WARN) << "ReadDDSFile warning: DDPF_BUMPLUMINANCE format is not supported" << std::endl;
+        OSG_WARN << "ReadDDSFile warning: DDPF_BUMPLUMINANCE format is not supported" << std::endl;
         return NULL;
 //         ddsd.ddpfPixelFormat.dwFlags = DDPF_RGB;
 //         // handle as RGB
@@ -306,7 +353,6 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     // while compressed formats will use DDPF_FOURCC with a four-character code.
     
     bool usingAlpha = ddsd.ddpfPixelFormat.dwFlags & DDPF_ALPHAPIXELS;
-    bool checkIfUsingOneBitAlpha = false;
 
     // Uncompressed formats.
     if(ddsd.ddpfPixelFormat.dwFlags & DDPF_RGB)
@@ -381,7 +427,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
                      f.pixelFormat    != UNSUPPORTED &&
                      f.dataType       != UNSUPPORTED )
                 {
-                    osg::notify(osg::INFO) << "ReadDDSFile info : format = " << f.name << std::endl;
+                    OSG_INFO << "ReadDDSFile info : format = " << f.name << std::endl;
                     internalFormat = f.internalFormat;
                     pixelFormat    = f.pixelFormat;
                     dataType       = f.dataType;
@@ -390,7 +436,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
                 }
                 else
                 {
-                    osg::notify(osg::INFO) << "ReadDDSFile info : " << f.name
+                    OSG_INFO << "ReadDDSFile info : " << f.name
                                            << " format is not supported" << std::endl;
                     return NULL;                   
                 }
@@ -399,19 +445,19 @@ osg::Image* ReadDDSFile(std::istream& _istream)
 
         if ( !found )
         {
-            osg::notify(osg::WARN) << "ReadDDSFile warning: unhandled RGB pixel format in dds file, image not loaded" << std::endl;
-            osg::notify(osg::INFO) << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwRGBBitCount     = "
+            OSG_WARN << "ReadDDSFile warning: unhandled RGB pixel format in dds file, image not loaded" << std::endl;
+            OSG_INFO << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwRGBBitCount     = "
                                    << ddsd.ddpfPixelFormat.dwRGBBitCount << std::endl;
-            osg::notify(osg::INFO) << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwRBitMask        = 0x"
+            OSG_INFO << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwRBitMask        = 0x"
                                    << std::hex << std::setw(8) << std::setfill('0') 
                                    << ddsd.ddpfPixelFormat.dwRBitMask << std::endl;
-            osg::notify(osg::INFO) << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwGBitMask        = 0x"
+            OSG_INFO << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwGBitMask        = 0x"
                                    << std::hex << std::setw(8) << std::setfill('0') 
                                    << ddsd.ddpfPixelFormat.dwGBitMask << std::endl;
-            osg::notify(osg::INFO) << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwBBitMask        = 0x"
+            OSG_INFO << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwBBitMask        = 0x"
                                    << std::hex << std::setw(8) << std::setfill('0') 
                                    << ddsd.ddpfPixelFormat.dwBBitMask << std::endl;
-            osg::notify(osg::INFO) << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwRGBAlphaBitMask = 0x"
+            OSG_INFO << "ReadDDSFile info : ddsd.ddpfPixelFormat.dwRGBAlphaBitMask = 0x"
                                    << std::hex << std::setw(8) << std::setfill('0') 
                                    << ddsd.ddpfPixelFormat.dwRGBAlphaBitMask << std::dec << std::endl;
             return NULL;
@@ -423,26 +469,26 @@ osg::Image* ReadDDSFile(std::istream& _istream)
             pixelFormat    = usingAlpha ? GL_LUMINANCE_ALPHA : GL_LUMINANCE;
             if ( usingAlpha && ddsd.ddpfPixelFormat.dwLuminanceBitDepth == 8 )
             {
-                osg::notify(osg::INFO) << "ReadDDSFile info : format = L4A4" << std::endl;
+                OSG_INFO << "ReadDDSFile info : format = L4A4" << std::endl;
                 pixelFormat = GL_LUMINANCE4_ALPHA4; // invalid enumerant?
             }
             else if ( usingAlpha && ddsd.ddpfPixelFormat.dwLuminanceBitDepth == 32 )
             {
-                osg::notify(osg::INFO) << "ReadDDSFile info : format = L16A16" << std::endl;
+                OSG_INFO << "ReadDDSFile info : format = L16A16" << std::endl;
                 dataType = GL_UNSIGNED_SHORT;
             }
             else if ( !usingAlpha && ddsd.ddpfPixelFormat.dwLuminanceBitDepth == 16 )
             {
-                osg::notify(osg::INFO) << "ReadDDSFile info : format = L16" << std::endl;
+                OSG_INFO << "ReadDDSFile info : format = L16" << std::endl;
                 dataType = GL_UNSIGNED_SHORT;
             }
             else if ( usingAlpha )
             {
-                osg::notify(osg::INFO) << "ReadDDSFile info : format = L8A8" << std::endl;
+                OSG_INFO << "ReadDDSFile info : format = L8A8" << std::endl;
             }
             else
             {
-                osg::notify(osg::INFO) << "ReadDDSFile info : format = L8" << std::endl;
+                OSG_INFO << "ReadDDSFile info : format = L8" << std::endl;
             }
 //             else if ( ddsd.ddpfPixelFormat.dwLuminanceBitDepth == (usingAlpha ? 64 : 32) )
 //             {
@@ -451,18 +497,17 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     }
     else if(ddsd.ddpfPixelFormat.dwFlags & DDPF_ALPHA)
     {
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = ALPHA" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = ALPHA" << std::endl;
             internalFormat = GL_ALPHA;
             pixelFormat    = GL_ALPHA;              
     }
     // Compressed formats
     else if(ddsd.ddpfPixelFormat.dwFlags & DDPF_FOURCC)
     {
-        // TODO: Image::isImageTranslucent() doesn't work with S3TC compressed files
         switch(ddsd.ddpfPixelFormat.dwFourCC)
         {
         case FOURCC_DXT1:
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = DXT1" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = DXT1, usingAlpha=" <<usingAlpha<< std::endl;
             if (usingAlpha)
             {
                 internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
@@ -472,39 +517,48 @@ osg::Image* ReadDDSFile(std::istream& _istream)
             {
                 internalFormat = GL_COMPRESSED_RGB_S3TC_DXT1_EXT;
                 pixelFormat    = GL_COMPRESSED_RGB_S3TC_DXT1_EXT;
-                checkIfUsingOneBitAlpha = true;
             }
             break;
         case FOURCC_DXT3:
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = DXT3" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = DXT3" << std::endl;
             internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
             pixelFormat    = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
             break;
         case FOURCC_DXT5:
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = DXT5" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = DXT5" << std::endl;
             internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
             pixelFormat    = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
             break;
+        case FOURCC_ATI1:
+            OSG_INFO << "ReadDDSFile info : format = ATI1" << std::endl;
+            internalFormat = GL_COMPRESSED_RED_RGTC1_EXT;
+            pixelFormat    = GL_COMPRESSED_RED_RGTC1_EXT;
+            break;
+        case FOURCC_ATI2:
+            OSG_INFO << "ReadDDSFile info : format = ATI2" << std::endl;
+            internalFormat = GL_COMPRESSED_RED_GREEN_RGTC2_EXT;
+            pixelFormat    = GL_COMPRESSED_RED_GREEN_RGTC2_EXT;
+            break;        
         case 0x00000024: // A16B16G16R16
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = A16B16G16R16" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = A16B16G16R16" << std::endl;
             internalFormat = GL_RGBA;
             pixelFormat    = GL_RGBA;
             dataType       = GL_UNSIGNED_SHORT;
             break;
         case 0x00000071: // A16B16G16R16F
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = A16B16G16R16F" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = A16B16G16R16F" << std::endl;
             internalFormat = GL_RGBA; // why no transparency?
             pixelFormat    = GL_RGBA;
             dataType       = GL_HALF_FLOAT_NV; 
             break;
         case 0x0000006E: // Q16W16V16U16
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = Q16W16V16U16" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = Q16W16V16U16" << std::endl;
             internalFormat = GL_RGBA;
             pixelFormat    = GL_RGBA;
             dataType       = GL_UNSIGNED_SHORT;
             break;
         case 0x00000070: // G16R16F
-            osg::notify(osg::INFO) << "ReadDDSFile info : G16R16F format is not supported"
+            OSG_INFO << "ReadDDSFile info : G16R16F format is not supported"
                                    << std::endl;
             return NULL;
 //             internalFormat = GL_RGB;
@@ -512,7 +566,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
 //             dataType       = GL_HALF_FLOAT_NV;
             break;
         case 0x00000073: // G32R32F
-            osg::notify(osg::INFO) << "ReadDDSFile info : G32R32F format is not supported"
+            OSG_INFO << "ReadDDSFile info : G32R32F format is not supported"
                                    << std::endl;
             return NULL;
 //             internalFormat = GL_RGB;
@@ -520,25 +574,25 @@ osg::Image* ReadDDSFile(std::istream& _istream)
 //             dataType       = GL_FLOAT;
             break;
         case 0x00000072: // R32F
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = R32F" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = R32F" << std::endl;
             internalFormat = GL_RGB;
             pixelFormat    = GL_RED;
             dataType       = GL_FLOAT;
             break;
         case 0x0000006F: // R16F
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = R16F" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = R16F" << std::endl;
             internalFormat = GL_RGB;
             pixelFormat    = GL_RED;
             dataType       = GL_HALF_FLOAT_NV;
             break;
         case 0x00000074: // A32B32G32R32F
-            osg::notify(osg::INFO) << "ReadDDSFile info : format = A32B32G32R32F" << std::endl;
+            OSG_INFO << "ReadDDSFile info : format = A32B32G32R32F" << std::endl;
             internalFormat = GL_RGBA;
             pixelFormat    = GL_RGBA;
             dataType       = GL_FLOAT;
             break;
         case 0x00000075: // CxV8U8
-            osg::notify(osg::INFO) << "ReadDDSFile info : CxV8U8 format is not supported" << std::endl;
+            OSG_INFO << "ReadDDSFile info : CxV8U8 format is not supported" << std::endl;
             return NULL;
 
         case MAKEFOURCC( 'U', 'Y', 'V', 'Y' ): // not supported in OSG
@@ -548,7 +602,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
             //break;
 
         default:
-            osg::notify(osg::WARN) << "ReadDDSFile warning: unhandled FOURCC pixel format ("
+            OSG_WARN << "ReadDDSFile warning: unhandled FOURCC pixel format ("
                                    << (char)((ddsd.ddpfPixelFormat.dwFourCC & 0x000000ff))
                                    << (char)((ddsd.ddpfPixelFormat.dwFourCC & 0x0000ff00) >> 8)
                                    << (char)((ddsd.ddpfPixelFormat.dwFourCC & 0x00ff0000) >> 16)
@@ -561,180 +615,90 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     }
     else 
     {
-        osg::notify(osg::WARN) << "ReadDDSFile warning: unhandled pixel format (ddsd.ddpfPixelFormat.dwFlags"
+        OSG_WARN << "ReadDDSFile warning: unhandled pixel format (ddsd.ddpfPixelFormat.dwFlags"
                                << " = 0x" << std::hex << std::setw(8) << std::setfill('0')
                                << ddsd.ddpfPixelFormat.dwFlags << std::dec
                                << ") in dds file, image not loaded."<<std::endl;
         return NULL;
     }
 
-    //###[afarre_051904]
-    /*if (is3dImage)
-        size = osg::Image::computeNumComponents(pixelFormat) * ddsd.dwWidth * ddsd.dwHeight * depth;
-
-
-    //delayed allocation og image data after all checks
-    unsigned char* imageData = new unsigned char [size];
-    if(!imageData)
-    {
-        return NULL;
-    }
-
-    // Read image data
-    _istream.read((char*)imageData, size);
-
-    
-    // NOTE: We need to set the image data before setting the mipmap data, this
-    // is because the setImage method clears the _mipmapdata vector in osg::Image.
-    // Set image data and properties.
-    osgImage->setImage(s,t,r, internalFormat, pixelFormat, dataType, imageData, osg::Image::USE_NEW_DELETE);
-    */
-
-    // Now set mipmap data (offsets into image raw data)
-    //###[afarre_051904]
-    osg::Image::MipmapDataType mipmaps; 
+    unsigned int size = ComputeImageSizeInBytes( s, t, r, pixelFormat, dataType );
 
     // Take care of mipmaps if any.
-    if (ddsd.dwMipMapCount>1)
-    {
-        // Now set mipmap data (offsets into image raw data).
-        //###[afarre_051904]osg::Image::MipmapDataType mipmaps;
+    unsigned int sizeWithMipmaps = size;
+    osg::Image::MipmapDataType mipmap_offsets;
+    if ( ddsd.dwMipMapCount>1 )
+    {        
+        unsigned numMipmaps = osg::Image::computeNumberOfMipmapLevels( s, t, r );
+        if( numMipmaps > ddsd.dwMipMapCount ) numMipmaps = ddsd.dwMipMapCount;
+        // array starts at 1 level offset, 0 level skipped
+        mipmap_offsets.resize( numMipmaps - 1 );
 
-        //This is to complete mipmap sequence until level Nx1
+        int width = s;
+        int height = t; 
+        int depth = r;
 
-        //debugging messages        
-        float power2_s = logf((float)s)/logf((float)2);
-        float power2_t = logf((float)t)/logf((float)2);
-
-        osg::notify(osg::INFO) << "ReadDDSFile INFO : ddsd.dwMipMapCount = "<<ddsd.dwMipMapCount<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile INFO : s = "<<s<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile INFO : t = "<<t<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile INFO : power2_s="<<power2_s<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile INFO : power2_t="<<power2_t<<std::endl;
-
-        mipmaps.resize((unsigned int)osg::maximum(power2_s,power2_t),0);
-
-        // Handle S3TC compressed mipmaps.
-        if( (ddsd.ddpfPixelFormat.dwFlags & DDPF_FOURCC)
-            &&
-            (ddsd.ddpfPixelFormat.dwFourCC == FOURCC_DXT1 ||
-             ddsd.ddpfPixelFormat.dwFourCC == FOURCC_DXT3 ||
-             ddsd.ddpfPixelFormat.dwFourCC == FOURCC_DXT5))
+        for( unsigned int k = 0; k < mipmap_offsets.size(); ++k  )
         {
-            int width = ddsd.dwWidth;
-            int height = ddsd.dwHeight;
-            int blockSize = (ddsd.ddpfPixelFormat.dwFourCC == FOURCC_DXT1) ? 8 : 16;
-            int offset = 0;
-            for (unsigned int k = 1; k < ddsd.dwMipMapCount && (width || height); ++k)
-            {
-                if (width == 0)
-                    width = 1;
-                if (height == 0)
-                    height = 1;
-                offset += (((width+3)/4) * ((height+3)/4) * blockSize);
-                mipmaps[k-1] = offset;
-                width >>= 1;
-                height >>= 1;
-            }
-            //###[afarre_051904] osgImage->setMipmapData(mipmaps);
-        }
-        // Handle uncompressed mipmaps
-        else
-        {
-            int offset = 0;
-            int width = ddsd.dwWidth;
-            int height = ddsd.dwHeight;
-            for (unsigned int k = 1; k < ddsd.dwMipMapCount && (width || height); ++k)
-            {
-                if (width == 0)
-                    width = 1;
-                if (height == 0)
-                    height = 1;
-                offset += height *
-                    osg::Image::computeRowWidthInBytes( width, pixelFormat, dataType, 1 );
-                mipmaps[k-1] = offset;
-                width >>= 1;
-                height >>= 1;
-            }
-            //###[afarre_051904] osgImage->setMipmapData(mipmaps);
+           mipmap_offsets[k] = sizeWithMipmaps;
+
+           width = osg::maximum( width >> 1, 1 );
+           height = osg::maximum( height >> 1, 1 );
+           depth = osg::maximum( depth >> 1, 1 );
+
+           sizeWithMipmaps += 
+                ComputeImageSizeInBytes( width, height, depth, pixelFormat, dataType );
         }
     }
-
-    osgImage->setImage(s,t,r, internalFormat, pixelFormat, dataType, 0, osg::Image::USE_NEW_DELETE);
-    if (mipmaps.size()>0)  osgImage->setMipmapLevels(mipmaps);
-    unsigned int size = osgImage->getTotalSizeInBytesIncludingMipmaps();
-
-    osg::notify(osg::INFO) << "ReadDDSFile INFO : size = " << size << std::endl;
-    
-    if(size <= 0)
-    {
-        osg::notify(osg::WARN) << "ReadDDSFile warning: size <= 0" << std::endl;
-        return NULL;
-    }
-
-    unsigned char* imageData = new unsigned char [size];
+     
+    unsigned char* imageData = new unsigned char [sizeWithMipmaps];
     if(!imageData)
     {
-        osg::notify(osg::WARN) << "ReadDDSFile warning: imageData == NULL" << std::endl;
+        OSG_WARN << "ReadDDSFile warning: imageData == NULL" << std::endl;
+        return NULL;
+    }
+    
+    // Read pixels in two chunks. First main image, next mipmaps. 
+    if ( !_istream.read( (char*)imageData, size ) )
+    {
+        delete [] imageData;
+        OSG_WARN << "ReadDDSFile warning: couldn't read imageData" << std::endl;
         return NULL;
     }
 
-    // Read image data
-    _istream.read((char*)imageData, size);
-
-    // Check if alpha information embedded in the 8-byte encoding blocks
-    if (checkIfUsingOneBitAlpha)
+    // If loading mipmaps in second chunk fails we may still use main image
+    if ( size < sizeWithMipmaps && !_istream.read( (char*)imageData + size, sizeWithMipmaps - size ) )
     {
-        const DXT1TexelsBlock *texelsBlock = reinterpret_cast<const DXT1TexelsBlock*>(imageData);
+        sizeWithMipmaps = size;
+        mipmap_offsets.resize( 0 );
+        OSG_WARN << "ReadDDSFile warning: couldn't read mipmapData" << std::endl;
 
-        // Only do the check on the first mipmap level
-        unsigned int numBlocks = mipmaps.size()>0 ? mipmaps[0] / 8 : size / 8;
-
-        for (int i=numBlocks; i>0; --i, ++texelsBlock)
-        {
-            if (texelsBlock->color_0<=texelsBlock->color_1)
-            {
-                // Texture is using the 1-bit alpha encoding, so we need to update the assumed pixel format
-                internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
-                pixelFormat    = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
-                break;
-            }
-        }
+        // if mipmaps read failed we leave some not used overhead memory allocated past main image
+        // this memory will not be used but it will not cause leak in worst meaning of this word.
     }
 
     osgImage->setImage(s,t,r, internalFormat, pixelFormat, dataType, imageData, osg::Image::USE_NEW_DELETE);
-    if (mipmaps.size()>0)  osgImage->setMipmapLevels(mipmaps);
-         
+
+    if (mipmap_offsets.size()>0) osgImage->setMipmapLevels(mipmap_offsets);
+
     // Return Image.
     return osgImage.release();
 }
 
-
-/*
-osg::Image::MipmapDataType mipmaps;
-osgImage->setMipmapData(mipmaps);
-osgImage->setImage(s,t,r, internalFormat, pixelFormat, dataType, 0, osg::Image::USE_NEW_DELETE);
-printf("INVENTO===> gtsibim:%d  grsib:%d   mi_size:%d   lPitch%d\n", 
-    osgImage->getTotalSizeInBytesIncludingMipmaps(), 
-    osgImage->getRowSizeInBytes(), size, ddsd.lPitch);
-printf("CORRECTO**> gtsibim:%d  grsib:%d   mi_size:%d   lPitch%d\n", 
-    osgImage->getTotalSizeInBytesIncludingMipmaps(), 
-    osgImage->getRowSizeInBytes(), size, ddsd.lPitch);
-
- */
-
-
 bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
 {
 
     // Initialize ddsd structure and its members 
     DDSURFACEDESC2 ddsd;
+    memset( &ddsd, 0, sizeof( ddsd ) );
     DDPIXELFORMAT  ddpf;
+    memset( &ddpf, 0, sizeof( ddpf ) );
     //DDCOLORKEY     ddckCKDestOverlay;
     //DDCOLORKEY     ddckCKDestBlt;
     //DDCOLORKEY     ddckCKSrcOverlay;
     //DDCOLORKEY     ddckCKSrcBlt;
     DDSCAPS2       ddsCaps;
+    memset( &ddsCaps, 0, sizeof( ddsCaps ) );
 
     ddsd.dwSize = sizeof(ddsd);  
     ddpf.dwSize = sizeof(ddpf);
@@ -752,7 +716,6 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
     //unsigned int components     = osg::Image::computeNumComponents(pixelFormat);
     unsigned int pixelSize      = osg::Image::computePixelSizeInBits(pixelFormat, dataType);
     unsigned int imageSize      = img->getImageSizeInBytes();
-    bool is3dImage = false;
 
     ddsd.dwWidth  = img->s();
     ddsd.dwHeight = img->t();
@@ -760,7 +723,6 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
 
     if(r > 1)  /* check for 3d image */
     {
-        is3dImage = true;
         ddsd.dwDepth = r;
         SD_flags    |= DDSD_DEPTH;
         CAPS_flags  |= DDSCAPS_COMPLEX;
@@ -816,6 +778,17 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
             SD_flags |= DDSD_PITCH;
         }
         break;
+    case GL_BGR:
+        {
+            ddpf.dwBBitMask        = 0x000000ff;
+            ddpf.dwGBitMask        = 0x0000ff00;
+            ddpf.dwRBitMask        = 0x00ff0000;  
+            PF_flags |= DDPF_RGB;
+            ddpf.dwRGBBitCount = pixelSize;
+            ddsd.lPitch = img->getRowSizeInBytes();
+            SD_flags |= DDSD_PITCH;
+        }
+        break;
     case GL_LUMINANCE:
         {
             ddpf.dwRBitMask         = 0x000000ff;
@@ -868,26 +841,65 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
             SD_flags |= DDSD_LINEARSIZE;
         }
         break;
+    case GL_COMPRESSED_SIGNED_RED_RGTC1_EXT:
+        {
+            ddpf.dwFourCC = FOURCC_ATI1;
+            PF_flags |= DDPF_FOURCC;  /* No alpha here */
+            ddsd.dwLinearSize = imageSize;
+            SD_flags |= DDSD_LINEARSIZE;
+        }
+        break;
+    case GL_COMPRESSED_RED_RGTC1_EXT:
+        {
+            ddpf.dwFourCC = FOURCC_ATI1;
+            PF_flags |= DDPF_FOURCC;  /* No alpha here */
+            ddsd.dwLinearSize = imageSize;
+            SD_flags |= DDSD_LINEARSIZE;
+        }
+        break;    
+    case GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT:
+        {
+            ddpf.dwFourCC = FOURCC_ATI2;
+            PF_flags |= DDPF_FOURCC;  /* No alpha here */
+            ddsd.dwLinearSize = imageSize;
+            SD_flags |= DDSD_LINEARSIZE;
+        }
+        break;    
+    case GL_COMPRESSED_RED_GREEN_RGTC2_EXT:
+        {
+            ddpf.dwFourCC = FOURCC_ATI2;
+            PF_flags |= DDPF_FOURCC;  /* No alpha here */
+            ddsd.dwLinearSize = imageSize;
+            SD_flags |= DDSD_LINEARSIZE;
+        }
+        break;    
     default:
-        osg::notify(osg::WARN)<<"Warning:: unhandled pixel format in image, file cannot be written."<<std::endl;
+        OSG_WARN<<"Warning:: unhandled pixel format in image, file cannot be written."<<std::endl;
         return false;
     }
 
-   
+    int size = img->getTotalSizeInBytes();
+
     // set even more flags
-    if(img->isMipmap() && !is3dImage)
-    {
+    if( !img->isMipmap() ) {
+
+       OSG_INFO<<"no mipmaps to write out."<<std::endl;
+
+    } else if( img->getPacking() > 1 ) {
+
+       OSG_WARN<<"Warning: mipmaps not written. DDS requires packing == 1."<<std::endl;
+
+    } else { // image contains mipmaps and has 1 byte alignment
+
         SD_flags   |= DDSD_MIPMAPCOUNT;
         CAPS_flags |= DDSCAPS_COMPLEX | DDSCAPS_MIPMAP;
-        ddsd.dwMipMapCount = img->getNumMipmapLevels();
         
-        osg::notify(osg::INFO)<<"writing out with mipmaps ddsd.dwMipMapCount"<<ddsd.dwMipMapCount<<std::endl;
-    }
-    else
-    {
-        osg::notify(osg::INFO)<<"no mipmaps to write out."<<std::endl;
-    }
+        ddsd.dwMipMapCount = img->getNumMipmapLevels();
 
+        size = img->getTotalSizeInBytesIncludingMipmaps();
+
+        OSG_INFO<<"writing out with mipmaps ddsd.dwMipMapCount"<<ddsd.dwMipMapCount<<std::endl;
+    }
 
     // Assign flags and structure members of ddsd
     ddsd.dwFlags    = SD_flags;
@@ -898,29 +910,14 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
     ddsd.ddpfPixelFormat = ddpf;
     ddsd.ddsCaps = ddsCaps;
 
-
     // Write DDS file
     fout.write("DDS ", 4); /* write FOURCC */
     fout.write(reinterpret_cast<char*>(&ddsd), sizeof(ddsd)); /* write file header */
-
-    //    int isize = img->getTotalSizeInBytesIncludingMipmaps();
-    if(!is3dImage)
-    {
-        fout.write(reinterpret_cast<const char*>(img->data()), img->getTotalSizeInBytesIncludingMipmaps());
-    }
-    else  /* 3d image */
-    {
-        for(int i = 0; i < r; ++i)
-        {
-            fout.write(reinterpret_cast<const char*>(img->data(0, 0, i)), imageSize);
-        }
-    }
+    fout.write(reinterpret_cast<const char*>(img->data()), size );
 
     // Check for correct saving
-    if (fout.fail())
-    {
+    if ( fout.fail() )
         return false;
-    }
 
     // If we get that far the file was saved properly //
     return true; 
@@ -934,6 +931,10 @@ public:
     ReaderWriterDDS()
     {
         supportsExtension("dds","DDS image format");
+        supportsOption("dds_dxt1_rgb","Set the pixel format of DXT1 encoded images to be RGB variant of DXT1");
+        supportsOption("dds_dxt1_rgba","Set the pixel format of DXT1 encoded images to be RGBA variant of DXT1");
+        supportsOption("dds_dxt1_detect_rgba","For DXT1 encode images set the pixel format according to presence of transparent pixels");
+        supportsOption("dds_flip","Flip the image about the horizontl axis");
     }
 
     virtual const char* className() const
@@ -971,7 +972,45 @@ public:
     {
         osg::Image* osgImage = ReadDDSFile(fin);
         if (osgImage==NULL) return ReadResult::FILE_NOT_HANDLED;
-        
+
+        if (osgImage->getPixelFormat()==GL_COMPRESSED_RGB_S3TC_DXT1_EXT ||
+            osgImage->getPixelFormat()==GL_COMPRESSED_RGBA_S3TC_DXT1_EXT)
+        {
+            if (options && options->getOptionString().find("dds_dxt1_rgba")!=std::string::npos)
+            {
+                osgImage->setPixelFormat(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT);
+                osgImage->setInternalTextureFormat(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT);
+            }
+            else if (options && options->getOptionString().find("dds_dxt1_rgb")!=std::string::npos)
+            {
+                osgImage->setPixelFormat(GL_COMPRESSED_RGB_S3TC_DXT1_EXT);
+                osgImage->setInternalTextureFormat(GL_COMPRESSED_RGB_S3TC_DXT1_EXT);
+            }
+            else if (options && options->getOptionString().find("dds_dxt1_detect_rgba")!=std::string::npos)
+            {
+                // check to see if DXT1c (RGB_S3TC_DXT1) format image might actually be
+                // a DXT1a format image
+
+                // temporarily set pixel format to GL_COMPRESSED_RGBA_S3TC_DXT1_EXT so
+                // that the isImageTranslucent() method assumes that RGBA is present and then
+                // checks the alpha values to see if they are all 1.0.
+                osgImage->setPixelFormat(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT);
+                osgImage->setInternalTextureFormat(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT);
+                if (!osgImage->isImageTranslucent())
+                {
+                    // image contains alpha's that are 1.0, so treat is as RGB
+                    OSG_INFO<<"Image with PixelFormat==GL_COMPRESSED_RGB_S3TC_DXT1_EXT is opaque."<<std::endl;
+                    osgImage->setPixelFormat(GL_COMPRESSED_RGB_S3TC_DXT1_EXT);
+                    osgImage->setInternalTextureFormat(GL_COMPRESSED_RGB_S3TC_DXT1_EXT);
+                }
+                else
+                {
+                    // image contains alpha's that are non 1.0, so treat is as RGBA
+                    OSG_INFO<<"Image with PixelFormat==GL_COMPRESSED_RGB_S3TC_DXT1_EXT has transparency, setting format to GL_COMPRESSED_RGBA_S3TC_DXT1_EXT."<<std::endl;
+                }
+            }
+        }
+
         if (options && options->getOptionString().find("dds_flip")!=std::string::npos)
         {
             osgImage->flipVertical();
