@@ -327,8 +327,9 @@ public:
         poses.nrmv=nrm;
         poses.idx=idx[j];
     }
-    void tessellate(const std::vector<Vec3> verts, const dwmaterial *themat, 
-          GLUtesselator *ts, _dwobj *dwob, const Matrix *tmat) const;
+    void tessellate(const std::vector<Vec3>& verts, const dwmaterial *themat, 
+          GLUtesselator *ts, _dwobj *dwob, const RefMatrix *tmat) const;
+          
     void link(const int idop, const _face *f2, const int idop2,const std::vector<Vec3> verts, const dwmaterial *themat) const; // to join up opposed faces of a hole
     inline const int getidx(int i) const { return idx[i];}
 private:
@@ -354,17 +355,14 @@ private:
 
 class prims {
 public:
-    prims() { nbegin=0; // primlengs=NULL; gsidx=NULL;nrmidx=NULL;
-     //   txidx=NULL;nrms=NULL;txcoords=NULL;
-     //   nload=0; nff=0; curmode=0;
+    prims() {
+        nbegin=0;
         vertices = new osg::Vec3Array;
         normals = new osg::Vec3Array;
         txc = new osg::Vec3Array;
         txcoords=new osg::Vec3Array; // new Vec2[6*nfnvf]; // one texture coord per vertex
-        tmat=NULL;
     }
-    ~prims() {    /*delete [] primlengs; delete [] nrms;
-        delete [] gsidx; delete [] nrmidx; delete [] txcoords;*/
+    ~prims() {
     }
     void addv(avertex *pos) { // tessellation callback
         vertices->push_back(osg::Vec3(pos->pos[0],pos->pos[1],pos->pos[2]));
@@ -440,32 +438,32 @@ public:
         drw=new osg::DrawArrays(osg::PrimitiveSet::QUADS,n1,4);
         gset->addPrimitiveSet(drw);
     }
-    void tessellate(_face &fc, const std::vector<Vec3> verts, const dwmaterial *themat,GLUtesselator* ts, _dwobj *dwob) 
+    void tessellate(_face &fc, const std::vector<Vec3>& verts, const dwmaterial *themat,GLUtesselator* ts, _dwobj *dwob) 
     {    // generates a set of primitives all of one type (eg tris, qstrip trifan...)
         fc.setNBegin(vertices->size());
-        fc.tessellate(verts, themat, ts, dwob, tmat);
+        fc.tessellate(verts, themat, ts, dwob, tmat.get());
     }
     void buildGeometry() { // at end of all faces, add collection of vertices to geometry
         gset->setNormalBinding(osg::Geometry::BIND_PER_VERTEX); //BIND_PERPRIM); //
-        gset->setNormalArray(normals);
-        gset->setTexCoordArray(0,txcoords);
-        gset->setVertexArray(vertices); // setCoords( vts, nusidx );
+        gset->setNormalArray(normals.get());
+        gset->setTexCoordArray(0,txcoords.get());
+        gset->setVertexArray(vertices.get()); // setCoords( vts, nusidx );
     }
     void setGeometry(osg::Geometry *gs) {
         gset=gs;
     }
-    void settmat(const Matrix *mx) {
+    void settmat(const RefMatrix *mx) {
         tmat= mx;
     }
 private:
-    osg::Geometry *gset;
-    osg::Vec3Array* vertices;
-    osg::Vec3Array* normals;
-    osg::Vec3Array* txc;
-    osg::Vec3Array* txcoords;
+    osg::ref_ptr<osg::Geometry> gset;
+    osg::ref_ptr<osg::Vec3Array> vertices;
+    osg::ref_ptr<osg::Vec3Array> normals;
+    osg::ref_ptr<osg::Vec3Array> txc;
+    osg::ref_ptr<osg::Vec3Array> txcoords;
     GLenum primType;
     int nbegin; // vertex indices for current primitive
-    const Matrix *tmat; // local texture matrix, or may be NULL for default mapping
+    osg::ref_ptr<const RefMatrix> tmat; // local texture matrix, or may be NULL for default mapping
 };
 
 static prims *prd=NULL; // OK not nice to have a static but the OpenGL Tessellator etc wants to be able to refer
@@ -526,7 +524,7 @@ public:
         nopens=nfaceverts=0; fc1=fc2=NULL; colour[0]=colour[1]=colour[2]=colour[3]=1;
     } 
     ~_dwobj() {/*delete verts; delete faces;delete openings;*/
-        delete fc1;delete fc2;
+        delete [] fc1;delete [] fc2;
     } 
     int readOpenings(FILE *fp, const int nexpected)
     { // read up to nexpected openings, each opening may have a number of vertices
@@ -631,7 +629,7 @@ public:
             }
             nverts++;
         }
-//    osg::notify(osg::NOTICE) << nverts<<" inp "<<verts[nverts-1].x()<<
+//    OSG_NOTICE << nverts<<" inp "<<verts[nverts-1].x()<<
 //        " "<<verts[nverts-1].y()<<" "<<verts[nverts-1].z()<<" "<<verts.size()<< std::endl;
 
         return nverts;
@@ -643,7 +641,7 @@ public:
         return nverts-1;
     }
     void settmat(const Matrix& mx) {
-        tmat= new Matrix(mx);
+        tmat= new RefMatrix(mx);
     }
     void makeuv(Vec2 &uv, const double pos[]) {
         Vec3 p;
@@ -653,7 +651,7 @@ public:
         uv[0]=txc[0];
         uv[1]=txc[1];
     }
-    inline void setmx(Matrix *m) { mx=m;}
+    inline void setmx(RefMatrix *m) { mx=m;}
 private:
     Vec4 colour;
     std::vector<Vec3> verts;
@@ -665,18 +663,18 @@ private:
     _dwedge *edges;
     int *openings;
     unsigned short *fc1, *fc2; // openings[i] is in faces[fc1[i]] to faces[fc2[i]]
-    Matrix *tmat;
-    Matrix *mx; // current uvw transform for currently tessealting face
+    osg::ref_ptr<RefMatrix> tmat;
+    osg::ref_ptr<RefMatrix> mx; // current uvw transform for currently tessealting face
 };
 
-void _face::tessellate(const std::vector<Vec3> verts, const dwmaterial *themat, 
-               GLUtesselator *ts, _dwobj *dwob, const Matrix * /*tmat*/) const {
+void _face::tessellate(const std::vector<Vec3>& verts, const dwmaterial *themat, 
+               GLUtesselator *ts, _dwobj *dwob, const RefMatrix * /*tmat*/) const {
     int nvall=getallverts();
     int nused=0;
     avertex *poses=new avertex[2*nvall]; // passed to Tessellator to redraw
     Matrix mx; // texture matrix transform to plane
     settrans(mx, nrm, verts,themat);
-    dwob->setmx(&mx); // may be used by combine callback to define txcoord
+    dwob->setmx(new RefMatrix(mx)); // may be used by combine callback to define txcoord
     gluTessBeginPolygon(ts, dwob); 
     gluTessBeginContour(ts); /**/
     for (int j=0; j<nv; j++) {
@@ -763,7 +761,7 @@ void _dwobj::buildDrawable(Group *grp, const osgDB::ReaderWriter::Options *optio
             // for Geometry we dont need to collect prim types individually
             //     prd.setmode(nvf , nfnvf); // filter out only this type of tessellated face
             prd=new prims;
-            prd->settmat(tmat);
+            prd->settmat(tmat.get());
             osg::Geometry *gset = new osg::Geometry;
             prd->setGeometry(gset);
             StateSet *dstate=themat->make(options);            
@@ -821,9 +819,9 @@ class ReaderWriterDW : public osgDB::ReaderWriter
 
             char buff[256];
 
-            notify(INFO)<<   "ReaderWriterDW::readNode( "<<fileName.c_str()<<" )\n";
+            OSG_INFO<<   "ReaderWriterDW::readNode( "<<fileName.c_str()<<" )\n";
 #ifdef _MSC_VER
-            notify(osg::NOTICE)<<   "MS Visual C++ version "<<_MSC_VER<<"\n";
+            OSG_INFO<<   "MS Visual C++ version "<<_MSC_VER<<"\n";
 #endif
 
             FILE *fp;
@@ -858,33 +856,33 @@ class ReaderWriterDW : public osgDB::ReaderWriter
                     } else if (strncmp(buff, "CurrPhase:",10)==0) {
                     } else if (strncmp(buff, "numPhases:",10)==0) {
                     } else if (strncmp(buff, "Opacity:",8)==0) {
-                        float opacity=atof(buff+8);
+                        float opacity=osg::asciiToFloat(buff+8);
                         matpalet[nmat].setopacity(opacity);
                     } else if (strncmp(buff, "FallOff:",8)==0) {
-                        float fo=atof(buff+8);
+                        float fo=osg::asciiToFloat(buff+8);
                         matpalet[nmat].setFallOff(fo);
                     } else if (strncmp(buff, "InnerHalfAngle:",15)==0) {
-                        float ha=atof(buff+15);
+                        float ha=osg::asciiToFloat(buff+15);
                         matpalet[nmat].setHalfAngleIn(ha);
                     } else if (strncmp(buff, "OuterHalfAngle:",15)==0) {
-                        float ha=atof(buff+15);
+                        float ha=osg::asciiToFloat(buff+15);
                         matpalet[nmat].setHalfAngleOut(ha);
                     } else if (strncmp(buff, "Brightness:",11)==0) {
-                        float br=atof(buff+11);
+                        float br=osg::asciiToFloat(buff+11);
                         matpalet[nmat].setBright(br);
                     } else if (strncmp(buff, "Attentuation:",13)==0) { // oops - they can't spell
                         matpalet[nmat].setAtten(buff+13);
                     } else if (strncmp(buff, "MaterialType:",13)==0) {
                         matpalet[nmat].setType(buff+14);
                     } else if (strncmp(buff, "SpecularReflectivity:",21)==0) {
-                        float spec=atof(buff+21);
+                        float spec=osg::asciiToFloat(buff+21);
                         matpalet[nmat].setspecular(spec);
                     } else if (strncmp(buff, "SmoothnessExponent:",19)==0) {
-                        float spec=atof(buff+19);
+                        float spec=osg::asciiToFloat(buff+19);
                         matpalet[nmat].setspecexp(spec*128.0f/100.0f); // convert to 0-128 range from percent
                     } else if (strncmp(buff, "TextureWidthAndHeight:",22)==0) {
                         char *ct=strchr(buff+22,',');
-                        float repx=atof(buff+23), repy=atof(ct+1);
+                        float repx=osg::asciiToFloat(buff+23), repy=osg::asciiToFloat(ct+1);
                         matpalet[nmat].settxrep(repx, repy);
                     } else if (strncmp(buff, "PictureFile:",12)==0) {
                         char *end=strchr(buff,'\n'); // end of line
