@@ -50,6 +50,15 @@ Vec3d scene::addNormal(Vec3d v)
     // to do: vertices are not always listed in order. find why.
     return v;
 }
+void 
+scene::addPoint(const std::string & l, unsigned short color, Vec3d & s) 
+{
+    dxfLayer* layer = _layerTable->findOrCreateLayer(l);
+    if (layer->getFrozen()) return;
+    sceneLayer* ly = findOrCreateSceneLayer(l);
+    Vec3d a(addVertex(s));
+    ly->_points[correctedColorIndex(l, color)].push_back(a);    
+}
 
 void 
 scene::addLine(const std::string & l, unsigned short color, Vec3d & s, Vec3d & e) 
@@ -156,14 +165,11 @@ void scene::addQuads(const std::string & l, unsigned short color, std::vector<Ve
                 n.normalize();
                 short cindex = correctedColorIndex(l, color);
                 ly->_quadnorms[cindex].push_back( n );
-                MapVList mvl = ly->_quads;
-                VList vl = mvl[cindex];
+                VList& vl = ly->_quads[cindex];
                 vl.push_back(addVertex(*a));
                 vl.push_back(addVertex(*b));
                 vl.push_back(addVertex(*c));
                 vl.push_back(addVertex(*d));
-                mvl[cindex] = vl;
-                ly->_quads = mvl;
             }
     }
 }    
