@@ -16,21 +16,56 @@
 #include <osgAnimation/Skeleton>
 #include <osgAnimation/StackedMatrixElement>
 #include <osgAnimation/StackedQuaternionElement>
+#include <osgAnimation/StackedRotateAxisElement>
 #include <osgAnimation/StackedScaleElement>
 #include <osgAnimation/StackedTranslateElement>
 #include <osgAnimation/UpdateBone>
 
 #if defined(_MSC_VER)
     #pragma warning( disable : 4505 )
+    #pragma warning( default : 4996 )
 #endif
 #include <fbxsdk.h>
 
-#include "fbxRAnimation.h"
-#include "fbxRCamera.h"
-#include "fbxRLight.h"
-#include "fbxRMesh.h"
-#include "fbxRNode.h"
-#include "fbxMaterialToOsgStateSet.h"
+#include "fbxReader.h"
+
+bool isAnimated(KFbxProperty& prop, KFbxScene& fbxScene)
+{
+    for (int i = 0; i < fbxScene.GetSrcObjectCount(FBX_TYPE(KFbxAnimStack)); ++i)
+    {
+        KFbxAnimStack* pAnimStack = KFbxCast<KFbxAnimStack>(fbxScene.GetSrcObject(FBX_TYPE(KFbxAnimStack), i));
+
+        const int nbAnimLayers = pAnimStack->GetMemberCount(FBX_TYPE(KFbxAnimLayer));
+        for (int j = 0; j < nbAnimLayers; j++)
+        {
+            KFbxAnimLayer* pAnimLayer = pAnimStack->GetMember(FBX_TYPE(KFbxAnimLayer), j);
+            if (prop.GetCurveNode(pAnimLayer, false))
+            {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+bool isAnimated(KFbxProperty& prop, const char* channel, KFbxScene& fbxScene)
+{
+    for (int i = 0; i < fbxScene.GetSrcObjectCount(FBX_TYPE(KFbxAnimStack)); ++i)
+    {
+        KFbxAnimStack* pAnimStack = KFbxCast<KFbxAnimStack>(fbxScene.GetSrcObject(FBX_TYPE(KFbxAnimStack), i));
+
+        const int nbAnimLayers = pAnimStack->GetMemberCount(FBX_TYPE(KFbxAnimLayer));
+        for (int j = 0; j < nbAnimLayers; j++)
+        {
+            KFbxAnimLayer* pAnimLayer = pAnimStack->GetMember(FBX_TYPE(KFbxAnimLayer), j);
+            if (prop.GetCurve<KFbxAnimCurve>(pAnimLayer, channel, false))
+            {
+                return true;
+            }
+        }
+    }
+    return false;
+}
 
 osg::Quat makeQuat(const fbxDouble3& degrees, ERotationOrder fbxRotOrder)
 {
@@ -78,7 +113,7 @@ osg::Quat makeQuat(const fbxDouble3& degrees, ERotationOrder fbxRotOrder)
             return quat;
         }
     default:
-        osg::notify(osg::WARN) << "Invalid FBX rotation mode." << std::endl;
+        OSG_WARN << "Invalid FBX rotation mode." << std::endl;
         return osg::Quat();
     }
 }
@@ -148,7 +183,8 @@ void makeLocalMatrix(const KFbxNode* pNode, osg::Matrix& m)
 
 void readTranslationElement(KFbxTypedProperty<fbxDouble3>& prop,
                             osgAnimation::UpdateMatrixTransform* pUpdate,
-                            osg::Matrix& staticTransform)
+                            osg::Matrix& staticTransform,
+                            KFbxScene& fbxScene)
 {
     fbxDouble3 fbxPropValue = prop.Get();
     osg::Vec3d val(
@@ -156,9 +192,7 @@ void readTranslationElement(KFbxTypedProperty<fbxDouble3>& prop,
         fbxPropValue[1],
         fbxPropValue[2]);
 
-    if (prop.GetKFCurve(KFCURVENODE_T_X) ||
-        prop.GetKFCurve(KFCURVENODE_T_Y) ||
-        prop.GetKFCurve(KFCURVENODE_T_Z))
+    if (isAnimated(prop, fbxScene))
     {
         if (!staticTransform.isIdentity())
         {
@@ -173,33 +207,94 @@ void readTranslationElement(KFbxTypedProperty<fbxDouble3>& prop,
     }
 }
 
+void getRotationOrder(ERotationOrder fbxRotOrder, int order[/*3*/])
+{
+    switch (fbxRotOrder)
+    {
+    case eEULER_XZY:
+        order[0] = 0; order[1] = 2; order[2] = 1;
+        break;
+    case eEULER_YZX:
+        order[0] = 1; order[1] = 2; order[2] = 0;
+        break;
+    case eEULER_YXZ:
+        order[0] = 1; order[1] = 0; order[2] = 2;
+        break;
+    case eEULER_ZXY:
+        order[0] = 2; order[1] = 0; order[2] = 1;
+        break;
+    case eEULER_ZYX:
+        order[0] = 2; order[1] = 1; order[2] = 0;
+        break;
+    default:
+        order[0] = 0; order[1] = 1; order[2] = 2;
+    }
+}
+
 void readRotationElement(KFbxTypedProperty<fbxDouble3>& prop,
                          ERotationOrder fbxRotOrder,
+                         bool quatInterpolate,
                          osgAnimation::UpdateMatrixTransform* pUpdate,
-                         osg::Matrix& staticTransform)
+                         osg::Matrix& staticTransform,
+                         KFbxScene& fbxScene)
 {
-    osg::Quat quat = makeQuat(prop.Get(), fbxRotOrder);
-
-    if (prop.GetKFCurve(KFCURVENODE_R_X) ||
-        prop.GetKFCurve(KFCURVENODE_R_Y) ||
-        prop.GetKFCurve(KFCURVENODE_R_Z))
+    if (isAnimated(prop, fbxScene))
     {
-        if (!staticTransform.isIdentity())
+        if (quatInterpolate)
         {
-            pUpdate->getStackedTransforms().push_back(new osgAnimation::StackedMatrixElement(staticTransform));
-            staticTransform.makeIdentity();
+            if (!staticTransform.isIdentity())
+            {
+                pUpdate->getStackedTransforms().push_back(
+                    new osgAnimation::StackedMatrixElement(staticTransform));
+                staticTransform.makeIdentity();
+            }
+            pUpdate->getStackedTransforms().push_back(new osgAnimation::StackedQuaternionElement(
+                "quaternion", makeQuat(prop.Get(), fbxRotOrder)));
+        }
+        else
+        {
+            const char* curveNames[3] = {KFCURVENODE_R_X, KFCURVENODE_R_Y, KFCURVENODE_R_Z};
+            osg::Vec3 axes[3] = {osg::Vec3(1,0,0), osg::Vec3(0,1,0), osg::Vec3(0,0,1)};
+
+            fbxDouble3 fbxPropValue = prop.Get();
+            fbxPropValue[0] = osg::DegreesToRadians(fbxPropValue[0]);
+            fbxPropValue[1] = osg::DegreesToRadians(fbxPropValue[1]);
+            fbxPropValue[2] = osg::DegreesToRadians(fbxPropValue[2]);
+
+            int order[3] = {0, 1, 2};
+            getRotationOrder(fbxRotOrder, order);
+
+            for (int i = 0; i < 3; ++i)
+            {
+                int j = order[2-i];
+                if (isAnimated(prop, curveNames[j], fbxScene))
+                {
+                    if (!staticTransform.isIdentity())
+                    {
+                        pUpdate->getStackedTransforms().push_back(new osgAnimation::StackedMatrixElement(staticTransform));
+                        staticTransform.makeIdentity();
+                    }
+
+                    pUpdate->getStackedTransforms().push_back(new osgAnimation::StackedRotateAxisElement(
+                        std::string("rotate") + curveNames[j], axes[j], fbxPropValue[j]));
+                }
+                else
+                {
+                    staticTransform.preMultRotate(osg::Quat(fbxPropValue[j], axes[j]));
+                }
+            }
         }
-        pUpdate->getStackedTransforms().push_back(new osgAnimation::StackedQuaternionElement("quaternion", quat));
     }
     else
     {
-        staticTransform.preMultRotate(quat);
+        staticTransform.preMultRotate(makeQuat(prop.Get(), fbxRotOrder));
     }
 }
 
 void readScaleElement(KFbxTypedProperty<fbxDouble3>& prop,
                       osgAnimation::UpdateMatrixTransform* pUpdate,
-                      osg::Matrix& staticTransform)
+                      osg::Matrix& staticTransform,
+                      KFbxScene& fbxScene)
 {
     fbxDouble3 fbxPropValue = prop.Get();
     osg::Vec3d val(
@@ -207,9 +302,7 @@ void readScaleElement(KFbxTypedProperty<fbxDouble3>& prop,
         fbxPropValue[1],
         fbxPropValue[2]);
 
-    if (prop.GetKFCurve(KFCURVENODE_S_X) ||
-        prop.GetKFCurve(KFCURVENODE_S_Y) ||
-        prop.GetKFCurve(KFCURVENODE_S_Z))
+    if (isAnimated(prop, fbxScene))
     {
         if (!staticTransform.isIdentity())
         {
@@ -224,11 +317,11 @@ void readScaleElement(KFbxTypedProperty<fbxDouble3>& prop,
     }
 }
 
-void readUpdateMatrixTransform(osgAnimation::UpdateMatrixTransform* pUpdate, KFbxNode* pNode)
+void readUpdateMatrixTransform(osgAnimation::UpdateMatrixTransform* pUpdate, KFbxNode* pNode, KFbxScene& fbxScene)
 {
     osg::Matrix staticTransform;
 
-    readTranslationElement(pNode->LclTranslation, pUpdate, staticTransform);
+    readTranslationElement(pNode->LclTranslation, pUpdate, staticTransform, fbxScene);
 
     fbxDouble3 fbxRotOffset = pNode->RotationOffset.Get();
     fbxDouble3 fbxRotPiv = pNode->RotationPivot.Get();
@@ -249,7 +342,9 @@ void readUpdateMatrixTransform(osgAnimation::UpdateMatrixTransform* pUpdate, KFb
         staticTransform.preMultRotate(makeQuat(pNode->PreRotation.Get(), fbxRotOrder));
     }
 
-    readRotationElement(pNode->LclRotation, fbxRotOrder, pUpdate, staticTransform);
+    readRotationElement(pNode->LclRotation, fbxRotOrder,
+        pNode->QuaternionInterpolate.IsValid() && pNode->QuaternionInterpolate.Get(),
+        pUpdate, staticTransform, fbxScene);
 
     if (rotationActive)
     {
@@ -263,7 +358,7 @@ void readUpdateMatrixTransform(osgAnimation::UpdateMatrixTransform* pUpdate, KFb
         fbxSclOffset[1] + fbxSclPiv[1] - fbxRotPiv[1],
         fbxSclOffset[2] + fbxSclPiv[2] - fbxRotPiv[2]));
 
-    readScaleElement(pNode->LclScaling, pUpdate, staticTransform);
+    readScaleElement(pNode->LclScaling, pUpdate, staticTransform, fbxScene);
 
     staticTransform.preMultTranslate(osg::Vec3d(
         -fbxSclPiv[0],
@@ -278,7 +373,7 @@ void readUpdateMatrixTransform(osgAnimation::UpdateMatrixTransform* pUpdate, KFb
 
 osg::Group* createGroupNode(KFbxSdkManager& pSdkManager, KFbxNode* pNode,
     const std::string& animName, const osg::Matrix& localMatrix, bool bNeedSkeleton,
-    std::map<KFbxNode*, osg::Node*>& nodeMap)
+    std::map<KFbxNode*, osg::Node*>& nodeMap, KFbxScene& fbxScene)
 {
     if (bNeedSkeleton)
     {
@@ -286,7 +381,7 @@ osg::Group* createGroupNode(KFbxSdkManager& pSdkManager, KFbxNode* pNode,
         osgBone->setDataVariance(osg::Object::DYNAMIC);
         osgBone->setName(pNode->GetName());
         osgAnimation::UpdateBone* pUpdate = new osgAnimation::UpdateBone(animName);
-        readUpdateMatrixTransform(pUpdate, pNode);
+        readUpdateMatrixTransform(pUpdate, pNode, fbxScene);
         osgBone->setUpdateCallback(pUpdate);
 
         nodeMap.insert(std::pair<KFbxNode*, osg::Node*>(pNode, osgBone));
@@ -309,7 +404,7 @@ osg::Group* createGroupNode(KFbxSdkManager& pSdkManager, KFbxNode* pNode,
         if (bAnimated)
         {
             osgAnimation::UpdateMatrixTransform* pUpdate = new osgAnimation::UpdateMatrixTransform(animName);
-            readUpdateMatrixTransform(pUpdate, pNode);
+            readUpdateMatrixTransform(pUpdate, pNode, fbxScene);
             pTransform->setUpdateCallback(pUpdate);
         }
 
@@ -317,24 +412,29 @@ osg::Group* createGroupNode(KFbxSdkManager& pSdkManager, KFbxNode* pNode,
     }
 }
 
-osgDB::ReaderWriter::ReadResult readFbxNode(
-    KFbxSdkManager& pSdkManager, KFbxNode* pNode,
-    osg::ref_ptr<osgAnimation::AnimationManagerBase>& pAnimationManager,
-    bool& bIsBone, int& nLightCount,
-    FbxMaterialToOsgStateSet& fbxMaterialToOsgStateSet,
-    std::map<KFbxNode*, osg::Node*>& nodeMap,
-    BindMatrixMap& boneBindMatrices,
-    const std::set<const KFbxNode*>& fbxSkeletons,
-    std::map<KFbxNode*, osgAnimation::Skeleton*>& skeletonMap,
-    const osgDB::ReaderWriter::Options* options)
+osgDB::ReaderWriter::ReadResult OsgFbxReader::readFbxNode(
+    KFbxNode* pNode,
+    bool& bIsBone, int& nLightCount)
 {
     if (KFbxNodeAttribute* lNodeAttribute = pNode->GetNodeAttribute())
     {
-        if (lNodeAttribute->GetAttributeType() == KFbxNodeAttribute::eNURB ||
-            lNodeAttribute->GetAttributeType() == KFbxNodeAttribute::ePATCH)
+        KFbxNodeAttribute::EAttributeType attrType = lNodeAttribute->GetAttributeType();
+        switch (attrType)
         {
-            KFbxGeometryConverter lConverter(&pSdkManager);
-            lConverter.TriangulateInPlace(pNode);
+        case KFbxNodeAttribute::eNURB:
+        case KFbxNodeAttribute::ePATCH:
+        case KFbxNodeAttribute::eNURBS_CURVE:
+        case KFbxNodeAttribute::eNURBS_SURFACE:
+            {
+                KFbxGeometryConverter lConverter(&pSdkManager);
+                if (!lConverter.TriangulateInPlace(pNode))
+                {
+                    OSG_WARN << "Unable to triangulate FBX NURBS " << pNode->GetName() << std::endl;
+                }
+            }
+            break;
+        default:
+            break;
         }
     }
 
@@ -381,9 +481,7 @@ osgDB::ReaderWriter::ReadResult readFbxNode(
 
         bool bChildIsBone = false;
         osgDB::ReaderWriter::ReadResult childResult = readFbxNode(
-            pSdkManager, pChildNode, pAnimationManager,
-            bChildIsBone, nLightCount, fbxMaterialToOsgStateSet, nodeMap,
-            boneBindMatrices, fbxSkeletons, skeletonMap, options);
+            pChildNode, bChildIsBone, nLightCount);
         if (childResult.error())
         {
             return childResult;
@@ -402,7 +500,7 @@ osgDB::ReaderWriter::ReadResult readFbxNode(
         }
     }
 
-    std::string animName = readFbxAnimation(pNode, pAnimationManager, pNode->GetName());
+    std::string animName = readFbxAnimation(pNode, pNode->GetName());
 
     osg::Matrix localMatrix;
     makeLocalMatrix(pNode, localMatrix);
@@ -414,25 +512,10 @@ osgDB::ReaderWriter::ReadResult readFbxNode(
 
     switch (lAttributeType)
     {
-    case KFbxNodeAttribute::eUNIDENTIFIED:
-        if (bLocalMatrixIdentity && children.size() + skeletal.size() == 1)
-        {
-            if (children.size() == 1)
-            {
-                return osgDB::ReaderWriter::ReadResult(children.front().get());
-            }
-            else
-            {
-                return osgDB::ReaderWriter::ReadResult(skeletal.front().get());
-            }
-        }
-        break;
     case KFbxNodeAttribute::eMESH:
         {
             size_t bindMatrixCount = boneBindMatrices.size();
-            osgDB::ReaderWriter::ReadResult meshRes = readFbxMesh(pSdkManager,
-                pNode, pAnimationManager, stateSetList, boneBindMatrices,
-                fbxSkeletons, skeletonMap);
+            osgDB::ReaderWriter::ReadResult meshRes = readFbxMesh(pNode, stateSetList);
             if (meshRes.error())
             {
                 return meshRes;
@@ -485,6 +568,8 @@ osgDB::ReaderWriter::ReadResult readFbxNode(
             }
         }
         break;
+    default:
+        break;
     }
 
     if (bEmpty)
@@ -492,7 +577,7 @@ osgDB::ReaderWriter::ReadResult readFbxNode(
         osgDB::ReaderWriter::ReadResult(0);
     }
 
-    if (!osgGroup) osgGroup = createGroupNode(pSdkManager, pNode, animName, localMatrix, bIsBone, nodeMap);
+    if (!osgGroup) osgGroup = createGroupNode(pSdkManager, pNode, animName, localMatrix, bIsBone, nodeMap, fbxScene);
 
     osg::Group* pAddChildrenTo = osgGroup.get();
     if (bCreateSkeleton)
