@@ -13,7 +13,7 @@
 
 #include "FreeTypeLibrary.h"
 #include <osg/Notify>
-#include <ft2build.h>
+//#include <ft2build.h>
 
 //#define PRINT_OUT_FONT_DETAILS
 #ifdef PRINT_OUT_FONT_DETAILS
@@ -24,10 +24,11 @@
 
 FreeTypeLibrary::FreeTypeLibrary()
 {
+    OSG_INFO << "FreeTypeLibrary::FreeTypeLibrary()" << std::endl; 
     FT_Error error = FT_Init_FreeType( &_ftlibrary );
     if (error)
     {
-        osg::notify(osg::WARN)<<"Warning: an error occurred during FT_Init_FreeType(..) initialisation, error code = "<<std::hex<<error<<std::dec<<std::endl;
+        OSG_WARN<<"Warning: an error occurred during FT_Init_FreeType(..) initialisation, error code = "<<std::hex<<error<<std::dec<<std::endl;
     }
 
 }
@@ -44,7 +45,8 @@ FreeTypeLibrary::~FreeTypeLibrary()
         FreeTypeFont* fontImplementation = *_fontImplementationSet.begin();
         _fontImplementationSet.erase(_fontImplementationSet.begin());
         osgText::Font* font = fontImplementation->_facade;
-        font->setImplementation(0);
+        if (font) font->setImplementation(0);
+        else fontImplementation->_facade = 0;
     }
     
     FT_Done_FreeType( _ftlibrary);
@@ -56,27 +58,27 @@ FreeTypeLibrary* FreeTypeLibrary::instance()
     return s_library.get();
 }
 
-osgText::Font* FreeTypeLibrary::getFont(const std::string& fontfile,unsigned int index, unsigned int flags)
+bool FreeTypeLibrary::getFace(const std::string& fontfile,unsigned int index, FT_Face & face)
 {
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(getMutex());
 
-    FT_Face face; /* handle to face object */
     FT_Error error = FT_New_Face( _ftlibrary, fontfile.c_str(), index, &face );
     if (error == FT_Err_Unknown_File_Format)
     {
-        osg::notify(osg::WARN)<<" .... the font file could be opened and read, but it appears"<<std::endl;
-        osg::notify(osg::WARN)<<" .... that its font format is unsupported"<<std::endl;
-        return 0;
+        OSG_WARN<<" .... the font file could be opened and read, but it appears"<<std::endl;
+        OSG_WARN<<" .... that its font format is unsupported"<<std::endl;
+        return false;
     }
     else if (error)
     {
-        osg::notify(osg::WARN)<<" .... another error code means that the font file could notd"<<std::endl;
-        osg::notify(osg::WARN)<<" .... be opened, read or simply that it is broken..d"<<std::endl;
-        return 0;
+        OSG_WARN<<" .... another error code means that the font file could not"<<std::endl;
+        OSG_WARN<<" .... be opened, read or simply that it is broken.."<<std::endl;
+        return false;
     }
     
 #ifdef PRINT_OUT_FONT_DETAILS
 
-    osg::notify(osg::NOTICE)<<"Face"<<face<<std::endl;
+    OSG_NOTICE<<"Face"<<face<<std::endl;
     unsigned int count = FT_Get_Sfnt_Name_Count(face);
     for(unsigned int i=0; i<count; ++i)
     {
@@ -85,10 +87,10 @@ osgText::Font* FreeTypeLibrary::getFont(const std::string& fontfile,unsigned int
         
         std::string name((char*)names.string, (char*)names.string + names.string_len);
         
-        osg::notify(osg::NOTICE)<<"names "<<name<<std::endl;
+        OSG_NOTICE<<"names "<<name<<std::endl;
     }
 
-    osg::notify(osg::NOTICE)<<std::endl;
+    OSG_NOTICE<<std::endl;
 #endif
 
     //
@@ -96,18 +98,13 @@ osgText::Font* FreeTypeLibrary::getFont(const std::string& fontfile,unsigned int
     //
     verifyCharacterMap(face);
     
-    FreeTypeFont* fontImp = new FreeTypeFont(fontfile,face,flags);
-    osgText::Font* font = new osgText::Font(fontImp);
-
-    _fontImplementationSet.insert(fontImp);
-    
-    return font;
-
+    return true;
 }
 
-osgText::Font* FreeTypeLibrary::getFont(std::istream& fontstream, unsigned int index, unsigned int flags)
+FT_Byte* FreeTypeLibrary::getFace(std::istream& fontstream, unsigned int index, FT_Face & face)
 {
-    FT_Face face;    /* handle to face object */
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(getMutex());
+
     FT_Open_Args args;
 
     std::streampos start = fontstream.tellg();
@@ -121,7 +118,8 @@ osgText::Font* FreeTypeLibrary::getFont(std::istream& fontstream, unsigned int i
     fontstream.read(reinterpret_cast<char*>(buffer), length);
     if (!fontstream || (static_cast<std::streampos>(fontstream.gcount()) != length))
     {
-        osg::notify(osg::WARN)<<" .... the font file could not be read from its stream"<<std::endl;
+        OSG_WARN<<" .... the font file could not be read from its stream"<<std::endl;
+        if (buffer) delete [] buffer;
         return 0;
     }
     args.flags = FT_OPEN_MEMORY;
@@ -132,22 +130,49 @@ osgText::Font* FreeTypeLibrary::getFont(std::istream& fontstream, unsigned int i
 
     if (error == FT_Err_Unknown_File_Format)
     {
-        osg::notify(osg::WARN)<<" .... the font file could be opened and read, but it appears"<<std::endl;
-        osg::notify(osg::WARN)<<" .... that its font format is unsupported"<<std::endl;
+        OSG_WARN<<" .... the font file could be opened and read, but it appears"<<std::endl;
+        OSG_WARN<<" .... that its font format is unsupported"<<std::endl;
         return 0;
     }
     else if (error)
     {
-        osg::notify(osg::WARN)<<" .... another error code means that the font file could not"<<std::endl;
-        osg::notify(osg::WARN)<<" .... be opened, read or simply that it is broken..."<<std::endl;
+        OSG_WARN<<" .... another error code means that the font file could not"<<std::endl;
+        OSG_WARN<<" .... be opened, read or simply that it is broken..."<<std::endl;
         return 0;
     }
-    
+
     //
     // GT: Fix to handle symbol fonts in MS Windows
     //
     verifyCharacterMap(face);
     
+    return buffer;
+}
+
+
+osgText::Font* FreeTypeLibrary::getFont(const std::string& fontfile, unsigned int index, unsigned int flags)
+{
+    FT_Face face;
+    if (getFace(fontfile, index, face) == false) return (0);
+
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(getMutex());
+    
+    FreeTypeFont* fontImp = new FreeTypeFont(fontfile,face,flags);
+    osgText::Font* font = new osgText::Font(fontImp);
+
+    _fontImplementationSet.insert(fontImp);
+    
+    return font;
+}
+osgText::Font* FreeTypeLibrary::getFont(std::istream& fontstream, unsigned int index, unsigned int flags)
+{
+    FT_Face face = 0;
+    FT_Byte * buffer = getFace(fontstream, index, face);
+    if (face == 0) return (0);
+
+    
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(getMutex());
+
     FreeTypeFont* fontImp = new FreeTypeFont(buffer,face,flags);
     osgText::Font* font = new osgText::Font(fontImp);
     
