@@ -32,6 +32,8 @@
 #include <memory.h>
 #include <stdio.h>
 
+#include <osgDB/FileUtils>
+
 typedef unsigned char RGBE[4];
 #define R            0
 #define G            1
@@ -49,14 +51,17 @@ static bool oldDecrunch(RGBE *scanline, int len, FILE *file);
 bool HDRLoader::isHDRFile(const char *_fileName)
 {
     FILE *file;
-    file = fopen(_fileName, "rb");
+    file = osgDB::fopen(_fileName, "rb");
     if (!file)
         return false;
 
     char str[10];
-    fread(str, 10, 1, file);
+    size_t numRead = fread(str, 10, 1, file);
+    
     fclose(file);
 
+    if (numRead<1) return false;
+    
     if (memcmp(str, "#?RADIANCE", 10) && memcmp(str, "#?RGBE", 6))
         return false;
 
@@ -69,22 +74,31 @@ bool HDRLoader::load(const char *_fileName, const bool _rawRGBE, HDRLoaderResult
     char str[200];
     FILE *file;
 
-    file = fopen(_fileName, "rb");
+    file = osgDB::fopen(_fileName, "rb");
     if (!file)
         return false;
 
-    fread(str, 10, 1, file);
-    if (memcmp(str, "#?RADIANCE", 10)) {
+    size_t numRead = fread(str, 10, 1, file);
+
+    if (numRead<1) 
+    {
+        fclose(file);
+        return false;
+    }
+    
+    if (memcmp(str, "#?RADIANCE", 10))
+    {
         fseek(file, 0, SEEK_SET);
-        fread(str, 6, 1, file);
-        if (memcmp(str, "#?RGBE", 6)) {
+        numRead = fread(str, 6, 1, file);
+        if (numRead<1 || memcmp(str, "#?RGBE", 6))
+        {
             fclose(file);
             return false;
         }
     }
     fseek(file, 1, SEEK_CUR);
 
-    char cmd[2000];
+    //char cmd[2000];
     i = 0;
     char c = 0, oldc;
     while(true) {
@@ -92,7 +106,7 @@ bool HDRLoader::load(const char *_fileName, const bool _rawRGBE, HDRLoaderResult
         c = fgetc(file);
         if (c == 0xa && oldc == 0xa)
             break;
-        cmd[i++] = c;
+        //cmd[i++] = c;
     }
 
     char reso[2000];
