@@ -26,7 +26,7 @@ void AutoTransform::write(DataOutputStream* out){
         ((ive::Transform*)(trans))->write(out);
     }
     else
-        throw Exception("AutoTransform::write(): Could not cast this osg::AutoTransform to an osg::Transform.");
+        out_THROW_EXCEPTION("AutoTransform::write(): Could not cast this osg::AutoTransform to an osg::Transform.");
     // Write AutoTransform's properties.
 
     out->writeVec3(getPosition());
@@ -41,7 +41,7 @@ void AutoTransform::write(DataOutputStream* out){
     {
         out->writeFloat(getMinimumScale());
         out->writeFloat(getMaximumScale());
-        out->writeFloat(getAutoScaleTransistionWidthRatio());
+        out->writeFloat(getAutoScaleTransitionWidthRatio());
     }
 
     out->writeQuat(getRotation());
@@ -61,7 +61,7 @@ void AutoTransform::read(DataInputStream* in){
             ((ive::Transform*)(trans))->read(in);
         }
         else
-            throw Exception("AutoTransform::read(): Could not cast this osg::AutoTransform to an osg::Transform.");
+            in_THROW_EXCEPTION("AutoTransform::read(): Could not cast this osg::AutoTransform to an osg::Transform.");
         // Read AutoTransform's properties
 
         setPosition(in->readVec3());
@@ -76,7 +76,7 @@ void AutoTransform::read(DataInputStream* in){
         {
             setMinimumScale(in->readFloat());
             setMaximumScale(in->readFloat());
-            setAutoScaleTransistionWidthRatio(in->readFloat());
+            setAutoScaleTransitionWidthRatio(in->readFloat());
         }
 
         setRotation(in->readQuat());
@@ -85,6 +85,6 @@ void AutoTransform::read(DataInputStream* in){
 
     }
     else{
-        throw Exception("AutoTransform::read(): Expected AutoTransform identification.");
+        in_THROW_EXCEPTION("AutoTransform::read(): Expected AutoTransform identification.");
     }
 }
