@@ -5,6 +5,7 @@
 
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 #include <osgDB/Registry>
 
 using namespace osg;
@@ -13,6 +14,23 @@ using namespace osgDB;
 class ReaderWriterIVE : public ReaderWriter
 {
     public:
+    
+        ReaderWriterIVE()
+        {
+            supportsExtension("ive","OpenSceneGraph native binary format");
+
+            supportsOption("compressed","Export option, use zlib compression to compress the data in the .ive ");
+            supportsOption("noTexturesInIVEFile","Export option");
+            supportsOption("includeImageFileInIVEFile","Export option");
+            supportsOption("compressImageData","Export option");
+            supportsOption("inlineExternalReferencesInIVEFile","Export option");
+            supportsOption("noWriteExternalReferenceFiles","Export option");
+            supportsOption("useOriginalExternalReferences","Export option");
+            supportsOption("TerrainMaximumErrorToSizeRatio=value","Export option that controls error matric used to determine terrain HeightField storage precision.");
+            supportsOption("noLoadExternalReferenceFiles","Import option");
+            supportsOption("OutputTextureFiles","Write out the texture images to file");
+        }
+    
         virtual const char* className() const { return "IVE Reader/Writer"; }
 
         virtual bool acceptsExtension(const std::string& extension) const
@@ -34,8 +52,9 @@ class ReaderWriterIVE : public ReaderWriter
 
             // code for setting up the database path so that internally referenced files are searched for on relative paths.
             osg::ref_ptr<Options> local_opt = options ? static_cast<Options*>(options->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
-            local_opt->setDatabasePath(osgDB::getFilePath(fileName));
-            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            local_opt->getDatabasePathList().push_front(osgDB::getFilePath(fileName));
+
+            osgDB::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
             return readImage(istream, local_opt.get());
         }
         
@@ -49,9 +68,9 @@ class ReaderWriterIVE : public ReaderWriter
 
             // code for setting up the database path so that internally referenced file are searched for on relative paths. 
             osg::ref_ptr<Options> local_opt = options ? static_cast<Options*>(options->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
-            local_opt->setDatabasePath(osgDB::getFilePath(fileName));
-            
-            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            local_opt->getDatabasePathList().push_front(osgDB::getFilePath(fileName));
+
+            osgDB::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
             return readNode(istream,local_opt.get());
         }
         
@@ -62,35 +81,26 @@ class ReaderWriterIVE : public ReaderWriter
 
         virtual ReadResult readImage(std::istream& fin, const Options* options) const
         {
-            try{
-                ive::DataInputStream in(&fin);
-                in.setOptions(options);
-                return in.readImage(ive::IMAGE_INCLUDE_DATA);
-            }
-            catch(ive::Exception e)
+            ive::DataInputStream in(&fin, options);
+            if (in.getException()) 
             {
-                osg::notify(osg::NOTICE)<<"Error reading image: "<<e.getError()<<std::endl;
-                return ReadResult::FILE_NOT_HANDLED;
+                return in.getException()->getError();
             }
+
+            return in.readImage(ive::IMAGE_INCLUDE_DATA);
         }
-        
+
         virtual ReadResult readNode(std::istream& fin, const Options* options) const
         {
-            try{
-                // Create datainputstream.
-                ive::DataInputStream in(&fin);
-                in.setOptions(options);
-
-                return in.readNode();
-            }
-            catch(ive::Exception e)
+            // Create datainputstream.
+            ive::DataInputStream in(&fin, options);
+            if (in.getException()) 
             {
-                osg::notify(osg::NOTICE)<<"Error reading file: "<< e.getError()<<std::endl;
-                return ReadResult::FILE_NOT_HANDLED;
+                return in.getException()->getError();
             }
-        }
 
-        
+            return in.readNode();
+        }
 
         virtual WriteResult writeObject(const Object& object,const std::string& fileName, const osgDB::ReaderWriter::Options* options) const
         {
@@ -109,7 +119,10 @@ class ReaderWriterIVE : public ReaderWriter
             osg::ref_ptr<Options> local_opt = options ? static_cast<Options*>(options->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
             if(local_opt->getDatabasePathList().empty())
                 local_opt->setDatabasePath(osgDB::getFilePath(fileName));
-            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+
+            local_opt->setPluginStringData("filename",fileName);
+
+            osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
             if (!fout) return WriteResult::ERROR_IN_WRITING_FILE;
             WriteResult result = writeImage(image, fout, local_opt.get());
             fout.close();
@@ -126,7 +139,9 @@ class ReaderWriterIVE : public ReaderWriter
             if(local_opt->getDatabasePathList().empty())
                 local_opt->setDatabasePath(osgDB::getFilePath(fileName));
 
-            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            local_opt->setPluginStringData("filename",fileName);
+
+            osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
             if (!fout) return WriteResult::ERROR_IN_WRITING_FILE;
     
             WriteResult result = writeNode(node, fout, local_opt.get());
@@ -145,41 +160,31 @@ class ReaderWriterIVE : public ReaderWriter
 
         virtual WriteResult writeImage(const Image& image,std::ostream& fout, const osgDB::ReaderWriter::Options* options) const
         {
-            try
+            ive::DataOutputStream out(&fout, options);
+            out.writeImage(ive::IMAGE_INCLUDE_DATA, const_cast<osg::Image*>(&image));
+            if (fout.fail()) return WriteResult::ERROR_IN_WRITING_FILE;
+            if (out.getException()) 
             {
-                ive::DataOutputStream out(&fout);
-                out.setOptions(options);
-                out.writeImage(ive::IMAGE_INCLUDE_DATA, const_cast<osg::Image*>(&image));
-                if (fout.fail()) return WriteResult::ERROR_IN_WRITING_FILE;
-                return WriteResult::FILE_SAVED;
+                OSG_WARN<<"Error writing IVE image: "<< out.getException()->getError() << std::endl;
+                return WriteResult::FILE_NOT_HANDLED;
             }
-            catch(ive::Exception e)
-            {
-                osg::notify(osg::WARN)<<"Error writing IVE image: "<< e.getError() << std::endl;
-            }
-            return WriteResult::FILE_NOT_HANDLED;
+            return WriteResult::FILE_SAVED;
         }
 
         virtual WriteResult writeNode(const Node& node,std::ostream& fout, const osgDB::ReaderWriter::Options* options) const
         {
-            try
+            ive::DataOutputStream out(&fout, options);
+
+            out.writeNode(const_cast<osg::Node*>(&node));
+
+            if ( fout.fail() ) return WriteResult::ERROR_IN_WRITING_FILE;
+            if (out.getException()) 
             {
-                ive::DataOutputStream out(&fout);
-
-                out.setOptions(options);
-
-                out.writeNode(const_cast<osg::Node*>(&node));
-
-                if ( fout.fail() ) return WriteResult::ERROR_IN_WRITING_FILE;
-
-                return WriteResult::FILE_SAVED;
+                OSG_WARN<<"Error writing IVE image: "<< out.getException()->getError() << std::endl;
+                return WriteResult::FILE_NOT_HANDLED;
             }
-            catch(ive::Exception e)
-            {
-                osg::notify(osg::WARN)<<"Error writing IVE file: "<< e.getError() << std::endl;
-            }
-            return WriteResult::FILE_NOT_HANDLED;
 
+            return WriteResult::FILE_SAVED;
         }
 
 };
