@@ -16,7 +16,9 @@
 #include "Group.h"
 #include "Layer.h"
 
+#include <osgDB/Options>
 #include <osgTerrain/GeometryTechnique>
+#include <osgTerrain/Terrain>
 
 using namespace ive;
 
@@ -29,7 +31,12 @@ void TerrainTile::write(DataOutputStream* out)
     if(group)
         ((ive::Group*)(group))->write(out);
     else
-        throw Exception("Terrain::write(): Could not cast this osgTerrain::Terrain to an osg::Group.");
+        out_THROW_EXCEPTION("Terrain::write(): Could not cast this osgTerrain::Terrain to an osg::Group.");
+
+    if (out->getVersion() >= VERSION_0044)
+    {
+        out->writeInt(getBlendingPolicy());
+    }
 
     if (out->getVersion() >= VERSION_0026)
     {
@@ -73,7 +80,7 @@ void TerrainTile::read(DataInputStream* in)
 {
     // Peek on Terrain's identification.
     int id = in->peekInt();
-    if (id != IVETERRAINTILE) throw Exception("TerrainTile::read(): Expected Terrain identification.");
+    if (id != IVETERRAINTILE) in_THROW_EXCEPTION("TerrainTile::read(): Expected Terrain identification.");
 
     // Read Terrain's identification.
     id = in->readInt();
@@ -82,14 +89,22 @@ void TerrainTile::read(DataInputStream* in)
     if(group)
         ((ive::Group*)(group))->read(in);
     else
-        throw Exception("Terrain::read(): Could not cast this osgTerrain::Terrain to an osg::Group.");
+        in_THROW_EXCEPTION("Terrain::read(): Could not cast this osgTerrain::Terrain to an osg::Group.");
+
+    if (in->getVersion() >= VERSION_0044)
+    {
+        setBlendingPolicy(static_cast<osgTerrain::TerrainTile::BlendingPolicy>(in->readInt()));
+    }
 
     if (in->getVersion() >= VERSION_0026)
     {
+    
         int level = in->readInt();
         int x = in->readInt();
         int y = in->readInt();
         setTileID(osgTerrain::TileID(level,x,y));
+
+        // OSG_NOTICE<<"Read TileID("<<level<<", "<<x<<", "<<y<<")"<<std::endl;
     }
 
     if (in->getVersion() >= VERSION_0023)
@@ -119,6 +134,15 @@ void TerrainTile::read(DataInputStream* in)
         
     setTerrainTechnique(readTerrainTechnique(in));
 
+    if (in->getOptions())
+    {
+        osg::ref_ptr<osg::Node> node;
+        if (in->getOptions()->getTerrain().lock(node))
+        {
+            setTerrain(node->asTerrain());
+        }
+    }
+
     if (osgTerrain::TerrainTile::getTileLoadedCallback().valid()) 
         osgTerrain::TerrainTile::getTileLoadedCallback()->loaded(this, in->getOptions());
 }
@@ -140,7 +164,7 @@ osgTerrain::TerrainTechnique* TerrainTile::readTerrainTechnique(DataInputStream*
 {
     bool hasTechnique = in->readBool();
     if (!hasTechnique) return 0;
-    
+
     int id = in->readInt();
     if (id==IVEGEOMETRYTECHNIQUE)
     {
