@@ -38,7 +38,6 @@ extern "C" {
         jas_matrix_t *data[4];
         jas_seqent_t *d[4];
         jas_seqent_t v;
-        int linelen;
         int width, height;
 
         width = jas_image_cmptwidth(image, 0);
@@ -64,7 +63,6 @@ extern "C" {
                 }
                 d[cmptno] = jas_matrix_getref(data[cmptno], 0, 0);
             }
-            linelen = 0;
             for (x = 0; x < width; ++x) {
                 for (cmptno = 0; cmptno < numcmpts; ++cmptno) {
                     v = *d[cmptno];
@@ -176,7 +174,9 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
         {
             supportsExtension("jp2","Jpeg2000 image format");
             supportsExtension("jpc","Jpeg2000 image format");
-        
+
+            jas_init();
+
             // little dance here to get around warnings created by jas_image_strtofmt use of char* rather than const char*
             // as a parameted and modern compilers deprecating "jp2" string being treated as char*.
             char* jp2 = strdup("jp2");
@@ -184,8 +184,13 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
             free(jp2);
         }
 
+        ~ReaderWriterJP2()
+        {
+            jas_cleanup();
+        }
+
         virtual const char* className() const { return "RGB Image Reader/Writer"; }
-        
+
         virtual ReadResult readObject(const std::string& file, const osgDB::ReaderWriter::Options* options) const
         {
             return readImage(file,options);
@@ -212,8 +217,6 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
                 return img;
             }
 
-            jas_init();
-
             jas_stream_t* in = jas_stream_fopen(fileName.c_str(), "rb");
 
             char* opt = 0;
@@ -239,7 +242,6 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
 
             jas_image_destroy(jimage);
             jas_stream_close(in);
-            jas_image_clearfmts();
 
             unsigned int pixelFormat =
                 internalFormat == 1 ? GL_LUMINANCE :
@@ -259,7 +261,7 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
 //                osg::Image::USE_NEW_DELETE);
                 osg::Image::NO_DELETE);
 
-            notify(INFO) << "image read ok "<<s<<"  "<<t<< std::endl;
+            OSG_INFO << "image read ok "<<s<<"  "<<t<< std::endl;
             return image;
 
         }
@@ -280,8 +282,6 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
 
             sdata = &vdata[0];
 
-            jas_init();
-
             jas_stream_t* in = jas_stream_memopen((char*)sdata, ssize);
 
             char* opt = 0;
@@ -309,7 +309,6 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
 
             jas_image_destroy(jimage);
             jas_stream_close(in);
-            jas_image_clearfmts();
 
             unsigned int pixelFormat =
                 internalFormat == 1 ? GL_LUMINANCE :
@@ -329,7 +328,7 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
 //                osg::Image::USE_NEW_DELETE);
                 osg::Image::NO_DELETE);
 
-            notify(INFO) << "image read ok "<<s<<"  "<<t<< std::endl;
+            OSG_INFO << "image read ok "<<s<<"  "<<t<< std::endl;
             return image;
         }
 
@@ -353,8 +352,6 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
         {
             std::string ext = osgDB::getFileExtension(fileName);
             if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
-            
-            jas_init();
 
             jas_image_cmptparm_t cmptparms[4];
             jas_image_cmptparm_t *cmptparm;
@@ -426,15 +423,12 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
 
             jas_stream_close(out);
             jas_image_destroy(jimage);
-            jas_image_clearfmts();
 
             return WriteResult::FILE_SAVED;
         }
 
         WriteResult writeImage(const osg::Image& img, std::ostream& fout, const Options* options) const
         {
-            jas_init();
-
             jas_image_cmptparm_t cmptparms[4];
             jas_image_cmptparm_t *cmptparm;
 
@@ -499,7 +493,7 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
                 opt = new char[options->getOptionString().size() + 1];
                 strcpy(opt, options->getOptionString().c_str());
             }
-            
+
             jas_image_encode(jimage, out, _fmt_jp2,  opt);
             if(opt) delete[] opt;
 
@@ -515,7 +509,6 @@ class ReaderWriterJP2 : public osgDB::ReaderWriter
             jas_stream_close(out);
 
             jas_image_destroy(jimage);
-            jas_image_clearfmts();
 
             return WriteResult::FILE_SAVED;
         }
