@@ -9,6 +9,11 @@
 
 #include <sstream>
 
+#if defined(_MSC_VER) && defined(OSG_DISABLE_MSVC_WARNINGS)
+    // disable "structure was padded due to __declspec(align())
+    #pragma warning( disable : 4324 )
+#endif
+
 /****************************************************************************
  *
  * Follows is code extracted from the simage library.  Original Authors:
@@ -70,6 +75,14 @@ namespace osgDBJPEG
 
 static int jpegerror = ERR_NO_ERROR;
 
+/* Some versions of jmorecfg.h define boolean, some don't...
+   Those that do also define HAVE_BOOLEAN, so we can guard using that. */
+#ifndef HAVE_BOOLEAN
+  typedef int boolean;
+  #define FALSE 0
+  #define TRUE 1
+#endif
+
 /* CODE FOR READING/WRITING JPEG FROM STREAMS
  *  This code was taken directly from jdatasrc.c and jdatadst.c (libjpeg source)
  *  and modified to use a std::istream/ostream* instead of a FILE*
@@ -421,6 +434,17 @@ my_error_exit (j_common_ptr cinfo)
     longjmp(myerr->setjmp_buffer, 1);
 }
 
+static void
+my_output_message (j_common_ptr cinfo)
+{
+  char buffer[JMSG_LENGTH_MAX];
+
+  /* Create the message */
+  (*cinfo->err->format_message) (cinfo, buffer);
+
+  OSG_WARN<<buffer<<std::endl;
+}
+
 
 int
 simage_jpeg_identify(const char *,
@@ -486,6 +510,7 @@ int *numComponents_ret)
     /* We set up the normal JPEG error routines, then override error_exit. */
     cinfo.err = jpeg_std_error(&jerr.pub);
     jerr.pub.error_exit = my_error_exit;
+    jerr.pub.output_message = my_output_message;
     /* Establish the setjmp return context for my_error_exit to use. */
     if (setjmp(jerr.setjmp_buffer))
     {
@@ -619,11 +644,40 @@ int *numComponents_ret)
 
 class ReaderWriterJPEG : public osgDB::ReaderWriter
 {
-    
-        WriteResult::WriteStatus write_JPEG_file (std::ostream &fout,int image_width,int image_height,JSAMPLE* image_buffer,int quality = 100) const
+
+        WriteResult::WriteStatus write_JPEG_file (std::ostream &fout, const osg::Image &img, int quality = 100) const
         {
+            int image_width = img.s();
+            int image_height = img.t();
             if ( (image_width == 0) || (image_height == 0) )
+            {
+                OSG_DEBUG << "ReaderWriterJPEG::write_JPEG_file - Error no size" << std::endl;
                 return WriteResult::ERROR_IN_WRITING_FILE;
+            }
+
+            J_COLOR_SPACE image_color_space = JCS_RGB;
+            int image_components = 3;
+            // Only cater for gray, alpha and RGB for now
+            switch(img.getPixelFormat()) {
+              case(GL_LUMINANCE):
+              case(GL_ALPHA): {
+                  image_color_space = JCS_GRAYSCALE;
+                  image_components = 1;
+                  break;
+              }
+              case(GL_RGB): {
+                  image_color_space = JCS_RGB;
+                  image_components = 3;
+                  break;
+              }
+              default:
+              {
+                  OSG_DEBUG << "ReaderWriterJPEG::write_JPEG_file - Error pixel format non supported" << std::endl;
+                return WriteResult::ERROR_IN_WRITING_FILE; break;              
+              }
+            }
+
+            JSAMPLE* image_buffer = (JSAMPLE*)(img.data());
 
             /* This struct contains the JPEG compression parameters and pointers to
             * working space (which is allocated as needed by the JPEG library).
@@ -680,8 +734,8 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             */
             cinfo.image_width = image_width;     /* image width and height, in pixels */
             cinfo.image_height = image_height;
-            cinfo.input_components = 3;        /* # of color components per pixel */
-            cinfo.in_color_space = JCS_RGB;     /* colorspace of input image */
+            cinfo.input_components = image_components;        /* # of color components per pixel */
+            cinfo.in_color_space = image_color_space;     /* colorspace of input image */
             /* Now use the library's routine to set default compression parameters.
             * (You must set at least cinfo.in_color_space before calling this,
             * since the defaults depend on the source color space.)
@@ -707,7 +761,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             * To keep things simple, we pass one scanline per call; you can pass
             * more if you wish, though.
             */
-            row_stride = image_width * 3;    /* JSAMPLEs per row in image_buffer */
+            row_stride = image_width * image_components;    /* JSAMPLEs per row in image_buffer */
 
             while (cinfo.next_scanline < cinfo.image_height)
             {
@@ -822,7 +876,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             std::string fileName = osgDB::findDataFile( file, options );
             if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
 
-            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            osgDB::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
             if(!istream) return ReadResult::ERROR_IN_READING_FILE;
             ReadResult rr = readJPGStream(istream);
             if(rr.validImage()) rr.getImage()->setFileName(file);
@@ -833,7 +887,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
         {
             osg::ref_ptr<osg::Image> tmp_img = new osg::Image(img);
             tmp_img->flipVertical();
-            WriteResult::WriteStatus ws = write_JPEG_file(fout,img.s(),img.t(),(JSAMPLE*)(tmp_img->data()),getQuality(options));
+            WriteResult::WriteStatus ws = write_JPEG_file(fout, *(tmp_img.get()), getQuality(options));
             return ws;
         }
 
@@ -842,7 +896,7 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
             std::string ext = osgDB::getFileExtension(fileName);
             if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
-            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
             if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
 
             return writeImage(img,fout,options);
