@@ -37,6 +37,8 @@
 #include <osgUtil/SmoothingVisitor>
 #include <osgUtil/Tessellator>
 
+#include <string.h>
+
 #include "Converter.h"
 #include "VertexMap.h"
 
@@ -46,12 +48,14 @@
 class ReaderWriterLWO : public osgDB::ReaderWriter
 {
 public:
-    ReaderWriterLWO() { }
-
-    virtual const char* className() const { return "Lightwave Object Reader"; }
-    virtual bool acceptsExtension(const std::string& extension) const {
-        return osgDB::equalCaseInsensitive(extension,"lwo") || osgDB::equalCaseInsensitive(extension,"lw") || osgDB::equalCaseInsensitive(extension,"geo");
+    ReaderWriterLWO()
+    {
+        supportsExtension("lwo","Lightwave object format");
+        supportsExtension("lw","Lightwave object format");
+        supportsExtension("geo","Lightwave geometry format");
     }
+    
+    virtual const char* className() const { return "Lightwave Object Reader"; }
 
     virtual ReadResult readNode(const std::string& file, const osgDB::ReaderWriter::Options* options) const
     {
@@ -186,9 +190,9 @@ osgDB::ReaderWriter::ReadResult ReaderWriterLWO::readNode_LWO1(const std::string
     if (!lw)
         return ReadResult::FILE_NOT_HANDLED;
 
-    osg::notify(osg::INFO) << "faces " << lw->face_cnt << std::endl;
-    osg::notify(osg::INFO) << "materials " << lw->material_cnt << std::endl;
-    osg::notify(osg::INFO) << "vertices " << lw->vertex_cnt << std::endl;
+    OSG_INFO << "faces " << lw->face_cnt << std::endl;
+    OSG_INFO << "materials " << lw->material_cnt << std::endl;
+    OSG_INFO << "vertices " << lw->vertex_cnt << std::endl;
 
     typedef std::map<int,GeometryCollection> MaterialToGeometryCollectionMap;
     MaterialToGeometryCollectionMap mtgcm;
@@ -237,16 +241,16 @@ osgDB::ReaderWriter::ReadResult ReaderWriterLWO::readNode_LWO1(const std::string
             {
                 if (lw_material.ctex.flags && strlen(lw_material.ctex.name)!=0)
                 {
-                    osg::notify(osg::INFO) << "ctex " << lw_material.ctex.name << std::endl;
-                    osg::Image* image = osgDB::readImageFile(lw_material.ctex.name);
-                    if (image)
+                    OSG_INFO << "ctex " << lw_material.ctex.name << std::endl;
+                    osg::ref_ptr<osg::Image> image = osgDB::readRefImageFile(lw_material.ctex.name);
+                    if (image.valid())
                     {
                         // create state
                         osg::StateSet* stateset = new osg::StateSet;
 
                         // create texture
                         osg::Texture2D* texture = new osg::Texture2D;
-                        texture->setImage(image);
+                        texture->setImage(image.get());
 
                         // texture wrap mode
                         static osg::Texture::WrapMode mode[] = {
