@@ -17,11 +17,16 @@
  */
 
 #include "old_lw.h"
+
+#include "lwo2read.h"
+
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <osgDB/FileUtils>
+
 #define MK_ID(a,b,c,d) ((((guint32)(a))<<24)| \
             (((guint32)(b))<<16)| \
             (((guint32)(c))<< 8)| \
@@ -78,10 +83,10 @@ static gint32 read_long(FILE *f)
   return (read_char(f)<<24) | (read_char(f)<<16) | (read_char(f)<<8) | read_char(f);
 }
 
+
 static GLfloat read_float(FILE *f)
 {
-  gint32 x = read_long(f);
-  return *(GLfloat*)&x;
+  return lwo2::changeType4<GLfloat, gint32>(read_long(f));
 }
 
 static gint read_string(FILE *f, char *s)
@@ -326,7 +331,7 @@ static void read_pnts(FILE *f, gint nbytes, lwObject *lwo)
 
 gint lw_is_lwobject(const char *lw_file)
 {
-  FILE *f = fopen(lw_file, "rb");
+  FILE *f = osgDB::fopen(lw_file, "rb");
   if (f) {
     gint32 form = read_long(f);
     gint32 nlen = read_long(f);
@@ -348,7 +353,7 @@ lwObject *lw_object_read(const char *lw_file, std::ostream& output)
   gint32 read_bytes = 0;
 
   /* open file */
-  f = fopen(lw_file, "rb");
+  f = osgDB::fopen(lw_file, "rb");
   if (f == NULL) {
     output << "can't open file "<<lw_file<<std::endl;
     return NULL;
