@@ -38,6 +38,23 @@
 
 #define SERIALIZER() OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex)  
 
+void CPL_STDCALL CPLOSGErrorHandler( CPLErr eErrClass, int nError, 
+                             const char * pszErrorMsg )
+{
+    if( eErrClass == CE_Debug )
+    {
+        OSG_DEBUG << pszErrorMsg << std::endl;
+    }
+    else if( eErrClass == CE_Warning )
+    {
+        OSG_WARN << nError << " " << pszErrorMsg << std::endl;
+    }
+    else
+    {
+        OSG_FATAL << nError << " " << pszErrorMsg << std::endl;
+    }
+}
+
 static osg::Material* createDefaultMaterial()
 {
     osg::Vec4 color;
@@ -85,20 +102,32 @@ public:
     {
         supportsExtension("ogr","OGR file reader");
         supportsOption("useRandomColorByFeature", "Assign a random color to each feature.");
-        supportsOption("addGroupPerFeature", "Places each feature in a seperate group.");
+        supportsOption("addGroupPerFeature", "Places each feature in a separate group.");
+        oldHandler = CPLSetErrorHandler(CPLOSGErrorHandler);
     }
+
+    virtual ~ReaderWriterOGR()
+    {
+        CPLSetErrorHandler(oldHandler); 
+    }
+    
     virtual const char* className() const { return "OGR file reader"; }
 
     virtual ReadResult readNode(const std::string& file, const osgDB::ReaderWriter::Options* options) const
     {
+        OSG_INFO<<"OGR::readNode("<<file<<")"<<std::endl;
+        
+        if (file.empty()) return ReadResult::FILE_NOT_FOUND;
+    
         if (osgDB::equalCaseInsensitive(osgDB::getFileExtension(file),"ogr"))
         {
-            return readObject(osgDB::getNameLessExtension(file),options);
+            OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex);
+            return readFile(osgDB::getNameLessExtension(file), options);
         }
 
         OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex);
         std::string fileName = osgDB::findDataFile( file, options );
-        if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
+        if (fileName.empty()) return readFile(file, options); // ReadResult::FILE_NOT_FOUND;
 
         return readFile(fileName, options);
     }
@@ -149,7 +178,6 @@ public:
         OGRFeature* ogrFeature = NULL;
         while ((ogrFeature = ogrLayer->GetNextFeature()) != NULL) 
         {
-
             osg::Geode* feature = readFeature(ogrFeature, useRandomColorByFeature);
             if (feature)
             {
@@ -178,6 +206,7 @@ public:
         pointGeom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, 1));
         return pointGeom;
     }
+
     osg::Geometry* linearRingToDrawable(OGRLinearRing* ring) const
     {
         osg::Geometry* contourGeom = new osg::Geometry();
@@ -209,6 +238,38 @@ public:
         return contourGeom;
     }
 
+    osg::Geometry* multiPointToDrawable(OGRMultiPoint* mpoint) const
+    {
+        osg::Geometry* geom = new osg::Geometry;
+
+        osg::Geometry* pointGeom = new osg::Geometry();
+        osg::Vec3Array* vertices = new osg::Vec3Array();
+        
+        vertices->reserve(mpoint->getNumGeometries());
+        for (int i = 0; i < mpoint->getNumGeometries(); i++ ) 
+        {
+            OGRGeometry* ogrGeom = mpoint->getGeometryRef(i);
+            OGRwkbGeometryType ogrGeomType = ogrGeom->getGeometryType();
+
+            if (wkbPoint != ogrGeomType && wkbPoint25D != ogrGeomType)
+                continue; // skip
+
+            OGRPoint* points = static_cast<OGRPoint*>(ogrGeom);
+        
+            vertices->push_back(osg::Vec3(points->getX(), points->getY(), points->getZ()));
+        }
+        
+        pointGeom->setVertexArray(vertices);
+        pointGeom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, vertices->size()));
+        
+        if (pointGeom->getVertexArray())
+        {
+            OSG_INFO << "osgOgrFeature::multiPointToDrawable " << geom->getVertexArray()->getNumElements() << " vertexes"<< std::endl;
+        }
+
+        return pointGeom;
+    }
+
     osg::Geometry* multiPolygonToDrawable(OGRMultiPolygon* mpolygon) const
     {
         osg::Geometry* geom = new osg::Geometry;
@@ -250,11 +311,15 @@ public:
                 }
             } 
             else
-                osg::notify(osg::WARN) << "Warning something wrong with a polygon in a multi polygon" << std::endl;
+            {
+                OSG_WARN << "Warning something wrong with a polygon in a multi polygon" << std::endl;
+            }
         }
 
         if (geom->getVertexArray())
-            osg::notify() << "osgOgrFeature::multiPolygonToGeode " << geom->getVertexArray()->getNumElements() << " vertexes"<< std::endl;
+        {
+            OSG_INFO << "osgOgrFeature::multiPolygonToDrawable " << geom->getVertexArray()->getNumElements() << " vertexes"<< std::endl;
+        }
 
         return geom;
     }
@@ -350,6 +415,7 @@ public:
           
         osg::Geometry* drawable = 0;
         bool disableCulling = false;
+
         // Read the geometry
         switch(ogrFeature->GetGeometryRef()->getGeometryType()) {
         case wkbPoint:
@@ -370,11 +436,13 @@ public:
 
         case wkbPolygon:
         case wkbPolygon25D:
-                drawable = polygonToDrawable(static_cast<OGRPolygon*>(ogrFeature->GetGeometryRef()));
+            drawable = polygonToDrawable(static_cast<OGRPolygon*>(ogrFeature->GetGeometryRef()));
             break;
 
         case wkbMultiPoint:
         case wkbMultiPoint25D:
+            drawable = multiPointToDrawable(static_cast<OGRMultiPoint*>(ogrFeature->GetGeometryRef()));
+            disableCulling = true;
             break;
 
         case wkbMultiLineString:
@@ -389,16 +457,16 @@ public:
 
         case wkbGeometryCollection:
         case wkbGeometryCollection25D:
-            osg::notify(osg::WARN) << "This geometry is not yet implemented " << OGRGeometryTypeToName(ogrFeature->GetGeometryRef()->getGeometryType()) << std::endl;
+            OSG_WARN << "This geometry is not yet implemented " << OGRGeometryTypeToName(ogrFeature->GetGeometryRef()->getGeometryType()) << std::endl;
             break;
 
         case wkbNone:
-            osg::notify(osg::WARN) << "No WKB Geometry " << OGRGeometryTypeToName(ogrFeature->GetGeometryRef()->getGeometryType()) << std::endl;
+            OSG_WARN << "No WKB Geometry " << OGRGeometryTypeToName(ogrFeature->GetGeometryRef()->getGeometryType()) << std::endl;
             break;
 
         case wkbUnknown:
         default:
-            osg::notify(osg::WARN) << "Unknown WKB Geometry " << OGRGeometryTypeToName(ogrFeature->GetGeometryRef()->getGeometryType()) << std::endl;
+            OSG_WARN << "Unknown WKB Geometry " << OGRGeometryTypeToName(ogrFeature->GetGeometryRef()->getGeometryType()) << std::endl;
             break;
         }
 
@@ -418,9 +486,9 @@ public:
     }
 
     mutable OpenThreads::ReentrantMutex _serializerMutex;
-        
+    CPLErrorHandler oldHandler;
 };
 
 // now register with Registry to instantiate the above
 // reader/writer.
-osgDB::RegisterReaderWriterProxy<ReaderWriterOGR> g_readerWriter_OGR_Proxy;
+REGISTER_OSGPLUGIN(ogr, ReaderWriterOGR)
