@@ -145,9 +145,9 @@ OSGA_Archive::IndexBlock* OSGA_Archive::IndexBlock::read(std::istream& in, bool
         osg::swapBytes(reinterpret_cast<char*>(&indexBlock->_offsetOfNextAvailableSpace), sizeof(indexBlock-> _offsetOfNextAvailableSpace));
     }
 
-//    osg::notify(osg::INFO)<<"indexBlock->_blockSize="<<indexBlock->_blockSize<<std::endl;
-//    osg::notify(osg::INFO)<<"indexBlock->_filePositionNextIndexBlock="<<indexBlock->_filePositionNextIndexBlock<<std::endl;
-//    osg::notify(osg::INFO)<<"indexBlock->_offsetOfNextAvailableSpace="<<indexBlock->_offsetOfNextAvailableSpace<<std::endl;
+//    OSG_INFO<<"indexBlock->_blockSize="<<indexBlock->_blockSize<<std::endl;
+//    OSG_INFO<<"indexBlock->_filePositionNextIndexBlock="<<indexBlock->_filePositionNextIndexBlock<<std::endl;
+//    OSG_INFO<<"indexBlock->_offsetOfNextAvailableSpace="<<indexBlock->_offsetOfNextAvailableSpace<<std::endl;
 
     indexBlock->allocateData(indexBlock->_blockSize);
     if (indexBlock->_data)
@@ -172,18 +172,18 @@ OSGA_Archive::IndexBlock* OSGA_Archive::IndexBlock::read(std::istream& in, bool
                 ptr += sizeof(unsigned int);
                 ptr += filename_size;
 
-                osg::notify(osg::INFO)<<"filename size="<<filename_size<<std::endl;
+                OSG_INFO<<"filename size="<<filename_size<<std::endl;
 
             }
         }
     }
     else
     {
-        osg::notify(osg::INFO)<<"Allocation Problem in OSGA_Archive::IndexBlock::read(std::istream& in)"<<std::endl;
+        OSG_INFO<<"Allocation Problem in OSGA_Archive::IndexBlock::read(std::istream& in)"<<std::endl;
         return 0;
     }
 
-    osg::notify(osg::INFO)<<"Read index block"<<std::endl;
+    OSG_INFO<<"Read index block"<<std::endl;
     
     return indexBlock.release();
     
@@ -251,14 +251,14 @@ void OSGA_Archive::IndexBlock::write(std::ostream& out)
 
     if (_filePosition==pos_type(0))
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::IndexBlock::write() setting _filePosition"<<std::endl;
+        OSG_INFO<<"OSGA_Archive::IndexBlock::write() setting _filePosition"<<std::endl;
         _filePosition = currentPos;
     }
     else
     {
          out.seekp( STREAM_POS( _filePosition ) );
     }
-    osg::notify(osg::INFO)<<"OSGA_Archive::IndexBlock::write() to _filePosition"<< ARCHIVE_POS( out.tellp() )<<std::endl;
+    OSG_INFO<<"OSGA_Archive::IndexBlock::write() to _filePosition"<< ARCHIVE_POS( out.tellp() )<<std::endl;
 
     out.write(reinterpret_cast<char*>(&_blockSize), sizeof(_blockSize));
     out.write(reinterpret_cast<char*>(&_filePositionNextIndexBlock), sizeof(_filePositionNextIndexBlock));
@@ -269,7 +269,7 @@ void OSGA_Archive::IndexBlock::write(std::ostream& out)
     if( _filePosition < currentPos ) // move file ptr to the end of file 
         out.seekp( STREAM_POS( currentPos ) );
     
-    osg::notify(osg::INFO)<<"OSGA_Archive::IndexBlock::write() end"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::IndexBlock::write() end"<<std::endl;
 }
 
 
@@ -300,7 +300,7 @@ bool OSGA_Archive::IndexBlock::addFileReference(pos_type position, size_type siz
         
         _requiresWrite = true;
 
-        osg::notify(osg::INFO)<<"OSGA_Archive::IndexBlock::addFileReference("<<(unsigned int)position<<", "<<filename<<")"<<std::endl;
+        OSG_INFO<<"OSGA_Archive::IndexBlock::addFileReference("<<(unsigned int)position<<", "<<filename<<")"<<std::endl;
         
         return true;
     }
@@ -330,6 +330,8 @@ bool OSGA_Archive::open(const std::string& filename, ArchiveStatus status, unsig
 {
     SERIALIZER();
 
+    _archiveFileName = filename;
+
     if (status==READ)
     {
         _status = status;
@@ -373,20 +375,20 @@ bool OSGA_Archive::open(const std::string& filename, ArchiveStatus status, unsig
 
             _output.open(filename.c_str(), std::ios_base::binary | std::ios_base::in | std::ios_base::out);
 
-            osg::notify(osg::INFO)<<"File position after open = "<<ARCHIVE_POS( _output.tellp() )<<" is_open "<<_output.is_open()<<std::endl;
+            OSG_INFO<<"File position after open = "<<ARCHIVE_POS( _output.tellp() )<<" is_open "<<_output.is_open()<<std::endl;
 
             // place write position at end of file. 
             _output.seekp( STREAM_POS( file_size ) );
 
-            osg::notify(osg::INFO)<<"File position after seekp = "<<ARCHIVE_POS( _output.tellp() )<<std::endl;
+            OSG_INFO<<"File position after seekp = "<<ARCHIVE_POS( _output.tellp() )<<std::endl;
 
-            osg::notify(osg::INFO)<<"OSGA_Archive::open("<<filename<<") open for writing"<<std::endl;
+            OSG_INFO<<"OSGA_Archive::open("<<filename<<") open for writing"<<std::endl;
 
             return true;
         }
         else // no file opened or using create so resort to creating the archive.
         {
-            osg::notify(osg::INFO)<<"OSGA_Archive::open("<<filename<<"), archive being created."<<std::endl;
+            OSG_INFO<<"OSGA_Archive::open("<<filename<<"), archive being created."<<std::endl;
 
             _status = WRITE;
             _output.open(filename.c_str(), std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
@@ -401,7 +403,7 @@ bool OSGA_Archive::open(const std::string& filename, ArchiveStatus status, unsig
                 _indexBlockList.push_back(indexBlock);
             }
 
-            osg::notify(osg::INFO)<<"File position after write = "<<ARCHIVE_POS( _output.tellp() )<<std::endl;
+            OSG_INFO<<"File position after write = "<<ARCHIVE_POS( _output.tellp() )<<std::endl;
 
             return true;
         }
@@ -413,7 +415,9 @@ bool OSGA_Archive::open(std::istream& fin)
 {
     SERIALIZER();
 
-    osg::notify(osg::NOTICE)<<"OSGA_Archive::open"<<std::endl;
+    _archiveFileName = "";
+
+    OSG_NOTICE<<"OSGA_Archive::open"<<std::endl;
     static_cast<std::istream&>(_input).rdbuf(fin.rdbuf());
     return _open(_input);
 }
@@ -439,8 +443,8 @@ bool OSGA_Archive::_open(std::istream& input)
                 osg::swapBytes(reinterpret_cast<char*>(&_version),sizeof(_version));
             }
 
-            osg::notify(osg::INFO)<<"OSGA_Archive::open() doEndianSwap="<<doEndianSwap<<std::endl;
-            osg::notify(osg::INFO)<<"OSGA_Archive::open() Version="<<_version<<std::endl;
+            OSG_INFO<<"OSGA_Archive::open() doEndianSwap="<<doEndianSwap<<std::endl;
+            OSG_INFO<<"OSGA_Archive::open() Version="<<_version<<std::endl;
 
             IndexBlock *indexBlock = 0;
 
@@ -471,7 +475,7 @@ bool OSGA_Archive::_open(std::istream& input)
                 mitr!=_indexMap.end();
                 ++mitr)
             {
-                osg::notify(osg::INFO)<<"    filename "<<(mitr->first)<<" pos="<<(int)((mitr->second).first)<<" size="<<(int)((mitr->second).second)<<std::endl;
+                OSG_INFO<<"    filename "<<(mitr->first)<<" pos="<<(int)((mitr->second).first)<<" size="<<(int)((mitr->second).second)<<std::endl;
             }
 
 
@@ -499,6 +503,12 @@ std::string OSGA_Archive::getMasterFileName() const
     return _masterFileName;
 }
 
+osgDB::FileType OSGA_Archive::getFileType(const std::string& filename) const
+{
+    if (_indexMap.count(filename)!=0) return osgDB::REGULAR_FILE;
+    return osgDB::FILE_NOT_FOUND;
+}
+
 bool OSGA_Archive::getFileNames(FileNameList& fileNameList) const
 {
     SERIALIZER();
@@ -514,7 +524,6 @@ bool OSGA_Archive::getFileNames(FileNameList& fileNameList) const
     return !fileNameList.empty();
 }
 
-
 void OSGA_Archive::writeIndexBlocks()
 {
     SERIALIZER();
@@ -544,13 +553,13 @@ bool OSGA_Archive::addFileReference(pos_type position, size_type size, const std
 
     if (_status==READ)
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::getPositionForNewEntry("<<fileName<<") failed, archive opened as read only."<<std::endl;
+        OSG_INFO<<"OSGA_Archive::getPositionForNewEntry("<<fileName<<") failed, archive opened as read only."<<std::endl;
         return false;
     }
     
     if (!_output)
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::getPositionForNewEntry("<<fileName<<") failed, _output set up."<<std::endl;
+        OSG_INFO<<"OSGA_Archive::getPositionForNewEntry("<<fileName<<") failed, _output set up."<<std::endl;
         return false;
     }
     
@@ -561,7 +570,7 @@ bool OSGA_Archive::addFileReference(pos_type position, size_type size, const std
 
     // get an IndexBlock with space available if possible
     unsigned int blockSize = 4096;
-    osg::ref_ptr<IndexBlock> indexBlock = _indexBlockList.empty() ? 0 : _indexBlockList.back();
+    osg::ref_ptr<IndexBlock> indexBlock = _indexBlockList.empty() ? 0 : _indexBlockList.back().get();
     osg::ref_ptr<IndexBlock> previousBlock = indexBlock;
     if (indexBlock.valid())
     {
@@ -652,7 +661,13 @@ struct OSGA_Archive::ReadHeightFieldFunctor : public OSGA_Archive::ReadFunctor
 struct OSGA_Archive::ReadNodeFunctor : public OSGA_Archive::ReadFunctor
 {
     ReadNodeFunctor(const std::string& filename, const ReaderWriter::Options* options):ReadFunctor(filename,options) {}
-    virtual ReaderWriter::ReadResult doRead(ReaderWriter& rw, std::istream& input) const { return rw.readNode(input, _options); }    
+    virtual ReaderWriter::ReadResult doRead(ReaderWriter& rw, std::istream& input) const { return rw.readNode(input, _options); }
+};
+
+struct OSGA_Archive::ReadShaderFunctor : public OSGA_Archive::ReadFunctor
+{
+    ReadShaderFunctor(const std::string& filename, const ReaderWriter::Options* options):ReadFunctor(filename,options) {}
+    virtual ReaderWriter::ReadResult doRead(ReaderWriter& rw, std::istream& input) const { return rw.readShader(input, _options); }
 };
 
 ReaderWriter::ReadResult OSGA_Archive::read(const ReadFunctor& readFunctor)
@@ -661,25 +676,25 @@ ReaderWriter::ReadResult OSGA_Archive::read(const ReadFunctor& readFunctor)
 
     if (_status!=READ) 
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<") failed, archive opened as write only."<<std::endl;
+        OSG_INFO<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<") failed, archive opened as write only."<<std::endl;
         return ReadResult(ReadResult::FILE_NOT_HANDLED);
     }
     
     FileNamePositionMap::const_iterator itr = _indexMap.find(readFunctor._filename);
     if (itr==_indexMap.end())
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<") failed, file not found in archive"<<std::endl;
+        OSG_INFO<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<") failed, file not found in archive"<<std::endl;
         return ReadResult(ReadResult::FILE_NOT_FOUND);
     }
     
     ReaderWriter* rw = osgDB::Registry::instance()->getReaderWriterForExtension(getLowerCaseFileExtension(readFunctor._filename));
     if (!rw)
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<") failed to find appropriate plugin to read file."<<std::endl;
+        OSG_INFO<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<") failed to find appropriate plugin to read file."<<std::endl;
         return ReadResult(ReadResult::FILE_NOT_HANDLED);
     }
     
-    osg::notify(osg::INFO)<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<")"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::readObject(obj, "<<readFunctor._filename<<")"<<std::endl;
     
     _input.seekg( STREAM_POS( itr->second.first ) );
 
@@ -715,6 +730,11 @@ ReaderWriter::ReadResult OSGA_Archive::readNode(const std::string& fileName,cons
     return const_cast<OSGA_Archive*>(this)->read(ReadNodeFunctor(fileName, options));
 }
 
+ReaderWriter::ReadResult OSGA_Archive::readShader(const std::string& fileName,const Options* options) const
+{
+    return const_cast<OSGA_Archive*>(this)->read(ReadShaderFunctor(fileName, options));
+}
+
 
 struct OSGA_Archive::WriteObjectFunctor : public OSGA_Archive::WriteFunctor
 {
@@ -753,7 +773,17 @@ struct OSGA_Archive::WriteNodeFunctor : public OSGA_Archive::WriteFunctor
         _object(object) {}
     const osg::Node& _object;
 
-    virtual ReaderWriter::WriteResult doWrite(ReaderWriter& rw, std::ostream& output) const { return rw.writeNode(_object, output, _options); }    
+    virtual ReaderWriter::WriteResult doWrite(ReaderWriter& rw, std::ostream& output) const { return rw.writeNode(_object, output, _options); }
+};
+
+struct OSGA_Archive::WriteShaderFunctor : public OSGA_Archive::WriteFunctor
+{
+    WriteShaderFunctor(const osg::Shader& object, const std::string& filename, const ReaderWriter::Options* options):
+        WriteFunctor(filename,options),
+        _object(object) {}
+    const osg::Shader& _object;
+
+    virtual ReaderWriter::WriteResult doWrite(ReaderWriter& rw, std::ostream& output) const { return rw.writeShader(_object, output, _options); }
 };
 
 ReaderWriter::WriteResult OSGA_Archive::write(const WriteFunctor& writeFunctor)
@@ -762,18 +792,18 @@ ReaderWriter::WriteResult OSGA_Archive::write(const WriteFunctor& writeFunctor)
 
     if (_status!=WRITE) 
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::write(obj, "<<writeFunctor._filename<<") failed, archive opened as read only."<<std::endl;
+        OSG_INFO<<"OSGA_Archive::write(obj, "<<writeFunctor._filename<<") failed, archive opened as read only."<<std::endl;
         return WriteResult(WriteResult::FILE_NOT_HANDLED);
     }
 
     ReaderWriter* rw = osgDB::Registry::instance()->getReaderWriterForExtension(getLowerCaseFileExtension(writeFunctor._filename));
     if (!rw)
     {
-        osg::notify(osg::INFO)<<"OSGA_Archive::write(obj, "<<writeFunctor._filename<<") failed to find appropriate plugin to write file."<<std::endl;
+        OSG_INFO<<"OSGA_Archive::write(obj, "<<writeFunctor._filename<<") failed to find appropriate plugin to write file."<<std::endl;
         return WriteResult(WriteResult::FILE_NOT_HANDLED);
     }
     
-    osg::notify(osg::INFO)<<"OSGA_Archive::write(obj, "<<writeFunctor._filename<<")"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::write(obj, "<<writeFunctor._filename<<")"<<std::endl;
     
     pos_type position = ARCHIVE_POS( _output.tellp() );
     
@@ -790,25 +820,31 @@ ReaderWriter::WriteResult OSGA_Archive::write(const WriteFunctor& writeFunctor)
 
 ReaderWriter::WriteResult OSGA_Archive::writeObject(const osg::Object& obj,const std::string& fileName,const Options* options) const
 {
-    osg::notify(osg::INFO)<<"OSGA_Archive::writeObject(obj, "<<fileName<<")"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::writeObject(obj, "<<fileName<<")"<<std::endl;
     return const_cast<OSGA_Archive*>(this)->write(WriteObjectFunctor(obj, fileName, options));
 }
 
 ReaderWriter::WriteResult OSGA_Archive::writeImage(const osg::Image& image,const std::string& fileName,const Options* options) const
 {
-    osg::notify(osg::INFO)<<"OSGA_Archive::writeImage(obj, "<<fileName<<")"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::writeImage(obj, "<<fileName<<")"<<std::endl;
     return const_cast<OSGA_Archive*>(this)->write(WriteImageFunctor(image, fileName, options));
 }
 
 ReaderWriter::WriteResult OSGA_Archive::writeHeightField(const osg::HeightField& heightField,const std::string& fileName,const Options* options) const
 {
-    osg::notify(osg::INFO)<<"OSGA_Archive::writeHeightField(obj, "<<fileName<<")"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::writeHeightField(obj, "<<fileName<<")"<<std::endl;
     return const_cast<OSGA_Archive*>(this)->write(WriteHeightFieldFunctor(heightField, fileName, options));
 }
 
 ReaderWriter::WriteResult OSGA_Archive::writeNode(const osg::Node& node,const std::string& fileName,const Options* options) const
 {
-    osg::notify(osg::INFO)<<"OSGA_Archive::writeNode(obj, "<<fileName<<")"<<std::endl;
+    OSG_INFO<<"OSGA_Archive::writeNode(obj, "<<fileName<<")"<<std::endl;
     return const_cast<OSGA_Archive*>(this)->write(WriteNodeFunctor(node, fileName, options));
 }
 
+ReaderWriter::WriteResult OSGA_Archive::writeShader(const osg::Shader& shader,const std::string& fileName,const Options* options) const
+{
+    OSG_INFO<<"OSGA_Archive::writeShader(obj, "<<fileName<<")"<<std::endl;
+    return const_cast<OSGA_Archive*>(this)->write(WriteShaderFunctor(shader, fileName, options));
+}
+
