@@ -47,6 +47,7 @@ WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 #include <string.h>
 
 #include <osg/Math>
+#include <osgDB/FileUtils>
 
 #if defined(_MSC_VER) && defined(OSG_DISABLE_MSVC_WARNINGS)
     #pragma warning( disable : 4996 )
@@ -314,7 +315,7 @@ PlyFile *ply_open_for_writing(
 
   /* open the file for writing */
 
-  fp = fopen (name, "wb");
+  fp = osgDB::fopen (name, "wb");
   free (name); //wjs remove memory leak//
   if (fp == NULL) {
     return (NULL);
@@ -827,6 +828,8 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
   /* create record for this object */
 
   plyfile = (PlyFile *) myalloc (sizeof (PlyFile));
+  if (!plyfile) return (NULL);
+  
   plyfile->nelems = 0;
   plyfile->comments = NULL;
   plyfile->num_comments = 0;
@@ -839,15 +842,25 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
 
   words = get_words (plyfile->fp, &nwords, &orig_line);
   if (!words || !equal_strings (words[0], "ply"))
+  {
+    if (words) free( words);
+    free( plyfile );
     return (NULL);
+  }
 
-  while (words) {
+  while (words)
+  {
 
     /* parse words */
 
-    if (equal_strings (words[0], "format")) {
+    if (equal_strings (words[0], "format"))
+    {
       if (nwords != 3)
+      {
+        free( words );
+        free( plyfile );
         return (NULL);
+      }
       if (equal_strings (words[1], "ascii"))
         plyfile->file_type = PLY_ASCII;
       else if (equal_strings (words[1], "binary_big_endian"))
@@ -855,10 +868,11 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
       else if (equal_strings (words[1], "binary_little_endian"))
         plyfile->file_type = PLY_BINARY_LE;
       else
-        {
-        free (words);
+      {
+        free( words );
+        free( plyfile );
         return (NULL);
-        }
+      }
       plyfile->version = osg::asciiToDouble (words[2]);
     }
     else if (equal_strings (words[0], "element"))
@@ -870,10 +884,10 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
     else if (equal_strings (words[0], "obj_info"))
       add_obj_info (plyfile, orig_line);
     else if (equal_strings (words[0], "end_header"))
-      {
+    {
       free (words);
       break;
-      }
+    }
 
     /* free up words space */
     free (words);
@@ -945,7 +959,7 @@ PlyFile *ply_open_for_reading(
 
   /* open the file for reading */
 
-  fp = fopen (name, "rb");
+  fp = osgDB::fopen (name, "rb");
   free(name);
   if (fp == NULL)
     return (NULL);
@@ -1288,7 +1302,7 @@ PlyOtherProp *ply_get_other_properties(
   }
 #endif
   other->size = elem->other_size;
-  other->props = (PlyProperty **) myalloc (sizeof(PlyProperty) * elem->nprops);
+  other->props = (PlyProperty **) myalloc (sizeof(PlyProperty*) * elem->nprops);
   
   /* save descriptions of each "other" property */
   nprops = 0;
