@@ -25,6 +25,7 @@ ESRIShapeParser::ESRIShapeParser( const std::string fileName, bool useDouble ):
         if( (fd = open( fileName.c_str(), O_RDONLY )) <= 0 )
 #endif
         {
+            if (fd) close ( fd );
             perror( fileName.c_str() );
             return ;
         }
@@ -217,7 +218,7 @@ void ESRIShapeParser::_combinePointToMultipoint()
 {
     if( !_valid ) return;
     
-    osg::notify(osg::NOTICE)<<"_combinePointToMultipoint()"<<std::endl;
+    OSG_NOTICE<<"_combinePointToMultipoint()"<<std::endl;
 
     ArrayHelper coords(_useDouble);
 
@@ -587,7 +588,7 @@ void ESRIShapeParser::_process( const std::vector<ESRIShape::MultiPatch> &mpatch
                 p->partTypes[i] == InnerRing || 
                 p->partTypes[i] == FirstRing || p->partTypes[i] == Ring )
             {
-                osg::notify(osg::WARN) << "ESRIShapeParser - MultiPatch type " << 
+                OSG_WARN << "ESRIShapeParser - MultiPatch type " << 
                     (p->partTypes[i] == TriangleStrip ? "TriangleStrip":
                      p->partTypes[i] == TriangleFan   ? "TriangleFan":
                      p->partTypes[i] == OuterRing     ? "OuterRing":
