@@ -6,6 +6,7 @@
 #include <osgDB/Registry>
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 
 #include <stdio.h>
 #include <assert.h>
@@ -467,8 +468,13 @@ int headerlen)
 class ReaderWriterTGA : public osgDB::ReaderWriter
 {
     public:
+    
+        ReaderWriterTGA()
+        {
+            supportsExtension("tga","Tga Image format");
+        }
+        
         virtual const char* className() const { return "TGA Image Reader"; }
-        virtual bool acceptsExtension(const std::string& extension) const { return osgDB::equalCaseInsensitive(extension,"tga"); }
 
         ReadResult readTGAStream(std::istream& fin) const
         {
@@ -530,12 +536,90 @@ class ReaderWriterTGA : public osgDB::ReaderWriter
             std::string fileName = osgDB::findDataFile( file, options );
             if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
 
-            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            osgDB::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
             if(!istream) return ReadResult::FILE_NOT_HANDLED;
             ReadResult rr = readTGAStream(istream);
             if(rr.validImage()) rr.getImage()->setFileName(file);
             return rr;
         }
+        
+        bool saveTGAStream(const osg::Image& image, std::ostream& fout) const
+        {
+            // At present, I will only save the image to unmapped RGB format
+            // Other data types can be added soon with different options
+            // The format description can be found at:
+            // http://local.wasp.uwa.edu.au/~pbourke/dataformats/tga/
+            unsigned int pixelFormat = image.getPixelFormat();
+            int width = image.s(), height = image.t();
+            int numPerPixel = image.computeNumComponents(pixelFormat);
+            int pixelMultiplier = (image.getDataType()==GL_FLOAT ? 255 : 1);
+            const unsigned char* data = image.data();
+            if ( !data ) return false;
+            
+            // Headers
+            fout.put(0);  // Identification field size
+            fout.put(0);  // Color map type
+            fout.put(2);  // Image type
+            fout.put(0); fout.put(0);  // Color map origin
+            fout.put(0); fout.put(0);  // Color map length
+            fout.put(0);  // Color map entry size
+            fout.put(0); fout.put(0);  // X origin of image
+            fout.put(0); fout.put(0);  // Y origin of image
+            fout.put(width&0xff); fout.put((width&0xff00)>>8);  // Width of image
+            fout.put(height&0xff); fout.put((height&0xff00)>>8);  // Height of image
+            fout.put(numPerPixel * 8);  // Image pixel size
+            fout.put(0);  // Image descriptor
+            
+            // Swap red/blue channels for BGR images
+            int r = 0, g = 1, b = 2;
+            if( pixelFormat == GL_BGR || pixelFormat == GL_BGRA )
+            {
+                r = 2;
+                b = 0;
+            }
+
+            // Data
+            for (int y=0; y<height; ++y)
+            {
+                const unsigned char* ptr = data + y * width * numPerPixel;
+                for (int x=0; x<width; ++x)
+                {
+                    int off = x * numPerPixel;
+                    switch ( numPerPixel )
+                    {
+                    case 3:  // BGR
+                        fout.put(ptr[off+b] * pixelMultiplier); fout.put(ptr[off+g] * pixelMultiplier);
+                        fout.put(ptr[off+r] * pixelMultiplier);
+                        break;
+                    case 4:  // BGRA
+                        fout.put(ptr[off+b] * pixelMultiplier); fout.put(ptr[off+g] * pixelMultiplier);
+                        fout.put(ptr[off+r] * pixelMultiplier); fout.put(ptr[off+3] * pixelMultiplier);
+                        break;
+                    default:
+                        return false;
+                    }
+                }
+            }
+            return true;
+        }
+        
+        virtual WriteResult writeImage(const osg::Image& image, std::ostream& fout, const Options*) const
+        {
+            if (saveTGAStream(image, fout))
+                return WriteResult::FILE_SAVED;
+            else
+                return WriteResult::ERROR_IN_WRITING_FILE;
+        }
+        
+        virtual WriteResult writeImage(const osg::Image& image, const std::string& fileName, const Options* options) const
+        {
+            std::string ext = osgDB::getFileExtension(fileName);
+            if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
+            
+            osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            if (!fout) return WriteResult::ERROR_IN_WRITING_FILE;
+            return writeImage(image, fout, options);
+        }
 };
 
 // now register with Registry to instantiate the above
