@@ -3,6 +3,7 @@
 #include <osg/Notify>
 #include <osg/Geode>
 #include <osg/GL>
+#include <osg/FrameBufferObject>
 
 #include <osgDB/Registry>
 #include <osgDB/FileUtils>
@@ -176,33 +177,63 @@ toff_t libtiffOStreamSeekProc(thandle_t fd, toff_t off, int i)
 {
     std::ostream *fout = (std::ostream*)fd;
 
-    toff_t ret;
+    toff_t pos_required = 0;
+    toff_t stream_end = 0;
     switch(i)
     {
         case SEEK_SET:
-            fout->seekp(off,std::ios::beg);
-            ret = fout->tellp();
-            if(fout->bad())
-                ret = 0;
-            break;
+        {
+            if (off==0)
+            {
+                std::streampos checkEmpty = fout->tellp();
+                if(checkEmpty < 0)
+                {
+                    return 0;
+                }
+            }
+            pos_required = off;
 
+            fout->seekp(0, std::ios::end);
+            stream_end = fout->tellp();
+            break;
+        }
         case SEEK_CUR:
-            fout->seekp(off,std::ios::cur);
-            ret = fout->tellp();
-            if(fout->bad())
-                ret = 0;
-            break;
+        {
+            toff_t stream_curr = fout->tellp();
+            pos_required = stream_curr + off;
 
+            fout->seekp(0, std::ios::end);
+            stream_end = fout->tellp();
+            break;
+        }
         case SEEK_END:
-            fout->seekp(off,std::ios::end);
-            ret = fout->tellp();
-            if(fout->bad())
-                ret = 0;
+        {
+            fout->seekp(0, std::ios::end);
+            stream_end = fout->tellp();
+            pos_required = stream_end + off;
             break;
+        }
         default:
-            ret = 0;
             break;
     }
+
+    if (pos_required>stream_end)
+    {
+        // position required past the end of the stream so we need to insert extra characters to
+        // ensure the stream is big enough to encompass the new the position.
+        fout->seekp(0, std::ios::end);
+        for(toff_t i=stream_end; i<pos_required; ++i)
+        {
+            fout->put(char(0));
+        }
+    }
+
+    fout->seekp(pos_required,std::ios::beg);
+    toff_t ret = fout->tellp();
+    if (fout->bad())
+    {
+        ret = 0;
+    }
     return ret;
 }
 
@@ -216,6 +247,9 @@ simage_tiff_error(char * buffer, int buflen)
         case ERR_OPEN:
             strncpy(buffer, "TIFF loader: Error opening file", buflen);
             break;
+        case ERR_READ:
+            strncpy(buffer, "TIFF loader: Error reding/decoding file", buflen);
+            break;
         case ERR_MEM:
             strncpy(buffer, "TIFF loader: Out of memory error", buflen);
             break;
@@ -225,28 +259,65 @@ simage_tiff_error(char * buffer, int buflen)
         case ERR_TIFFLIB:
             strncpy(buffer, "TIFF loader: Illegal tiff file", buflen);
             break;
+        default:
+            strncpy(buffer, "TIFF loader: unknown error", buflen);
+            break;
     }
     return tifferror;
 }
 
 
+/// Generates a std::string from a printf format string and a va_list.
+/// Took & adapted from the man page of printf.
+///\todo Externalize this function to make is useable for all OSG?
+std::string doFormat(const char* fmt, va_list ap) {
+    static const int MSG_BUFSIZE = 256;            // Initial size of the buffer used for formatting
+    static const int MAX_BUFSIZE = 256*1024;    // Maximum size of the buffer used for formatting
+    for(int size=MSG_BUFSIZE; size<MAX_BUFSIZE; )
+    {
+        // Sukender: Here we could try/catch(std::bad_alloc &), but this is clearly an and-of-all-things condition knowing the fact 'size' is kept small.
+        //           Hence the commented code, to avoid the burden.
+        //try {
+        char * p = new char[size];
+        //} catch (std::bad_alloc &) {
+        //    return std::string();
+        //}
 
+        /* Try to print in the allocated space. */
+        int n = vsnprintf (p, size, fmt, ap);
+        // Now reset the state of the va_list (TIFF calling method will call
 
-
+        /* If that worked, return the string. */
+        if (n >= 0 && n < size) {
+            std::string res(p);
+            delete[] p;
+            return res;
+        }
+        /* Else try again with more space. */
+        if (n > 0)      /* glibc 2.1 */
+            size = n+1; /* precisely what is needed */
+        else            /* glibc 2.0 */
+            size *= 2;  /* twice the old size */
+        delete[] p;
+    }
+    return std::string(fmt, fmt+MSG_BUFSIZE) + "...";        // Fallback: Message is not formatted and truncated, but that's better than no message
+}
 
 static void
-tiff_error(const char*, const char*, va_list)
+tiff_error(const char*, const char* fmt, va_list ap)
 {
     // values are (const char* module, const char* fmt, va_list list)
     /* FIXME: store error message ? */
+    OSG_WARN << "TIFF rader: " << doFormat(fmt, ap) << std::endl;
 }
 
 
 static void
-tiff_warn(const char *, const char *, va_list)
+tiff_warn(const char*, const char* fmt, va_list ap)
 {
     // values are (const char* module, const char* fmt, va_list list)
     /* FIXME: notify? */
+    OSG_NOTICE << "TIFF rader: " << doFormat(fmt, ap) << std::endl;
 }
 
 
@@ -263,7 +334,7 @@ checkcmap(int n, uint16* r, uint16* g, uint16* b)
 static void
 invert_row(unsigned char *ptr, unsigned char *data, int n, int invert, uint16 bitspersample)
 {
-    // osg::notify(osg::NOTICE)<<"invert_row "<<invert<<std::endl;
+    // OSG_NOTICE<<"invert_row "<<invert<<std::endl;
     if (bitspersample == 8)
     {
         while (n--)
@@ -301,7 +372,7 @@ static void
 remap_row(unsigned char *ptr, unsigned char *data, int n,
 unsigned short *rmap, unsigned short *gmap, unsigned short *bmap)
 {
-    // osg::notify(osg::NOTICE)<<"remap row"<<std::endl;
+    // OSG_NOTICE<<"remap row"<<std::endl;
     unsigned int ix;
     while (n--)
     {
@@ -316,7 +387,7 @@ static void interleave_row(unsigned char *ptr,
                            unsigned char *red, unsigned char *green, unsigned char *blue,
                            int n, int numSamples, uint16 bitspersample)
 {
-    // osg::notify(osg::NOTICE)<<"Interleave row RGB"<<std::endl;
+    // OSG_NOTICE<<"Interleave row RGB"<<std::endl;
     if (bitspersample == 8)
     {
         while (n--)
@@ -363,7 +434,7 @@ static void interleave_row(unsigned char *ptr,
                            unsigned char *red, unsigned char *green, unsigned char *blue, unsigned char *alpha,
                            int n, int numSamples, uint16 bitspersample)
 {
-    // osg::notify(osg::NOTICE)<<"Interleave row RGBA"<<std::endl;
+    // OSG_NOTICE<<"Interleave row RGBA"<<std::endl;
     if (bitspersample == 8)
     {
         while (n--)
@@ -475,7 +546,7 @@ simage_tiff_load(std::istream& fin,
             photometric != PHOTOMETRIC_MINISWHITE &&
             photometric != PHOTOMETRIC_MINISBLACK)
         {
-            osg::notify(osg::NOTICE) << "Photometric type "<<photometric<<" not handled; can only handle Grayscale, RGB and Palette images" << std::endl;
+            OSG_NOTICE << "Photometric type "<<photometric<<" not handled; can only handle Grayscale, RGB and Palette images" << std::endl;
             TIFFClose(in);
             tifferror = ERR_UNSUPPORTED;
             return NULL;
@@ -495,7 +566,7 @@ simage_tiff_load(std::istream& fin,
             samplesperpixel != 3 &&
             samplesperpixel != 4)
         {
-            osg::notify(osg::DEBUG_INFO) << "Bad samples/pixel" << std::endl;
+            OSG_DEBUG << "Bad samples/pixel" << std::endl;
             tifferror = ERR_UNSUPPORTED;
             TIFFClose(in);
             return NULL;
@@ -512,7 +583,7 @@ simage_tiff_load(std::istream& fin,
     {
          if (bitspersample != 8 && bitspersample != 16 && bitspersample != 32)
         {
-            osg::notify(osg::NOTICE) << "can only handle 8, 16 and 32 bit samples" << std::endl;
+            OSG_NOTICE << "can only handle 8, 16 and 32 bit samples" << std::endl;
             TIFFClose(in);
             tifferror = ERR_UNSUPPORTED;
             return NULL;
@@ -536,7 +607,7 @@ simage_tiff_load(std::istream& fin,
 
         
     TIFFGetField(in, TIFFTAG_DATATYPE, &dataType);
-    osg::notify(osg::INFO)<<"TIFFTAG_DATATYPE="<<dataType<<std::endl;
+    OSG_INFO<<"TIFFTAG_DATATYPE="<<dataType<<std::endl;
 
 
     /*
@@ -557,9 +628,9 @@ simage_tiff_load(std::istream& fin,
     int bytespersample = bitspersample / 8;
     int bytesperpixel = bytespersample * samplesperpixel;
     
-    osg::notify(osg::INFO)<<"format="<<format<<std::endl;
-    osg::notify(osg::INFO)<<"bytespersample="<<bytespersample<<std::endl;
-    osg::notify(osg::INFO)<<"bytesperpixel="<<bytesperpixel<<std::endl;
+    OSG_INFO<<"format="<<format<<std::endl;
+    OSG_INFO<<"bytespersample="<<bytespersample<<std::endl;
+    OSG_INFO<<"bytesperpixel="<<bytesperpixel<<std::endl;
     
     buffer = new unsigned char [w*h*format];
 
@@ -725,7 +796,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             {
                 char err_msg[256];
                 simage_tiff_error( err_msg, sizeof(err_msg)); 
-                osg::notify(osg::WARN) << err_msg << std::endl;
+                OSG_WARN << err_msg << std::endl;
                 return ReadResult::FILE_NOT_HANDLED;
             }
 
@@ -783,6 +854,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             }
 
             switch(img.getPixelFormat()) {
+                case GL_DEPTH_COMPONENT:
                 case GL_LUMINANCE:
                 case GL_ALPHA:
                     photometric = PHOTOMETRIC_MINISBLACK;
@@ -866,7 +938,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             std::string fileName = osgDB::findDataFile( file, options );
             if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
 
-            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            osgDB::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
             if(!istream) return ReadResult::FILE_NOT_HANDLED;
             ReadResult rr = readTIFStream(istream);
             if(rr.validImage()) rr.getImage()->setFileName(file);
@@ -884,7 +956,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             std::string ext = osgDB::getFileExtension(fileName);
             if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
-            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
             if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
 
             return writeImage(img,fout,options);
