@@ -20,7 +20,7 @@
 #include "TXPSeamLOD.h"
 #include "TileMapper.h"
 
-#define ReaderWriterTXPERROR(s) osg::notify(osg::NOTICE) << "txp::ReaderWriterTXP::" << (s) << " error: "
+#define ReaderWriterTXPERROR(s) OSG_NOTICE << "txp::ReaderWriterTXP::" << (s) << " error: "
 
 
 
@@ -51,14 +51,14 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
         {
             txpNode->setOptions(options->getOptionString());
         }
-        
+
 
         //modified by Brad Anderegg on May-27-08
         //calling getArchive will create a new TXPArchive if the specified one does not exist
         //we will set our osgdb loader options on the archive and set the appropriate archive on 
         //the txpNode.
         int id = ++_archiveId;
-        TXPArchive* archive = getArchive(id,osgDB::getFilePath(fileName));
+        osg::ref_ptr< TXPArchive > archive = createArchive(id,osgDB::getFilePath(fileName));
 
         if (archive != NULL)
         {
@@ -69,8 +69,8 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
                archive->SetMaterialAttributesToStateSetVar(true);
             }
 
-            txpNode->loadArchive(archive);
-            
+            txpNode->loadArchive(archive.get());
+
             return txpNode.get();
         }
         else
@@ -85,7 +85,9 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
         int x,y,lod;
         unsigned int id;
         sscanf(name.c_str(),"tile%d_%dx%d_%u",&lod,&x,&y,&id);
-        TXPArchive* archive = getArchive(id,osgDB::getFilePath(file));
+        osg::ref_ptr< TXPArchive > archive = getArchive(id,osgDB::getFilePath(file));
+        if (archive == NULL)
+            return ReadResult::ERROR_IN_READING_FILE;
 
         // The way this is done a 'tile' should only be created for lod 0 only,
         // something is wrong if this is no the case
@@ -106,7 +108,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
             return ReadResult::ERROR_IN_READING_FILE;
 
         std::vector<TXPArchive::TileLocationInfo> childrenLoc;
-        osg::ref_ptr<osg::Node> tileContent = getTileContent(info,x,y,lod,archive, childrenLoc);
+        osg::ref_ptr<osg::Node> tileContent = getTileContent(info,x,y,lod,archive.get(), childrenLoc);
         
         tileContent->setName("TileContent");
 
@@ -119,25 +121,24 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
         archive->GetVersion(majorVersion, minorVersion);
         if(majorVersion ==2 && minorVersion >=1)
         {
-        // Version 2.1 and over
-        // The tile table only contains lod 0 and the children
-        // info are stored in its parent. SO if we do not want
-        // to be forced to reparse the parent we need to save that
-        // info. For now we just add it to the node name
+            // Version 2.1 and over
+            // The tile table only contains lod 0 and the children
+            // info are stored in its parent. SO if we do not want
+            // to be forced to reparse the parent we need to save that
+            // info. For now we just add it to the node name
 
-        if(childrenLoc.size() > 0)
-        {
-        asChildren = true;
-        createChildrenLocationString(childrenLoc, childrenInfoStr);
-        }
+            if(childrenLoc.size() > 0)
+            {
+                asChildren = true;
+                createChildrenLocationString(childrenLoc, childrenInfoStr);
+            }
         }
         else
         {
-        if (lod < (numLods-1))
-        asChildren = true;
+            if (lod < (numLods-1)) asChildren = true;
         }
 
-    if (asChildren)
+        if (asChildren)
         {
             char pagedLODfile[1024];
             sprintf(pagedLODfile,"%s\\subtiles%d_%dx%d_%d",
@@ -199,7 +200,9 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
         int x,y,lod;
         unsigned int id;
         sscanf(name.c_str(),"subtiles%d_%dx%d_%u",&lod,&x,&y,&id);
-        TXPArchive* archive = getArchive(id,osgDB::getFilePath(file));
+        osg::ref_ptr< TXPArchive > archive = getArchive(id,osgDB::getFilePath(file));
+        if (archive == NULL)
+            return ReadResult::ERROR_IN_READING_FILE;
 
         int majorVersion, minorVersion;
         archive->GetVersion(majorVersion, minorVersion);
@@ -218,64 +221,64 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
             std::vector<TXPArchive::TileLocationInfo> locs;
             bool status = true;
             status = extractChildrenLocations(name, lod, locs, nbChild);
-        if(majorVersion >= TRPG_NOMERGE_VERSION_MAJOR && minorVersion >=TRPG_NOMERGE_VERSION_MINOR && archive->GetHeader()->GetIsMaster())
-        {
-        for(int idx=0;idx<nbChild;idx++)
-        {
-            //figure out the block row/col
-            int blockx,blocky;
-            unsigned int denom = (1 << locs[idx].lod); // this should work up to lod 31
-            blockx = locs[idx].x/denom;
-            blocky = locs[idx].y/denom;
-            locs[idx].addr.col = blockx;
-            locs[idx].addr.row = blocky;
-        }
-        }
+            if(majorVersion >= TRPG_NOMERGE_VERSION_MAJOR && minorVersion >=TRPG_NOMERGE_VERSION_MINOR && archive->GetHeader()->GetIsMaster())
+            {
+                for(int idx=0;idx<nbChild;idx++)
+                {
+                    //figure out the block row/col
+                    int blockx,blocky;
+                    unsigned int denom = (1 << locs[idx].lod); // this should work up to lod 31
+                    blockx = locs[idx].x/denom;
+                    blocky = locs[idx].y/denom;
+                    locs[idx].addr.col = blockx;
+                    locs[idx].addr.row = blocky;
+                }
+            }
 
             if(!status)
             {
-        ReaderWriterTXPERROR("ReaderWriterTXP::local_readNode()") << "'subtile' filename children parsing failed " << std::endl;
-        return ReadResult::ERROR_IN_READING_FILE;
+                ReaderWriterTXPERROR("ReaderWriterTXP::local_readNode()") << "'subtile' filename children parsing failed " << std::endl;
+                return ReadResult::ERROR_IN_READING_FILE;
             }
 
             const trpgHeader* header = archive->GetHeader();
             trpgHeader::trpgTileType tileType;
             header->GetTileOriginType(tileType);
 
-        TXPArchive::TileLocationInfo plInfo;
-        plInfo.x = x;
-        plInfo.y = y;
-        plInfo.lod = lod;
+            TXPArchive::TileLocationInfo plInfo;
+            plInfo.x = x;
+            plInfo.y = y;
+            plInfo.lod = lod;
             TXPArchive::TileInfo parentInfo;
             archive->getTileInfo(plInfo,parentInfo);
 
             for(int idx = 0; idx < nbChild; ++idx)
             {
-        std::vector<TXPArchive::TileLocationInfo> childrenChildLoc;
-               
-        TXPArchive::TileLocationInfo& loc = locs[idx];
-               
-        TXPArchive::TileInfo info;
-        if (!archive->getTileInfo(loc,info))
-            continue;
+                std::vector<TXPArchive::TileLocationInfo> childrenChildLoc;
 
-        osg::ref_ptr<osg::Node> tileContent = getTileContent(info, loc, archive, childrenChildLoc);
+                TXPArchive::TileLocationInfo& loc = locs[idx];
 
-        tileContent->setName("TileContent");
+                TXPArchive::TileInfo info;
+                if (!archive->getTileInfo(loc,info))
+                    continue;
 
-        if(childrenChildLoc.size() > 0)
-        {
-            std::string childInfoStr;
-            createChildrenLocationString(childrenChildLoc, childInfoStr);
+                osg::ref_ptr<osg::Node> tileContent = getTileContent(info, loc, archive.get(), childrenChildLoc);
 
-            char pagedLODfile[1024];
-            sprintf(pagedLODfile,"%s\\subtiles%d_%dx%d_%d%s.txp",
-                archive->getDir(),
-                loc.lod,
-                loc.x,
-                loc.y,
-                archive->getId(),
-                childInfoStr.c_str());
+                tileContent->setName("TileContent");
+
+                if(childrenChildLoc.size() > 0)
+                {
+                    std::string childInfoStr;
+                    createChildrenLocationString(childrenChildLoc, childInfoStr);
+
+                    char pagedLODfile[1024];
+                    sprintf(pagedLODfile,"%s\\subtiles%d_%dx%d_%d%s.txp",
+                        archive->getDir(),
+                        loc.lod,
+                        loc.x,
+                        loc.y,
+                        archive->getId(),
+                        childInfoStr.c_str());
 
                     // there are tile sets which do not maintain the z extents in
                     // the tile table.  This attempt to address the issue by using
@@ -289,154 +292,154 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
                         info.radius = bSphere.radius();
                     }
 
-            osg::ref_ptr<TXPPagedLOD> pagedLOD = new TXPPagedLOD;
-                    // note: use maximum(info.maxRange,1e7) as just maxRange would result in some corner tiles from being culled out.
-            pagedLOD->addChild(tileContent.get(),info.minRange,osg::maximum(info.maxRange,1e7));
-            pagedLOD->setFileName(1,pagedLODfile);
-            pagedLOD->setRange(1,0,info.minRange);
-            pagedLOD->setCenter(info.center);
-            pagedLOD->setRadius(info.radius);
-            pagedLOD->setPriorityOffset(0,numLods - loc.lod);
-            pagedLOD->setPriorityScale(0,1.0f);
-            pagedLOD->setNumChildrenThatCannotBeExpired(1);
-            pagedLOD->setTileId(loc.x, loc.y, loc.lod);
+                    osg::ref_ptr<TXPPagedLOD> pagedLOD = new TXPPagedLOD;
+                            // note: use maximum(info.maxRange,1e7) as just maxRange would result in some corner tiles from being culled out.
+                    pagedLOD->addChild(tileContent.get(),info.minRange,osg::maximum(info.maxRange,1e7));
+                    pagedLOD->setFileName(1,pagedLODfile);
+                    pagedLOD->setRange(1,0,info.minRange);
+                    pagedLOD->setCenter(info.center);
+                    pagedLOD->setRadius(info.radius);
+                    pagedLOD->setPriorityOffset(0,numLods - loc.lod);
+                    pagedLOD->setPriorityScale(0,1.0f);
+                    pagedLOD->setNumChildrenThatCannotBeExpired(1);
+                    pagedLOD->setTileId(loc.x, loc.y, loc.lod);
 
-            if(tileType == trpgHeader::TileLocal)
-            {
-            osg::Vec3d center(info.center - parentInfo.bbox._min);
-            osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
-            sw[2] = 0.0;
-            pagedLOD->setCenter(center - sw);
-            osg::Matrix offset;
-            offset.setTrans(sw);
-            osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
-            tform->addChild(pagedLOD.get());
-            subtiles->addChild(tform);
-            }
-            else
-            subtiles->addChild(pagedLOD.get());
-            subtiles->setUserData(new TileIdentifier(loc.x, loc.y, loc.lod)); // is this really needed?
-        }
-        else
-        {
-            subtiles->setUserData(new TileIdentifier(loc.x, loc.y, loc.lod));
-            if(tileType == trpgHeader::TileLocal)
-            {
-            osg::Vec3d center(info.center - parentInfo.bbox._min);
-            osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
-            sw[2] = 0.0;
-            osg::Matrix offset;
-            offset.setTrans(sw);
-            osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
-            tform->addChild(tileContent.get());
-            subtiles->addChild(tform);
-            }
-            else
-            subtiles->addChild(tileContent.get());
-        }
+                    if(tileType == trpgHeader::TileLocal)
+                    {
+                        osg::Vec3d center(info.center - parentInfo.bbox._min);
+                        osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
+                        sw[2] = 0.0;
+                        pagedLOD->setCenter(center - sw);
+                        osg::Matrix offset;
+                        offset.setTrans(sw);
+                        osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
+                        tform->addChild(pagedLOD.get());
+                        subtiles->addChild(tform);
+                    }
+                    else
+                        subtiles->addChild(pagedLOD.get());
+                        subtiles->setUserData(new TileIdentifier(loc.x, loc.y, loc.lod)); // is this really needed?
+                }
+                else
+                {
+                    subtiles->setUserData(new TileIdentifier(loc.x, loc.y, loc.lod));
+                    if(tileType == trpgHeader::TileLocal)
+                    {
+                        osg::Vec3d center(info.center - parentInfo.bbox._min);
+                        osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
+                        sw[2] = 0.0;
+                        osg::Matrix offset;
+                        offset.setTrans(sw);
+                        osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
+                        tform->addChild(tileContent.get());
+                        subtiles->addChild(tform);
+                    }
+                    else
+                        subtiles->addChild(tileContent.get());
+                }
             }
         }
         else
         {
-            
-        int sizeX, sizeY;
-        archive->getLODSize(lod+1,sizeX,sizeY);
 
-        const trpgHeader* header = archive->GetHeader();
-        trpgHeader::trpgTileType tileType;
-        header->GetTileOriginType(tileType);
+            int sizeX, sizeY;
+            archive->getLODSize(lod+1,sizeX,sizeY);
 
-        TXPArchive::TileInfo parentInfo;
-        archive->getTileInfo(x,y,lod,parentInfo);
+            const trpgHeader* header = archive->GetHeader();
+            trpgHeader::trpgTileType tileType;
+            header->GetTileOriginType(tileType);
 
-        for (int ix = 0; ix < 2; ix++)
-        {
-        for (int iy = 0; iy < 2; iy++)
-        {
-            int tileX = x*2+ix;
-            int tileY = y*2+iy;
-            int tileLOD = lod+1;
+            TXPArchive::TileInfo parentInfo;
+            archive->getTileInfo(x,y,lod,parentInfo);
 
-            TXPArchive::TileInfo info;
-            if (!archive->getTileInfo(tileX,tileY,tileLOD,info))
-            continue;
-
-            osg::ref_ptr<osg::Node> tileContent = getTileContent(info,tileX,tileY,tileLOD,archive, childrenLoc);
-
-            tileContent->setName("TileContent");
-
-            if (tileLOD < (numLods-1))
+            for (int ix = 0; ix < 2; ix++)
             {
-            char pagedLODfile[1024];
-            sprintf(pagedLODfile,"%s\\subtiles%d_%dx%d_%d.txp",
-                archive->getDir(),
-                tileLOD,
-                tileX,
-                tileY,
-                archive->getId());
+                for (int iy = 0; iy < 2; iy++)
+                {
+                    int tileX = x*2+ix;
+                    int tileY = y*2+iy;
+                    int tileLOD = lod+1;
 
-            // there are tile sets which do not maintain the z extents in
-            // the tile table.  This attempt to address the issue by using
-            // the geometry bounding sphere.  The downside is that this is
-            // not coupled to the generation and may result in runtime cracks
-            if (info.center.z() == 0)
-            {
-                osg::BoundingSphere bSphere = tileContent->getBound();
+                    TXPArchive::TileInfo info;
+                    if (!archive->getTileInfo(tileX,tileY,tileLOD,info))
+                    continue;
 
-                info.center.z() = bSphere.center().z();
-                info.radius = bSphere.radius();
-            }
+                    osg::ref_ptr<osg::Node> tileContent = getTileContent(info,tileX,tileY,tileLOD,archive.get(), childrenLoc);
 
-            osg::ref_ptr<TXPPagedLOD> pagedLOD = new TXPPagedLOD;
-                        // note: use maximum(info.maxRange,1e7) as just maxRange would result in some corner tiles from being culled out.
-            pagedLOD->addChild(tileContent.get(),info.minRange,osg::maximum(info.maxRange,1e7));
-            pagedLOD->setFileName(1,pagedLODfile);
-            pagedLOD->setRange(1,0,info.minRange);
-            pagedLOD->setCenter(info.center);
-            pagedLOD->setRadius(info.radius);
-            pagedLOD->setPriorityOffset(0,numLods-lod);
-            pagedLOD->setPriorityScale(0,1.0f);
-            pagedLOD->setNumChildrenThatCannotBeExpired(1);
-            pagedLOD->setTileId(tileX,tileY,tileLOD);
+                    tileContent->setName("TileContent");
 
-            if(tileType == trpgHeader::TileLocal)
-            {
-                osg::Vec3d center(info.center - parentInfo.bbox._min);
-                osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
-                sw[2] = 0.0;
-                pagedLOD->setCenter(center - sw);
-                osg::Matrix offset;
-                offset.setTrans(sw);
-                osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
-                tform->addChild(pagedLOD.get());
-                subtiles->addChild(tform);
-            }
-            else
-                subtiles->addChild(pagedLOD.get());
-            }
-            else
-            {
-            subtiles->setUserData(new TileIdentifier(tileX,tileY,tileLOD));
-            if(tileType == trpgHeader::TileLocal)
-            {
-                osg::Vec3d center(info.center - parentInfo.bbox._min);
-                osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
-                sw[2] = 0.0;
-                osg::Matrix offset;
-                offset.setTrans(sw);
-                osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
-                tform->addChild(tileContent.get());
-                subtiles->addChild(tform);
-            }
-            else
-                subtiles->addChild(tileContent.get());
-            }
+                    if (tileLOD < (numLods-1))
+                    {
+                        char pagedLODfile[1024];
+                        sprintf(pagedLODfile,"%s\\subtiles%d_%dx%d_%d.txp",
+                            archive->getDir(),
+                            tileLOD,
+                            tileX,
+                            tileY,
+                            archive->getId());
 
-        }
-        }
+                        // there are tile sets which do not maintain the z extents in
+                        // the tile table.  This attempt to address the issue by using
+                        // the geometry bounding sphere.  The downside is that this is
+                        // not coupled to the generation and may result in runtime cracks
+                        if (info.center.z() == 0)
+                        {
+                            osg::BoundingSphere bSphere = tileContent->getBound();
+
+                            info.center.z() = bSphere.center().z();
+                            info.radius = bSphere.radius();
+                        }
+
+                        osg::ref_ptr<TXPPagedLOD> pagedLOD = new TXPPagedLOD;
+                                    // note: use maximum(info.maxRange,1e7) as just maxRange would result in some corner tiles from being culled out.
+                        pagedLOD->addChild(tileContent.get(),info.minRange,osg::maximum(info.maxRange,1e7));
+                        pagedLOD->setFileName(1,pagedLODfile);
+                        pagedLOD->setRange(1,0,info.minRange);
+                        pagedLOD->setCenter(info.center);
+                        pagedLOD->setRadius(info.radius);
+                        pagedLOD->setPriorityOffset(0,numLods-lod);
+                        pagedLOD->setPriorityScale(0,1.0f);
+                        pagedLOD->setNumChildrenThatCannotBeExpired(1);
+                        pagedLOD->setTileId(tileX,tileY,tileLOD);
+
+                        if(tileType == trpgHeader::TileLocal)
+                        {
+                            osg::Vec3d center(info.center - parentInfo.bbox._min);
+                            osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
+                            sw[2] = 0.0;
+                            pagedLOD->setCenter(center - sw);
+                            osg::Matrix offset;
+                            offset.setTrans(sw);
+                            osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
+                            tform->addChild(pagedLOD.get());
+                            subtiles->addChild(tform);
+                        }
+                        else
+                            subtiles->addChild(pagedLOD.get());
+                    }
+                    else
+                    {
+                        subtiles->setUserData(new TileIdentifier(tileX,tileY,tileLOD));
+                        if(tileType == trpgHeader::TileLocal)
+                        {
+                            osg::Vec3d center(info.center - parentInfo.bbox._min);
+                            osg::Vec3d sw(info.bbox._min - parentInfo.bbox._min);
+                            sw[2] = 0.0;
+                            osg::Matrix offset;
+                            offset.setTrans(sw);
+                            osg::MatrixTransform *tform = new osg::MatrixTransform(offset);
+                            tform->addChild(tileContent.get());
+                            subtiles->addChild(tform);
+                        }
+                        else
+                            subtiles->addChild(tileContent.get());
+                    }
+
+                }
+            }
         }
 
-        //osg::notify(osg::NOTICE) << "Subtiles for " << x << " " << y << " " << lod << " lodaded" << std::endl;
+        //OSG_NOTICE << "Subtiles for " << x << " " << y << " " << lod << " lodaded" << std::endl;
         return subtiles.get();
     }
     
@@ -564,67 +567,97 @@ bool ReaderWriterTXP::extractChildrenLocations(const std::string& name, int pare
 
 }
 
-TXPArchive *ReaderWriterTXP::getArchive(int id, const std::string& dir)
+std::string ReaderWriterTXP::getArchiveName(const std::string& dir)
 {
-    TXPArchive* archive = NULL;
+#ifdef _WIN32
+    const char _PATHD = '\\';
+#elif defined(macintosh)
+    const char _PATHD = ':';
+#else
+    const char _PATHD = '/';
+#endif
+
+    return dir+_PATHD+"archive.txp";
+}
+
+osg::ref_ptr< TXPArchive > ReaderWriterTXP::getArchive(int id, const std::string& dir)
+{
+    osg::ref_ptr< TXPArchive > archive = NULL;
 
     std::map< int,osg::ref_ptr<TXPArchive> >::iterator iter = _archives.find(id);
     
-
     if (iter != _archives.end())
     {
         archive = iter->second.get();
     }
-
-    if (archive == NULL)
+    else
     {
-#ifdef _WIN32
-        const char _PATHD = '\\';
-#elif defined(macintosh)
-        const char _PATHD = ':';
-#else
-        const char _PATHD = '/';
-#endif
-        std::string archiveName = dir+_PATHD+"archive.txp";
-        archive = new TXPArchive;
-        if (archive->openFile(archiveName) == false)
-        {
-            ReaderWriterTXPERROR("getArchive()") << "failed to load archive: \"" << archiveName << "\"" << std::endl;
-            return NULL;
-        }
+        std::string archiveName = getArchiveName(dir);
+        ReaderWriterTXPERROR("getArchive()") << "archive id " << id << " not found: \"" << archiveName << "\"" << std::endl;
+    }
+    return archive;
+}
 
-        if (archive->loadMaterials() == false)
-        {
-            ReaderWriterTXPERROR("getArchive()") << "failed to load materials from archive: \"" << archiveName << "\"" << std::endl;
-            return NULL;
-        }
+osg::ref_ptr< TXPArchive > ReaderWriterTXP::createArchive(int id, const std::string& dir)
+{
+    std::string archiveName = getArchiveName(dir);
 
-        if (archive->loadModels() == false)
-        {
-            ReaderWriterTXPERROR("getArchive()") << "failed to load models from archive: \"" << archiveName << "\"" << std::endl;
-            return NULL;
-        }
-
-        if (archive->loadLightAttributes() == false)
-        {
-            ReaderWriterTXPERROR("getArchive()") << "failed to load light attributes from archive: \"" << archiveName << "\"" << std::endl;
-            return NULL;
-        }
-
-        if (archive->loadTextStyles() == false)
-        {
-            ReaderWriterTXPERROR("getArchive()") << "failed to load text styles from archive: \"" << archiveName << "\"" << std::endl;
-            return NULL;
-        }
-
-        archive->setId(id);
-
-        _archives[id] = archive;
+    osg::ref_ptr< TXPArchive > archive = getArchive(id, dir);
+    if (archive != NULL)
+    {
+        ReaderWriterTXPERROR("createArchive()") << "archive id " << id << " already exists: \"" << archiveName << "\"" << std::endl;
+        return NULL;
     }
 
+    archive = new TXPArchive;
+    if (archive->openFile(archiveName) == false)
+    {
+        ReaderWriterTXPERROR("createArchive()") << "failed to load archive: \"" << archiveName << "\"" << std::endl;
+        return NULL;
+    }
+
+    if (archive->loadMaterials() == false)
+    {
+        ReaderWriterTXPERROR("createArchive()") << "failed to load materials from archive: \"" << archiveName << "\"" << std::endl;
+        return NULL;
+    }
+
+    if (archive->loadModels() == false)
+    {
+        ReaderWriterTXPERROR("createArchive()") << "failed to load models from archive: \"" << archiveName << "\"" << std::endl;
+        return NULL;
+    }
+
+    if (archive->loadLightAttributes() == false)
+    {
+        ReaderWriterTXPERROR("createArchive()") << "failed to load light attributes from archive: \"" << archiveName << "\"" << std::endl;
+        return NULL;
+    }
+
+    if (archive->loadTextStyles() == false)
+    {
+        ReaderWriterTXPERROR("createArchive()") << "failed to load text styles from archive: \"" << archiveName << "\"" << std::endl;
+        return NULL;
+    }
+
+    archive->setId(id);
+
+    _archives[id] = archive;
+
     return archive;
 }
 
+bool ReaderWriterTXP::removeArchive( int id )
+{
+    OSG_INFO<<"ReaderWriterTXP::removeArchive(id="<<id<<")"<<std::endl;
+    //return (_archives.erase(id) >= 1);
+    bool result=_archives.erase(id) >= 1;
+    OSG_WARN<<"remove archive " << id << " size " << _archives.size()
+        << " result " << result << std::endl;
+    return result;
+
+}
+
 class SeamFinder: public osg::NodeVisitor
 {
 public:
@@ -637,16 +670,16 @@ public:
     {
         for (unsigned int i = 0; i < group.getNumChildren(); i++)
         {
-        osg::Node* child = group.getChild(i);
-        osg::Node* seam = seamReplacement(child);
-        if (child != seam)
-        {
-            group.replaceChild(child,seam);
-        }
-        else
-        {
-            child->accept(*this);
-        }
+            osg::Node* child = group.getChild(i);
+            osg::Node* seam = seamReplacement(child);
+            if (child != seam)
+            {
+                group.replaceChild(child,seam);
+            }
+            else
+            {
+                child->accept(*this);
+            }
         }
     }
 
@@ -660,6 +693,8 @@ protected:
     TXPArchive *_archive;
 };
 
+#define equalDoubles(a, b) (fabs(a-b) < 0.001)
+
 osg::Node* SeamFinder::seamReplacement(osg::Node* node)
 {
     osg::Group* group = node->asGroup();
@@ -685,10 +720,10 @@ osg::Node* SeamFinder::seamReplacement(osg::Node* node)
 
         bool nonSeamChild = true;
 
-    // looks like the problem is in here - likely due to seamLOD info
-    // not being adjusted properly in tiled databases
-        // seam center is outside the bounding box of the tile
-    osg::Vec3 lodCenter = lod->getCenter();
+        // looks like the problem is in here - likely due to seamLOD info
+        // not being adjusted properly in tiled databases
+            // seam center is outside the bounding box of the tile
+        osg::Vec3 lodCenter = lod->getCenter();
 
         if(tileType == trpgHeader::TileLocal)
         {
@@ -704,32 +739,40 @@ osg::Node* SeamFinder::seamReplacement(osg::Node* node)
             tileExtents.y /= divider;
             offset[0] = _x*tileExtents.x;// + tileExtents.x*0.5;
             offset[1] = _y*tileExtents.y;// + tileExtents.y*0.5;
-        lodCenter += offset;
+            lodCenter += offset;
         }
 
         if (!_info.bbox.contains(lodCenter))
         {
-            // seams have center as the neighbour tile
-            osg::Vec3 d = _info.center - lodCenter;
-            if (((fabs(d.x())-_info.size.x()) > 0.0001) && ((fabs(d.y())-_info.size.y()) > 0.0001))
+            const osg::LOD::RangeList& rangeList = lod->getRangeList();
+            if (!rangeList.size())
             {
-                nonSeamChildren.push_back(lod);
+                // TODO: Warn here
                 continue;
             }
 
-            // low res seam has min/max ranges of lod+1 range/lod 0 range
-            if ((fabs((float)_info.minRange-lod->getMinRange(0))<0.001)&&(fabs((float)_info.lod0Range-lod->getMaxRange(0))<0.001))
+            TXPArchive::TileInfo lod_plus_one_info;
+            if (!this->_archive->getTileInfo(_x,_y,_lod+1,lod_plus_one_info))
             {
+                // TODO: Warn here
+                continue;
+            }
 
+            double lod_plus_oneSwitchInDistance =  lod_plus_one_info.maxRange;
+            double lod0SwitchInDistance =  _info.lod0Range;
+
+            // low res seam has min/max ranges of lod+1 range/lod 0 range
+            if (equalDoubles(lod_plus_oneSwitchInDistance,rangeList.at(0).first) && equalDoubles(lod0SwitchInDistance,rangeList.at(0).second))
+            {
                 if (loRes==0)
                 {
                     loRes = lod;
                     nonSeamChild = false;
                 }
             }
-
+            else
             // hi res seam has min/max ranges of 0 range/lod+1 range
-            if ((lod->getMinRange(0)==0.0f)&&(fabs(_info.minRange-lod->getMaxRange(0))<0.001))
+            if (rangeList.at(0).first==0.0 && equalDoubles(lod_plus_oneSwitchInDistance,rangeList.at(0).second))
             {
                 if (hiRes==0)
                 {
@@ -749,7 +792,7 @@ osg::Node* SeamFinder::seamReplacement(osg::Node* node)
         int dx = 0;
         int dy = 0;
         int lod = _lod;
-    osg::Vec3 lodCenter = loRes->getCenter();
+        osg::Vec3 lodCenter = loRes->getCenter();
 
         if(tileType == trpgHeader::TileLocal)
         {
@@ -765,7 +808,7 @@ osg::Node* SeamFinder::seamReplacement(osg::Node* node)
             tileExtents.y /= divider;
             offset[0] = _x*tileExtents.x;// + tileExtents.x*0.5;
             offset[1] = _y*tileExtents.y;// + tileExtents.y*0.5;
-        lodCenter += offset;
+            lodCenter += offset;
         }
 
         osg::Vec3 delta = lodCenter-_info.center;
@@ -785,7 +828,7 @@ osg::Node* SeamFinder::seamReplacement(osg::Node* node)
         }
 
         TXPSeamLOD* seam = new TXPSeamLOD(_x, _y, lod, dx, dy);
-    seam->setCenter(loRes->getCenter());
+        seam->setCenter(loRes->getCenter());
         seam->addChild(loRes->getChild(0));        // low res
         if (hiRes) 
         {
