@@ -22,6 +22,14 @@
 
 #include <trpage_sys.h>
 
+#ifndef TeAttrH
+/* was unsigned long, but it is used as if it was signed in the constructor
+ * and code that set the handle are using a signed value.
+ * 01-10-2006 David Fries
+ */
+typedef long TeAttrHdl;
+#endif
+
 // Macros we may need
 #ifndef MIN
 // {secret}
@@ -38,7 +46,12 @@
 // Current TerraPage major version
 #define TRPG_VERSION_MAJOR 2
 // Current TerraPage minor version
-#define TRPG_VERSION_MINOR 0
+#define TRPG_VERSION_MINOR 2
+
+// defined values for the version that doesn't need 
+// a merge
+#define TRPG_NOMERGE_VERSION_MAJOR 2
+#define TRPG_NOMERGE_VERSION_MINOR 2
 
 // Non-existent token
 // {secret}
@@ -91,6 +104,8 @@ typedef short trpgToken;
 #define TRPGMAT_BUMP        408
 // {secret}
 #define TRPGMAT_ATTR        409
+// {secret}
+#define TRPGMAT_HANDLE        410
 
 // {secret}
 #define TRPGMAT_TEXENV        500
@@ -112,7 +127,8 @@ typedef short trpgToken;
 
 // {secret}
 #define TRPGMODELREF        700
-
+// {secret}
+#define TRPGMODELREF2        701
 // {secret}
 #define TRPGMODELTABLE        800
 
@@ -155,6 +171,8 @@ typedef short trpgToken;
 // {secret}
 #define TRPGLIGHTATTR_COMMENT        1156
 // {secret}
+#define TRPGLIGHTATTR_HANDLE        1157
+// {secret}
 #define TRPGLIGHT                    1160
 // {secret}
 #define TRPG_LIGHT                    1160
@@ -231,7 +249,10 @@ typedef short trpgToken;
 #define TRPG_ATTACH            4000
 
 // {secret}
-#define TRPG_MAXTOKEN        4000
+#define TRPG_CHILDREF        5000
+
+// {secret}
+#define TRPG_MAXTOKEN        5000
 
 // Basic data types
 
@@ -406,6 +427,12 @@ public:
     /* Adds the TRPG_POP token to the current buffer.  This is done by objects
        that have defined children.  See Push. */
     virtual void Pop(void);
+   /* Take out the pop from the end of the buffer, if there is one */
+   /* Will return true if a pop was actually taken out */
+   virtual bool UnPop();
+  /* Take out the push from the end of the buffer, if there is one */
+   /* Will return true if a push was actually taken out */
+   virtual bool UnPush();
 
 protected:
     virtual void append(unsigned int,const char *);
@@ -472,6 +499,11 @@ public:
     // Buffer is empty
     virtual bool isEmpty(void) = 0;
 
+    // MD: making this public to unravel trpgModel read problem.
+    /* A utility function for subclasses to use to see if they would exceed an
+        outside imposed limit by reading the given number of bytes. */
+    virtual bool TestLimit(int);
+
 protected:
     trpgEndian ness;                     // Endianness of the source we're reading
     trpgEndian cpuNess;                     // Endiannees of the CPU
@@ -486,9 +518,6 @@ protected:
     /* This virtual method must be filled in by the subclass so that SkipToLimit
         will work correctly. */
     virtual bool Skip(int) = 0;             
-    /* A utility function for subclasses to use to see if they would exceed an
-        outside imposed limit by reading the given number of bytes. */
-    virtual bool TestLimit(int);
     /* Utility function that must be called after a successfull read to update
         the outside imposed read limits. */
     virtual void UpdateLimits(int);
@@ -509,6 +538,8 @@ public:
     bool isEmpty(void);                        
     // Sets the size of this read buffer.
     void SetLength(int);
+    // Gets the size of the buffer.
+    int GetLength(void) { return len; }
     /* Return a pointer to the raw data cache for this object.  Data will
         be dumped straight into here (from disk, for example) and then parsed
         by something that takes a trpgReadBuffer as input. */
@@ -533,10 +564,21 @@ public:
     virtual ~trpgCheckable(void);
     // Returns the state of the valid flag, or can be overriden by a subclass to do a more complex check.
     bool isValid(void) const;
+    
+    virtual TeAttrHdl GetHandle() const {
+        return handle;    
+    }
+    virtual void SetHandle(TeAttrHdl hdl) {
+        writeHandle = true;
+        handle = hdl;
+    }
+
 protected:
     /* Set this flag to true if your checkable structure doesn't have a complex
         check it needs to do. */
     bool valid;
+    TeAttrHdl handle;
+    bool writeHandle;
 };
 
 class trpgPrintBuffer;
@@ -589,12 +631,17 @@ protected:
  */
 TX_EXDECL class TX_CLDECL trpgwAppAddress {
 public:
-    trpgwAppAddress() {file = -1; offset = -1;};
+    trpgwAppAddress() {file = -1; offset = -1; row = -1; col = -1;};
     // Which file
     int32 file;
     // Offset within the file
     // Note: This is not a 64 bit value
     int32 offset;
+    // Row and col are used for TerraPage 2.3 archives, so we
+    // can know which block to get the appropriate file from
+    int32 row;
+    int32 col;
+
 };
 
 /* Archive File.
@@ -604,13 +651,15 @@ public:
  */
 TX_EXDECL class TX_CLDECL trpgwAppFile {
 public:
-    trpgwAppFile(trpgEndian,const char *);
+    trpgwAppFile() {valid=false;};
+    trpgwAppFile(trpgEndian,const char *,bool reuse=false);
     virtual ~trpgwAppFile(void);
     virtual bool Append(const trpgMemWriteBuffer *,const trpgMemWriteBuffer *);
     virtual bool Append(const char *,int size);
     virtual int64 Pos(void) const;
     virtual int GetLengthWritten();
     virtual bool Flush(void);
+    virtual void Init(trpgEndian,const char *,bool reuse=false);
 
     bool isValid(void) const;
 protected:
@@ -627,7 +676,10 @@ protected:
  */
 TX_EXDECL class TX_CLDECL trpgrAppFile {
 public:
+    trpgrAppFile() {valid=false;};
     trpgrAppFile(trpgEndian,const char *);
+    // real construction is here
+    virtual void Init(trpgEndian,const char *);
     virtual ~trpgrAppFile(void);
     virtual bool Read(trpgMemReadBuffer *,int32 offset);
     virtual bool Read(char *data,int32 baseOffset,int32 objOffset,int32 dataSize);
@@ -644,9 +696,14 @@ protected:
  */
 TX_EXDECL class TX_CLDECL trpgrAppFileCache {
 public:
+    trpgrAppFileCache(){;};
     trpgrAppFileCache(const char *prefix,const char *ext,int noFiles=32);
-    ~trpgrAppFileCache(void);
-    trpgrAppFile *GetFile(trpgEndian ness,int id);
+    // real construction is here
+    virtual void Init(const char *prefix,const char *ext,int noFiles);
+    virtual ~trpgrAppFileCache(void);
+    virtual trpgrAppFile *GetFile(trpgEndian ness,int id,int col,int row);
+    virtual trpgrAppFile *GetFile(trpgEndian ness,int id);
+    virtual trpgrAppFile *GetNewRAppFile(trpgEndian ness, const char *fileName);
 protected:
     // Prefix name and extension
     char baseName[1024],ext[20];
@@ -655,6 +712,8 @@ protected:
     public:
         OpenFile(void);
         int id;  // ID of open file
+        int row;
+        int col;
         trpgrAppFile *afile;
         int lastUsed;  // When the file was last accessed
     };
