@@ -31,7 +31,7 @@
 */
 TX_EXDECL class TX_CLDECL trpgr_Callback
 {
- public:
+public:
     virtual ~trpgr_Callback(void) { };
     virtual void *Parse(trpgToken,trpgReadBuffer &) { return (void *)1; };
 };
@@ -42,12 +42,12 @@ TX_EXDECL class TX_CLDECL trpgr_Callback
 */
 TX_EXDECL class TX_CLDECL trpgr_Token
 {
- public:
+public:
     trpgr_Token(void);
     trpgr_Token(int,trpgr_Callback *,bool destroy=true);
     ~trpgr_Token(void);
     void init(int,trpgr_Callback *,bool destroy=true);
-    int Token;			// Constant token value
+    int Token;            // Constant token value
     trpgr_Callback *cb; // Callback when we hit this token
     bool destroy;       // Should we call delete on the callback or not
     void Destruct(void);    // Not quite like delete
@@ -59,7 +59,7 @@ TX_EXDECL class TX_CLDECL trpgr_Token
 */
 TX_EXDECL class TX_CLDECL trpgr_Parser
 {
- public:
+public:
     trpgr_Parser(void);
     virtual ~trpgr_Parser(void);
     bool isValid(void) const;
@@ -74,11 +74,11 @@ TX_EXDECL class TX_CLDECL trpgr_Parser
     // Parse a read buffer
     virtual bool Parse(trpgReadBuffer &);
     virtual bool TokenIsValid(trpgToken);  // Check token validity
- protected:
+protected:
     void *lastObject;
- private:
+private:
     // Note: Just how slow is a map<> anyway?
-    //		 This usage is self-contained and could be replaced with an array
+    //         This usage is self-contained and could be replaced with an array
 #if defined(_WIN32)
     typedef std::map<trpgToken,trpgr_Token> tok_map;
 #else
@@ -121,14 +121,14 @@ TX_EXDECL class TX_CLDECL trpgrImageHelper
        This routine also calculates the total size, including mipmaps if they're there.
     */
     virtual bool GetImageInfoForLocalMat(const trpgLocalMaterial *locMat,
-					 const trpgMaterial **retMat,const trpgTexture **retTex,
-					 int &totSize);
+                     const trpgMaterial **retMat,const trpgTexture **retTex,
+                     int &totSize);
 
     /* Same as above, but gets info for nth image associated with this local material
      */
     virtual bool GetNthImageInfoForLocalMat(const trpgLocalMaterial *locMat, int index,
-					    const trpgMaterial **retMat,const trpgTexture **retTex,
-					    int &totSize);
+                        const trpgMaterial **retMat,const trpgTexture **retTex,
+                        int &totSize);
 
     /* Fetch the bytes for the given Local Material (and
        associated texture).  This is for Tile Local and
@@ -160,11 +160,11 @@ TX_EXDECL class TX_CLDECL trpgrImageHelper
 
     trpgrAppFileCache *GetGeoTypCache()
     {
-	return geotypCache;
+    return geotypCache;
     }
     void SetTexTable(trpgTexTable *texTable)
     {
-	this->texTable = texTable;
+    this->texTable = texTable;
     }
  protected:
     char dir[1024];
@@ -184,14 +184,14 @@ TX_EXDECL class TX_CLDECL trpgrImageHelper
 */
 TX_EXDECL class TX_CLDECL trpgr_Archive : public trpgCheckable
 {
- public:
+public:
     trpgr_Archive(void);
     virtual ~trpgr_Archive(void);
 
     virtual void SetDirectory(const char *);
-    virtual bool OpenFile(const char *);	// Open File
+    virtual bool OpenFile(const char *);    // Open File
     virtual void CloseFile(void);
-    virtual bool ReadHeader(void);		// Read header (materials, tile table. etc..)
+    virtual bool ReadHeader(void);        // Read header (materials, tile table. etc..)
     //overload that lets you specify if you want to read all the blocks now,
     //or defer reading them for later.
     virtual bool ReadHeader(bool readAllBlocks); 
@@ -215,7 +215,7 @@ TX_EXDECL class TX_CLDECL trpgr_Archive : public trpgCheckable
 
     // Utility routine to calculate the MBR of a given tile
     virtual bool trpgGetTileMBR(uint32 x,uint32 y,uint32 lod,
-				trpg3dPoint &ll,trpg3dPoint &ur) const;
+                trpg3dPoint &ll,trpg3dPoint &ur) const;
 
     trpgEndian GetEndian(void) const;
     char* getDir(void){return dir;};
@@ -223,7 +223,7 @@ TX_EXDECL class TX_CLDECL trpgr_Archive : public trpgCheckable
     virtual trpgrAppFileCache* GetNewRAppFileCache(const char *fullBase, const char *ext);
     virtual int32 GetHeaderData(char *dataPtr,int32 length,FILE *filehandle);
     virtual int32 GetMagicNumber() {return TRPG_MAGIC;};
- protected:
+protected:
     bool headerRead;
     trpgEndian ness;
     FILE *fp;
@@ -258,10 +258,10 @@ TX_EXDECL class TX_CLDECL trpgSceneParser : public trpgr_Parser
     friend class trpgSceneHelperPush;
     friend class trpgSceneHelperPop;
     friend class trpgSceneHelperDefault;
- public:
+public:
     trpgSceneParser(void);
     virtual ~trpgSceneParser(void);
- protected:
+protected:
     // Start defining children for the given object
     virtual bool StartChildren(void *) { return true;};
     virtual bool EndChildren(void *) { return true;};
