@@ -33,9 +33,9 @@ Locator::Locator():
 Locator::Locator(const Locator& locator,const osg::CopyOp& copyop):
     osg::Object(locator,copyop),
     _coordinateSystemType(locator._coordinateSystemType),
-    _ellipsoidModel(locator._ellipsoidModel),
     _format(locator._format),
     _cs(locator._cs),
+    _ellipsoidModel(locator._ellipsoidModel),
     _transform(locator._transform),
     _definedInFile(locator._definedInFile),
     _transformScaledByResolution(locator._transformScaledByResolution)
@@ -52,9 +52,11 @@ void Locator::setTransformAsExtents(double minX, double minY, double maxX, doubl
                    0.0,       maxY-minY, 0.0, 0.0,
                    0.0,       0.0,       1.0, 0.0,
                    minX,      minY,      0.0, 1.0); 
+
+    _inverse.invert(_transform);
 }
 
-bool Locator::computeLocalBounds(Locator& source, osg::Vec3d& bottomLeft, osg::Vec3d& topRight)
+bool Locator::computeLocalBounds(Locator& source, osg::Vec3d& bottomLeft, osg::Vec3d& topRight) const
 {
     typedef std::list<osg::Vec3d> Corners;
     Corners corners;
@@ -83,7 +85,14 @@ bool Locator::computeLocalBounds(Locator& source, osg::Vec3d& bottomLeft, osg::V
     if (corners.empty()) return false;
 
 
-    for(Corners::iterator itr = corners.begin();
+    Corners::iterator itr = corners.begin();
+
+    bottomLeft.x() = topRight.x() = itr->x();
+    bottomLeft.y() = topRight.y() = itr->y();
+
+    ++itr;
+
+    for(;
         itr != corners.end();
         ++itr)
     {
