@@ -58,10 +58,11 @@ osg::ref_ptr<TerrainTile::TileLoadedCallback>& TerrainTile::getTileLoadedCallbac
 
 TerrainTile::TerrainTile():
     _terrain(0),
-    _dirty(false),
+    _dirtyMask(NOT_DIRTY),
     _hasBeenTraversal(false),
     _requiresNormals(true),
-    _treatBoundariesToValidDataAsDefaultValue(false)
+    _treatBoundariesToValidDataAsDefaultValue(false),
+    _blendingPolicy(INHERIT)
 {
     setThreadSafeRefUnref(true);
 }
@@ -69,12 +70,13 @@ TerrainTile::TerrainTile():
 TerrainTile::TerrainTile(const TerrainTile& terrain,const osg::CopyOp& copyop):
     Group(terrain,copyop),
     _terrain(0),
-    _dirty(false),
+    _dirtyMask(NOT_DIRTY),
     _hasBeenTraversal(false),
     _elevationLayer(terrain._elevationLayer),
     _colorLayers(terrain._colorLayers),
     _requiresNormals(terrain._requiresNormals),
-    _treatBoundariesToValidDataAsDefaultValue(terrain._treatBoundariesToValidDataAsDefaultValue)
+    _treatBoundariesToValidDataAsDefaultValue(terrain._treatBoundariesToValidDataAsDefaultValue),
+    _blendingPolicy(terrain._blendingPolicy)
 {
     if (terrain.getTerrainTechnique()) 
     {
@@ -84,6 +86,11 @@ TerrainTile::TerrainTile(const TerrainTile& terrain,const osg::CopyOp& copyop):
 
 TerrainTile::~TerrainTile()
 {
+    if (_terrainTechnique.valid())
+    {
+        _terrainTechnique->setTerrainTile(0);
+    }
+
     if (_terrain) setTerrain(0);
 }
 
@@ -126,15 +133,15 @@ void TerrainTile::traverse(osg::NodeVisitor& nv)
                     osgTerrain::Terrain* ts = dynamic_cast<Terrain*>(*itr);
                     if (ts) 
                     {
-                        osg::notify(osg::INFO)<<"Assigning terrain system "<<ts<<std::endl;                        
+                        OSG_INFO<<"Assigning terrain system "<<ts<<std::endl;
                         setTerrain(ts);
                     }
                 }
             }
         }
-        
-        init();
-                    
+
+        init(getDirtyMask(), false);
+
         _hasBeenTraversal = true;
     }
 
@@ -157,10 +164,10 @@ void TerrainTile::traverse(osg::NodeVisitor& nv)
     }
 }
 
-void TerrainTile::init()
+void TerrainTile::init(int dirtyMask, bool assumeMultiThreaded)
 {
     if (!_terrainTechnique)
-    {        
+    {
         if (_terrain && _terrain->getTerrainTechniquePrototype())
         {            
             osg::ref_ptr<osg::Object> object = _terrain->getTerrainTechniquePrototype()->clone(osg::CopyOp::DEEP_COPY_ALL);
@@ -172,51 +179,63 @@ void TerrainTile::init()
         }
     }
 
-    if (_terrainTechnique.valid() && getDirty())
+    if (_terrainTechnique.valid())
     {
-        _terrainTechnique->init();
-        
-        setDirty(false);
-    }    
+        _terrainTechnique->init(dirtyMask, assumeMultiThreaded);
+    }
 }
 
 void TerrainTile::setTerrainTechnique(TerrainTechnique* terrainTechnique)
 {
     if (_terrainTechnique == terrainTechnique) return; 
 
-    int dirtyDelta = _dirty ? -1 : 0;
+    int dirtyDelta = (_dirtyMask==NOT_DIRTY) ? 0 : -1;
 
     if (_terrainTechnique.valid()) 
     {
-        _terrainTechnique->_terrainTile = 0;
+        _terrainTechnique->setTerrainTile(0);
     }
 
     _terrainTechnique = terrainTechnique;
     
     if (_terrainTechnique.valid()) 
     {
-        _terrainTechnique->_terrainTile = this;
+        _terrainTechnique->setTerrainTile(this);
         ++dirtyDelta;        
     }
     
-    if (dirtyDelta>0) setDirty(true);
-    else if (dirtyDelta<0) setDirty(false);
+    if (dirtyDelta>0) setDirtyMask(ALL_DIRTY);
+    else if (dirtyDelta<0) setDirtyMask(NOT_DIRTY);
 }
 
-void TerrainTile::setDirty(bool dirty)
+void TerrainTile::setDirtyMask(int dirtyMask)
 {
-    if (_dirty==dirty) return;
+    if (_dirtyMask==dirtyMask) return;
 
-    _dirty = dirty;
+    int dirtyDelta = (_dirtyMask==NOT_DIRTY) ? 0 : -1;
 
-    if (_dirty)
+    _dirtyMask = dirtyMask;
+
+    if (_dirtyMask!=NOT_DIRTY) dirtyDelta += 1;
+
+#if 1
+    if (dirtyDelta>0)
+    {
+        // need to signal that we need an update
+        if (_terrain) _terrain->updateTerrainTileOnNextFrame(this);
+    }
+#else
+
+    // setNumChildrenRequeingUpdateTraversal() isn't thread safe so should avoid using it.
+    if (dirtyDelta>0)
     {
         setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()+1);
     }
-    else if (getNumChildrenRequiringUpdateTraversal()>0) 
+    else if (dirtyDelta<0 && getNumChildrenRequiringUpdateTraversal()>0)
     {
         setNumChildrenRequiringUpdateTraversal(getNumChildrenRequiringUpdateTraversal()-1);
     }
+#endif
 }
 
 
@@ -463,3 +482,14 @@ void WhiteListTileLoadedCallback::loaded(osgTerrain::TerrainTile* tile, const os
 
     }
 }
+
+void TerrainTile::releaseGLObjects(osg::State* state) const
+{
+    Group::releaseGLObjects(state);
+
+    if (_terrainTechnique.valid())
+    {
+        _terrainTechnique->releaseGLObjects( state );
+    }
+}
+
