@@ -13,6 +13,7 @@
 
 
 #include <osgText/TextBase>
+#include <osgText/Font>
 
 #include <osg/Math>
 #include <osg/GL>
@@ -24,17 +25,15 @@
 
 #include <osgDB/ReadFile>
 
-#include "DefaultFont.h"
-
 using namespace osg;
 using namespace osgText;
 
 //#define TREES_CODE_FOR_MAKING_SPACES_EDITABLE
 
 TextBase::TextBase():
+    _color(1.0f,1.0f,1.0f,1.0f),
     _fontSize(32,32),
     _characterHeight(32),
-    _characterAspectRatio(1.0f),
     _characterSizeMode(OBJECT_COORDS),
     _maximumWidth(0.0f),
     _maximumHeight(0.0f),
@@ -44,19 +43,23 @@ TextBase::TextBase():
     _autoRotateToScreen(false),
     _layout(LEFT_TO_RIGHT),
     _drawMode(TEXT),
+    _textBBMargin(0.0f),
+    _textBBColor(0.0, 0.0, 0.0, 0.5),
     _kerningType(KERNING_DEFAULT),
     _lineCount(0)
 {
-    setStateSet(DefaultFont::instance()->getStateSet());
+    setStateSet(Font::getDefaultFont()->getStateSet());
     setUseDisplayList(false);
     setSupportsDisplayList(false);
 }
 
 TextBase::TextBase(const TextBase& textBase,const osg::CopyOp& copyop):
     osg::Drawable(textBase,copyop),
+    _color(textBase._color),
+    _font(textBase._font),
+    _style(textBase._style),
     _fontSize(textBase._fontSize),
     _characterHeight(textBase._characterHeight),
-    _characterAspectRatio(textBase._characterAspectRatio),
     _characterSizeMode(textBase._characterSizeMode),
     _maximumWidth(textBase._maximumWidth),
     _maximumHeight(textBase._maximumHeight),
@@ -69,6 +72,8 @@ TextBase::TextBase(const TextBase& textBase,const osg::CopyOp& copyop):
     _autoRotateToScreen(textBase._autoRotateToScreen),
     _layout(textBase._layout),
     _drawMode(textBase._drawMode),
+    _textBBMargin(textBase._textBBMargin),
+    _textBBColor(textBase._textBBColor),
     _kerningType(textBase._kerningType),
     _lineCount(textBase._lineCount)
 {
@@ -78,19 +83,47 @@ TextBase::~TextBase()
 {
 }
 
+void TextBase::setColor(const osg::Vec4& color)
+{
+    _color = color;
+}
+
+
+void TextBase::setFont(osg::ref_ptr<Font> font)
+{
+    if (_font==font) return;
+
+    _font = font;
+
+    computeGlyphRepresentation();
+}
+
+void TextBase::setFont(const std::string& fontfile)
+{
+    setFont(readRefFontFile(fontfile));
+}
+
 void TextBase::setFontResolution(unsigned int width, unsigned int height)
 {
     _fontSize = FontResolution(width,height);
     computeGlyphRepresentation();
 }
 
-void TextBase::setCharacterSize(float height,float aspectRatio)
+void TextBase::setCharacterSize(float height)
 {
     _characterHeight = height;
-    _characterAspectRatio = aspectRatio;
     computeGlyphRepresentation();
 }
 
+void TextBase::setCharacterSize(float height, float aspectRatio)
+{
+    if (getCharacterAspectRatio()!=aspectRatio)
+    {
+        getOrCreateStyle()->setWidthRatio(aspectRatio);
+    }
+    setCharacterSize(height);
+}
+
 void TextBase::setMaximumWidth(float maximumWidth)
 {
     _maximumWidth = maximumWidth;
@@ -147,7 +180,7 @@ void TextBase::setAlignment(AlignmentType alignment)
     if (_alignment==alignment) return;
     
     _alignment = alignment;
-    computePositions();
+    computeGlyphRepresentation();
 }
 
 void TextBase::setAxisAlignment(AxisAlignment axis)
@@ -225,6 +258,16 @@ void TextBase::setDrawMode(unsigned int mode)
 }
 
 
+void TextBase::setBoundingBoxMargin(float margin)
+{
+    if (_textBBMargin == margin)
+        return;
+
+    _textBBMargin = margin;
+    computeGlyphRepresentation();
+}
+
+
 osg::BoundingBox TextBase::computeBound() const
 {
     osg::BoundingBox  bbox;
@@ -233,27 +276,40 @@ osg::BoundingBox TextBase::computeBound() const
     {
         for(unsigned int i=0;i<_autoTransformCache.size();++i)
         {
-            if (_autoTransformCache[i]._traversalNumber<0 && (_characterSizeMode!=OBJECT_COORDS || _autoRotateToScreen))
+            if (_autoTransformCache[i]._traversalNumber>=0)
             {
-                // _autoTransformCache is not valid so don't take it into accoumt when compute bounding volume.
-#if 0
-                // so fallback to estimating the bounding box size by assuming a scale of 1
-                // but might cause problems due to small feature culling...
-                osg::Matrix matrix;
-                matrix.makeTranslate(_position);
-                bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMin(),_textBB.zMin())*matrix);
-                bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMax(),_textBB.zMax())*matrix);
-#endif
-            }
-            else
-            {            
                 osg::Matrix& matrix = _autoTransformCache[i]._matrix;
                 bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMin(),_textBB.zMin())*matrix);
                 bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMax(),_textBB.zMax())*matrix);
             }
         }
+
+
+        if (!bbox.valid())
+        {
+            // Provide a fallback in cases where no bounding box has been been setup so far.
+            // Note, assume a scaling of 1.0 for _characterSizeMode!=OBJECT_COORDS as the
+            // for screen space coordinates size modes we don't know what scale will be used until
+            // the text is actually rendered, but we haven't to assume something otherwise the
+            // text label will be culled by view or small feature culling on first frame.
+            if (_autoRotateToScreen)
+            {
+                // use bounding sphere encompassing the maximum size of the text centered on the _position
+                double radius = _textBB.radius();
+                osg::Vec3 diagonal(radius, radius, radius);
+                bbox.set(_position-diagonal, _position+diagonal);
+            }
+            else
+            {
+                osg::Matrix matrix;
+                matrix.makeTranslate(_position);
+                matrix.preMultRotate(_rotation);
+                bbox.expandBy(osg::Vec3(_textBB.xMin(),_textBB.yMin(),_textBB.zMin())*matrix);
+                bbox.expandBy(osg::Vec3(_textBB.xMax(),_textBB.yMax(),_textBB.zMax())*matrix);
+            }
+        }
     }
-    
+
     return bbox;
 }
 
