@@ -27,7 +27,7 @@ void TangentSpaceGenerator::generate(osg::Geometry *geo, int normal_map_tex_unit
     if (geo->suitableForOptimization())
     {
         // removing coord indices so we don't have to deal with them in the binormal code.
-        osg::notify(osg::INFO)<<"TangentSpaceGenerator::generate(Geometry*,int): Removing attribute indices"<<std::endl;
+        OSG_INFO<<"TangentSpaceGenerator::generate(Geometry*,int): Removing attribute indices"<<std::endl;
         geo->copyToAndOptimize(*geo);
     }
 
@@ -55,12 +55,12 @@ void TangentSpaceGenerator::generate(osg::Geometry *geo, int normal_map_tex_unit
         }
     }
 
-    unsigned i; // VC6 doesn't like for-scoped variables
+    unsigned int i; // VC6 doesn't like for-scoped variables
 
-    for (unsigned pri=0; pri<geo->getNumPrimitiveSets(); ++pri) {
+    for (unsigned int pri=0; pri<geo->getNumPrimitiveSets(); ++pri) {
         osg::PrimitiveSet *pset = geo->getPrimitiveSet(pri);
 
-        unsigned N = pset->getNumIndices();
+        unsigned int N = pset->getNumIndices();
 
         switch (pset->getMode()) {
 
@@ -80,9 +80,9 @@ void TangentSpaceGenerator::generate(osg::Geometry *geo, int normal_map_tex_unit
             case osg::PrimitiveSet::TRIANGLE_STRIP:
                 if (pset->getType() == osg::PrimitiveSet::DrawArrayLengthsPrimitiveType) {
                     osg::DrawArrayLengths *dal = static_cast<osg::DrawArrayLengths *>(pset);
-                    unsigned j = 0;
+                    unsigned int j = 0;
                     for (osg::DrawArrayLengths::const_iterator pi=dal->begin(); pi!=dal->end(); ++pi) {
-                        unsigned iN = static_cast<unsigned>(*pi-2);
+                        unsigned int iN = static_cast<unsigned int>(*pi-2);
                         for (i=0; i<iN; ++i, ++j) {
                             if ((i%2) == 0) {
                                 compute(pset, vx, nx, tx, j, j+1, j+2);
@@ -104,11 +104,12 @@ void TangentSpaceGenerator::generate(osg::Geometry *geo, int normal_map_tex_unit
                 break;
 
             case osg::PrimitiveSet::TRIANGLE_FAN:
+            case osg::PrimitiveSet::POLYGON:
                 if (pset->getType() == osg::PrimitiveSet::DrawArrayLengthsPrimitiveType) {
                     osg::DrawArrayLengths *dal = static_cast<osg::DrawArrayLengths *>(pset);
-                    unsigned j = 0;
+                    unsigned int j = 0;
                     for (osg::DrawArrayLengths::const_iterator pi=dal->begin(); pi!=dal->end(); ++pi) {
-                        unsigned iN = static_cast<unsigned>(*pi-2);
+                        unsigned int iN = static_cast<unsigned int>(*pi-2);
                         for (i=0; i<iN; ++i) {
                             compute(pset, vx, nx, tx, 0, j+1, j+2);
                         }
@@ -116,7 +117,7 @@ void TangentSpaceGenerator::generate(osg::Geometry *geo, int normal_map_tex_unit
                     }
                 } else {
                     for (i=0; i<N-2; ++i) {
-                        compute(pset, vx, nx, tx, 2, i+1, i+2);
+                        compute(pset, vx, nx, tx, 0, i+1, i+2);
                     }
                 }
                 break;
@@ -125,9 +126,11 @@ void TangentSpaceGenerator::generate(osg::Geometry *geo, int normal_map_tex_unit
             case osg::PrimitiveSet::LINES:
             case osg::PrimitiveSet::LINE_STRIP:
             case osg::PrimitiveSet::LINE_LOOP:
+            case osg::PrimitiveSet::LINES_ADJACENCY:
+            case osg::PrimitiveSet::LINE_STRIP_ADJACENCY:
                 break;
 
-            default: osg::notify(osg::WARN) << "Warning: TangentSpaceGenerator: unknown primitive mode " << pset->getMode() << "\n";
+            default: OSG_WARN << "Warning: TangentSpaceGenerator: unknown primitive mode " << pset->getMode() << "\n";
         }
     }
 
@@ -199,7 +202,7 @@ void TangentSpaceGenerator::compute(osg::PrimitiveSet *pset,
         break;
 
     default:
-        osg::notify(osg::WARN) << "Warning: TangentSpaceGenerator: vertex array must be Vec2Array, Vec3Array or Vec4Array" << std::endl;
+        OSG_WARN << "Warning: TangentSpaceGenerator: vertex array must be Vec2Array, Vec3Array or Vec4Array" << std::endl;
     }
 
     osg::Vec3 N1;
@@ -233,7 +236,7 @@ void TangentSpaceGenerator::compute(osg::PrimitiveSet *pset,
             break;
 
         default:
-            osg::notify(osg::WARN) << "Warning: TangentSpaceGenerator: normal array must be Vec2Array, Vec3Array or Vec4Array" << std::endl;
+            OSG_WARN << "Warning: TangentSpaceGenerator: normal array must be Vec2Array, Vec3Array or Vec4Array" << std::endl;
         }
     }
 
@@ -266,7 +269,7 @@ void TangentSpaceGenerator::compute(osg::PrimitiveSet *pset,
         break;
 
     default:
-        osg::notify(osg::WARN) << "Warning: TangentSpaceGenerator: texture coord array must be Vec2Array, Vec3Array or Vec4Array" << std::endl;
+        OSG_WARN << "Warning: TangentSpaceGenerator: texture coord array must be Vec2Array, Vec3Array or Vec4Array" << std::endl;
     }
 
     if(nx){
