@@ -39,7 +39,7 @@ static pascal OSStatus GraphicsWindowEventHandler(EventHandlerCallRef nextHandle
     Rect                bounds;
     OSStatus            result = eventNotHandledErr; /* report failure by default */
     
-    osg::notify(osg::INFO) << "GraphicsWindowEventHandler" << std::endl;
+    OSG_INFO << "GraphicsWindowEventHandler" << std::endl;
 
     GraphicsWindowCarbon* w = (GraphicsWindowCarbon*)userData;
     if (!w)
@@ -246,7 +246,9 @@ void GraphicsWindowCarbon::init()
     _window = NULL;
     _pixelFormat = PixelBufferCarbon::createPixelFormat(_traits.get());
     if (!_pixelFormat)
-        osg::notify(osg::WARN) << "GraphicsWindowCarbon::init could not create a valid pixelformat" << std::endl;
+    {
+        OSG_WARN << "GraphicsWindowCarbon::init could not create a valid pixelformat" << std::endl;
+    }
     _valid = (_pixelFormat != NULL);
     _initialized = true;
 }
@@ -274,7 +276,7 @@ bool GraphicsWindowCarbon::setWindowDecorationImplementation(bool flag)
 
         if (err != noErr)
         {
-            osg::notify(osg::WARN) << "GraphicsWindowCarbon::setWindowDecoration failed with " << err << std::endl;
+            OSG_WARN << "GraphicsWindowCarbon::setWindowDecoration failed with " << err << std::endl;
             return false;
         }
         
@@ -343,19 +345,18 @@ void GraphicsWindowCarbon::installEventHandler() {
 
 bool GraphicsWindowCarbon::realizeImplementation()
 {
-    
     if (!_initialized) init();
     if (!_initialized) return false;
     if (!_traits) return false;
-    
-    osg::notify(osg::INFO) << "GraphicsWindowCarbon:: realizeIMplementation" << std::endl;
+ 
+    OSG_INFO << "GraphicsWindowCarbon::realizeImplementation" << std::endl;
     
     setWindowDecoration(_traits->windowDecoration);
     useCursor(_traits->useCursor);
 
     // move the window to the right screen
     DarwinWindowingSystemInterface* wsi = dynamic_cast<DarwinWindowingSystemInterface*>(osg::GraphicsContext::getWindowingSystemInterface());
-    int screenLeft(0), screenTop(0);
+    int screenLeft = 0, screenTop = 0;
     if (wsi) 
     {
         wsi->getScreenTopLeft((*_traits), screenLeft, screenTop);
@@ -375,10 +376,10 @@ bool GraphicsWindowCarbon::realizeImplementation()
         err = CreateNewWindow(kDocumentWindowClass, attr, &bounds, &_window);
 
         if (err) {
-            osg::notify(osg::WARN) << "GraphicsWindowCarbon::realizeImplementation() failed creating a window: " << err << std::endl;
+            OSG_WARN << "GraphicsWindowCarbon::realizeImplementation: failed to create window: " << err << std::endl;
             return false;
         } else {
-            osg::notify(osg::INFO) << "GraphicsWindowCarbon::realizeImplementation() - window created with bounds(" << bounds.top << ", " << bounds.left << ", " << bounds.bottom << ", " << bounds.right << ")" << std::endl;
+            OSG_INFO << "GraphicsWindowCarbon::realizeImplementation: window created with bounds(" << bounds.top << ", " << bounds.left << ", " << bounds.bottom << ", " << bounds.right << ")" << std::endl;
         }
     }
     else {
@@ -398,23 +399,15 @@ bool GraphicsWindowCarbon::realizeImplementation()
     // create the context
     AGLContext sharedContextCarbon = NULL;
     
-    GraphicsWindowCarbon* graphicsWindowCarbon = dynamic_cast<GraphicsWindowCarbon*>(_traits->sharedContext);
-    if (graphicsWindowCarbon) 
+    GraphicsHandleCarbon* graphicsHandleCarbon = dynamic_cast<GraphicsHandleCarbon*>(_traits->sharedContext);
+    if (graphicsHandleCarbon) 
     {
-        sharedContextCarbon = graphicsWindowCarbon->getAGLContext();
-    }
-    else
-    {
-        PixelBufferCarbon* pixelbuffer = dynamic_cast<PixelBufferCarbon*>(_traits->sharedContext);
-        if (pixelbuffer) {
-            sharedContextCarbon = pixelbuffer->getAGLContext();
-        }
+        sharedContextCarbon = graphicsHandleCarbon->getAGLContext();
     }
+
     _context = aglCreateContext (_pixelFormat, sharedContextCarbon);
-
-
     if (!_context) {
-        osg::notify(osg::WARN) << "GraphicsWindowCarbon::realizeImplementation failed creating a context: " << aglGetError() << std::endl;
+        OSG_WARN << "GraphicsWindowCarbon::realizeImplementation: failed to create context: " << aglGetError() << std::endl;
         return false;
     }
 
@@ -445,17 +438,18 @@ bool GraphicsWindowCarbon::realizeImplementation()
 #endif    
         if (cgerr != kCGLNoError )
         {
-            osg::notify(osg::INFO) << "GraphicsWindowCarbon:: Multi-threaded OpenGL Execution not available" << std::endl;
+            OSG_INFO << "GraphicsWindowCarbon::realizeImplementation: multi-threaded OpenGL Execution not available" << std::endl;
         } 
     }
-    
+
     InitCursor();
-    
-    //enable vsync
+
+    // enable vsync
     if (_traits->vsync) {
         GLint swap = 1;
         aglSetInteger (_context, AGL_SWAP_INTERVAL, &swap);
     }
+    _currentVSync = _traits->vsync;
 
     _realized = true;
     return _realized;
@@ -473,12 +467,12 @@ bool GraphicsWindowCarbon::releaseContextImplementation()
 {
     if (!_realized)
     {
-        osg::notify(osg::NOTICE)<<"Warning: GraphicsWindow not realized, cannot do makeCurrent."<<std::endl;
+        OSG_NOTICE<<"Warning: GraphicsWindow not realized, cannot do makeCurrent."<<std::endl;
         return false;
     }
 
-    // osg::notify(osg::NOTICE)<<"makeCurrentImplementation "<<this<<" "<<OpenThreads::Thread::CurrentThread()<<std::endl;
-    // osg::notify(osg::NOTICE)<<"   glXMakeCurrent ("<<_display<<","<<_window<<","<<_glxContext<<std::endl;
+    // OSG_NOTICE<<"makeCurrentImplementation "<<this<<" "<<OpenThreads::Thread::CurrentThread()<<std::endl;
+    // OSG_NOTICE<<"   glXMakeCurrent ("<<_display<<","<<_window<<","<<_glxContext<<std::endl;
     return (aglSetCurrentContext(NULL) == GL_TRUE);
 }
 
@@ -486,10 +480,10 @@ bool GraphicsWindowCarbon::releaseContextImplementation()
 
 void GraphicsWindowCarbon::closeImplementation()
 {
-    // osg::notify(osg::INFO) << "GraphicsWindowCarbon::closeImplementation" << std::endl;
+    // OSG_INFO << "GraphicsWindowCarbon::closeImplementation" << std::endl;
     _valid = false;
     _realized = false;
-    
+
     // there's a possibility that the MenubarController is destructed already, so prevent a crash:
     MenubarController* mbc = MenubarController::instance();
     if (mbc) mbc->detachWindow(this);
@@ -499,7 +493,7 @@ void GraphicsWindowCarbon::closeImplementation()
         aglDestroyPixelFormat(_pixelFormat);
         _pixelFormat = NULL;
     }
-    
+ 
     if (_context) 
     {
         aglSetDrawable(_context, NULL);
@@ -507,7 +501,7 @@ void GraphicsWindowCarbon::closeImplementation()
         aglDestroyContext(_context);
         _context = NULL;
     }
-    
+ 
     if (_ownsWindow && _window) DisposeWindow(_window);
     _window = NULL;
 }
@@ -516,6 +510,16 @@ void GraphicsWindowCarbon::closeImplementation()
 
 void GraphicsWindowCarbon::swapBuffersImplementation()
 {
+    // check for vsync change
+    if (_traits.valid() && _traits->vsync != _currentVSync)
+    {
+        const bool on = _traits->vsync;
+        GLint swap = (on ? 1 : 0);
+        aglSetInteger (_context, AGL_SWAP_INTERVAL, &swap);
+        OSG_NOTICE << "GraphicsWindowCarbon: VSync=" << (on ? "on" : "off") << std::endl;
+        _currentVSync = on;
+    }
+
     aglSwapBuffers(_context);
 }
 
@@ -570,7 +574,7 @@ bool GraphicsWindowCarbon::handleMouseEvent(EventRef theEvent)
         if (mouseButton==3) mouseButton = 2;
         else if (mouseButton==2) mouseButton = 3;
         
-        // check tablet pointer device and map it to a musebutton
+        // check tablet pointer device and map it to a mouse button
         TabletProximityRec    theTabletRecord;    // The Tablet Proximity Record
         // Extract the Tablet Proximity reccord from the event.
         if(noErr == GetEventParameter(theEvent, kEventParamTabletProximityRec,
@@ -589,7 +593,7 @@ bool GraphicsWindowCarbon::handleMouseEvent(EventRef theEvent)
                     pointerType = osgGA::GUIEventAdapter::PUCK;
                     break;
                     
-                case 3: //eraser
+                case 3: // eraser
                     pointerType = osgGA::GUIEventAdapter::ERASER;
                     break;
 
@@ -619,8 +623,8 @@ bool GraphicsWindowCarbon::handleMouseEvent(EventRef theEvent)
         {
             case kEventMouseDown:
                 {
-                    float mx =wheresMyMouse.h;
-                    float my =wheresMyMouse.v;
+                    float mx = wheresMyMouse.h;
+                    float my = wheresMyMouse.v;
                     transformMouseXY(mx, my);
                     
                     lastEmulatedMouseButton = 0;
@@ -645,8 +649,8 @@ bool GraphicsWindowCarbon::handleMouseEvent(EventRef theEvent)
                 break;
             case kEventMouseUp:
                 {
-                    float mx =wheresMyMouse.h;
-                    float my =wheresMyMouse.v;
+                    float mx = wheresMyMouse.h;
+                    float my = wheresMyMouse.v;
                     transformMouseXY(mx, my);
                     if (lastEmulatedMouseButton > 0) {
                         getEventQueue()->mouseButtonRelease(mx, my, lastEmulatedMouseButton);
@@ -666,11 +670,10 @@ bool GraphicsWindowCarbon::handleMouseEvent(EventRef theEvent)
                                     sizeof(TabletPointRec), NULL, (void *)&theTabletRecord)) {
                     
                         getEventQueue()->penPressure(theTabletRecord.pressure / 65535.0f);
-            
                     }
                     
-                    float mx =wheresMyMouse.h;
-                    float my =wheresMyMouse.v;
+                    float mx = wheresMyMouse.h;
+                    float my = wheresMyMouse.v;
                     transformMouseXY(mx, my);
                     getEventQueue()->mouseMotion(mx, my);
                 }
@@ -709,23 +712,23 @@ bool GraphicsWindowCarbon::handleMouseEvent(EventRef theEvent)
             case 11:
                 {
                     enum
-                        {
+                    {
                         kEventParamMouseWheelSmoothVerticalDelta       = 'saxy', // typeSInt32
                         kEventParamMouseWheelSmoothHorizontalDelta     = 'saxx' // typeSInt32
-                        };
+                    };
                     
                     SInt32 scroll_delta_x = 0;
                     SInt32 scroll_delta_y = 0;
                     OSErr err = noErr;
                     err = GetEventParameter( theEvent, kEventParamMouseWheelSmoothVerticalDelta, typeLongInteger, NULL, sizeof(scroll_delta_y), NULL, &scroll_delta_y );
                     err = GetEventParameter( theEvent, kEventParamMouseWheelSmoothHorizontalDelta, typeLongInteger, NULL, sizeof(scroll_delta_x), NULL, &scroll_delta_x );
-                    
+
                     if ((scroll_delta_x != 0) || (scroll_delta_y != 0)) {
                         getEventQueue()->mouseScroll2D( scroll_delta_x, scroll_delta_y);
                     }
                 }
                 break;
-            
+ 
             default:
                 return false;
         }
@@ -745,7 +748,7 @@ bool GraphicsWindowCarbon::handleKeyboardEvent(EventRef theEvent)
     UInt32 rawkey;
     GetEventParameter (theEvent,kEventParamKeyCode,typeUInt32, NULL,sizeof(rawkey), NULL,&rawkey);
     
-    // osg::notify(osg::INFO) << "key code: " << rawkey << " modifiers: " << modifierKeys << std::endl;
+    // OSG_INFO << "key code: " << rawkey << " modifiers: " << modifierKeys << std::endl;
             
     UInt32 dataSize;
     /* jbw check return status so that we don't allocate a huge array */
@@ -763,15 +766,16 @@ bool GraphicsWindowCarbon::handleKeyboardEvent(EventRef theEvent)
         case kEventRawKeyDown:
         case kEventRawKeyRepeat:
         {
+            //OSG_INFO << "GraphicsWindowCarbon::keyPress Up" << std::endl;
             //getEventQueue()->getCurrentEventState()->setModKeyMask(modifierMask);
-            osg::notify(osg::INFO) << "GraphicsWindowCarbon::keyPress" << std::endl;
+            //OSG_INFO << "GraphicsWindowCarbon::keyPress" << std::endl;
             getEventQueue()->keyPress(keychar);
             break;
         }
         
         case kEventRawKeyUp:
         {                 
-            osg::notify(osg::INFO) << "GraphicsWindowCarbon::keyPress" << std::endl;
+            //OSG_INFO << "GraphicsWindowCarbon::keyPress" << std::endl;
             //getEventQueue()->getCurrentEventState()->setModKeyMask(modifierMask);
             getEventQueue()->keyRelease(keychar);
             break;
@@ -944,34 +948,24 @@ void GraphicsWindowCarbon::grabFocus()
 void GraphicsWindowCarbon::grabFocusIfPointerInWindow()
 {
    // TODO: implement
-   osg::notify(osg::ALWAYS) << "GraphicsWindowCarbon::grabFocusIfPointerInWindow" << std::endl;
+   OSG_NOTIFY(osg::ALWAYS) << "GraphicsWindowCarbon::grabFocusIfPointerInWindow: not implemented" << std::endl;
 }
 
 
 void GraphicsWindowCarbon::useCursor(bool cursorOn)
 {
-
     if (_traits.valid())
         _traits->useCursor = cursorOn;
     DarwinWindowingSystemInterface* wsi = dynamic_cast<DarwinWindowingSystemInterface*>(osg::GraphicsContext::getWindowingSystemInterface());
     if (wsi == NULL) {
-        osg::notify(osg::WARN) << "GraphicsWindowCarbon::useCursor :: could not get OSXCarbonWindowingSystemInterface" << std::endl;
+        OSG_WARN << "GraphicsWindowCarbon::useCursor: could not get OSXCarbonWindowingSystemInterface" << std::endl;
         return;
     }
     
     CGDirectDisplayID displayId = wsi->getDisplayID((*_traits));
-    CGDisplayErr err = kCGErrorSuccess;
-    switch (cursorOn)
-    {
-        case true:
-            err = CGDisplayShowCursor(displayId);
-            break;
-        case false:
-            err = CGDisplayHideCursor(displayId);
-            break;
-    }
+    CGDisplayErr err = (cursorOn ? CGDisplayShowCursor(displayId) : CGDisplayHideCursor(displayId));
     if (err != kCGErrorSuccess) {
-        osg::notify(osg::WARN) << "GraphicsWindowCarbon::useCursor failed with " << err << std::endl;
+        OSG_WARN << "GraphicsWindowCarbon::useCursor: failed with " << err << std::endl;
     }
 }
 
@@ -979,9 +973,10 @@ void GraphicsWindowCarbon::useCursor(bool cursorOn)
 // FIXME: I used deprecated functions, but don't know if there are any substitutable newer functions...
 void GraphicsWindowCarbon::setCursor(MouseCursor mouseCursor)
 {
-    UInt32 cursor;
     if (_currentCursor == mouseCursor)
       return;
+
+    UInt32 cursor;
     switch (mouseCursor) 
     {
         case NoCursor:
@@ -1005,7 +1000,7 @@ void GraphicsWindowCarbon::setCursor(MouseCursor mouseCursor)
             break;
         default:
             cursor = kThemeArrowCursor;
-            osg::notify(osg::WARN) << "GraphicsWindowCarbon::setCursor doesn't implement cursor: type = " << mouseCursor << std::endl;
+            OSG_WARN << "GraphicsWindowCarbon::setCursor doesn't implement cursor: type = " << mouseCursor << std::endl;
     }
     
     _currentCursor = mouseCursor;
@@ -1013,6 +1008,12 @@ void GraphicsWindowCarbon::setCursor(MouseCursor mouseCursor)
     ShowCursor();
 }
 
+void GraphicsWindowCarbon::setSyncToVBlank(bool on)
+{
+    if (_traits.valid()) {
+        _traits->vsync = on;
+    }
+}
 
 void GraphicsWindowCarbon::setWindowName (const std::string& name) 
 {
@@ -1027,10 +1028,16 @@ void GraphicsWindowCarbon::setWindowName (const std::string& name)
 
 void GraphicsWindowCarbon::requestWarpPointer(float x,float y)
 {
+    if (!_realized)
+    {
+        OSG_INFO<<"GraphicsWindowCarbon::requestWarpPointer() - Window not realized; cannot warp pointer, screenNum="<< _traits->screenNum<<std::endl;
+        return;
+    }
 
     DarwinWindowingSystemInterface* wsi = dynamic_cast<DarwinWindowingSystemInterface*>(osg::GraphicsContext::getWindowingSystemInterface());
-    if (wsi == NULL) {
-        osg::notify(osg::WARN) << "GraphicsWindowCarbon::useCursor :: could not get OSXCarbonWindowingSystemInterface" << std::endl;
+    if (wsi == NULL)
+    {
+        OSG_WARN << "GraphicsWindowCarbon::useCursor: could not get OSXCarbonWindowingSystemInterface" << std::endl;
         return;
     }
 
