@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
 *
-* This library is open source and may be redistributed and/or modified under  
-* the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+* This library is open source and may be redistributed and/or modified under
+* the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
-* 
+*
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
 */
 
@@ -48,19 +48,20 @@ class WindowCaptureCallback : public osg::Camera::DrawCallback
             END_FRAME
         };
 
-        WindowCaptureCallback(Mode mode, FramePosition position, GLenum readBuffer);
+        WindowCaptureCallback(int numFrames, Mode mode, FramePosition position, GLenum readBuffer);
 
         FramePosition getFramePosition() const { return _position; }
 
         void setCaptureOperation(ScreenCaptureHandler::CaptureOperation* operation);
         ScreenCaptureHandler::CaptureOperation* getCaptureOperation() { return _contextDataMap.begin()->second->_captureOperation.get(); }
 
+        void setFramesToCapture(int numFrames) { _numFrames = numFrames; }
+        int getFramesToCapture() const { return _numFrames; }
+
         virtual void operator () (osg::RenderInfo& renderInfo) const;
 
         struct OSGVIEWER_EXPORT ContextData : public osg::Referenced
         {
-            static unsigned int COUNTER;
-
             ContextData(osg::GraphicsContext* gc, Mode mode, GLenum readBuffer);
 
             void getSize(osg::GraphicsContext* gc, int& width, int& height);
@@ -71,10 +72,10 @@ class WindowCaptureCallback : public osg::Camera::DrawCallback
                                osg::Timer_t tick_afterCaptureOperation,
                                unsigned int dataSize);
 
-            void read();            
+            void read();
             void readPixels();
-            void singlePBO(osg::BufferObject::Extensions* ext);
-            void multiPBO(osg::BufferObject::Extensions* ext);
+            void singlePBO(osg::GLBufferObject::Extensions* ext);
+            void multiPBO(osg::GLBufferObject::Extensions* ext);
 
             typedef std::vector< osg::ref_ptr<osg::Image> >             ImageBuffer;
             typedef std::vector< GLuint > PBOBuffer;
@@ -112,21 +113,20 @@ class WindowCaptureCallback : public osg::Camera::DrawCallback
         ContextData* createContextData(osg::GraphicsContext* gc) const;
         ContextData* getContextData(osg::GraphicsContext* gc) const;
 
-        Mode                        _mode;        
+        Mode                        _mode;
         FramePosition               _position;
         GLenum                      _readBuffer;
         mutable OpenThreads::Mutex  _mutex;
         mutable ContextDataMap      _contextDataMap;
+        mutable int                 _numFrames;
 
         osg::ref_ptr<ScreenCaptureHandler::CaptureOperation> _defaultCaptureOperation;
 };
 
 
-unsigned int WindowCaptureCallback::ContextData::COUNTER = 0;
-
 WindowCaptureCallback::ContextData::ContextData(osg::GraphicsContext* gc, Mode mode, GLenum readBuffer)
     : _gc(gc),
-      _index(COUNTER++),
+      _index(_gc->getState()->getContextID()),
       _mode(mode),
       _readBuffer(readBuffer),
       _pixelFormat(GL_RGBA),
@@ -145,26 +145,26 @@ WindowCaptureCallback::ContextData::ContextData(osg::GraphicsContext* gc, Mode m
       _previousFrameTick(0)
 {
     _previousFrameTick = osg::Timer::instance()->tick();
-    
+
     osg::NotifySeverity level = osg::INFO;
 
     if (gc->getTraits())
     {
         if (gc->getTraits()->alpha)
         {
-            osg::notify(level)<<"ScreenCaptureHandler: Selected GL_RGBA read back format"<<std::endl;
+            OSG_NOTIFY(level)<<"ScreenCaptureHandler: Selected GL_RGBA read back format"<<std::endl;
             _pixelFormat = GL_RGBA;
         }
-        else 
+        else
         {
-            osg::notify(level)<<"ScreenCaptureHandler: Selected GL_RGB read back format"<<std::endl;
-            _pixelFormat = GL_RGB; 
+            OSG_NOTIFY(level)<<"ScreenCaptureHandler: Selected GL_RGB read back format"<<std::endl;
+            _pixelFormat = GL_RGB;
         }
     }
 
     getSize(gc, _width, _height);
 
-    //osg::notify(osg::NOTICE)<<"Window size "<<_width<<", "<<_height<<std::endl;
+    //OSG_NOTICE<<"Window size "<<_width<<", "<<_height<<std::endl;
 
     // single buffered image
     _imageBuffer.push_back(new osg::Image);
@@ -173,25 +173,25 @@ WindowCaptureCallback::ContextData::ContextData(osg::GraphicsContext* gc, Mode m
     switch(_mode)
     {
         case(READ_PIXELS):
-            osg::notify(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, without PixelBufferObject."<<std::endl;
+            OSG_NOTIFY(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, without PixelBufferObject."<<std::endl;
             break;
-        case(SINGLE_PBO): 
-            osg::notify(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, with a single PixelBufferObject."<<std::endl;
-            _pboBuffer.push_back(0); 
+        case(SINGLE_PBO):
+            OSG_NOTIFY(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, with a single PixelBufferObject."<<std::endl;
+            _pboBuffer.push_back(0);
             break;
-        case(DOUBLE_PBO): 
-            osg::notify(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, with a double buffer PixelBufferObject."<<std::endl;
-            _pboBuffer.push_back(0); 
-            _pboBuffer.push_back(0); 
+        case(DOUBLE_PBO):
+            OSG_NOTIFY(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, with a double buffer PixelBufferObject."<<std::endl;
+            _pboBuffer.push_back(0);
+            _pboBuffer.push_back(0);
             break;
-        case(TRIPLE_PBO): 
-            osg::notify(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, with a triple buffer PixelBufferObject."<<std::endl;
-            _pboBuffer.push_back(0); 
-            _pboBuffer.push_back(0); 
-            _pboBuffer.push_back(0); 
+        case(TRIPLE_PBO):
+            OSG_NOTIFY(level)<<"ScreenCaptureHandler: Reading window using glReadPixels, with a triple buffer PixelBufferObject."<<std::endl;
+            _pboBuffer.push_back(0);
+            _pboBuffer.push_back(0);
+            _pboBuffer.push_back(0);
             break;
         default:
-            break;                                
+            break;
     }
 }
 
@@ -220,7 +220,7 @@ void WindowCaptureCallback::ContextData::updateTimings(osg::Timer_t tick_start,
 
 void WindowCaptureCallback::ContextData::read()
 {
-    osg::BufferObject::Extensions* ext = osg::BufferObject::getExtensions(_gc->getState()->getContextID(),true);
+    osg::GLBufferObject::Extensions* ext = osg::GLBufferObject::getExtensions(_gc->getState()->getContextID(),true);
 
     if (ext->isPBOSupported() && !_pboBuffer.empty())
     {
@@ -249,7 +249,7 @@ void WindowCaptureCallback::ContextData::readPixels()
     getSize(_gc, width, height);
     if (width!=_width || _height!=height)
     {
-        //osg::notify(osg::NOTICE)<<"   Window resized "<<width<<", "<<height<<std::endl;
+        //OSG_NOTICE<<"   Window resized "<<width<<", "<<height<<std::endl;
         _width = width;
         _height = height;
     }
@@ -277,7 +277,7 @@ void WindowCaptureCallback::ContextData::readPixels()
     _currentPboIndex = nextPboIndex;
 }
 
-void WindowCaptureCallback::ContextData::singlePBO(osg::BufferObject::Extensions* ext)
+void WindowCaptureCallback::ContextData::singlePBO(osg::GLBufferObject::Extensions* ext)
 {
     unsigned int nextImageIndex = (_currentImageIndex+1)%_imageBuffer.size();
 
@@ -285,36 +285,36 @@ void WindowCaptureCallback::ContextData::singlePBO(osg::BufferObject::Extensions
     getSize(_gc, width, height);
     if (width!=_width || _height!=height)
     {
-        //osg::notify(osg::NOTICE)<<"   Window resized "<<width<<", "<<height<<std::endl;
+        //OSG_NOTICE<<"   Window resized "<<width<<", "<<height<<std::endl;
         _width = width;
         _height = height;
     }
 
     GLuint& pbo = _pboBuffer[0];
-    
+
     osg::Image* image = _imageBuffer[_currentImageIndex].get();
-    if (image->s() != _width || 
+    if (image->s() != _width ||
         image->t() != _height)
     {
-        //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: Allocating image "<<std::endl;
+        //OSG_NOTICE<<"ScreenCaptureHandler: Allocating image "<<std::endl;
         image->allocateImage(_width, _height, 1, _pixelFormat, _type);
-        
+
         if (pbo!=0)
         {
-            //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: deleting pbo "<<pbo<<std::endl;
+            //OSG_NOTICE<<"ScreenCaptureHandler: deleting pbo "<<pbo<<std::endl;
             ext->glDeleteBuffers (1, &pbo);
             pbo = 0;
         }
     }
-    
-    
+
+
     if (pbo==0)
     {
         ext->glGenBuffers(1, &pbo);
         ext->glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, pbo);
         ext->glBufferData(GL_PIXEL_PACK_BUFFER_ARB, image->getTotalSizeInBytes(), 0, GL_STREAM_READ);
 
-        //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: Generating pbo "<<pbo<<std::endl;
+        //OSG_NOTICE<<"ScreenCaptureHandler: Generating pbo "<<pbo<<std::endl;
     }
     else
     {
@@ -352,7 +352,7 @@ void WindowCaptureCallback::ContextData::singlePBO(osg::BufferObject::Extensions
     _currentImageIndex = nextImageIndex;
 }
 
-void WindowCaptureCallback::ContextData::multiPBO(osg::BufferObject::Extensions* ext)
+void WindowCaptureCallback::ContextData::multiPBO(osg::GLBufferObject::Extensions* ext)
 {
     unsigned int nextImageIndex = (_currentImageIndex+1)%_imageBuffer.size();
     unsigned int nextPboIndex = (_currentPboIndex+1)%_pboBuffer.size();
@@ -361,37 +361,37 @@ void WindowCaptureCallback::ContextData::multiPBO(osg::BufferObject::Extensions*
     getSize(_gc, width, height);
     if (width!=_width || _height!=height)
     {
-        //osg::notify(osg::NOTICE)<<"   Window resized "<<width<<", "<<height<<std::endl;
+        //OSG_NOTICE<<"   Window resized "<<width<<", "<<height<<std::endl;
         _width = width;
         _height = height;
     }
 
     GLuint& copy_pbo = _pboBuffer[_currentPboIndex];
     GLuint& read_pbo = _pboBuffer[nextPboIndex];
-    
+
     osg::Image* image = _imageBuffer[_currentImageIndex].get();
-    if (image->s() != _width || 
+    if (image->s() != _width ||
         image->t() != _height)
     {
-        //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: Allocating image "<<std::endl;
+        //OSG_NOTICE<<"ScreenCaptureHandler: Allocating image "<<std::endl;
         image->allocateImage(_width, _height, 1, _pixelFormat, _type);
-        
+
         if (read_pbo!=0)
         {
-            //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: deleting pbo "<<read_pbo<<std::endl;
+            //OSG_NOTICE<<"ScreenCaptureHandler: deleting pbo "<<read_pbo<<std::endl;
             ext->glDeleteBuffers (1, &read_pbo);
             read_pbo = 0;
         }
 
         if (copy_pbo!=0)
         {
-            //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: deleting pbo "<<copy_pbo<<std::endl;
+            //OSG_NOTICE<<"ScreenCaptureHandler: deleting pbo "<<copy_pbo<<std::endl;
             ext->glDeleteBuffers (1, &copy_pbo);
             copy_pbo = 0;
         }
     }
-    
-    
+
+
     bool doCopy = copy_pbo!=0;
     if (copy_pbo==0)
     {
@@ -399,7 +399,7 @@ void WindowCaptureCallback::ContextData::multiPBO(osg::BufferObject::Extensions*
         ext->glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, copy_pbo);
         ext->glBufferData(GL_PIXEL_PACK_BUFFER_ARB, image->getTotalSizeInBytes(), 0, GL_STREAM_READ);
 
-        //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: Generating pbo "<<read_pbo<<std::endl;
+        //OSG_NOTICE<<"ScreenCaptureHandler: Generating pbo "<<read_pbo<<std::endl;
     }
 
     if (read_pbo==0)
@@ -408,7 +408,7 @@ void WindowCaptureCallback::ContextData::multiPBO(osg::BufferObject::Extensions*
         ext->glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, read_pbo);
         ext->glBufferData(GL_PIXEL_PACK_BUFFER_ARB, image->getTotalSizeInBytes(), 0, GL_STREAM_READ);
 
-        //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: Generating pbo "<<read_pbo<<std::endl;
+        //OSG_NOTICE<<"ScreenCaptureHandler: Generating pbo "<<read_pbo<<std::endl;
     }
     else
     {
@@ -441,22 +441,22 @@ void WindowCaptureCallback::ContextData::multiPBO(osg::BufferObject::Extensions*
             (*_captureOperation)(*image, _index);
         }
     }
-    
+
     ext->glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, 0);
 
     osg::Timer_t tick_afterMemCpy = osg::Timer::instance()->tick();
-    
+
     updateTimings(tick_start, tick_afterReadPixels, tick_afterMemCpy, tick_afterMemCpy, image->getTotalSizeInBytes());
 
     _currentImageIndex = nextImageIndex;
     _currentPboIndex = nextPboIndex;
 }
 
-
-WindowCaptureCallback::WindowCaptureCallback(Mode mode, FramePosition position, GLenum readBuffer)
+WindowCaptureCallback::WindowCaptureCallback(int numFrames, Mode mode, FramePosition position, GLenum readBuffer)
     : _mode(mode),
       _position(position),
-      _readBuffer(readBuffer)
+      _readBuffer(readBuffer),
+      _numFrames(numFrames)
 {
 }
 
@@ -472,7 +472,7 @@ WindowCaptureCallback::ContextData* WindowCaptureCallback::getContextData(osg::G
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
     osg::ref_ptr<ContextData>& data = _contextDataMap[gc];
     if (!data) data = createContextData(gc);
-    
+
     return data.get();
 }
 
@@ -490,21 +490,30 @@ void WindowCaptureCallback::setCaptureOperation(ScreenCaptureHandler::CaptureOpe
 
 void WindowCaptureCallback::operator () (osg::RenderInfo& renderInfo) const
 {
+#if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
     glReadBuffer(_readBuffer);
+#endif
 
     osg::GraphicsContext* gc = renderInfo.getState()->getGraphicsContext();
     osg::ref_ptr<ContextData> cd = getContextData(gc);
     cd->read();
 
-    // Since we just want to take one screenshot, the callback must remove 
-    // itself when it's done.
-    if (_position == START_FRAME)
-        renderInfo.getCurrentCamera()->setInitialDrawCallback(0);
-    if (_position == END_FRAME)
-        renderInfo.getCurrentCamera()->setFinalDrawCallback(0);
+    // If _numFrames is > 0 it means capture that number of frames.
+    if (_numFrames > 0)
+    {
+        --_numFrames;
+        if (_numFrames == 0)
+        {
+            // the callback must remove itself when it's done.
+            if (_position == START_FRAME)
+                renderInfo.getCurrentCamera()->setInitialDrawCallback(0);
+            if (_position == END_FRAME)
+                renderInfo.getCurrentCamera()->setFinalDrawCallback(0);
+        }
+    }
 
     int prec = osg::notify(osg::INFO).precision(5);
-    osg::notify(osg::INFO) << "ScreenCaptureHandler: "
+    OSG_INFO << "ScreenCaptureHandler: "
                            << "copy="      << (cd->_timeForFullCopy*1000.0f)             << "ms, "
                            << "operation=" << (cd->_timeForCaptureOperation*1000.0f)     << "ms, "
                            << "total="     << (cd->_timeForFullCopyAndOperation*1000.0f) << std::endl;
@@ -519,9 +528,9 @@ void WindowCaptureCallback::operator () (osg::RenderInfo& renderInfo) const
 //
 //  ScreenCaptureHandler::WriteToFile
 //
-ScreenCaptureHandler::WriteToFile::WriteToFile(const std::string& filename, 
-                                                         const std::string& extension,
-                                                         SavePolicy savePolicy)
+ScreenCaptureHandler::WriteToFile::WriteToFile(const std::string& filename,
+                                               const std::string& extension,
+                                               SavePolicy savePolicy)
     : _filename(filename), _extension(extension), _savePolicy(savePolicy)
 {
 }
@@ -532,8 +541,11 @@ void ScreenCaptureHandler::WriteToFile::operator () (const osg::Image& image, co
     {
         if (_contextSaveCounter.size() <= context_id)
         {
+            unsigned int oldSize = _contextSaveCounter.size();
             _contextSaveCounter.resize(context_id + 1);
-            _contextSaveCounter[context_id] = 0;
+            // Initialize all new values to 0 since context ids may not be consecutive.
+            for (unsigned int i = oldSize; i <= context_id; i++)
+                _contextSaveCounter[i] = 0;
         }
     }
 
@@ -547,7 +559,7 @@ void ScreenCaptureHandler::WriteToFile::operator () (const osg::Image& image, co
 
     osgDB::writeImageFile(image, filename.str());
 
-    osg::notify(osg::INFO)<<"ScreenCaptureHandler: Taking a screenshot, saved as '"<<filename.str()<<"'"<<std::endl;
+    OSG_INFO<<"ScreenCaptureHandler: Taking a screenshot, saved as '"<<filename.str()<<"'"<<std::endl;
 
     if (_savePolicy == SEQUENTIAL_NUMBER)
     {
@@ -560,14 +572,18 @@ void ScreenCaptureHandler::WriteToFile::operator () (const osg::Image& image, co
 //
 //  ScreenCaptureHandler
 //
-ScreenCaptureHandler::ScreenCaptureHandler(CaptureOperation* defaultOperation)
-    : _keyEventTakeScreenShot('c'),
-      _callback(new WindowCaptureCallback(
-                                          WindowCaptureCallback::READ_PIXELS, 
-//                                          WindowCaptureCallback::SINGLE_PBO, 
-//                                          WindowCaptureCallback::DOUBLE_PBO, 
-//                                          WindowCaptureCallback::TRIPLE_PBO, 
-                                          WindowCaptureCallback::END_FRAME, GL_BACK))
+ScreenCaptureHandler::ScreenCaptureHandler(CaptureOperation* defaultOperation,
+                                           int numFrames)
+    : _startCapture(false),
+      _stopCapture(false),
+      _keyEventTakeScreenShot('c'),
+      _keyEventToggleContinuousCapture('C'),
+      _callback(new WindowCaptureCallback( numFrames,
+                                           WindowCaptureCallback::READ_PIXELS,
+//                                          WindowCaptureCallback::SINGLE_PBO,
+//                                          WindowCaptureCallback::DOUBLE_PBO,
+//                                          WindowCaptureCallback::TRIPLE_PBO,
+                                           WindowCaptureCallback::END_FRAME, GL_BACK))
 {
     if (defaultOperation)
         setCaptureOperation(defaultOperation);
@@ -577,35 +593,68 @@ ScreenCaptureHandler::ScreenCaptureHandler(CaptureOperation* defaultOperation)
 
 void ScreenCaptureHandler::setCaptureOperation(CaptureOperation* operation)
 {
-    static_cast<WindowCaptureCallback*>(_callback.get())->setCaptureOperation(operation);
+    WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+    callback->setCaptureOperation(operation);
 }
 
 ScreenCaptureHandler::CaptureOperation* ScreenCaptureHandler::getCaptureOperation() const
 {
-    return static_cast<WindowCaptureCallback*>(_callback.get())->getCaptureOperation();
+    WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+    return callback->getCaptureOperation();
 }
 
-
 void ScreenCaptureHandler::addCallbackToViewer(osgViewer::ViewerBase& viewer)
 {
-    // Select either the first or the last active camera, depending on the 
+    osg::Camera* camera = findAppropriateCameraForCallback(viewer);
+    if (!camera) return;
+
+    WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+    if (callback && callback->getFramePosition() == WindowCaptureCallback::START_FRAME)
+    {
+        camera->setInitialDrawCallback(_callback.get());
+    }
+    else
+    {
+        camera->setFinalDrawCallback(_callback.get());
+    }
+}
+
+void ScreenCaptureHandler::removeCallbackFromViewer(osgViewer::ViewerBase& viewer)
+{
+    osg::Camera* camera = findAppropriateCameraForCallback(viewer);
+    if (!camera) return;
+
+    WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+    if (callback && callback->getFramePosition() == WindowCaptureCallback::START_FRAME)
+    {
+        camera->setInitialDrawCallback(0);
+    }
+    else
+    {
+        camera->setFinalDrawCallback(0);
+    }
+}
+
+osg::Camera* ScreenCaptureHandler::findAppropriateCameraForCallback(osgViewer::ViewerBase& viewer)
+{
+    // Select either the first or the last active camera, depending on the
     // frame position set in the callback.
     // One case where testing the node mask is important is when the stats
-    // handler has been initialized, but stats are not displayed. In that 
+    // handler has been initialized, but stats are not displayed. In that
     // case, there is a post render camera on the viewer, but its node mask
     // is zero, so the callback added to that camera would never be called.
     WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
 
     if (callback->getFramePosition() == WindowCaptureCallback::START_FRAME)
     {
-        osgViewer::ViewerBase::Windows windows;
-        viewer.getWindows(windows);
-        for(osgViewer::ViewerBase::Windows::iterator itr = windows.begin();
-            itr != windows.end();
+        osgViewer::ViewerBase::Contexts contexts;
+        viewer.getContexts(contexts);
+        for(osgViewer::ViewerBase::Contexts::iterator itr = contexts.begin();
+            itr != contexts.end();
             ++itr)
         {
-            osgViewer::GraphicsWindow* window = *itr;
-            osg::GraphicsContext::Cameras& cameras = window->getCameras();
+            osg::GraphicsContext* context = *itr;
+            osg::GraphicsContext::Cameras& cameras = context->getCameras();
             osg::Camera* firstCamera = 0;
             for(osg::GraphicsContext::Cameras::iterator cam_itr = cameras.begin();
                 cam_itr != cameras.end();
@@ -634,26 +683,26 @@ void ScreenCaptureHandler::addCallbackToViewer(osgViewer::ViewerBase& viewer)
 
             if (firstCamera)
             {
-                //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: First camera "<<firstCamera<<std::endl;
+                //OSG_NOTICE<<"ScreenCaptureHandler: First camera "<<firstCamera<<std::endl;
 
-                firstCamera->setInitialDrawCallback(_callback.get());
+                return firstCamera;
             }
             else
             {
-                osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: No camera found"<<std::endl;
+                OSG_NOTICE<<"ScreenCaptureHandler: No camera found"<<std::endl;
             }
         }
     }
     else
-    {    
-        osgViewer::ViewerBase::Windows windows;
-        viewer.getWindows(windows);
-        for(osgViewer::ViewerBase::Windows::iterator itr = windows.begin();
-            itr != windows.end();
+    {
+        osgViewer::ViewerBase::Contexts contexts;
+        viewer.getContexts(contexts);
+        for(osgViewer::ViewerBase::Contexts::iterator itr = contexts.begin();
+            itr != contexts.end();
             ++itr)
         {
-            osgViewer::GraphicsWindow* window = *itr;
-            osg::GraphicsContext::Cameras& cameras = window->getCameras();
+            osg::GraphicsContext* context = *itr;
+            osg::GraphicsContext::Cameras& cameras = context->getCameras();
             osg::Camera* lastCamera = 0;
             for(osg::GraphicsContext::Cameras::iterator cam_itr = cameras.begin();
                 cam_itr != cameras.end();
@@ -682,16 +731,18 @@ void ScreenCaptureHandler::addCallbackToViewer(osgViewer::ViewerBase& viewer)
 
             if (lastCamera)
             {
-                //osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: Last camera "<<lastCamera<<std::endl;
+                //OSG_NOTICE<<"ScreenCaptureHandler: Last camera "<<lastCamera<<std::endl;
 
-                lastCamera->setFinalDrawCallback(_callback.get());
+                return lastCamera;
             }
             else
             {
-                osg::notify(osg::NOTICE)<<"ScreenCaptureHandler: No camera found"<<std::endl;
+                OSG_NOTICE<<"ScreenCaptureHandler: No camera found"<<std::endl;
             }
         }
     }
+
+    return 0;
 }
 
 // aa will point to an osgViewer::View, so we will take a screenshot
@@ -701,18 +752,60 @@ bool ScreenCaptureHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIAc
     osgViewer::ViewerBase* viewer = dynamic_cast<osgViewer::View*>(&aa)->getViewerBase();
     if (!viewer) return false;
 
-    if (ea.getHandled()) return false;
-
     switch(ea.getEventType())
     {
+        case (osgGA::GUIEventAdapter::FRAME):
+        {
+            // Booleans aren't the best way of doing this, but I want to do
+            // the actual adding here because I don't want to require
+            // startCapture() take a viewer as argument, which could not be
+            // the right one.
+            if (_startCapture)
+            {
+                // Start capturing with the currently set number of frames.
+                // If set to -1 it will capture continuously, if set to >0
+                // it will capture that number of frames.
+                _startCapture = false;
+                addCallbackToViewer(*viewer);
+            }
+            else if (_stopCapture)
+            {
+                _stopCapture = false;
+                removeCallbackFromViewer(*viewer);
+            }
+        }
+
         case(osgGA::GUIEventAdapter::KEYUP):
         {
             if (ea.getKey() == _keyEventTakeScreenShot)
             {
+                // Check that we will capture at least one frame.
+                // Just check for ==0, because >0 is means we're already
+                // capturing and <0 means it will capture all frames.
+                WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+                if (callback->getFramesToCapture() == 0)
+                {
+                    setFramesToCapture(1);
+                }
                 addCallbackToViewer(*viewer);
                 return true;
             }
 
+            if (ea.getKey() == _keyEventToggleContinuousCapture)
+            {
+                if (getFramesToCapture() < 0)
+                {
+                    setFramesToCapture(0);
+                    removeCallbackFromViewer(*viewer);
+                }
+                else
+                {
+                    setFramesToCapture(-1);
+                    addCallbackToViewer(*viewer);
+                }
+                return true;
+            }
+
             break;
         }
     default:
@@ -728,6 +821,33 @@ void ScreenCaptureHandler::captureNextFrame(osgViewer::ViewerBase& viewer)
     addCallbackToViewer(viewer);
 }
 
+/** Set the number of frames to capture. */
+void ScreenCaptureHandler::setFramesToCapture(int numFrames)
+{
+    WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+    callback->setFramesToCapture(numFrames);
+}
+
+/** Get the number of frames to capture. */
+int ScreenCaptureHandler::getFramesToCapture() const
+{
+    WindowCaptureCallback* callback = static_cast<WindowCaptureCallback*>(_callback.get());
+    return callback->getFramesToCapture();
+}
+
+/** Start capturing at the end of the next frame. */
+void ScreenCaptureHandler::startCapture()
+{
+    if (getFramesToCapture() != 0)
+        _startCapture = true;
+}
+
+/** Stop capturing. */
+void ScreenCaptureHandler::stopCapture()
+{
+    _stopCapture = true;
+}
+
 /** Get the keyboard and mouse usage of this manipulator.*/
 void ScreenCaptureHandler::getUsage(osg::ApplicationUsage& usage) const
 {
@@ -736,7 +856,12 @@ void ScreenCaptureHandler::getUsage(osg::ApplicationUsage& usage) const
         ostr<<char(_keyEventTakeScreenShot);
         usage.addKeyboardMouseBinding(ostr.str(),"Take screenshot.");
     }
+
+    {
+        std::ostringstream ostr;
+        ostr<<char(_keyEventToggleContinuousCapture);
+        usage.addKeyboardMouseBinding(ostr.str(),"Toggle continuous screen capture.");
+    }
 }
 
-
 }
