@@ -1,13 +1,13 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -33,25 +33,24 @@ static osg::ApplicationUsageProxy ViewerBase_e0(osg::ApplicationUsage::ENVIRONME
 static osg::ApplicationUsageProxy ViewerBase_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_THREADING <value>","Set the threading model using by Viewer, <value> can be SingleThreaded, CullDrawThreadPerContext, DrawThreadPerContext or CullThreadPerCameraDrawThreadPerContext.");
 static osg::ApplicationUsageProxy ViewerBase_e2(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_SCREEN <value>","Set the default screen that windows should open up on.");
 static osg::ApplicationUsageProxy ViewerBase_e3(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_WINDOW x y width height","Set the default window dimensions that windows should open up on.");
-
+static osg::ApplicationUsageProxy ViewerBase_e4(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_RUN_FRAME_SCHEME","Frame rate manage scheme that viewer run should use,  ON_DEMAND or CONTINUOUS (default).");
+static osg::ApplicationUsageProxy ViewerBase_e5(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_RUN_MAX_FRAME_RATE","Set the maximum number of frame as second that viewer run. 0.0 is default and disables an frame rate capping.");
 
 using namespace osgViewer;
 
 ViewerBase::ViewerBase():
     osg::Object(true)
 {
-    _firstFrame = true;
-    _done = false;
-    _keyEventSetsDone = osgGA::GUIEventAdapter::KEY_Escape;
-    _quitEventSetsDone = true;
-    _releaseContextAtEndOfFrameHint = true;
-    _threadingModel = AutomaticSelection;
-    _threadsRunning = false;
-    _endBarrierPosition = AfterSwapBuffers;
+    viewerBaseInit();
 }
 
-ViewerBase::ViewerBase(const ViewerBase& base):
+ViewerBase::ViewerBase(const ViewerBase&):
     osg::Object(true)
+{
+    viewerBaseInit();
+}
+
+void ViewerBase::viewerBaseInit()
 {
     _firstFrame = true;
     _done = false;
@@ -61,14 +60,32 @@ ViewerBase::ViewerBase(const ViewerBase& base):
     _threadingModel = AutomaticSelection;
     _threadsRunning = false;
     _endBarrierPosition = AfterSwapBuffers;
+    _requestRedraw = true;
+    _requestContinousUpdate = false;
+
+    _runFrameScheme = CONTINUOUS;
+    _runMaxFrameRate = 0.0f;
+
+    const char* str = getenv("OSG_RUN_FRAME_SCHEME");
+    if (str)
+    {
+        if      (strcmp(str, "ON_DEMAND")==0) _runFrameScheme = ON_DEMAND;
+        else if (strcmp(str, "CONTINUOUS")==0) _runFrameScheme = CONTINUOUS;
+    }
+
+    str = getenv("OSG_RUN_MAX_FRAME_RATE");
+    if (str)
+    {
+        _runMaxFrameRate = osg::asciiToDouble(str);
+    }
 }
 
 void ViewerBase::setThreadingModel(ThreadingModel threadingModel)
 {
     if (_threadingModel == threadingModel) return;
-    
+
     if (_threadsRunning) stopThreading();
-    
+
     _threadingModel = threadingModel;
 
     if (isRealized() && _threadingModel!=SingleThreaded) startThreading();
@@ -87,7 +104,7 @@ ViewerBase::ThreadingModel ViewerBase::suggestBestThreadingModel()
 
     Contexts contexts;
     getContexts(contexts);
-    
+
     if (contexts.empty()) return SingleThreaded;
 
 #if 0
@@ -109,7 +126,7 @@ ViewerBase::ThreadingModel ViewerBase::suggestBestThreadingModel()
         if (numProcessors==1) return SingleThreaded;
         else return DrawThreadPerContext;
     }
-    
+
 #if 1
     if (numProcessors >= static_cast<int>(cameras.size()+contexts.size()))
     {
@@ -133,30 +150,13 @@ void ViewerBase::setUpThreading()
             // we'll set processor affinity here to help single threaded apps
             // with multiple processor cores, and using the database pager.
             int numProcessors = OpenThreads::GetNumberOfProcessors();
-            bool affinity = numProcessors>1;    
-            if (affinity) 
+            bool affinity = numProcessors>1;
+            if (affinity)
             {
                 OpenThreads::SetProcessorAffinityOfCurrentThread(0);
 
                 Scenes scenes;
                 getScenes(scenes);
-                
-                for(Scenes::iterator itr = scenes.begin();
-                    itr != scenes.end();
-                    ++itr)
-                {
-                    Scene* scene = *itr;
-                    osgDB::DatabasePager* dp = scene->getDatabasePager();
-                    if (dp)
-                    {
-                        for(unsigned int i=0; i<dp->getNumDatabaseThreads(); ++i)
-                        {
-                            osgDB::DatabasePager::DatabaseThread* dt = dp->getDatabaseThread(i);
-                            dt->setProcessorAffinity(1);
-                        }
-                    }
-                }
-            
             }
         }
     }
@@ -164,15 +164,15 @@ void ViewerBase::setUpThreading()
     {
         if (!_threadsRunning) startThreading();
     }
-    
+
 }
 
 void ViewerBase::setEndBarrierPosition(BarrierPosition bp)
 {
     if (_endBarrierPosition == bp) return;
-    
+
     if (_threadsRunning) stopThreading();
-    
+
     _endBarrierPosition = bp;
 
     if (_threadingModel!=SingleThreaded) startThreading();
@@ -183,7 +183,7 @@ void ViewerBase::stopThreading()
 {
     if (!_threadsRunning) return;
 
-    osg::notify(osg::INFO)<<"ViewerBase::stopThreading() - stopping threading"<<std::endl;
+    OSG_INFO<<"ViewerBase::stopThreading() - stopping threading"<<std::endl;
 
     Contexts contexts;
     getContexts(contexts);
@@ -203,7 +203,7 @@ void ViewerBase::stopThreading()
         if (renderer) renderer->release();
     }
 
-    // delete all the graphics threads.    
+    // delete all the graphics threads.
     for(gcitr = contexts.begin();
         gcitr != contexts.end();
         ++gcitr)
@@ -211,7 +211,7 @@ void ViewerBase::stopThreading()
         (*gcitr)->setGraphicsThread(0);
     }
 
-    // delete all the camera threads.    
+    // delete all the camera threads.
     for(citr = cameras.begin();
         citr != cameras.end();
         ++citr)
@@ -238,15 +238,15 @@ void ViewerBase::stopThreading()
     _endRenderingDispatchBarrier = 0;
     _endDynamicDrawBlock = 0;
 
-    osg::notify(osg::INFO)<<"Viewer::stopThreading() - stopped threading."<<std::endl;
+    OSG_INFO<<"Viewer::stopThreading() - stopped threading."<<std::endl;
 }
 
 void ViewerBase::startThreading()
 {
     if (_threadsRunning) return;
-    
-    osg::notify(osg::INFO)<<"Viewer::startThreading() - starting threading"<<std::endl;
-    
+
+    OSG_INFO<<"Viewer::startThreading() - starting threading"<<std::endl;
+
     // release any context held by the main thread.
     releaseContext();
 
@@ -254,34 +254,34 @@ void ViewerBase::startThreading()
 
     Contexts contexts;
     getContexts(contexts);
-    
-    osg::notify(osg::INFO)<<"Viewer::startThreading() - contexts.size()="<<contexts.size()<<std::endl;
+
+    OSG_INFO<<"Viewer::startThreading() - contexts.size()="<<contexts.size()<<std::endl;
 
     Cameras cameras;
     getCameras(cameras);
-    
+
     unsigned int numThreadsOnStartBarrier = 0;
     unsigned int numThreadsOnEndBarrier = 0;
     switch(_threadingModel)
     {
-        case(SingleThreaded): 
+        case(SingleThreaded):
             numThreadsOnStartBarrier = 1;
             numThreadsOnEndBarrier = 1;
             return;
-        case(CullDrawThreadPerContext): 
+        case(CullDrawThreadPerContext):
             numThreadsOnStartBarrier = contexts.size()+1;
             numThreadsOnEndBarrier = contexts.size()+1;
             break;
-        case(DrawThreadPerContext): 
+        case(DrawThreadPerContext):
             numThreadsOnStartBarrier = 1;
             numThreadsOnEndBarrier = 1;
             break;
-        case(CullThreadPerCameraDrawThreadPerContext): 
+        case(CullThreadPerCameraDrawThreadPerContext):
             numThreadsOnStartBarrier = cameras.size()+1;
-            numThreadsOnEndBarrier = 1; 
+            numThreadsOnEndBarrier = 1;
             break;
         default:
-            osg::notify(osg::NOTICE)<<"Error: Threading model not selected"<<std::endl;
+            OSG_NOTICE<<"Error: Threading model not selected"<<std::endl;
             return;
     }
 
@@ -296,7 +296,7 @@ void ViewerBase::startThreading()
     {
         if ((*scitr)->getSceneData())
         {
-            osg::notify(osg::INFO)<<"Making scene thread safe"<<std::endl;
+            OSG_INFO<<"Making scene thread safe"<<std::endl;
 
             // make sure that existing scene graph objects are allocated with thread safe ref/unref
             (*scitr)->getSceneData()->setThreadSafeRefUnref(true);
@@ -305,10 +305,10 @@ void ViewerBase::startThreading()
             (*scitr)->getSceneData()->resizeGLObjectBuffers(osg::DisplaySettings::instance()->getMaxNumberOfGraphicsContexts());
         }
     }
-        
+
     int numProcessors = OpenThreads::GetNumberOfProcessors();
-    bool affinity = numProcessors>1;    
-    
+    bool affinity = numProcessors>1;
+
     Contexts::iterator citr;
 
     unsigned int numViewerDoubleBufferedRenderingOperation = 0;
@@ -335,19 +335,19 @@ void ViewerBase::startThreading()
         _endRenderingDispatchBarrier = 0;
         _endDynamicDrawBlock = 0;
     }
-    else if (_threadingModel==DrawThreadPerContext || 
+    else if (_threadingModel==DrawThreadPerContext ||
              _threadingModel==CullThreadPerCameraDrawThreadPerContext)
     {
         _startRenderingBarrier = 0;
         _endRenderingDispatchBarrier = 0;
         _endDynamicDrawBlock = new osg::EndOfDynamicDrawBlock(numViewerDoubleBufferedRenderingOperation);
-        
+
 #ifndef OSGUTIL_RENDERBACKEND_USE_REF_PTR
         if (!osg::Referenced::getDeleteHandler()) osg::Referenced::setDeleteHandler(new osg::DeleteHandler(2));
         else osg::Referenced::getDeleteHandler()->setNumFramesToRetainObjects(2);
 #endif
     }
-    
+
     if (numThreadsOnStartBarrier>1)
     {
         _startRenderingBarrier = new osg::BarrierOperation(numThreadsOnStartBarrier, osg::BarrierOperation::NO_OPERATION);
@@ -372,13 +372,13 @@ void ViewerBase::startThreading()
         ++citr, ++processNum)
     {
         osg::GraphicsContext* gc = (*citr);
-        
+
         if (!gc->isRealized())
         {
-            osg::notify(osg::INFO)<<"ViewerBase::startThreading() : Realizng window "<<gc<<std::endl;
+            OSG_INFO<<"ViewerBase::startThreading() : Realizng window "<<gc<<std::endl;
             gc->realize();
         }
-        
+
         gc->getState()->setDynamicObjectRenderingCompletedCallback(_endDynamicDrawBlock.get());
 
         // create the a graphics thread for this context
@@ -434,7 +434,7 @@ void ViewerBase::startThreading()
             Renderer* renderer = dynamic_cast<Renderer*>(camera->getRenderer());
             renderer->setGraphicsThreadDoesCull(false);
             camera->getCameraThread()->add(renderer);
-            
+
             if (_endRenderingDispatchBarrier.valid())
             {
                 // add the endRenderingDispatchBarrier
@@ -450,19 +450,19 @@ void ViewerBase::startThreading()
             osg::Camera* camera = *camItr;
             if (camera->getCameraThread() && !camera->getCameraThread()->isRunning())
             {
-                osg::notify(osg::INFO)<<"  camera->getCameraThread()-> "<<camera->getCameraThread()<<std::endl;
+                OSG_INFO<<"  camera->getCameraThread()-> "<<camera->getCameraThread()<<std::endl;
                 camera->getCameraThread()->startThread();
             }
         }
     }
 
-#if 0    
-    if (affinity) 
+#if 0
+    if (affinity)
     {
         OpenThreads::SetProcessorAffinityOfCurrentThread(0);
         if (_scene.valid() && _scene->getDatabasePager())
         {
-#if 0        
+#if 0
             _scene->getDatabasePager()->setProcessorAffinity(1);
 #else
             _scene->getDatabasePager()->setProcessorAffinity(0);
@@ -491,7 +491,7 @@ void ViewerBase::startThreading()
         osg::GraphicsContext* gc = (*citr);
         if (gc->getGraphicsThread() && !gc->getGraphicsThread()->isRunning())
         {
-            osg::notify(osg::INFO)<<"  gc->getGraphicsThread()->startThread() "<<gc->getGraphicsThread()<<std::endl;
+            OSG_INFO<<"  gc->getGraphicsThread()->startThread() "<<gc->getGraphicsThread()<<std::endl;
             gc->getGraphicsThread()->startThread();
             // OpenThreads::Thread::YieldCurrentThread();
         }
@@ -499,7 +499,7 @@ void ViewerBase::startThreading()
 
     _threadsRunning = true;
 
-    osg::notify(osg::INFO)<<"Set up threading"<<std::endl;
+    OSG_INFO<<"Set up threading"<<std::endl;
 }
 
 void ViewerBase::getWindows(Windows& windows, bool onlyValid)
@@ -508,7 +508,7 @@ void ViewerBase::getWindows(Windows& windows, bool onlyValid)
 
     Contexts contexts;
     getContexts(contexts, onlyValid);
-    
+
     for(Contexts::iterator itr = contexts.begin();
         itr != contexts.end();
         ++itr)
@@ -522,9 +522,11 @@ void ViewerBase::checkWindowStatus()
 {
     Contexts contexts;
     getContexts(contexts);
-    
-    // osg::notify(osg::NOTICE)<<"Viewer::checkWindowStatus() - "<<contexts.size()<<std::endl;
-    
+    checkWindowStatus(contexts);
+}
+
+void ViewerBase::checkWindowStatus(const Contexts& contexts)
+{
     if (contexts.size()==0)
     {
         _done = true;
@@ -537,7 +539,7 @@ void ViewerBase::addUpdateOperation(osg::Operation* operation)
     if (!operation) return;
 
     if (!_updateOperations) _updateOperations = new osg::OperationQueue;
-    
+
     _updateOperations->add(operation);
 }
 
@@ -548,7 +550,33 @@ void ViewerBase::removeUpdateOperation(osg::Operation* operation)
     if (_updateOperations.valid())
     {
         _updateOperations->remove(operation);
-    } 
+    }
+}
+
+void ViewerBase::setIncrementalCompileOperation(osgUtil::IncrementalCompileOperation* ico)
+{
+    if (_incrementalCompileOperation == ico) return;
+
+    Contexts contexts;
+    getContexts(contexts, false);
+
+    if (_incrementalCompileOperation.valid()) _incrementalCompileOperation->removeContexts(contexts);
+
+    // assign new operation
+    _incrementalCompileOperation = ico;
+
+    Scenes scenes;
+    getScenes(scenes,false);
+    for(Scenes::iterator itr = scenes.begin();
+        itr != scenes.end();
+        ++itr)
+    {
+        osgDB::DatabasePager* dp = (*itr)->getDatabasePager();
+        dp->setIncrementalCompileOperation(ico);
+    }
+
+
+    if (_incrementalCompileOperation) _incrementalCompileOperation->assignContexts(contexts);
 }
 
 int ViewerBase::run()
@@ -558,30 +586,38 @@ int ViewerBase::run()
         realize();
     }
 
-#if 0
-    while (!done())
-    {
-        frame();
-    }
-#else
+    const char* run_frame_count_str = getenv("OSG_RUN_FRAME_COUNT");
+    unsigned int runTillFrameNumber = run_frame_count_str==0 ? osg::UNINITIALIZED_FRAME_NUMBER : atoi(run_frame_count_str);
 
-    const char* str = getenv("OSG_RUN_FRAME_COUNT");
-    if (str)
+    while(!done() && (run_frame_count_str==0 || getViewerFrameStamp()->getFrameNumber()<runTillFrameNumber))
     {
-        int runTillFrameNumber = atoi(str);
-        while (!done() && getViewerFrameStamp()->getFrameNumber()<runTillFrameNumber)
+        double minFrameTime = _runMaxFrameRate>0.0 ? 1.0/_runMaxFrameRate : 0.0;
+        osg::Timer_t startFrameTick = osg::Timer::instance()->tick();
+        if (_runFrameScheme==ON_DEMAND)
+        {
+            if (checkNeedToDoFrame())
+            {
+                frame();
+            }
+            else
+            {
+                // we don't need to render a frame but we don't want to spin the run loop so make sure the minimum
+                // loop time is 1/100th of second, if not otherwise set, so enabling the frame microSleep below to
+                // avoid consume excessive CPU resources.
+                if (minFrameTime==0.0) minFrameTime=0.01;
+            }
+        }
+        else
         {
             frame();
         }
+
+        // work out if we need to force a sleep to hold back the frame rate
+        osg::Timer_t endFrameTick = osg::Timer::instance()->tick();
+        double frameTime = osg::Timer::instance()->delta_s(startFrameTick, endFrameTick);
+        if (frameTime < minFrameTime) OpenThreads::Thread::microSleep(static_cast<unsigned int>(1000000.0*(minFrameTime-frameTime)));
     }
-    else
-    {
-        while (!done())
-        {
-            frame();
-        }
-    }
-#endif    
+
     return 0;
 }
 
@@ -589,21 +625,21 @@ void ViewerBase::frame(double simulationTime)
 {
     if (_done) return;
 
-    // osg::notify(osg::NOTICE)<<std::endl<<"CompositeViewer::frame()"<<std::endl<<std::endl;
+    // OSG_NOTICE<<std::endl<<"CompositeViewer::frame()"<<std::endl<<std::endl;
 
     if (_firstFrame)
     {
         viewerInit();
-        
+
         if (!isRealized())
         {
             realize();
         }
-        
+
         _firstFrame = false;
     }
     advance(simulationTime);
-    
+
     eventTraversal();
     updateTraversal();
     renderingTraversals();
@@ -626,14 +662,16 @@ void ViewerBase::renderingTraversals()
             if (view)
             {
                 const osg::Matrixd& m = view->getCamera()->getInverseViewMatrix();
-                osg::notify(osg::NOTICE)<<"View "<<view<<", Master Camera position("<<m.getTrans()<<"), rotation("<<m.getRotate()<<")"<<std::endl;
+                OSG_NOTICE<<"View "<<view<<", Master Camera position("<<m.getTrans()<<"), rotation("<<m.getRotate()<<")"<<std::endl;
             }
         }
     }
-        
-    // check to see if windows are still valid
-    checkWindowStatus();
 
+    Contexts contexts;
+    getContexts(contexts);
+
+    // check to see if windows are still valid
+    checkWindowStatus(contexts);
     if (_done) return;
 
     double beginRenderingTraversals = elapsedTime();
@@ -642,8 +680,8 @@ void ViewerBase::renderingTraversals()
 
     if (getViewerStats() && getViewerStats()->collectStats("scene"))
     {
-        int frameNumber = frameStamp ? frameStamp->getFrameNumber() : 0;
-    
+        unsigned int frameNumber = frameStamp ? frameStamp->getFrameNumber() : 0;
+
         Views views;
         getViews(views);
         for(Views::iterator vitr = views.begin();
@@ -651,14 +689,14 @@ void ViewerBase::renderingTraversals()
             ++vitr)
         {
             View* view = *vitr;
-            osg::Stats* stats = view->getStats();            
+            osg::Stats* stats = view->getStats();
             osg::Node* sceneRoot = view->getSceneData();
             if (sceneRoot && stats)
             {
                 osgUtil::StatsVisitor statsVisitor;
                 sceneRoot->accept(statsVisitor);
                 statsVisitor.totalUpStats();
-                
+
                 unsigned int unique_primitives = 0;
                 osgUtil::Statistics::PrimitiveCountMap::iterator pcmitr;
                 for(pcmitr = statsVisitor._uniqueStats.GetPrimitivesBegin();
@@ -703,7 +741,7 @@ void ViewerBase::renderingTraversals()
 
     Scenes scenes;
     getScenes(scenes);
-    
+
     for(Scenes::iterator sitr = scenes.begin();
         sitr != scenes.end();
         ++sitr)
@@ -714,33 +752,30 @@ void ViewerBase::renderingTraversals()
         {
             dp->signalBeginFrame(frameStamp);
         }
-        
+
         if (scene->getSceneData())
         {
-            // fire off a build of the bounding volumes while we 
+            // fire off a build of the bounding volumes while we
             // are still running single threaded.
             scene->getSceneData()->getBound();
         }
     }
 
-    // osg::notify(osg::NOTICE)<<std::endl<<"Start frame"<<std::endl;
-    
+    // OSG_NOTICE<<std::endl<<"Start frame"<<std::endl;
 
-    Contexts contexts;
-    getContexts(contexts);
 
     Cameras cameras;
     getCameras(cameras);
-    
+
     Contexts::iterator itr;
-    
+
     bool doneMakeCurrentInThisThread = false;
 
     if (_endDynamicDrawBlock.valid())
     {
         _endDynamicDrawBlock->reset();
     }
-    
+
     // dispatch the rendering threads
     if (_startRenderingBarrier.valid()) _startRenderingBarrier->block();
 
@@ -767,13 +802,13 @@ void ViewerBase::renderingTraversals()
         if (_done) return;
         if (!((*itr)->getGraphicsThread()) && (*itr)->valid())
         {
-            doneMakeCurrentInThisThread = true; 
+            doneMakeCurrentInThisThread = true;
             makeCurrent(*itr);
             (*itr)->runOperations();
         }
     }
 
-    // osg::notify(osg::NOTICE)<<"Joing _endRenderingDispatchBarrier block "<<_endRenderingDispatchBarrier.get()<<std::endl;
+    // OSG_NOTICE<<"Joing _endRenderingDispatchBarrier block "<<_endRenderingDispatchBarrier.get()<<std::endl;
 
     // wait till the rendering dispatch is done.
     if (_endRenderingDispatchBarrier.valid()) _endRenderingDispatchBarrier->block();
@@ -785,8 +820,8 @@ void ViewerBase::renderingTraversals()
         if (_done) return;
 
         if (!((*itr)->getGraphicsThread()) && (*itr)->valid())
-        { 
-            doneMakeCurrentInThisThread = true; 
+        {
+            doneMakeCurrentInThisThread = true;
             makeCurrent(*itr);
             (*itr)->swapBuffers();
         }
@@ -805,16 +840,16 @@ void ViewerBase::renderingTraversals()
     }
 
     // wait till the dynamic draw is complete.
-    if (_endDynamicDrawBlock.valid()) 
+    if (_endDynamicDrawBlock.valid())
     {
         // osg::Timer_t startTick = osg::Timer::instance()->tick();
         _endDynamicDrawBlock->block();
-        // osg::notify(osg::NOTICE)<<"Time waiting "<<osg::Timer::instance()->delta_m(startTick, osg::Timer::instance()->tick())<<std::endl;;
+        // OSG_NOTICE<<"Time waiting "<<osg::Timer::instance()->delta_m(startTick, osg::Timer::instance()->tick())<<std::endl;;
     }
-    
+
     if (_releaseContextAtEndOfFrameHint && doneMakeCurrentInThisThread)
     {
-        //osg::notify(osg::NOTICE)<<"Doing release context"<<std::endl;
+        //OSG_NOTICE<<"Doing release context"<<std::endl;
         releaseContext();
     }
 
@@ -827,5 +862,7 @@ void ViewerBase::renderingTraversals()
         getViewerStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals end time ", endRenderingTraversals);
         getViewerStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals time taken", endRenderingTraversals-beginRenderingTraversals);
     }
+
+    _requestRedraw = false;
 }
 
